(in-package :cl-info)
(let (
(deffn-defvr-pairs '(
; CONTENT: (<INDEX TOPIC> . (<FILENAME> <BYTE OFFSET> <LENGTH IN CHARACTERS> <NODE NAME>))
("!" . ("maxima.info-2" 52378 3782 "Funciones combinatorias"))
("!!" . ("maxima.info-2" 50349 731 "Funciones combinatorias"))
("#" . ("maxima.info-1" 197779 773 "Operadores para ecuaciones"))
("%" . ("maxima.info-1" 49410 730 "Funciones y variables para la línea de comandos"))
("%%" . ("maxima.info-1" 50141 1685 "Funciones y variables para la línea de comandos"))
("%and" . ("maxima.info-8" 125097 2725 "Funciones y variables para to_poly_solve"))
("%e" . ("maxima.info-1" 116272 224 "Funciones y variables para Constantes"))
("%e_to_numlog" . ("maxima.info-2" 57835 410 "Funciones radicales exponenciales y logarítmicas"))
("%edispflag" . ("maxima.info-1" 73700 621 "Funciones y variables para la impresión"))
("%emode" . ("maxima.info-2" 58246 606 "Funciones radicales exponenciales y logarítmicas"))
("%enumer" . ("maxima.info-2" 58853 345 "Funciones radicales exponenciales y logarítmicas"))
("%f" . ("maxima.info-3" 38771 183 "Funciones hipergeométricas"))
("%gamma" . ("maxima.info-1" 116723 88 "Funciones y variables para Constantes"))
("%i" . ("maxima.info-1" 116497 83 "Funciones y variables para Constantes"))
("%iargs" . ("maxima.info-2" 71211 1080 "Funciones y variables para trigonometría"))
("%if" . ("maxima.info-8" 127823 1425 "Funciones y variables para to_poly_solve"))
("%m" . ("maxima.info-3" 38559 134 "Funciones hipergeométricas"))
("%or" . ("maxima.info-8" 129249 2563 "Funciones y variables para to_poly_solve"))
("%phi" . ("maxima.info-1" 117268 2470 "Funciones y variables para Constantes"))
("%pi" . ("maxima.info-1" 119739 210 "Funciones y variables para Constantes"))
("%piargs" . ("maxima.info-2" 69214 1996 "Funciones y variables para trigonometría"))
("%rnum_list" . ("maxima.info-3" 146673 968 "Funciones y variable para las ecuaciones"))
("%s" . ("maxima.info-2" 289446 99 "Funciones de Bessel"))
("%th" . ("maxima.info-1" 51827 1126 "Funciones y variables para la línea de comandos"))
("%unitexpand" . ("maxima.info-8" 192122 158 "Funciones y variables para units"))
("%w" . ("maxima.info-3" 38694 76 "Funciones hipergeométricas"))
("'" . ("maxima.info-1" 227913 3782 "Funciones y variables para la evaluación"))
("'%c'" . ("maxima.info-5" 218141 88 "Funciones y variables para contrib_ode"))
("'%k1'" . ("maxima.info-5" 218230 104 "Funciones y variables para contrib_ode"))
("'%k2'" . ("maxima.info-5" 218335 104 "Funciones y variables para contrib_ode"))
("''" . ("maxima.info-1" 231696 5858 "Funciones y variables para la evaluación"))
("'method'" . ("maxima.info-5" 218046 94 "Funciones y variables para contrib_ode"))
("*" . ("maxima.info-1" 182434 7400 "Operadores aritméticos"))
("**" . ("maxima.info-1" 189835 831 "Operadores aritméticos"))
("+" . ("maxima.info-1" 182417 7417 "Operadores aritméticos"))
("-" . ("maxima.info-1" 182418 7416 "Operadores aritméticos"))
("." . ("maxima.info-1" 191477 425 "Operadores aritméticos"))
("/" . ("maxima.info-1" 182450 7384 "Operadores aritméticos"))
(":" . ("maxima.info-1" 202942 2849 "Operadores de asignación"))
("::" . ("maxima.info-1" 205792 746 "Operadores de asignación"))
("::=" . ("maxima.info-1" 206539 3164 "Operadores de asignación"))
(":=" . ("maxima.info-1" 209704 3198 "Operadores de asignación"))
("<" . ("maxima.info-1" 192086 2531 "Operadores relacionales"))
("<=" . ("maxima.info-1" 192087 2530 "Operadores relacionales"))
("=" . ("maxima.info-1" 198553 4181 "Operadores para ecuaciones"))
(">" . ("maxima.info-1" 192120 2497 "Operadores relacionales"))
(">=" . ("maxima.info-1" 192103 2514 "Operadores relacionales"))
("?" . ("maxima.info-1" 52954 565 "Funciones y variables para la línea de comandos"))
("??" . ("maxima.info-1" 53520 313 "Funciones y variables para la línea de comandos"))
("@" . ("maxima.info-1" 174177 1745 "Funciones y variables para las estructuras"))
("Acotador de listas" . ("maxima.info-1" 122341 1104 "Funciones y variables para listas"))
("AntiDifference" . ("maxima.info-8" 194812 177 "Funciones y variables para zeilberger"))
("Ayuda" . ("maxima.info-1" 34507 2961 "Funciones y variables para la ayuda"))
("Base de los logaritmos naturales" . ("maxima.info-1" 116273 223 "Funciones y variables para Constantes"))
("Busca coincidencias inexactas en la documentaci&oacute;n" . ("maxima.info-1" 53521 312 "Funciones y variables para la línea de comandos"))
("Busca en la documentaci&oacute;n" . ("maxima.info-1" 52955 564 "Funciones y variables para la línea de comandos"))
("Conjunción lógica" . ("maxima.info-1" 194796 1084 "Operadores lógicos"))
("Conjunción lógica <1>" . ("maxima.info-8" 125098 2724 "Funciones y variables para to_poly_solve"))
("Constante de Euler-Mascheroni" . ("maxima.info-1" 116724 87 "Funciones y variables para Constantes"))
("Contracción con vector" . ("maxima.info-4" 62030 1322 "Funciones y variables para itensor"))
("Delimitadores de listas" . ("maxima.info-3" 285702 1171 "Funciones y variables para las matrices y el álgebra lineal"))
("Disyunción lógica" . ("maxima.info-1" 196506 1078 "Operadores lógicos"))
("Disyunción lógica <1>" . ("maxima.info-8" 129250 2562 "Funciones y variables para to_poly_solve"))
("División" . ("maxima.info-1" 182466 7368 "Operadores aritméticos"))
("Doble factorial" . ("maxima.info-2" 50350 730 "Funciones combinatorias"))
("Entrada anterior" . ("maxima.info-1" 47789 1620 "Funciones y variables para la línea de comandos"))
("Evaluación condicionada" . ("maxima.info-8" 127824 1424 "Funciones y variables para to_poly_solve"))
("Exponenciación" . ("maxima.info-1" 182482 7352 "Operadores aritméticos"))
("Exponenciación no conmutativa" . ("maxima.info-1" 190668 808 "Operadores aritméticos"))
("Expresión actual de entrada" . ("maxima.info-1" 45838 1949 "Funciones y variables para la línea de comandos"))
("GGFCFMAX" . ("maxima.info-7" 76596 573 "Funciones y variables para ggf"))
("GGFINFINITY" . ("maxima.info-7" 75983 612 "Funciones y variables para ggf"))
("Gosper" . ("maxima.info-8" 194990 264 "Funciones y variables para zeilberger"))
("GosperSum" . ("maxima.info-8" 195255 2053 "Funciones y variables para zeilberger"))
("Gosper_in_Zeilberger" . ("maxima.info-8" 199238 283 "Funciones y variables para zeilberger"))
("Igualdad sintáctica" . ("maxima.info-1" 198554 4180 "Operadores para ecuaciones"))
("Indefinido" . ("maxima.info-1" 120092 198 "Funciones y variables para Constantes"))
("Indeterminado" . ("maxima.info-1" 116813 207 "Funciones y variables para Constantes"))
("Infinito complejo" . ("maxima.info-1" 117102 83 "Funciones y variables para Constantes"))
("Infinito negativo" . ("maxima.info-1" 117187 80 "Funciones y variables para Constantes"))
("JF" . ("maxima.info-6" 23123 837 "Funciones y variables para diag"))
("Lindstedt" . ("maxima.info-7" 197362 1103 "Funciones y variables para lindstedt"))
("Mayor o igual que" . ("maxima.info-1" 192120 2497 "Operadores relacionales"))
("Mayor que" . ("maxima.info-1" 192137 2480 "Operadores relacionales"))
("Menor o igual que" . ("maxima.info-1" 192103 2514 "Operadores relacionales"))
("Menor que" . ("maxima.info-1" 192087 2530 "Operadores relacionales"))
("Menos infinito" . ("maxima.info-1" 117187 80 "Funciones y variables para Constantes"))
("ModeMatrix" . ("maxima.info-6" 27406 1468 "Funciones y variables para diag"))
("Multiplicación" . ("maxima.info-1" 182450 7384 "Operadores aritméticos"))
("Multiplicación no conmutativa" . ("maxima.info-1" 191478 424 "Operadores aritméticos"))
("Más infinito" . ("maxima.info-1" 117022 78 "Funciones y variables para Constantes"))
("Negación de la igualdad sintáctica" . ("maxima.info-1" 197780 772 "Operadores para ecuaciones"))
("Negación lógica" . ("maxima.info-1" 195882 622 "Operadores lógicos"))
("Número de Euler" . ("maxima.info-1" 116273 223 "Funciones y variables para Constantes"))
("Operador comilla" . ("maxima.info-1" 227914 3781 "Funciones y variables para la evaluación"))
("Operador comilla-comilla" . ("maxima.info-1" 231697 5857 "Funciones y variables para la evaluación"))
("Operador de asignación" . ("maxima.info-1" 202943 2848 "Operadores de asignación"))
("Operador de asignación (evalúa el miembro izquierdo)" . ("maxima.info-1" 205793 745 "Operadores de asignación"))
("Operador de definición de funciones" . ("maxima.info-1" 209705 3197 "Operadores de asignación"))
("Operador de definición de funciones macro" . ("maxima.info-1" 206540 3163 "Operadores de asignación"))
("Operador de ecuación" . ("maxima.info-1" 198554 4180 "Operadores para ecuaciones"))
("Operador de subíndice" . ("maxima.info-3" 285702 1171 "Funciones y variables para las matrices y el álgebra lineal"))
("Operador para subíndices" . ("maxima.info-1" 122341 1104 "Funciones y variables para listas"))
("Producto exterior" . ("maxima.info-4" 60419 1609 "Funciones y variables para itensor"))
("Razón áurea" . ("maxima.info-1" 117269 2469 "Funciones y variables para Constantes"))
("Resta" . ("maxima.info-1" 182434 7400 "Operadores aritméticos"))
("Resultado anterior en una sentencia compuesta" . ("maxima.info-1" 50142 1684 "Funciones y variables para la línea de comandos"))
("Salida anterior" . ("maxima.info-1" 49411 729 "Funciones y variables para la línea de comandos"))
("Suma" . ("maxima.info-1" 182418 7416 "Operadores aritméticos"))
("Unidad imaginaria" . ("maxima.info-1" 116498 82 "Funciones y variables para Constantes"))
("Zeilberger" . ("maxima.info-8" 197671 784 "Funciones y variables para zeilberger"))
("[" . ("maxima.info-1" 122324 1121 "Funciones y variables para listas"))
("[ <1>" . ("maxima.info-3" 285677 1196 "Funciones y variables para las matrices y el álgebra lineal"))
("[ <2>" . ("maxima.info-3" 285678 1195 "Funciones y variables para las matrices y el álgebra lineal"))
("]" . ("maxima.info-1" 122325 1120 "Funciones y variables para listas"))
("^" . ("maxima.info-1" 182466 7368 "Operadores aritméticos"))
("^^" . ("maxima.info-1" 190667 809 "Operadores aritméticos"))
("_" . ("maxima.info-1" 47788 1621 "Funciones y variables para la línea de comandos"))
("__" . ("maxima.info-1" 45837 1950 "Funciones y variables para la línea de comandos"))
("`" . ("maxima.info-7" 26065 3299 "Funciones y variables para ezunits"))
("` <1>" . ("maxima.info-7" 29365 7642 "Funciones y variables para ezunits"))
("abasep" . ("maxima.info-4" 142651 261 "Funciones y variables para atensor"))
("abs" . ("maxima.info-2" 33453 3472 "Funciones para los números"))
("absboxchar" . ("maxima.info-1" 74322 358 "Funciones y variables para la impresión"))
("absint" . ("maxima.info-4" 186537 606 "Funciones y variables para series de Fourier"))
("absint <1>" . ("maxima.info-4" 186538 605 "Funciones y variables para series de Fourier"))
("absint <2>" . ("maxima.info-4" 186582 561 "Funciones y variables para series de Fourier"))
("absolute_real_time" . ("maxima.info-4" 280892 472 "Funciones y variables para el entorno de ejecución"))
("acos" . ("maxima.info-2" 72292 43 "Funciones y variables para trigonometría"))
("acosh" . ("maxima.info-2" 72336 55 "Funciones y variables para trigonometría"))
("acot" . ("maxima.info-2" 72392 46 "Funciones y variables para trigonometría"))
("acoth" . ("maxima.info-2" 72439 59 "Funciones y variables para trigonometría"))
("acsc" . ("maxima.info-2" 72499 45 "Funciones y variables para trigonometría"))
("acsch" . ("maxima.info-2" 72545 58 "Funciones y variables para trigonometría"))
("activate" . ("maxima.info-2" 112262 409 "Funciones y variables para los hechos"))
("activecontexts" . ("maxima.info-2" 112672 266 "Funciones y variables para los hechos"))
("adapt_depth" . ("maxima.info-2" 158266 190 "Opciones gráficas"))
("adapt_depth <1>" . ("maxima.info-6" 128346 246 "Funciones y variables para draw"))
("add_edge" . ("maxima.info-7" 116190 412 "Funciones y variables para graphs"))
("add_edges" . ("maxima.info-7" 116603 386 "Funciones y variables para graphs"))
("add_vertex" . ("maxima.info-7" 116990 346 "Funciones y variables para graphs"))
("add_vertices" . ("maxima.info-7" 117337 103 "Funciones y variables para graphs"))
("addcol" . ("maxima.info-3" 235165 138 "Funciones y variables para las matrices y el álgebra lineal"))
("additive" . ("maxima.info-2" 5079 928 "Funciones y variables para simplificación"))
("addmatrices" . ("maxima.info-7" 203430 570 "Funciones y variables para linearalgebra"))
("addrow" . ("maxima.info-3" 235304 135 "Funciones y variables para las matrices y el álgebra lineal"))
("adim" . ("maxima.info-4" 140538 225 "Funciones y variables para atensor"))
("adjacency_matrix" . ("maxima.info-7" 88635 612 "Funciones y variables para graphs"))
("adjoin" . ("maxima.info-5" 11266 535 "Funciones y variables para los conjuntos"))
("adjoint" . ("maxima.info-3" 235440 147 "Funciones y variables para las matrices y el álgebra lineal"))
("af" . ("maxima.info-4" 141334 295 "Funciones y variables para atensor"))
("aform" . ("maxima.info-4" 140764 188 "Funciones y variables para atensor"))
("agd" . ("maxima.info-8" 25222 170 "Paquete functs"))
("airy_ai" . ("maxima.info-3" 651 177 "Funciones de Airy"))
("airy_bi" . ("maxima.info-3" 941 551 "Funciones de Airy"))
("airy_dai" . ("maxima.info-3" 829 111 "Funciones de Airy"))
("airy_dbi" . ("maxima.info-3" 1493 126 "Funciones de Airy"))
("alg_type" . ("maxima.info-4" 140385 152 "Funciones y variables para atensor"))
("algebraic" . ("maxima.info-2" 219125 184 "Funciones y variables para polinomios"))
("algepsilon" . ("maxima.info-3" 2210 117 "Funciones Gamma y factorial"))
("algepsilon <1>" . ("maxima.info-3" 147642 121 "Funciones y variable para las ecuaciones"))
("algexact" . ("maxima.info-3" 147764 650 "Funciones y variable para las ecuaciones"))
("algsys" . ("maxima.info-3" 148415 4733 "Funciones y variable para las ecuaciones"))
("algsys <1>" . ("maxima.info-3" 148416 4732 "Funciones y variable para las ecuaciones"))
("alias" . ("maxima.info-1" 261622 281 "Funciones y variables para expresiones"))
("aliases" . ("maxima.info-1" 261904 307 "Funciones y variables para expresiones"))
("all_dotsimp_denoms" . ("maxima.info-3" 292029 369 "Funciones y variables para Afines"))
("allbut" . ("maxima.info-1" 262212 1028 "Funciones y variables para expresiones"))
("allocation" . ("maxima.info-6" 128593 1483 "Funciones y variables para draw"))
("allroots" . ("maxima.info-3" 153149 2281 "Funciones y variable para las ecuaciones"))
("allroots <1>" . ("maxima.info-3" 153150 2280 "Funciones y variable para las ecuaciones"))
("allsym" . ("maxima.info-4" 24302 436 "Funciones y variables para itensor"))
("alphabetic" . ("maxima.info-2" 91004 678 "Funciones y variables para las propiedades"))
("alphacharp" . ("maxima.info-8" 107146 92 "Funciones y variables para caracteres"))
("alphanumericp" . ("maxima.info-8" 107239 107 "Funciones y variables para caracteres"))
("amortization" . ("maxima.info-7" 61026 1395 "Funciones y Variables para finance"))
("and" . ("maxima.info-1" 194795 1085 "Operadores lógicos"))
("animation" . ("maxima.info-7" 7411 660 "Visualización usando VTK"))
("annuity_fv" . ("maxima.info-7" 59653 410 "Funciones y Variables para finance"))
("annuity_pv" . ("maxima.info-7" 59251 401 "Funciones y Variables para finance"))
("antid" . ("maxima.info-3" 61909 2515 "Funciones y variables para la diferenciación"))
("antidiff" . ("maxima.info-3" 64425 2369 "Funciones y variables para la diferenciación"))
("antisymmetric" . ("maxima.info-2" 6008 1046 "Funciones y variables para simplificación"))
("append" . ("maxima.info-1" 123446 368 "Funciones y variables para listas"))
("appendfile" . ("maxima.info-2" 179596 341 "Funciones y variables para lectura y escritura"))
("apply" . ("maxima.info-5" 77771 2377 "Funciones y variables para la definición de funciones"))
("apply1" . ("maxima.info-4" 291895 739 "Funciones y variables sobre reglas y patrones"))
("apply2" . ("maxima.info-4" 292635 586 "Funciones y variables sobre reglas y patrones"))
("applyb1" . ("maxima.info-4" 293222 801 "Funciones y variables sobre reglas y patrones"))
("apropos" . ("maxima.info-1" 30144 1293 "Funciones y variables para la ayuda"))
("args" . ("maxima.info-1" 263241 551 "Funciones y variables para expresiones"))
("arit_amortization" . ("maxima.info-7" 62422 1709 "Funciones y Variables para finance"))
("arithmetic" . ("maxima.info-8" 23710 227 "Paquete functs"))
("arithsum" . ("maxima.info-8" 24415 189 "Paquete functs"))
("array" . ("maxima.info-1" 150180 1639 "Funciones y variables para los arrays"))
("array <1>" . ("maxima.info-1" 150181 1638 "Funciones y variables para los arrays"))
("array <2>" . ("maxima.info-1" 150234 1585 "Funciones y variables para los arrays"))
("arrayapply" . ("maxima.info-1" 151820 268 "Funciones y variables para los arrays"))
("arrayinfo" . ("maxima.info-1" 152089 4023 "Funciones y variables para los arrays"))
("arraymake" . ("maxima.info-1" 156113 1817 "Funciones y variables para los arrays"))
("arrays" . ("maxima.info-1" 157931 1156 "Funciones y variables para los arrays"))
("arraysetapply" . ("maxima.info-1" 159088 203 "Funciones y variables para los arrays"))
("ascii" . ("maxima.info-8" 107347 396 "Funciones y variables para caracteres"))
("asec" . ("maxima.info-2" 72604 43 "Funciones y variables para trigonometría"))
("asech" . ("maxima.info-2" 72648 56 "Funciones y variables para trigonometría"))
("asin" . ("maxima.info-2" 72705 40 "Funciones y variables para trigonometría"))
("asinh" . ("maxima.info-2" 72746 53 "Funciones y variables para trigonometría"))
("askexp" . ("maxima.info-4" 284177 133 "Funciones y variables para la miscelánea de opciones"))
("askinteger" . ("maxima.info-2" 112939 710 "Funciones y variables para los hechos"))
("askinteger <1>" . ("maxima.info-2" 112940 709 "Funciones y variables para los hechos"))
("askinteger <2>" . ("maxima.info-2" 112982 667 "Funciones y variables para los hechos"))
("askinteger <3>" . ("maxima.info-2" 113015 634 "Funciones y variables para los hechos"))
("asksign" . ("maxima.info-2" 113650 412 "Funciones y variables para los hechos"))
("assoc" . ("maxima.info-1" 123815 688 "Funciones y variables para listas"))
("assoc <1>" . ("maxima.info-1" 123816 687 "Funciones y variables para listas"))
("assoc_legendre_p" . ("maxima.info-7" 298010 264 "Funciones y variables para polinomios ortogonales"))
("assoc_legendre_q" . ("maxima.info-7" 298275 195 "Funciones y variables para polinomios ortogonales"))
("assume" . ("maxima.info-2" 114063 2557 "Funciones y variables para los hechos"))
("assume_external_byte_order" . ("maxima.info-7" 272638 535 "Funciones y variables para entrada y salida en formato binario"))
("assume_pos" . ("maxima.info-2" 117617 1390 "Funciones y variables para los hechos"))
("assume_pos_pred" . ("maxima.info-2" 119008 2284 "Funciones y variables para los hechos"))
("assumescalar" . ("maxima.info-2" 116621 995 "Funciones y variables para los hechos"))
("asymbol" . ("maxima.info-4" 140953 88 "Funciones y variables para atensor"))
("at" . ("maxima.info-3" 66795 1921 "Funciones y variables para la diferenciación"))
("at <1>" . ("maxima.info-3" 66796 1920 "Funciones y variables para la diferenciación"))
("atan" . ("maxima.info-2" 72800 44 "Funciones y variables para trigonometría"))
("atan2" . ("maxima.info-2" 72845 111 "Funciones y variables para trigonometría"))
("atanh" . ("maxima.info-2" 72957 57 "Funciones y variables para trigonometría"))
("atensimp" . ("maxima.info-4" 139941 443 "Funciones y variables para atensor"))
("atom" . ("maxima.info-1" 263793 337 "Funciones y variables para expresiones"))
("atomgrad" . ("maxima.info-3" 68717 80 "Funciones y variables para la diferenciación"))
("atrig1" . ("maxima.info-2" 73015 464 "Funciones y variables para trigonometría"))
("atvalue" . ("maxima.info-3" 68798 2614 "Funciones y variables para la diferenciación"))
("atvalue <1>" . ("maxima.info-3" 68799 2613 "Funciones y variables para la diferenciación"))
("augcoefmatrix" . ("maxima.info-3" 235588 706 "Funciones y variables para las matrices y el álgebra lineal"))
("augmented_lagrangian_method" . ("maxima.info-5" 190441 3982 "Funciones y variables para augmented_lagrangian"))
("augmented_lagrangian_method <1>" . ("maxima.info-5" 190442 3981 "Funciones y variables para augmented_lagrangian"))
("augmented_lagrangian_method <2>" . ("maxima.info-5" 190574 3849 "Funciones y variables para augmented_lagrangian"))
("augmented_lagrangian_method <3>" . ("maxima.info-5" 190669 3754 "Funciones y variables para augmented_lagrangian"))
("av" . ("maxima.info-4" 141630 1020 "Funciones y variables para atensor"))
("average_degree" . ("maxima.info-7" 89248 345 "Funciones y variables para graphs"))
("axes" . ("maxima.info-2" 158457 377 "Opciones gráficas"))
("axis_3d" . ("maxima.info-6" 130077 529 "Funciones y variables para draw"))
("axis_bottom" . ("maxima.info-6" 130607 490 "Funciones y variables para draw"))
("axis_left" . ("maxima.info-6" 131098 487 "Funciones y variables para draw"))
("axis_right" . ("maxima.info-6" 131586 487 "Funciones y variables para draw"))
("axis_top" . ("maxima.info-6" 132074 489 "Funciones y variables para draw"))
("azimuth" . ("maxima.info-2" 158835 518 "Opciones gráficas"))
("azimuth <1>" . ("maxima.info-7" 2910 309 "Visualización usando VTK"))
("background" . ("maxima.info-7" 3220 268 "Visualización usando VTK"))
("background_color" . ("maxima.info-6" 132564 444 "Funciones y variables para draw"))
("backsubst" . ("maxima.info-3" 155882 749 "Funciones y variable para las ecuaciones"))
("backtrace" . ("maxima.info-5" 141088 1258 "Funciones y variables para la programación"))
("backtrace <1>" . ("maxima.info-5" 141089 1257 "Funciones y variables para la programación"))
("bars" . ("maxima.info-6" 219370 907 "Funciones y variables para draw"))
("barsplot" . ("maxima.info-6" 334 5803 "Funciones y variables para gráficos estadísticos"))
("barsplot_description" . ("maxima.info-6" 405 5732 "Funciones y variables para gráficos estadísticos"))
("base64" . ("maxima.info-8" 110214 347 "Funciones y variables para cadenas"))
("base64_decode" . ("maxima.info-8" 110562 200 "Funciones y variables para cadenas"))
("bashindices" . ("maxima.info-4" 143617 406 "Funciones y variables para sumas y productos"))
("batch" . ("maxima.info-2" 179938 2520 "Funciones y variables para lectura y escritura"))
("batch <1>" . ("maxima.info-2" 179939 2519 "Funciones y variables para lectura y escritura"))
("batchload" . ("maxima.info-2" 182459 653 "Funciones y variables para lectura y escritura"))
("bc2" . ("maxima.info-3" 190848 599 "Funciones y variables para ecuaciones diferenciales"))
("bdvac" . ("maxima.info-4" 109351 482 "Funciones y variables para ctensor"))
("belln" . ("maxima.info-5" 11802 967 "Funciones y variables para los conjuntos"))
("benefit_cost" . ("maxima.info-7" 68144 702 "Funciones y Variables para finance"))
("berlefact" . ("maxima.info-2" 219310 267 "Funciones y variables para polinomios"))
("bern" . ("maxima.info-4" 191926 780 "Funciones y variables para teoría de números"))
("bernpoly" . ("maxima.info-4" 192707 100 "Funciones y variables para teoría de números"))
("bernstein_approx" . ("maxima.info-5" 197802 1224 "Funciones y variables para Bernstein"))
("bernstein_expand" . ("maxima.info-5" 199027 528 "Funciones y variables para Bernstein"))
("bernstein_explicit" . ("maxima.info-5" 196707 669 "Funciones y variables para Bernstein"))
("bernstein_poly" . ("maxima.info-5" 194776 1930 "Funciones y variables para Bernstein"))
("bessel_i" . ("maxima.info-2" 284016 545 "Funciones de Bessel"))
("bessel_j" . ("maxima.info-2" 283043 525 "Funciones de Bessel"))
("bessel_k" . ("maxima.info-2" 284562 461 "Funciones de Bessel"))
("bessel_y" . ("maxima.info-2" 283569 446 "Funciones de Bessel"))
("besselexpand" . ("maxima.info-2" 287800 1016 "Funciones de Bessel"))
("beta" . ("maxima.info-3" 10353 4306 "Funciones Gamma y factorial"))
("beta_args_sum_to_integer" . ("maxima.info-3" 31234 225 "Funciones Gamma y factorial"))
("beta_expand" . ("maxima.info-3" 31006 227 "Funciones Gamma y factorial"))
("beta_incomplete" . ("maxima.info-3" 14660 6248 "Funciones Gamma y factorial"))
("beta_incomplete_generalized" . ("maxima.info-3" 25331 5674 "Funciones Gamma y factorial"))
("beta_incomplete_regularized" . ("maxima.info-3" 20909 4421 "Funciones Gamma y factorial"))
("bezout" . ("maxima.info-2" 219578 574 "Funciones y variables para polinomios"))
("bf_find_root" . ("maxima.info-3" 209385 4078 "Funciones para la resolución numérica de ecuaciones"))
("bf_find_root <1>" . ("maxima.info-3" 209446 4017 "Funciones para la resolución numérica de ecuaciones"))
("bf_fmin_cobyla" . ("maxima.info-5" 208119 399 "Funciones y variables para cobyla"))
("bf_fmin_cobyla <1>" . ("maxima.info-5" 208120 398 "Funciones y variables para cobyla"))
("bfallroots" . ("maxima.info-3" 155431 450 "Funciones y variable para las ecuaciones"))
("bfallroots <1>" . ("maxima.info-3" 155432 449 "Funciones y variable para las ecuaciones"))
("bffac" . ("maxima.info-3" 1968 241 "Funciones Gamma y factorial"))
("bfhzeta" . ("maxima.info-4" 193083 629 "Funciones y variables para teoría de números"))
("bfloat" . ("maxima.info-1" 99609 507 "Funciones y variables para los números"))
("bfloatp" . ("maxima.info-1" 100117 163 "Funciones y variables para los números"))
("bfpsi" . ("maxima.info-3" 2328 414 "Funciones Gamma y factorial"))
("bfpsi0" . ("maxima.info-3" 2329 413 "Funciones Gamma y factorial"))
("bftorat" . ("maxima.info-1" 100281 511 "Funciones y variables para los números"))
("bftrunc" . ("maxima.info-1" 100793 326 "Funciones y variables para los números"))
("bfzeta" . ("maxima.info-4" 192808 274 "Funciones y variables para teoría de números"))
("biconnected_components" . ("maxima.info-7" 89594 492 "Funciones y variables para graphs"))
("bindtest" . ("maxima.info-2" 91683 626 "Funciones y variables para las propiedades"))
("binomial" . ("maxima.info-2" 51081 989 "Funciones combinatorias"))
("bipartition" . ("maxima.info-7" 90087 405 "Funciones y variables para graphs"))
("block" . ("maxima.info-5" 80149 3235 "Funciones y variables para la definición de funciones"))
("block <1>" . ("maxima.info-5" 80150 3234 "Funciones y variables para la definición de funciones"))
("blockmatrixp" . ("maxima.info-7" 204001 138 "Funciones y variables para linearalgebra"))
("bode_gain" . ("maxima.info-5" 199854 1428 "Funciones y variables para bode"))
("bode_phase" . ("maxima.info-5" 201283 1673 "Funciones y variables para bode"))
("border" . ("maxima.info-6" 133009 617 "Funciones y variables para draw"))
("bothcoef" . ("maxima.info-2" 220153 629 "Funciones y variables para polinomios"))
("boundaries_array" . ("maxima.info-6" 257852 401 "Funciones y variables para worldmap"))
("box" . ("maxima.info-2" 159354 163 "Opciones gráficas"))
("box <1>" . ("maxima.info-1" 264131 2020 "Funciones y variables para expresiones"))
("box <2>" . ("maxima.info-1" 264132 2019 "Funciones y variables para expresiones"))
("boxchar" . ("maxima.info-1" 266152 332 "Funciones y variables para expresiones"))
("boxplot" . ("maxima.info-6" 6138 4129 "Funciones y variables para gráficos estadísticos"))
("boxplot <1>" . ("maxima.info-6" 6139 4128 "Funciones y variables para gráficos estadísticos"))
("boxplot_description" . ("maxima.info-6" 6169 4098 "Funciones y variables para gráficos estadísticos"))
("break" . ("maxima.info-5" 83385 281 "Funciones y variables para la definición de funciones"))
("breakup" . ("maxima.info-3" 156632 3315 "Funciones y variable para las ecuaciones"))
("bug_report" . ("maxima.info-1" 26639 490 "Funciones y variables para la detección e informe de fallos"))
("build_info" . ("maxima.info-1" 27130 1569 "Funciones y variables para la detección e informe de fallos"))
("build_sample" . ("maxima.info-5" 229551 2396 "Funciones y variables para el tratamiento de datos"))
("build_sample <1>" . ("maxima.info-5" 229552 2395 "Funciones y variables para el tratamiento de datos"))
("buildq" . ("maxima.info-5" 68531 4544 "Macros"))
("burn" . ("maxima.info-4" 193713 996 "Funciones y variables para teoría de números"))
("cabs" . ("maxima.info-2" 45207 2550 "Funciones para los números complejos"))
("canform" . ("maxima.info-4" 27461 988 "Funciones y variables para itensor"))
("canform <1>" . ("maxima.info-4" 27462 987 "Funciones y variables para itensor"))
("canten" . ("maxima.info-4" 23529 601 "Funciones y variables para itensor"))
("capping" . ("maxima.info-6" 133627 595 "Funciones y variables para draw"))
("capping <1>" . ("maxima.info-7" 8072 263 "Visualización usando VTK"))
("cardinality" . ("maxima.info-5" 12770 556 "Funciones y variables para los conjuntos"))
("carg" . ("maxima.info-2" 47758 1048 "Funciones para los números complejos"))
("cartan" . ("maxima.info-3" 71413 643 "Funciones y variables para la diferenciación"))
("cartesian_product" . ("maxima.info-5" 13327 785 "Funciones y variables para los conjuntos"))
("catch" . ("maxima.info-5" 83667 1159 "Funciones y variables para la definición de funciones"))
("cauchy_matrix" . ("maxima.info-3" 236295 1381 "Funciones y variables para las matrices y el álgebra lineal"))
("cauchy_matrix <1>" . ("maxima.info-3" 236365 1311 "Funciones y variables para las matrices y el álgebra lineal"))
("cauchysum" . ("maxima.info-4" 156921 1270 "Funciones y variables para las series"))
("cbffac" . ("maxima.info-3" 2743 173 "Funciones Gamma y factorial"))
("cbrange" . ("maxima.info-6" 134223 975 "Funciones y variables para draw"))
("cbtics" . ("maxima.info-6" 135199 734 "Funciones y variables para draw"))
("cdf_bernoulli" . ("maxima.info-6" 106740 259 "Funciones y variables para distribuciones discretas"))
("cdf_beta" . ("maxima.info-6" 75098 441 "Funciones y variables para distribuciones continuas"))
("cdf_binomial" . ("maxima.info-6" 101380 482 "Funciones y variables para distribuciones discretas"))
("cdf_cauchy" . ("maxima.info-6" 93385 241 "Funciones y variables para distribuciones continuas"))
("cdf_chi2" . ("maxima.info-6" 57617 491 "Funciones y variables para distribuciones continuas"))
("cdf_continuous_uniform" . ("maxima.info-6" 77572 267 "Funciones y variables para distribuciones continuas"))
("cdf_discrete_uniform" . ("maxima.info-6" 112164 273 "Funciones y variables para distribuciones discretas"))
("cdf_empirical" . ("maxima.info-5" 261619 1009 "Funciones y variables de parámetros descriptivos"))
("cdf_empirical <1>" . ("maxima.info-5" 261620 1008 "Funciones y variables de parámetros descriptivos"))
("cdf_exp" . ("maxima.info-6" 66251 392 "Funciones y variables para distribuciones continuas"))
("cdf_f" . ("maxima.info-6" 63276 478 "Funciones y variables para distribuciones continuas"))
("cdf_gamma" . ("maxima.info-6" 72354 485 "Funciones y variables para distribuciones continuas"))
("cdf_general_finite_discrete" . ("maxima.info-6" 98128 896 "Funciones y variables para distribuciones discretas"))
("cdf_geometric" . ("maxima.info-6" 109872 254 "Funciones y variables para distribuciones discretas"))
("cdf_gumbel" . ("maxima.info-6" 94568 241 "Funciones y variables para distribuciones continuas"))
("cdf_hypergeometric" . ("maxima.info-6" 114966 381 "Funciones y variables para distribuciones discretas"))
("cdf_laplace" . ("maxima.info-6" 91187 243 "Funciones y variables para distribuciones continuas"))
("cdf_logistic" . ("maxima.info-6" 80009 243 "Funciones y variables para distribuciones continuas"))
("cdf_lognormal" . ("maxima.info-6" 69704 632 "Funciones y variables para distribuciones continuas"))
("cdf_negative_binomial" . ("maxima.info-6" 117729 427 "Funciones y variables para distribuciones discretas"))
("cdf_noncentral_chi2" . ("maxima.info-6" 60961 248 "Funciones y variables para distribuciones continuas"))
("cdf_noncentral_student_t" . ("maxima.info-6" 53271 542 "Funciones y variables para distribuciones continuas"))
("cdf_normal" . ("maxima.info-6" 46190 632 "Funciones y variables para distribuciones continuas"))
("cdf_pareto" . ("maxima.info-6" 82214 243 "Funciones y variables para distribuciones continuas"))
("cdf_poisson" . ("maxima.info-6" 104176 363 "Funciones y variables para distribuciones discretas"))
("cdf_rank_sum" . ("maxima.info-8" 88595 288 "Funciones y variables para distribuciones especiales"))
("cdf_rayleigh" . ("maxima.info-6" 86851 448 "Funciones y variables para distribuciones continuas"))
("cdf_signed_rank" . ("maxima.info-8" 88026 279 "Funciones y variables para distribuciones especiales"))
("cdf_student_t" . ("maxima.info-6" 49183 603 "Funciones y variables para distribuciones continuas"))
("cdf_weibull" . ("maxima.info-6" 84428 245 "Funciones y variables para distribuciones continuas"))
("cdisplay" . ("maxima.info-4" 110392 13864 "Funciones y variables para ctensor"))
("ceiling" . ("maxima.info-2" 36926 3407 "Funciones para los números"))
("center" . ("maxima.info-7" 8336 425 "Visualización usando VTK"))
("central_moment" . ("maxima.info-5" 247463 928 "Funciones y variables de parámetros descriptivos"))
("central_moment <1>" . ("maxima.info-5" 247464 927 "Funciones y variables de parámetros descriptivos"))
("cequal" . ("maxima.info-8" 107744 108 "Funciones y variables para caracteres"))
("cequalignore" . ("maxima.info-8" 107853 132 "Funciones y variables para caracteres"))
("cf" . ("maxima.info-4" 195666 3176 "Funciones y variables para teoría de números"))
("cfdisrep" . ("maxima.info-4" 198843 700 "Funciones y variables para teoría de números"))
("cfexpand" . ("maxima.info-4" 199544 638 "Funciones y variables para teoría de números"))
("cflength" . ("maxima.info-4" 200183 654 "Funciones y variables para teoría de números"))
("cframe_flag" . ("maxima.info-4" 125296 127 "Funciones y variables para ctensor"))
("cgeodesic" . ("maxima.info-4" 109063 287 "Funciones y variables para ctensor"))
("cgreaterp" . ("maxima.info-8" 107986 130 "Funciones y variables para caracteres"))
("cgreaterpignore" . ("maxima.info-8" 108117 138 "Funciones y variables para caracteres"))
("changename" . ("maxima.info-4" 903 401 "Funciones y variables para itensor"))
("changevar" . ("maxima.info-3" 91869 2347 "Funciones y variables para integración"))
("chaosgame" . ("maxima.info-6" 281776 835 "Análisis gráfico de sistemas dinámicos discretos"))
("charat" . ("maxima.info-8" 110763 225 "Funciones y variables para cadenas"))
("charfun" . ("maxima.info-2" 129076 514 "Funciones y variables para los predicados"))
("charfun2" . ("maxima.info-7" 150914 141 "Funciones y variables para interpol"))
("charlist" . ("maxima.info-8" 110989 245 "Funciones y variables para cadenas"))
("charp" . ("maxima.info-8" 108256 83 "Funciones y variables para caracteres"))
("charpoly" . ("maxima.info-3" 237677 1622 "Funciones y variables para las matrices y el álgebra lineal"))
("chebyshev_t" . ("maxima.info-7" 298471 151 "Funciones y variables para polinomios ortogonales"))
("chebyshev_u" . ("maxima.info-7" 298623 151 "Funciones y variables para polinomios ortogonales"))
("check_overlaps" . ("maxima.info-3" 290327 642 "Funciones y variables para Afines"))
("checkdiv" . ("maxima.info-4" 108781 281 "Funciones y variables para ctensor"))
("chinese" . ("maxima.info-4" 194710 955 "Funciones y variables para teoría de números"))
("cholesky" . ("maxima.info-7" 205631 309 "Funciones y variables para linearalgebra"))
("cholesky <1>" . ("maxima.info-7" 205632 308 "Funciones y variables para linearalgebra"))
("christof" . ("maxima.info-4" 84925 856 "Funciones y variables para ctensor"))
("chromatic_index" . ("maxima.info-7" 90493 258 "Funciones y variables para graphs"))
("chromatic_number" . ("maxima.info-7" 90752 336 "Funciones y variables para graphs"))
("cint" . ("maxima.info-8" 108340 68 "Funciones y variables para caracteres"))
("circulant_graph" . ("maxima.info-7" 81207 575 "Funciones y variables para graphs"))
("clear_edge_weight" . ("maxima.info-7" 91089 441 "Funciones y variables para graphs"))
("clear_rules" . ("maxima.info-4" 331044 185 "Funciones y variables sobre reglas y patrones"))
("clear_vertex_label" . ("maxima.info-7" 91531 493 "Funciones y variables para graphs"))
("clebsch_graph" . ("maxima.info-7" 81783 65 "Funciones y variables para graphs"))
("clessp" . ("maxima.info-8" 108409 127 "Funciones y variables para caracteres"))
("clesspignore" . ("maxima.info-8" 108537 132 "Funciones y variables para caracteres"))
("close" . ("maxima.info-8" 97465 124 "Funciones y variables para entrada y salida"))
("closefile" . ("maxima.info-2" 183113 120 "Funciones y variables para lectura y escritura"))
("cmetric" . ("maxima.info-4" 76683 944 "Funciones y variables para ctensor"))
("cmetric <1>" . ("maxima.info-4" 76684 943 "Funciones y variables para ctensor"))
("cnonmet_flag" . ("maxima.info-4" 125704 311 "Funciones y variables para ctensor"))
("coeff" . ("maxima.info-2" 220783 3702 "Funciones y variables para polinomios"))
("coeff <1>" . ("maxima.info-2" 220784 3701 "Funciones y variables para polinomios"))
("coefmatrix" . ("maxima.info-3" 239300 451 "Funciones y variables para las matrices y el álgebra lineal"))
("cograd" . ("maxima.info-4" 106479 251 "Funciones y variables para ctensor"))
("col" . ("maxima.info-3" 239752 135 "Funciones y variables para las matrices y el álgebra lineal"))
("collapse" . ("maxima.info-1" 266485 546 "Funciones y variables para expresiones"))
("collectterms" . ("maxima.info-8" 18696 813 "Paquete facexp"))
("color" . ("maxima.info-2" 159518 928 "Opciones gráficas"))
("color <1>" . ("maxima.info-6" 135934 1983 "Funciones y variables para draw"))
("color <2>" . ("maxima.info-7" 8762 248 "Visualización usando VTK"))
("colorbox" . ("maxima.info-2" 160447 410 "Opciones gráficas"))
("colorbox <1>" . ("maxima.info-6" 137918 1078 "Funciones y variables para draw"))
("columnop" . ("maxima.info-7" 204140 259 "Funciones y variables para linearalgebra"))
("columns" . ("maxima.info-6" 138997 660 "Funciones y variables para draw"))
("columnspace" . ("maxima.info-7" 204598 195 "Funciones y variables para linearalgebra"))
("columnswap" . ("maxima.info-7" 204400 197 "Funciones y variables para linearalgebra"))
("columnvector" . ("maxima.info-3" 239888 859 "Funciones y variables para las matrices y el álgebra lineal"))
("combination" . ("maxima.info-8" 25866 181 "Paquete functs"))
("combine" . ("maxima.info-2" 7055 145 "Funciones y variables para simplificación"))
("commutative" . ("maxima.info-2" 7201 234 "Funciones y variables para simplificación"))
("comp2pui" . ("maxima.info-4" 231610 583 "Funciones y variables para simetrías"))
("compare" . ("maxima.info-2" 129591 968 "Funciones y variables para los predicados"))
("compfile" . ("maxima.info-5" 84827 837 "Funciones y variables para la definición de funciones"))
("compfile <1>" . ("maxima.info-5" 84828 836 "Funciones y variables para la definición de funciones"))
("compfile <2>" . ("maxima.info-5" 84883 781 "Funciones y variables para la definición de funciones"))
("compile" . ("maxima.info-5" 85665 596 "Funciones y variables para la definición de funciones"))
("compile <1>" . ("maxima.info-5" 85666 595 "Funciones y variables para la definición de funciones"))
("compile <2>" . ("maxima.info-5" 85707 554 "Funciones y variables para la definición de funciones"))
("compile_file" . ("maxima.info-5" 133111 1410 "Funciones y variables para la definición de funciones"))
("compile_file <1>" . ("maxima.info-5" 133112 1409 "Funciones y variables para la definición de funciones"))
("compile_file <2>" . ("maxima.info-5" 133154 1367 "Funciones y variables para la definición de funciones"))
("complement_graph" . ("maxima.info-7" 81849 80 "Funciones y variables para graphs"))
("complete_bipartite_graph" . ("maxima.info-7" 81930 110 "Funciones y variables para graphs"))
("complete_graph" . ("maxima.info-7" 82041 83 "Funciones y variables para graphs"))
("complex" . ("maxima.info-2" 109627 268 "Funciones y variables para las propiedades"))
("complex_number_p" . ("maxima.info-8" 131813 795 "Funciones y variables para to_poly_solve"))
("components" . ("maxima.info-4" 9540 4082 "Funciones y variables para itensor"))
("compose_functions" . ("maxima.info-8" 132609 1951 "Funciones y variables para to_poly_solve"))
("concan" . ("maxima.info-4" 24131 112 "Funciones y variables para itensor"))
("concat" . ("maxima.info-1" 113160 1327 "Funciones y variables para las cadenas de texto"))
("cone" . ("maxima.info-7" 5300 882 "Visualización usando VTK"))
("conjugate" . ("maxima.info-2" 48807 536 "Funciones para los números complejos"))
("conmetderiv" . ("maxima.info-4" 36018 859 "Funciones y variables para itensor"))
("connect_vertices" . ("maxima.info-7" 117441 567 "Funciones y variables para graphs"))
("connected_components" . ("maxima.info-7" 92025 311 "Funciones y variables para graphs"))
("cons" . ("maxima.info-1" 124504 281 "Funciones y variables para listas"))
("constant" . ("maxima.info-2" 92310 552 "Funciones y variables para las propiedades"))
("constantp" . ("maxima.info-2" 92863 1189 "Funciones y variables para las propiedades"))
("constituent" . ("maxima.info-8" 108670 602 "Funciones y variables para caracteres"))
("constvalue" . ("maxima.info-7" 37008 1542 "Funciones y variables para ezunits"))
("cont2part" . ("maxima.info-4" 232194 598 "Funciones y variables para simetrías"))
("content" . ("maxima.info-2" 224486 467 "Funciones y variables para polinomios"))
("context" . ("maxima.info-2" 121293 587 "Funciones y variables para los hechos"))
("contexts" . ("maxima.info-2" 121881 1495 "Funciones y variables para los hechos"))
("continuous_freq" . ("maxima.info-5" 231948 1882 "Funciones y variables para el tratamiento de datos"))
("continuous_freq <1>" . ("maxima.info-5" 231949 1881 "Funciones y variables para el tratamiento de datos"))
("contortion" . ("maxima.info-4" 103565 115 "Funciones y variables para ctensor"))
("contour" . ("maxima.info-6" 139658 1092 "Funciones y variables para draw"))
("contour_levels" . ("maxima.info-6" 140751 2108 "Funciones y variables para draw"))
("contour_plot" . ("maxima.info-2" 142033 933 "Funciones y variables para gráficos"))
("contract" . ("maxima.info-4" 8825 475 "Funciones y variables para itensor"))
("contract <1>" . ("maxima.info-4" 232793 993 "Funciones y variables para simetrías"))
("contract_edge" . ("maxima.info-7" 118009 779 "Funciones y variables para graphs"))
("contragrad" . ("maxima.info-4" 106731 1162 "Funciones y variables para ctensor"))
("contrib_ode" . ("maxima.info-5" 215323 197 "Funciones y variables para contrib_ode"))
("convert" . ("maxima.info-8" 178863 3414 "Funciones y variables para units"))
("coord" . ("maxima.info-4" 35006 437 "Funciones y variables para itensor"))
("copy" . ("maxima.info-7" 204794 836 "Funciones y variables para linearalgebra"))
("copy_graph" . ("maxima.info-7" 81137 69 "Funciones y variables para graphs"))
("copylist" . ("maxima.info-1" 124786 72 "Funciones y variables para listas"))
("copymatrix" . ("maxima.info-3" 240748 503 "Funciones y variables para las matrices y el álgebra lineal"))
("cor" . ("maxima.info-5" 267029 2215 "Funciones y variables de parámetros descriptivos"))
("cor <1>" . ("maxima.info-5" 267030 2214 "Funciones y variables de parámetros descriptivos"))
("cos" . ("maxima.info-2" 73480 36 "Funciones y variables para trigonometría"))
("cosh" . ("maxima.info-2" 73517 49 "Funciones y variables para trigonometría"))
("cosnpiflag" . ("maxima.info-4" 187535 89 "Funciones y variables para series de Fourier"))
("cot" . ("maxima.info-2" 73567 40 "Funciones y variables para trigonometría"))
("coth" . ("maxima.info-2" 73608 53 "Funciones y variables para trigonometría"))
("cov" . ("maxima.info-5" 262629 1277 "Funciones y variables de parámetros descriptivos"))
("cov1" . ("maxima.info-5" 263907 1280 "Funciones y variables de parámetros descriptivos"))
("covdiff" . ("maxima.info-4" 42060 1900 "Funciones y variables para itensor"))
("covect" . ("maxima.info-3" 239889 858 "Funciones y variables para las matrices y el álgebra lineal"))
("covers" . ("maxima.info-8" 25509 119 "Paquete functs"))
("create_graph" . ("maxima.info-7" 79278 1858 "Funciones y variables para graphs"))
("create_graph <1>" . ("maxima.info-7" 79279 1857 "Funciones y variables para graphs"))
("create_graph <2>" . ("maxima.info-7" 79326 1810 "Funciones y variables para graphs"))
("create_list" . ("maxima.info-1" 124859 1306 "Funciones y variables para listas"))
("csc" . ("maxima.info-2" 73662 39 "Funciones y variables para trigonometría"))
("csch" . ("maxima.info-2" 73702 52 "Funciones y variables para trigonometría"))
("csetup" . ("maxima.info-4" 76481 201 "Funciones y variables para ctensor"))
("cspline" . ("maxima.info-7" 153661 4665 "Funciones y variables para interpol"))
("cspline <1>" . ("maxima.info-7" 153662 4664 "Funciones y variables para interpol"))
("ct_coords" . ("maxima.info-4" 130951 384 "Funciones y variables para ctensor"))
("ct_coordsys" . ("maxima.info-4" 77628 5828 "Funciones y variables para ctensor"))
("ct_coordsys <1>" . ("maxima.info-4" 77629 5827 "Funciones y variables para ctensor"))
("ctaylor" . ("maxima.info-4" 91155 5445 "Funciones y variables para ctensor"))
("ctaypov" . ("maxima.info-4" 126427 131 "Funciones y variables para ctensor"))
("ctaypt" . ("maxima.info-4" 126559 134 "Funciones y variables para ctensor"))
("ctayswitch" . ("maxima.info-4" 126016 276 "Funciones y variables para ctensor"))
("ctayvar" . ("maxima.info-4" 126293 133 "Funciones y variables para ctensor"))
("ctorsion_flag" . ("maxima.info-4" 125424 279 "Funciones y variables para ctensor"))
("ctransform" . ("maxima.info-4" 103867 262 "Funciones y variables para ctensor"))
("ctranspose" . ("maxima.info-7" 205941 211 "Funciones y variables para linearalgebra"))
("ctrgsimp" . ("maxima.info-4" 125054 241 "Funciones y variables para ctensor"))
("cube" . ("maxima.info-7" 6183 351 "Visualización usando VTK"))
("cube_graph" . ("maxima.info-7" 82359 72 "Funciones y variables para graphs"))
("cuboctahedron_graph" . ("maxima.info-7" 82280 78 "Funciones y variables para graphs"))
("cunlisp" . ("maxima.info-8" 109273 155 "Funciones y variables para caracteres"))
("current_let_rule_package" . ("maxima.info-4" 294024 647 "Funciones y variables sobre reglas y patrones"))
("cv" . ("maxima.info-5" 248392 629 "Funciones y variables de parámetros descriptivos"))
("cv <1>" . ("maxima.info-5" 248393 628 "Funciones y variables de parámetros descriptivos"))
("cycle_digraph" . ("maxima.info-7" 82125 82 "Funciones y variables para graphs"))
("cycle_graph" . ("maxima.info-7" 82208 71 "Funciones y variables para graphs"))
("cylinder" . ("maxima.info-7" 6535 682 "Visualización usando VTK"))
("cylindrical" . ("maxima.info-6" 220278 730 "Funciones y variables para draw"))
("data_file_name" . ("maxima.info-6" 142860 457 "Funciones y variables para draw"))
("days360" . ("maxima.info-7" 57830 298 "Funciones y Variables para finance"))
("dblint" . ("maxima.info-3" 94217 2360 "Funciones y variables para integración"))
("deactivate" . ("maxima.info-2" 123377 139 "Funciones y variables para los hechos"))
("debugmode" . ("maxima.info-5" 179277 479 "Funciones y variables para depurado"))
("declare" . ("maxima.info-2" 94053 5548 "Funciones y variables para las propiedades"))
("declare_constvalue" . ("maxima.info-7" 37009 1541 "Funciones y variables para ezunits"))
("declare_dimensions" . ("maxima.info-7" 45868 1139 "Funciones y variables para ezunits"))
("declare_fundamental_dimensions" . ("maxima.info-7" 47008 1393 "Funciones y variables para ezunits"))
("declare_fundamental_units" . ("maxima.info-7" 48402 1685 "Funciones y variables para ezunits"))
("declare_qty" . ("maxima.info-7" 40980 1708 "Funciones y variables para ezunits"))
("declare_translated" . ("maxima.info-5" 134522 459 "Funciones y variables para la definición de funciones"))
("declare_unit_conversion" . ("maxima.info-7" 43946 1921 "Funciones y variables para ezunits"))
("declare_units" . ("maxima.info-7" 38552 2426 "Funciones y variables para ezunits"))
("declare_weights" . ("maxima.info-3" 289175 276 "Funciones y variables para Afines"))
("decreasing" . ("maxima.info-2" 99602 621 "Funciones y variables para las propiedades"))
("decsym" . ("maxima.info-4" 24739 2558 "Funciones y variables para itensor"))
("default_let_rule_package" . ("maxima.info-4" 294672 318 "Funciones y variables sobre reglas y patrones"))
("defcon" . ("maxima.info-4" 7510 1044 "Funciones y variables para itensor"))
("defcon <1>" . ("maxima.info-4" 7511 1043 "Funciones y variables para itensor"))
("define" . ("maxima.info-5" 86262 4182 "Funciones y variables para la definición de funciones"))
("define <1>" . ("maxima.info-5" 86263 4181 "Funciones y variables para la definición de funciones"))
("define <2>" . ("maxima.info-5" 86316 4128 "Funciones y variables para la definición de funciones"))
("define <3>" . ("maxima.info-5" 86369 4075 "Funciones y variables para la definición de funciones"))
("define <4>" . ("maxima.info-5" 86434 4010 "Funciones y variables para la definición de funciones"))
("define_variable" . ("maxima.info-5" 90445 4013 "Funciones y variables para la definición de funciones"))
("defint" . ("maxima.info-3" 96578 569 "Funciones y variables para integración"))
("defmatch" . ("maxima.info-4" 294991 4752 "Funciones y variables sobre reglas y patrones"))
("defmatch <1>" . ("maxima.info-4" 294992 4751 "Funciones y variables sobre reglas y patrones"))
("defrule" . ("maxima.info-4" 299744 659 "Funciones y variables sobre reglas y patrones"))
("defstruct" . ("maxima.info-1" 171625 1682 "Funciones y variables para las estructuras"))
("defstruct <1>" . ("maxima.info-1" 171626 1681 "Funciones y variables para las estructuras"))
("deftaylor" . ("maxima.info-4" 158192 1329 "Funciones y variables para las series"))
("degree_sequence" . ("maxima.info-7" 93309 268 "Funciones y variables para graphs"))
("del" . ("maxima.info-3" 72057 722 "Funciones y variables para la diferenciación"))
("delay" . ("maxima.info-6" 143318 922 "Funciones y variables para draw"))
("delete" . ("maxima.info-1" 126166 2481 "Funciones y variables para listas"))
("delete <1>" . ("maxima.info-1" 126167 2480 "Funciones y variables para listas"))
("deleten" . ("maxima.info-4" 124257 114 "Funciones y variables para ctensor"))
("delta" . ("maxima.info-3" 72780 391 "Funciones y variables para la diferenciación"))
("demo" . ("maxima.info-1" 31438 2994 "Funciones y variables para la ayuda"))
("demoivre" . ("maxima.info-2" 7436 732 "Funciones y variables para simplificación"))
("demoivre <1>" . ("maxima.info-2" 7437 731 "Funciones y variables para simplificación"))
("denom" . ("maxima.info-2" 224954 90 "Funciones y variables para polinomios"))
("dependencies" . ("maxima.info-3" 73172 361 "Funciones y variables para la diferenciación"))
("depends" . ("maxima.info-3" 73534 2643 "Funciones y variables para la diferenciación"))
("derivabbrev" . ("maxima.info-3" 76178 278 "Funciones y variables para la diferenciación"))
("derivdegree" . ("maxima.info-3" 76457 606 "Funciones y variables para la diferenciación"))
("derivlist" . ("maxima.info-3" 77064 164 "Funciones y variables para la diferenciación"))
("derivsubst" . ("maxima.info-3" 77229 220 "Funciones y variables para la diferenciación"))
("describe" . ("maxima.info-1" 34433 3035 "Funciones y variables para la ayuda"))
("describe <1>" . ("maxima.info-1" 34434 3034 "Funciones y variables para la ayuda"))
("describe <2>" . ("maxima.info-1" 34467 3001 "Funciones y variables para la ayuda"))
("desolve" . ("maxima.info-3" 191448 2241 "Funciones y variables para ecuaciones diferenciales"))
("desolve <1>" . ("maxima.info-3" 191449 2240 "Funciones y variables para ecuaciones diferenciales"))
("determinant" . ("maxima.info-3" 241252 379 "Funciones y variables para las matrices y el álgebra lineal"))
("detout" . ("maxima.info-3" 241632 972 "Funciones y variables para las matrices y el álgebra lineal"))
("dfloat" . ("maxima.info-8" 134561 840 "Funciones y variables para to_poly_solve"))
("dgauss_a" . ("maxima.info-5" 219107 104 "Funciones y variables para contrib_ode"))
("dgauss_b" . ("maxima.info-5" 219212 104 "Funciones y variables para contrib_ode"))
("dgeev" . ("maxima.info-7" 160397 3071 "Funciones y variables para lapack"))
("dgeev <1>" . ("maxima.info-7" 160398 3070 "Funciones y variables para lapack"))
("dgemm" . ("maxima.info-7" 174712 5507 "Funciones y variables para lapack"))
("dgemm <1>" . ("maxima.info-7" 174713 5506 "Funciones y variables para lapack"))
("dgeqrf" . ("maxima.info-7" 163469 1433 "Funciones y variables para lapack"))
("dgesv" . ("maxima.info-7" 164903 3644 "Funciones y variables para lapack"))
("dgesvd" . ("maxima.info-7" 168548 5372 "Funciones y variables para lapack"))
("dgesvd <1>" . ("maxima.info-7" 168549 5371 "Funciones y variables para lapack"))
("diag" . ("maxima.info-6" 22179 943 "Funciones y variables para diag"))
("diag_matrix" . ("maxima.info-7" 206153 984 "Funciones y variables para linearalgebra"))
("diagmatrix" . ("maxima.info-3" 242605 525 "Funciones y variables para las matrices y el álgebra lineal"))
("diagmatrixp" . ("maxima.info-4" 110073 116 "Funciones y variables para ctensor"))
("diagmetric" . ("maxima.info-4" 124660 393 "Funciones y variables para ctensor"))
("diameter" . ("maxima.info-7" 92337 216 "Funciones y variables para graphs"))
("diff" . ("maxima.info-3" 80770 190 "Funciones y variables para la diferenciación"))
("diff <1>" . ("maxima.info-3" 77450 3319 "Funciones y variables para la diferenciación"))
("diff <2>" . ("maxima.info-3" 77451 3318 "Funciones y variables para la diferenciación"))
("diff <3>" . ("maxima.info-3" 77511 3258 "Funciones y variables para la diferenciación"))
("diff <4>" . ("maxima.info-3" 77548 3221 "Funciones y variables para la diferenciación"))
("diff <5>" . ("maxima.info-4" 28518 1389 "Funciones y variables para itensor"))
("digitcharp" . ("maxima.info-8" 109429 78 "Funciones y variables para caracteres"))
("dim" . ("maxima.info-4" 124456 203 "Funciones y variables para ctensor"))
("dimacs_export" . ("maxima.info-7" 119797 236 "Funciones y variables para graphs"))
("dimacs_export <1>" . ("maxima.info-7" 119798 235 "Funciones y variables para graphs"))
("dimacs_import" . ("maxima.info-7" 120034 101 "Funciones y variables para graphs"))
("dimension" . ("maxima.info-3" 159948 247 "Funciones y variable para las ecuaciones"))
("dimension <1>" . ("maxima.info-3" 159949 246 "Funciones y variable para las ecuaciones"))
("dimensionless" . ("maxima.info-7" 53276 1116 "Funciones y variables para ezunits"))
("dimensions" . ("maxima.info-6" 144241 1647 "Funciones y variables para draw"))
("dimensions <1>" . ("maxima.info-7" 50088 1785 "Funciones y variables para ezunits"))
("dimensions_as_list" . ("maxima.info-7" 50089 1784 "Funciones y variables para ezunits"))
("direct" . ("maxima.info-4" 233787 3720 "Funciones y variables para simetrías"))
("discrete_freq" . ("maxima.info-5" 233831 631 "Funciones y variables para el tratamiento de datos"))
("disjoin" . ("maxima.info-5" 14113 759 "Funciones y variables para los conjuntos"))
("disjointp" . ("maxima.info-5" 14873 417 "Funciones y variables para los conjuntos"))
("disolate" . ("maxima.info-1" 267032 477 "Funciones y variables para expresiones"))
("disp" . ("maxima.info-1" 74681 696 "Funciones y variables para la impresión"))
("dispJordan" . ("maxima.info-6" 25654 1068 "Funciones y variables para diag"))
("dispcon" . ("maxima.info-4" 343 278 "Funciones y variables para itensor"))
("dispcon <1>" . ("maxima.info-4" 344 277 "Funciones y variables para itensor"))
("dispflag" . ("maxima.info-3" 160196 373 "Funciones y variable para las ecuaciones"))
("dispform" . ("maxima.info-1" 267510 1845 "Funciones y variables para expresiones"))
("dispform <1>" . ("maxima.info-1" 267511 1844 "Funciones y variables para expresiones"))
("dispfun" . ("maxima.info-5" 94459 3834 "Funciones y variables para la definición de funciones"))
("dispfun <1>" . ("maxima.info-5" 94460 3833 "Funciones y variables para la definición de funciones"))
("display" . ("maxima.info-1" 75378 868 "Funciones y variables para la impresión"))
("display2d" . ("maxima.info-1" 76247 614 "Funciones y variables para la impresión"))
("display_format_internal" . ("maxima.info-1" 76862 695 "Funciones y variables para la impresión"))
("disprule" . ("maxima.info-4" 300404 1431 "Funciones y variables sobre reglas y patrones"))
("disprule <1>" . ("maxima.info-4" 300405 1430 "Funciones y variables sobre reglas y patrones"))
("dispterms" . ("maxima.info-1" 77558 1057 "Funciones y variables para la impresión"))
("distrib" . ("maxima.info-2" 8169 868 "Funciones y variables para simplificación"))
("distribute_over" . ("maxima.info-2" 9038 1846 "Funciones y variables para simplificación"))
("divide" . ("maxima.info-2" 225045 609 "Funciones y variables para polinomios"))
("divisors" . ("maxima.info-5" 15291 1669 "Funciones y variables para los conjuntos"))
("divsum" . ("maxima.info-4" 200838 580 "Funciones y variables para teoría de números"))
("divsum <1>" . ("maxima.info-4" 200839 579 "Funciones y variables para teoría de números"))
("dkummer_m" . ("maxima.info-5" 219967 97 "Funciones y variables para contrib_ode"))
("dkummer_u" . ("maxima.info-5" 220065 99 "Funciones y variables para contrib_ode"))
("dlange" . ("maxima.info-7" 173921 790 "Funciones y variables para lapack"))
("do" . ("maxima.info-5" 142347 8632 "Funciones y variables para la programación"))
("doallmxops" . ("maxima.info-3" 243131 277 "Funciones y variables para las matrices y el álgebra lineal"))
("dodecahedron_graph" . ("maxima.info-7" 82432 74 "Funciones y variables para graphs"))
("domain" . ("maxima.info-2" 10885 174 "Funciones y variables para simplificación"))
("domxexpt" . ("maxima.info-3" 243409 1329 "Funciones y variables para las matrices y el álgebra lineal"))
("domxmxops" . ("maxima.info-3" 244739 308 "Funciones y variables para las matrices y el álgebra lineal"))
("domxnctimes" . ("maxima.info-3" 245048 170 "Funciones y variables para las matrices y el álgebra lineal"))
("dontfactor" . ("maxima.info-3" 245219 227 "Funciones y variables para las matrices y el álgebra lineal"))
("doscmxops" . ("maxima.info-3" 245447 164 "Funciones y variables para las matrices y el álgebra lineal"))
("doscmxplus" . ("maxima.info-3" 245612 184 "Funciones y variables para las matrices y el álgebra lineal"))
("dot0nscsimp" . ("maxima.info-3" 245797 137 "Funciones y variables para las matrices y el álgebra lineal"))
("dotassoc" . ("maxima.info-3" 245935 162 "Funciones y variables para las matrices y el álgebra lineal"))
("dotconstrules" . ("maxima.info-3" 246098 223 "Funciones y variables para las matrices y el álgebra lineal"))
("dotdistrib" . ("maxima.info-3" 246322 171 "Funciones y variables para las matrices y el álgebra lineal"))
("dotexptsimp" . ("maxima.info-3" 246494 161 "Funciones y variables para las matrices y el álgebra lineal"))
("dotident" . ("maxima.info-3" 246656 141 "Funciones y variables para las matrices y el álgebra lineal"))
("dotproduct" . ("maxima.info-7" 207138 210 "Funciones y variables para linearalgebra"))
("dotscrules" . ("maxima.info-3" 246798 197 "Funciones y variables para las matrices y el álgebra lineal"))
("dotsimp" . ("maxima.info-3" 289635 267 "Funciones y variables para Afines"))
("dpart" . ("maxima.info-1" 269356 583 "Funciones y variables para expresiones"))
("draw" . ("maxima.info-6" 123170 1871 "Funciones y variables para draw"))
("draw2d" . ("maxima.info-6" 125042 316 "Funciones y variables para draw"))
("draw3d" . ("maxima.info-6" 125359 316 "Funciones y variables para draw"))
("draw_file" . ("maxima.info-6" 125676 484 "Funciones y variables para draw"))
("draw_graph" . ("maxima.info-7" 121113 2962 "Funciones y variables para graphs"))
("draw_graph <1>" . ("maxima.info-7" 121114 2961 "Funciones y variables para graphs"))
("draw_graph_program" . ("maxima.info-7" 124076 185 "Funciones y variables para graphs"))
("draw_realpart" . ("maxima.info-6" 145889 826 "Funciones y variables para draw"))
("drawdf" . ("maxima.info-6" 272264 8304 "Funciones y variables para drawdf"))
("drawdf <1>" . ("maxima.info-6" 272265 8303 "Funciones y variables para drawdf"))
("drawdf <2>" . ("maxima.info-6" 272321 8247 "Funciones y variables para drawdf"))
("drawdf <3>" . ("maxima.info-6" 272445 8123 "Funciones y variables para drawdf"))
("drawdf <4>" . ("maxima.info-6" 272506 8062 "Funciones y variables para drawdf"))
("drawdf <5>" . ("maxima.info-6" 272645 7923 "Funciones y variables para drawdf"))
("dscalar" . ("maxima.info-4" 107894 886 "Funciones y variables para ctensor"))
("e" . ("maxima.info-1" 116273 223 "Funciones y variables para Constantes"))
("echelon" . ("maxima.info-3" 246996 1561 "Funciones y variables para las matrices y el álgebra lineal"))
("edge_color" . ("maxima.info-7" 126203 75 "Funciones y variables para graphs"))
("edge_coloring" . ("maxima.info-7" 127123 170 "Funciones y variables para graphs"))
("edge_coloring <1>" . ("maxima.info-7" 92554 754 "Funciones y variables para graphs"))
("edge_connectivity" . ("maxima.info-7" 93578 135 "Funciones y variables para graphs"))
("edge_partition" . ("maxima.info-7" 126939 183 "Funciones y variables para graphs"))
("edge_type" . ("maxima.info-7" 126344 136 "Funciones y variables para graphs"))
("edge_width" . ("maxima.info-7" 126279 64 "Funciones y variables para graphs"))
("edges" . ("maxima.info-7" 93714 261 "Funciones y variables para graphs"))
("eigens_by_jacobi" . ("maxima.info-7" 207349 3211 "Funciones y variables para linearalgebra"))
("eigens_by_jacobi <1>" . ("maxima.info-7" 207350 3210 "Funciones y variables para linearalgebra"))
("eigenvalues" . ("maxima.info-3" 248558 1094 "Funciones y variables para las matrices y el álgebra lineal"))
("eigenvectors" . ("maxima.info-3" 249653 4898 "Funciones y variables para las matrices y el álgebra lineal"))
("eighth" . ("maxima.info-1" 128648 134 "Funciones y variables para listas"))
("einstein" . ("maxima.info-4" 86742 606 "Funciones y variables para ctensor"))
("eivals" . ("maxima.info-3" 248559 1093 "Funciones y variables para las matrices y el álgebra lineal"))
("eivects" . ("maxima.info-3" 249654 4897 "Funciones y variables para las matrices y el álgebra lineal"))
("elapsed_real_time" . ("maxima.info-4" 281365 514 "Funciones y variables para el entorno de ejecución"))
("elapsed_run_time" . ("maxima.info-4" 281880 561 "Funciones y variables para el entorno de ejecución"))
("ele2comp" . ("maxima.info-4" 237508 356 "Funciones y variables para simetrías"))
("ele2polynome" . ("maxima.info-4" 237865 840 "Funciones y variables para simetrías"))
("ele2pui" . ("maxima.info-4" 238706 357 "Funciones y variables para simetrías"))
("elem" . ("maxima.info-4" 239064 1652 "Funciones y variables para simetrías"))
("elementp" . ("maxima.info-5" 16961 424 "Funciones y variables para los conjuntos"))
("elevation" . ("maxima.info-2" 160858 522 "Opciones gráficas"))
("elevation <1>" . ("maxima.info-7" 3489 338 "Visualización usando VTK"))
("elevation_grid" . ("maxima.info-6" 221009 1049 "Funciones y variables para draw"))
("elim" . ("maxima.info-8" 135402 2601 "Funciones y variables para to_poly_solve"))
("elim_allbut" . ("maxima.info-8" 138004 574 "Funciones y variables para to_poly_solve"))
("eliminate" . ("maxima.info-2" 225655 1317 "Funciones y variables para polinomios"))
("eliminate_using" . ("maxima.info-8" 138579 1253 "Funciones y variables para to_poly_solve"))
("ellipse" . ("maxima.info-6" 222059 1198 "Funciones y variables para draw"))
("elliptic_e" . ("maxima.info-3" 56046 226 "Funciones y variables para integrales elípticas"))
("elliptic_ec" . ("maxima.info-3" 57177 313 "Funciones y variables para integrales elípticas"))
("elliptic_eu" . ("maxima.info-3" 56273 357 "Funciones y variables para integrales elípticas"))
("elliptic_f" . ("maxima.info-3" 55838 207 "Funciones y variables para integrales elípticas"))
("elliptic_kc" . ("maxima.info-3" 56863 313 "Funciones y variables para integrales elípticas"))
("elliptic_pi" . ("maxima.info-3" 56631 231 "Funciones y variables para integrales elípticas"))
("ematrix" . ("maxima.info-3" 254552 202 "Funciones y variables para las matrices y el álgebra lineal"))
("empty_graph" . ("maxima.info-7" 82507 77 "Funciones y variables para graphs"))
("emptyp" . ("maxima.info-5" 17386 318 "Funciones y variables para los conjuntos"))
("endcons" . ("maxima.info-1" 128783 265 "Funciones y variables para listas"))
("endphi" . ("maxima.info-7" 9011 461 "Visualización usando VTK"))
("endtheta" . ("maxima.info-7" 9473 466 "Visualización usando VTK"))
("enhanced3d" . ("maxima.info-6" 146716 5866 "Funciones y variables para draw"))
("entermatrix" . ("maxima.info-3" 254755 1505 "Funciones y variables para las matrices y el álgebra lineal"))
("entertensor" . ("maxima.info-4" 622 280 "Funciones y variables para itensor"))
("entier" . ("maxima.info-2" 40334 209 "Funciones para los números"))
("epsilon_lp" . ("maxima.info-8" 8473 186 "Funciones y variables para simplex"))
("equal" . ("maxima.info-2" 130560 4247 "Funciones y variables para los predicados"))
("equalp" . ("maxima.info-4" 185628 204 "Funciones y variables para series de Fourier"))
("equiv_classes" . ("maxima.info-5" 17705 1388 "Funciones y variables para los conjuntos"))
("erf" . ("maxima.info-3" 35947 95 "Función de error"))
("erf_generalized" . ("maxima.info-3" 36246 87 "Función de error"))
("erf_representation" . ("maxima.info-3" 37214 176 "Función de error"))
("erfc" . ("maxima.info-3" 36043 111 "Función de error"))
("erfflag" . ("maxima.info-3" 97148 203 "Funciones y variables para integración"))
("erfi" . ("maxima.info-3" 36155 90 "Función de error"))
("errcatch" . ("maxima.info-5" 150980 598 "Funciones y variables para la programación"))
("error" . ("maxima.info-5" 151579 648 "Funciones y variables para la programación"))
("error <1>" . ("maxima.info-5" 151580 647 "Funciones y variables para la programación"))
("error_size" . ("maxima.info-5" 152228 1578 "Funciones y variables para la programación"))
("error_syms" . ("maxima.info-5" 153807 685 "Funciones y variables para la programación"))
("error_type" . ("maxima.info-6" 152583 331 "Funciones y variables para draw"))
("errormsg" . ("maxima.info-5" 154649 1311 "Funciones y variables para la programación"))
("errormsg <1>" . ("maxima.info-5" 154493 155 "Funciones y variables para la programación"))
("errors" . ("maxima.info-6" 223258 1390 "Funciones y variables para draw"))
("euler" . ("maxima.info-4" 201419 541 "Funciones y variables para teoría de números"))
("ev" . ("maxima.info-1" 237555 9464 "Funciones y variables para la evaluación"))
("ev_point" . ("maxima.info-8" 200147 212 "Funciones y variables para zeilberger"))
("eval" . ("maxima.info-1" 247020 566 "Funciones y variables para la evaluación"))
("eval_string" . ("maxima.info-8" 111235 642 "Funciones y variables para cadenas"))
("even" . ("maxima.info-2" 100224 648 "Funciones y variables para las propiedades"))
("evenfun" . ("maxima.info-2" 11060 683 "Funciones y variables para simplificación"))
("evenp" . ("maxima.info-1" 101120 115 "Funciones y variables para los números"))
("every" . ("maxima.info-5" 19094 3019 "Funciones y variables para los conjuntos"))
("every <1>" . ("maxima.info-5" 19095 3018 "Funciones y variables para los conjuntos"))
("evflag" . ("maxima.info-1" 247587 2516 "Funciones y variables para la evaluación"))
("evfun" . ("maxima.info-1" 250104 2925 "Funciones y variables para la evaluación"))
("evolution" . ("maxima.info-6" 282612 739 "Análisis gráfico de sistemas dinámicos discretos"))
("evolution2d" . ("maxima.info-6" 283352 987 "Análisis gráfico de sistemas dinámicos discretos"))
("evundiff" . ("maxima.info-4" 31554 2376 "Funciones y variables para itensor"))
("example" . ("maxima.info-1" 37469 1906 "Funciones y variables para la ayuda"))
("example <1>" . ("maxima.info-1" 37470 1905 "Funciones y variables para la ayuda"))
("exp" . ("maxima.info-2" 59199 606 "Funciones radicales exponenciales y logarítmicas"))
("expand" . ("maxima.info-2" 11744 4470 "Funciones y variables para simplificación"))
("expand <1>" . ("maxima.info-2" 11745 4469 "Funciones y variables para simplificación"))
("expandwrt" . ("maxima.info-2" 16215 634 "Funciones y variables para simplificación"))
("expandwrt_denom" . ("maxima.info-2" 16850 407 "Funciones y variables para simplificación"))
("expandwrt_factored" . ("maxima.info-2" 17258 386 "Funciones y variables para simplificación"))
("expintegral_chi" . ("maxima.info-3" 35040 83 "Integral exponencial"))
("expintegral_ci" . ("maxima.info-3" 34874 81 "Integral exponencial"))
("expintegral_e" . ("maxima.info-3" 34707 84 "Integral exponencial"))
("expintegral_e1" . ("maxima.info-3" 34461 81 "Integral exponencial"))
("expintegral_ei" . ("maxima.info-3" 34543 81 "Integral exponencial"))
("expintegral_li" . ("maxima.info-3" 34625 81 "Integral exponencial"))
("expintegral_shi" . ("maxima.info-3" 34956 83 "Integral exponencial"))
("expintegral_si" . ("maxima.info-3" 34792 81 "Integral exponencial"))
("expintexpand" . ("maxima.info-3" 35406 242 "Integral exponencial"))
("expintrep" . ("maxima.info-3" 35124 281 "Integral exponencial"))
("explicit" . ("maxima.info-6" 224649 1830 "Funciones y variables para draw"))
("explicit <1>" . ("maxima.info-6" 224650 1829 "Funciones y variables para draw"))
("explose" . ("maxima.info-4" 240717 406 "Funciones y variables para simetrías"))
("expon" . ("maxima.info-2" 17645 286 "Funciones y variables para simplificación"))
("exponentialize" . ("maxima.info-2" 17932 636 "Funciones y variables para simplificación"))
("exponentialize <1>" . ("maxima.info-2" 17933 635 "Funciones y variables para simplificación"))
("expop" . ("maxima.info-2" 18569 410 "Funciones y variables para simplificación"))
("express" . ("maxima.info-3" 80961 2912 "Funciones y variables para la diferenciación"))
("expt" . ("maxima.info-1" 78616 330 "Funciones y variables para la impresión"))
("exptdispflag" . ("maxima.info-1" 78947 701 "Funciones y variables para la impresión"))
("exptisolate" . ("maxima.info-1" 269940 90 "Funciones y variables para expresiones"))
("exptsubst" . ("maxima.info-1" 270031 156 "Funciones y variables para expresiones"))
("exsec" . ("maxima.info-8" 25629 118 "Paquete functs"))
("extdiff" . ("maxima.info-4" 63353 1496 "Funciones y variables para itensor"))
("extract_linear_equations" . ("maxima.info-3" 291313 419 "Funciones y variables para Afines"))
("extremal_subset" . ("maxima.info-5" 22114 765 "Funciones y variables para los conjuntos"))
("extremal_subset <1>" . ("maxima.info-5" 22115 764 "Funciones y variables para los conjuntos"))
("ezgcd" . ("maxima.info-2" 226973 1217 "Funciones y variables para polinomios"))
("f90" . ("maxima.info-7" 54930 1928 "Funciones y variables para f90"))
("facexpand" . ("maxima.info-2" 228191 232 "Funciones y variables para polinomios"))
("facsum" . ("maxima.info-8" 15341 1843 "Paquete facexp"))
("facsum_combine" . ("maxima.info-8" 17965 387 "Paquete facexp"))
("factcomb" . ("maxima.info-2" 52071 305 "Funciones combinatorias"))
("factlim" . ("maxima.info-2" 56161 224 "Funciones combinatorias"))
("factor" . ("maxima.info-2" 228424 6357 "Funciones y variables para polinomios"))
("factor <1>" . ("maxima.info-2" 228425 6356 "Funciones y variables para polinomios"))
("factorfacsum" . ("maxima.info-8" 18353 342 "Paquete facexp"))
("factorflag" . ("maxima.info-2" 234782 179 "Funciones y variables para polinomios"))
("factorial" . ("maxima.info-2" 52377 3783 "Funciones combinatorias"))
("factorial_expand" . ("maxima.info-2" 56386 230 "Funciones combinatorias"))
("factorout" . ("maxima.info-2" 234962 689 "Funciones y variables para polinomios"))
("factors_only" . ("maxima.info-4" 201961 399 "Funciones y variables para teoría de números"))
("factorsum" . ("maxima.info-2" 235652 884 "Funciones y variables para polinomios"))
("facts" . ("maxima.info-2" 123517 502 "Funciones y variables para los hechos"))
("facts <1>" . ("maxima.info-2" 123518 501 "Funciones y variables para los hechos"))
("false" . ("maxima.info-1" 116581 141 "Funciones y variables para Constantes"))
("fast_central_elements" . ("maxima.info-3" 289903 423 "Funciones y variables para Afines"))
("fast_linsolve" . ("maxima.info-3" 287213 757 "Funciones y variables para Afines"))
("fasttimes" . ("maxima.info-2" 236537 422 "Funciones y variables para polinomios"))
("fb" . ("maxima.info-4" 129271 122 "Funciones y variables para ctensor"))
("feature" . ("maxima.info-2" 100873 381 "Funciones y variables para las propiedades"))
("featurep" . ("maxima.info-2" 101255 553 "Funciones y variables para las propiedades"))
("features" . ("maxima.info-2" 101809 1397 "Funciones y variables para las propiedades"))
("fernfale" . ("maxima.info-7" 71210 367 "Definiciones para IFS fractals"))
("fft" . ("maxima.info-3" 203871 3939 "Funciones y variables para la transformada rápida de Fourier"))
("fib" . ("maxima.info-4" 202361 493 "Funciones y variables para teoría de números"))
("fibtophi" . ("maxima.info-4" 202855 1203 "Funciones y variables para teoría de números"))
("fifth" . ("maxima.info-1" 129049 133 "Funciones y variables para listas"))
("file_name" . ("maxima.info-6" 152915 646 "Funciones y variables para draw"))
("file_name <1>" . ("maxima.info-7" 128061 116 "Funciones y variables para graphs"))
("file_output_append" . ("maxima.info-2" 183234 994 "Funciones y variables para lectura y escritura"))
("file_search" . ("maxima.info-2" 184618 2000 "Funciones y variables para lectura y escritura"))
("file_search <1>" . ("maxima.info-2" 184619 1999 "Funciones y variables para lectura y escritura"))
("file_search_demo" . ("maxima.info-2" 186662 1598 "Funciones y variables para lectura y escritura"))
("file_search_lisp" . ("maxima.info-2" 186620 1640 "Funciones y variables para lectura y escritura"))
("file_search_maxima" . ("maxima.info-2" 186619 1641 "Funciones y variables para lectura y escritura"))
("file_search_tests" . ("maxima.info-2" 186742 1518 "Funciones y variables para lectura y escritura"))
("file_search_usage" . ("maxima.info-2" 186702 1558 "Funciones y variables para lectura y escritura"))
("file_type" . ("maxima.info-2" 188261 731 "Funciones y variables para lectura y escritura"))
("file_type_lisp" . ("maxima.info-2" 188993 221 "Funciones y variables para lectura y escritura"))
("file_type_maxima" . ("maxima.info-2" 189215 253 "Funciones y variables para lectura y escritura"))
("filename_merge" . ("maxima.info-2" 184229 388 "Funciones y variables para lectura y escritura"))
("fill_color" . ("maxima.info-6" 153562 240 "Funciones y variables para draw"))
("fill_density" . ("maxima.info-6" 153803 246 "Funciones y variables para draw"))
("fillarray" . ("maxima.info-1" 159292 2262 "Funciones y variables para los arrays"))
("filled_func" . ("maxima.info-6" 154050 1498 "Funciones y variables para draw"))
("find_root" . ("maxima.info-3" 209315 4148 "Funciones para la resolución numérica de ecuaciones"))
("find_root <1>" . ("maxima.info-3" 209316 4147 "Funciones para la resolución numérica de ecuaciones"))
("find_root_abs" . ("maxima.info-3" 209582 3881 "Funciones para la resolución numérica de ecuaciones"))
("find_root_error" . ("maxima.info-3" 209518 3945 "Funciones para la resolución numérica de ecuaciones"))
("find_root_rel" . ("maxima.info-3" 209621 3842 "Funciones para la resolución numérica de ecuaciones"))
("findde" . ("maxima.info-4" 104130 2348 "Funciones y variables para ctensor"))
("first" . ("maxima.info-1" 129183 903 "Funciones y variables para listas"))
("fix" . ("maxima.info-2" 42561 62 "Funciones para los números"))
("fixed_vertices" . ("maxima.info-7" 128644 188 "Funciones y variables para graphs"))
("flatten" . ("maxima.info-5" 22880 2661 "Funciones y variables para los conjuntos"))
("flength" . ("maxima.info-8" 97590 148 "Funciones y variables para entrada y salida"))
("flipflag" . ("maxima.info-4" 6848 661 "Funciones y variables para itensor"))
("float" . ("maxima.info-1" 101236 238 "Funciones y variables para los números"))
("float2bf" . ("maxima.info-1" 101475 290 "Funciones y variables para los números"))
("floatnump" . ("maxima.info-1" 101766 134 "Funciones y variables para los números"))
("floor" . ("maxima.info-2" 40544 2016 "Funciones para los números"))
("flower_snark" . ("maxima.info-7" 82585 254 "Funciones y variables para graphs"))
("flush" . ("maxima.info-4" 33931 177 "Funciones y variables para itensor"))
("flush1deriv" . ("maxima.info-4" 39883 157 "Funciones y variables para itensor"))
("flushd" . ("maxima.info-4" 34109 174 "Funciones y variables para itensor"))
("flushnd" . ("maxima.info-4" 34284 721 "Funciones y variables para itensor"))
("fmin_cobyla" . ("maxima.info-5" 205666 2452 "Funciones y variables para cobyla"))
("fmin_cobyla <1>" . ("maxima.info-5" 205667 2451 "Funciones y variables para cobyla"))
("font" . ("maxima.info-6" 155549 3569 "Funciones y variables para draw"))
("font_size" . ("maxima.info-6" 159119 473 "Funciones y variables para draw"))
("for" . ("maxima.info-5" 155961 140 "Funciones y variables para la programación"))
("forget" . ("maxima.info-2" 124020 376 "Funciones y variables para los hechos"))
("forget <1>" . ("maxima.info-2" 124021 375 "Funciones y variables para los hechos"))
("fortindent" . ("maxima.info-2" 212685 322 "Funciones y variables para salida Fortran"))
("fortran" . ("maxima.info-2" 213008 2222 "Funciones y variables para salida Fortran"))
("fortspaces" . ("maxima.info-2" 215231 166 "Funciones y variables para salida Fortran"))
("fourcos" . ("maxima.info-4" 187914 137 "Funciones y variables para series de Fourier"))
("fourexpand" . ("maxima.info-4" 187625 288 "Funciones y variables para series de Fourier"))
("fourier" . ("maxima.info-4" 187144 148 "Funciones y variables para series de Fourier"))
("fourier_elim" . ("maxima.info-8" 139833 3299 "Funciones y variables para to_poly_solve"))
("fourint" . ("maxima.info-4" 188317 153 "Funciones y variables para series de Fourier"))
("fourintcos" . ("maxima.info-4" 188471 123 "Funciones y variables para series de Fourier"))
("fourintsin" . ("maxima.info-4" 188595 123 "Funciones y variables para series de Fourier"))
("foursimp" . ("maxima.info-4" 187293 151 "Funciones y variables para series de Fourier"))
("foursin" . ("maxima.info-4" 188052 135 "Funciones y variables para series de Fourier"))
("fourth" . ("maxima.info-1" 130087 134 "Funciones y variables para listas"))
("fposition" . ("maxima.info-8" 97739 412 "Funciones y variables para entrada y salida"))
("fposition <1>" . ("maxima.info-8" 97740 411 "Funciones y variables para entrada y salida"))
("fpprec" . ("maxima.info-1" 101901 368 "Funciones y variables para los números"))
("fpprintprec" . ("maxima.info-1" 102270 944 "Funciones y variables para los números"))
("frame_bracket" . ("maxima.info-4" 96813 355 "Funciones y variables para ctensor"))
("freeof" . ("maxima.info-1" 270188 3589 "Funciones y variables para expresiones"))
("freshline" . ("maxima.info-8" 98152 217 "Funciones y variables para entrada y salida"))
("freshline <1>" . ("maxima.info-8" 98153 216 "Funciones y variables para entrada y salida"))
("fresnel_c" . ("maxima.info-3" 36334 442 "Función de error"))
("fresnel_s" . ("maxima.info-3" 36777 436 "Función de error"))
("from_adjacency_matrix" . ("maxima.info-7" 82840 106 "Funciones y variables para graphs"))
("frucht_graph" . ("maxima.info-7" 82947 63 "Funciones y variables para graphs"))
("full_listify" . ("maxima.info-5" 25542 617 "Funciones y variables para los conjuntos"))
("fullmap" . ("maxima.info-5" 98294 726 "Funciones y variables para la definición de funciones"))
("fullmapl" . ("maxima.info-5" 99021 252 "Funciones y variables para la definición de funciones"))
("fullratsimp" . ("maxima.info-2" 236960 1545 "Funciones y variables para polinomios"))
("fullratsubst" . ("maxima.info-2" 238506 2012 "Funciones y variables para polinomios"))
("fullsetify" . ("maxima.info-5" 26160 668 "Funciones y variables para los conjuntos"))
("funcsolve" . ("maxima.info-3" 160570 909 "Funciones y variable para las ecuaciones"))
("functions" . ("maxima.info-5" 99274 1959 "Funciones y variables para la definición de funciones"))
("fundamental_dimensions" . ("maxima.info-7" 47080 1321 "Funciones y variables para ezunits"))
("fundamental_units" . ("maxima.info-7" 51874 1401 "Funciones y variables para ezunits"))
("fundamental_units <1>" . ("maxima.info-7" 51875 1400 "Funciones y variables para ezunits"))
("fundef" . ("maxima.info-5" 101234 939 "Funciones y variables para la definición de funciones"))
("funmake" . ("maxima.info-5" 102174 3243 "Funciones y variables para la definición de funciones"))
("funp" . ("maxima.info-4" 186187 349 "Funciones y variables para series de Fourier"))
("funp <1>" . ("maxima.info-4" 186188 348 "Funciones y variables para series de Fourier"))
("fv" . ("maxima.info-7" 58129 357 "Funciones y Variables para finance"))
("gamma" . ("maxima.info-3" 2917 3447 "Funciones Gamma y factorial"))
("gamma_expand" . ("maxima.info-3" 8901 856 "Funciones Gamma y factorial"))
("gamma_incomplete" . ("maxima.info-3" 6982 682 "Funciones Gamma y factorial"))
("gamma_incomplete_generalized" . ("maxima.info-3" 8228 672 "Funciones Gamma y factorial"))
("gamma_incomplete_lower" . ("maxima.info-3" 6439 542 "Funciones Gamma y factorial"))
("gamma_incomplete_regularized" . ("maxima.info-3" 7665 562 "Funciones Gamma y factorial"))
("gammalim" . ("maxima.info-3" 9758 416 "Funciones Gamma y factorial"))
("gauss_a" . ("maxima.info-5" 218440 593 "Funciones y variables para contrib_ode"))
("gauss_b" . ("maxima.info-5" 219034 72 "Funciones y variables para contrib_ode"))
("gaussprob" . ("maxima.info-8" 24887 188 "Paquete functs"))
("gcd" . ("maxima.info-2" 240519 1206 "Funciones y variables para polinomios"))
("gcdex" . ("maxima.info-2" 241726 1016 "Funciones y variables para polinomios"))
("gcdex <1>" . ("maxima.info-2" 241727 1015 "Funciones y variables para polinomios"))
("gcdivide" . ("maxima.info-8" 21999 1710 "Paquete functs"))
("gcfac" . ("maxima.info-8" 32439 1006 "Paquete scifac"))
("gcfactor" . ("maxima.info-2" 242743 320 "Funciones y variables para polinomios"))
("gd" . ("maxima.info-8" 25076 145 "Paquete functs"))
("gdet" . ("maxima.info-4" 126694 142 "Funciones y variables para ctensor"))
("gen_laguerre" . ("maxima.info-7" 298775 165 "Funciones y variables para polinomios ortogonales"))
("generalized_lambert_w" . ("maxima.info-3" 48085 376 "Funciones y variables para las funciones especiales"))
("genfact" . ("maxima.info-2" 56617 225 "Funciones combinatorias"))
("genindex" . ("maxima.info-4" 284311 198 "Funciones y variables para la miscelánea de opciones"))
("genmatrix" . ("maxima.info-3" 256261 2735 "Funciones y variables para las matrices y el álgebra lineal"))
("genmatrix <1>" . ("maxima.info-3" 256262 2734 "Funciones y variables para las matrices y el álgebra lineal"))
("genmatrix <2>" . ("maxima.info-3" 256319 2677 "Funciones y variables para las matrices y el álgebra lineal"))
("gensumnum" . ("maxima.info-4" 284510 277 "Funciones y variables para la miscelánea de opciones"))
("gensym" . ("maxima.info-4" 284788 1195 "Funciones y variables para la miscelánea de opciones"))
("gensym <1>" . ("maxima.info-4" 284789 1194 "Funciones y variables para la miscelánea de opciones"))
("geo_amortization" . ("maxima.info-7" 64132 1744 "Funciones y Variables para finance"))
("geo_annuity_fv" . ("maxima.info-7" 60545 480 "Funciones y Variables para finance"))
("geo_annuity_pv" . ("maxima.info-7" 60064 480 "Funciones y Variables para finance"))
("geomap" . ("maxima.info-6" 264086 5947 "Funciones y variables para worldmap"))
("geomap <1>" . ("maxima.info-6" 264087 5946 "Funciones y variables para worldmap"))
("geometric" . ("maxima.info-8" 23938 220 "Paquete functs"))
("geometric_mean" . ("maxima.info-5" 255759 897 "Funciones y variables de parámetros descriptivos"))
("geometric_mean <1>" . ("maxima.info-5" 255760 896 "Funciones y variables de parámetros descriptivos"))
("geosum" . ("maxima.info-8" 24605 281 "Paquete functs"))
("get" . ("maxima.info-2" 103207 1044 "Funciones y variables para las propiedades"))
("get_edge_weight" . ("maxima.info-7" 93976 702 "Funciones y variables para graphs"))
("get_edge_weight <1>" . ("maxima.info-7" 93977 701 "Funciones y variables para graphs"))
("get_lu_factors" . ("maxima.info-7" 210561 314 "Funciones y variables para linearalgebra"))
("get_output_stream_string" . ("maxima.info-8" 98370 294 "Funciones y variables para entrada y salida"))
("get_pixel" . ("maxima.info-6" 253163 171 "Funciones y variables para picture"))
("get_plot_option" . ("maxima.info-2" 142967 389 "Funciones y variables para gráficos"))
("get_tex_environment" . ("maxima.info-2" 210152 1090 "Funciones y variables para salida TeX"))
("get_tex_environment_default" . ("maxima.info-2" 211243 1205 "Funciones y variables para salida TeX"))
("get_vertex_label" . ("maxima.info-7" 94679 307 "Funciones y variables para graphs"))
("gfactor" . ("maxima.info-2" 243064 365 "Funciones y variables para polinomios"))
("gfactorsum" . ("maxima.info-2" 243430 125 "Funciones y variables para polinomios"))
("ggf" . ("maxima.info-7" 77170 564 "Funciones y variables para ggf"))
("girth" . ("maxima.info-7" 96749 248 "Funciones y variables para graphs"))
("global_variances" . ("maxima.info-5" 265188 1840 "Funciones y variables de parámetros descriptivos"))
("global_variances <1>" . ("maxima.info-5" 265189 1839 "Funciones y variables de parámetros descriptivos"))
("globalsolve" . ("maxima.info-3" 161480 2136 "Funciones y variable para las ecuaciones"))
("gnuplot_close" . ("maxima.info-2" 175757 118 "Funciones para el formato Gnuplot_pipes"))
("gnuplot_curve_styles" . ("maxima.info-2" 174330 101 "Opciones para Gnuplot"))
("gnuplot_curve_titles" . ("maxima.info-2" 174227 102 "Opciones para Gnuplot"))
("gnuplot_default_term_command" . ("maxima.info-2" 174432 170 "Opciones para Gnuplot"))
("gnuplot_dumb_term_command" . ("maxima.info-2" 174603 246 "Opciones para Gnuplot"))
("gnuplot_file_name" . ("maxima.info-6" 159593 738 "Funciones y variables para draw"))
("gnuplot_out_file" . ("maxima.info-2" 173208 410 "Opciones para Gnuplot"))
("gnuplot_pm3d" . ("maxima.info-2" 173619 284 "Opciones para Gnuplot"))
("gnuplot_preamble" . ("maxima.info-2" 173904 322 "Opciones para Gnuplot"))
("gnuplot_ps_term_command" . ("maxima.info-2" 174850 480 "Opciones para Gnuplot"))
("gnuplot_replot" . ("maxima.info-2" 176020 256 "Funciones para el formato Gnuplot_pipes"))
("gnuplot_replot <1>" . ("maxima.info-2" 176021 255 "Funciones para el formato Gnuplot_pipes"))
("gnuplot_reset" . ("maxima.info-2" 176277 206 "Funciones para el formato Gnuplot_pipes"))
("gnuplot_restart" . ("maxima.info-2" 175876 143 "Funciones para el formato Gnuplot_pipes"))
("gnuplot_start" . ("maxima.info-2" 175534 222 "Funciones para el formato Gnuplot_pipes"))
("gnuplot_term" . ("maxima.info-2" 171698 1509 "Opciones para Gnuplot"))
("go" . ("maxima.info-5" 156102 632 "Funciones y variables para la programación"))
("gr2d" . ("maxima.info-6" 121293 906 "Funciones y variables para draw"))
("gr3d" . ("maxima.info-6" 122200 935 "Funciones y variables para draw"))
("gradef" . ("maxima.info-3" 83874 2118 "Funciones y variables para la diferenciación"))
("gradef <1>" . ("maxima.info-3" 83875 2117 "Funciones y variables para la diferenciación"))
("gradefs" . ("maxima.info-3" 85993 303 "Funciones y variables para la diferenciación"))
("gramschmidt" . ("maxima.info-3" 258997 2481 "Funciones y variables para las matrices y el álgebra lineal"))
("gramschmidt <1>" . ("maxima.info-3" 258998 2480 "Funciones y variables para las matrices y el álgebra lineal"))
("graph6_decode" . ("maxima.info-7" 120136 107 "Funciones y variables para graphs"))
("graph6_encode" . ("maxima.info-7" 120244 104 "Funciones y variables para graphs"))
("graph6_export" . ("maxima.info-7" 120349 131 "Funciones y variables para graphs"))
("graph6_import" . ("maxima.info-7" 120481 117 "Funciones y variables para graphs"))
("graph_center" . ("maxima.info-7" 95363 241 "Funciones y variables para graphs"))
("graph_charpoly" . ("maxima.info-7" 94987 375 "Funciones y variables para graphs"))
("graph_eigenvalues" . ("maxima.info-7" 95605 386 "Funciones y variables para graphs"))
("graph_flow" . ("maxima.info-7" 58826 424 "Funciones y Variables para finance"))
("graph_order" . ("maxima.info-7" 96497 251 "Funciones y variables para graphs"))
("graph_periphery" . ("maxima.info-7" 95992 255 "Funciones y variables para graphs"))
("graph_product" . ("maxima.info-7" 83011 251 "Funciones y variables para graphs"))
("graph_size" . ("maxima.info-7" 96248 248 "Funciones y variables para graphs"))
("graph_union" . ("maxima.info-7" 83263 95 "Funciones y variables para graphs"))
("great_rhombicosidodecahedron_graph" . ("maxima.info-7" 83429 102 "Funciones y variables para graphs"))
("great_rhombicuboctahedron_graph" . ("maxima.info-7" 83532 101 "Funciones y variables para graphs"))
("grid" . ("maxima.info-2" 161381 198 "Opciones gráficas"))
("grid <1>" . ("maxima.info-6" 160332 729 "Funciones y variables para draw"))
("grid_graph" . ("maxima.info-7" 83359 69 "Funciones y variables para graphs"))
("grind" . ("maxima.info-1" 79649 4690 "Funciones y variables para la impresión"))
("grind <1>" . ("maxima.info-1" 79650 4689 "Funciones y variables para la impresión"))
("grobner_basis" . ("maxima.info-3" 287971 631 "Funciones y variables para Afines"))
("grotzch_graph" . ("maxima.info-7" 83634 65 "Funciones y variables para graphs"))
("halfangles" . ("maxima.info-2" 73755 2256 "Funciones y variables para trigonometría"))
("hamilton_cycle" . ("maxima.info-7" 96998 380 "Funciones y variables para graphs"))
("hamilton_path" . ("maxima.info-7" 97379 382 "Funciones y variables para graphs"))
("hankel" . ("maxima.info-7" 210876 320 "Funciones y variables para linearalgebra"))
("hankel <1>" . ("maxima.info-7" 210877 319 "Funciones y variables para linearalgebra"))
("hankel_1" . ("maxima.info-2" 285024 2023 "Funciones de Bessel"))
("hankel_2" . ("maxima.info-2" 287048 751 "Funciones de Bessel"))
("harmonic" . ("maxima.info-8" 24159 255 "Paquete functs"))
("harmonic_mean" . ("maxima.info-5" 254853 905 "Funciones y variables de parámetros descriptivos"))
("harmonic_mean <1>" . ("maxima.info-5" 254854 904 "Funciones y variables de parámetros descriptivos"))
("hav" . ("maxima.info-8" 25748 117 "Paquete functs"))
("head_angle" . ("maxima.info-6" 161062 1228 "Funciones y variables para draw"))
("head_angle <1>" . ("maxima.info-7" 127521 131 "Funciones y variables para graphs"))
("head_both" . ("maxima.info-6" 162291 631 "Funciones y variables para draw"))
("head_length" . ("maxima.info-6" 162923 728 "Funciones y variables para draw"))
("head_length <1>" . ("maxima.info-7" 127653 135 "Funciones y variables para graphs"))
("head_type" . ("maxima.info-6" 163652 886 "Funciones y variables para draw"))
("heawood_graph" . ("maxima.info-7" 83700 65 "Funciones y variables para graphs"))
("height" . ("maxima.info-7" 3828 179 "Visualización usando VTK"))
("height <1>" . ("maxima.info-7" 9940 166 "Visualización usando VTK"))
("hermite" . ("maxima.info-7" 298941 128 "Funciones y variables para polinomios ortogonales"))
("hessian" . ("maxima.info-7" 211197 1123 "Funciones y variables para linearalgebra"))
("hgfred" . ("maxima.info-3" 45728 2212 "Funciones y variables para las funciones especiales"))
("hilbert_matrix" . ("maxima.info-7" 212321 149 "Funciones y variables para linearalgebra"))
("hilbertmap" . ("maxima.info-7" 75164 259 "Definiciones para curvas de Peano"))
("hipow" . ("maxima.info-2" 243556 924 "Funciones y variables para polinomios"))
("histogram" . ("maxima.info-6" 10268 3630 "Funciones y variables para gráficos estadísticos"))
("histogram <1>" . ("maxima.info-6" 10269 3629 "Funciones y variables para gráficos estadísticos"))
("histogram <2>" . ("maxima.info-6" 10301 3597 "Funciones y variables para gráficos estadísticos"))
("histogram <3>" . ("maxima.info-6" 10362 3536 "Funciones y variables para gráficos estadísticos"))
("histogram <4>" . ("maxima.info-6" 10476 3422 "Funciones y variables para gráficos estadísticos"))
("histogram <5>" . ("maxima.info-6" 10491 3407 "Funciones y variables para gráficos estadísticos"))
("histogram_description" . ("maxima.info-6" 10533 3365 "Funciones y variables para gráficos estadísticos"))
("hodge" . ("maxima.info-4" 64850 1493 "Funciones y variables para itensor"))
("horner" . ("maxima.info-3" 208172 1142 "Funciones para la resolución numérica de ecuaciones"))
("horner <1>" . ("maxima.info-3" 208173 1141 "Funciones para la resolución numérica de ecuaciones"))
("hypergeometric" . ("maxima.info-3" 38955 1648 "Funciones hipergeométricas"))
("hypergeometric_representation" . ("maxima.info-3" 37391 186 "Función de error"))
("i" . ("maxima.info-1" 116498 82 "Funciones y variables para Constantes"))
("i-ésima salida anterior" . ("maxima.info-1" 51828 1125 "Funciones y variables para la línea de comandos"))
("ibase" . ("maxima.info-1" 84340 2027 "Funciones y variables para la impresión"))
("ic1" . ("maxima.info-3" 193690 452 "Funciones y variables para ecuaciones diferenciales"))
("ic2" . ("maxima.info-3" 194143 641 "Funciones y variables para ecuaciones diferenciales"))
("ic_convert" . ("maxima.info-4" 68714 2072 "Funciones y variables para itensor"))
("icc1" . ("maxima.info-4" 50753 567 "Funciones y variables para itensor"))
("icc2" . ("maxima.info-4" 51321 615 "Funciones y variables para itensor"))
("ichr1" . ("maxima.info-4" 40761 399 "Funciones y variables para itensor"))
("ichr2" . ("maxima.info-4" 41161 290 "Funciones y variables para itensor"))
("icosahedron_graph" . ("maxima.info-7" 83766 70 "Funciones y variables para graphs"))
("icosidodecahedron_graph" . ("maxima.info-7" 83837 82 "Funciones y variables para graphs"))
("icounter" . ("maxima.info-4" 16611 231 "Funciones y variables para itensor"))
("icurvature" . ("maxima.info-4" 41452 607 "Funciones y variables para itensor"))
("ident" . ("maxima.info-3" 261479 73 "Funciones y variables para las matrices y el álgebra lineal"))
("identfor" . ("maxima.info-7" 212471 576 "Funciones y variables para linearalgebra"))
("identfor <1>" . ("maxima.info-7" 212472 575 "Funciones y variables para linearalgebra"))
("identity" . ("maxima.info-5" 26829 331 "Funciones y variables para los conjuntos"))
("idiff" . ("maxima.info-4" 29908 521 "Funciones y variables para itensor"))
("idim" . ("maxima.info-4" 40573 187 "Funciones y variables para itensor"))
("idummy" . ("maxima.info-4" 16167 323 "Funciones y variables para itensor"))
("idummyx" . ("maxima.info-4" 16491 119 "Funciones y variables para itensor"))
("ieqn" . ("maxima.info-3" 163617 1400 "Funciones y variable para las ecuaciones"))
("ieqnprint" . ("maxima.info-3" 165018 701 "Funciones y variable para las ecuaciones"))
("if" . ("maxima.info-5" 156735 2759 "Funciones y variables para la programación"))
("ifactors" . ("maxima.info-4" 204059 1074 "Funciones y variables para teoría de números"))
("ifb" . ("maxima.info-4" 49705 1047 "Funciones y variables para itensor"))
("ifc1" . ("maxima.info-4" 51937 490 "Funciones y variables para itensor"))
("ifc2" . ("maxima.info-4" 52428 329 "Funciones y variables para itensor"))
("ifg" . ("maxima.info-4" 53072 148 "Funciones y variables para itensor"))
("ifgi" . ("maxima.info-4" 53221 128 "Funciones y variables para itensor"))
("ifr" . ("maxima.info-4" 52758 167 "Funciones y variables para itensor"))
("iframe_bracket_form" . ("maxima.info-4" 53350 115 "Funciones y variables para itensor"))
("ifri" . ("maxima.info-4" 52926 145 "Funciones y variables para itensor"))
("ifs" . ("maxima.info-6" 284340 1550 "Análisis gráfico de sistemas dinámicos discretos"))
("igcdex" . ("maxima.info-4" 205134 686 "Funciones y variables para teoría de números"))
("igeodesic_coords" . ("maxima.info-4" 44168 2000 "Funciones y variables para itensor"))
("igeowedge_flag" . ("maxima.info-4" 66344 405 "Funciones y variables para itensor"))
("ikt1" . ("maxima.info-4" 55131 479 "Funciones y variables para itensor"))
("ikt2" . ("maxima.info-4" 55611 314 "Funciones y variables para itensor"))
("ilt" . ("maxima.info-3" 97352 2140 "Funciones y variables para integración"))
("image" . ("maxima.info-6" 226480 3940 "Funciones y variables para draw"))
("imaginary" . ("maxima.info-2" 109607 288 "Funciones y variables para las propiedades"))
("imagpart" . ("maxima.info-2" 49344 228 "Funciones para los números complejos"))
("imetric" . ("maxima.info-4" 40111 461 "Funciones y variables para itensor"))
("imetric <1>" . ("maxima.info-4" 40112 460 "Funciones y variables para itensor"))
("implicit" . ("maxima.info-6" 230421 1981 "Funciones y variables para draw"))
("implicit <1>" . ("maxima.info-6" 230422 1980 "Funciones y variables para draw"))
("implicit_derivative" . ("maxima.info-7" 145458 469 "Funciones y variables para impdiff"))
("in_neighbors" . ("maxima.info-7" 98258 387 "Funciones y variables para graphs"))
("inchar" . ("maxima.info-1" 53834 971 "Funciones y variables para la línea de comandos"))
("increasing" . ("maxima.info-2" 99603 620 "Funciones y variables para las propiedades"))
("ind" . ("maxima.info-1" 116812 208 "Funciones y variables para Constantes"))
("indexed_tensor" . ("maxima.info-4" 9301 238 "Funciones y variables para itensor"))
("indices" . ("maxima.info-4" 2249 1031 "Funciones y variables para itensor"))
("induced_subgraph" . ("maxima.info-7" 83920 499 "Funciones y variables para graphs"))
("inf" . ("maxima.info-1" 117021 79 "Funciones y variables para Constantes"))
("inference_result" . ("maxima.info-8" 46660 1888 "Funciones y variables para inference_result"))
("inferencep" . ("maxima.info-8" 48549 142 "Funciones y variables para inference_result"))
("infeval" . ("maxima.info-1" 253030 402 "Funciones y variables para la evaluación"))
("infinity" . ("maxima.info-1" 117101 84 "Funciones y variables para Constantes"))
("infix" . ("maxima.info-1" 213097 4337 "Operadores definidos por el usuario"))
("infix <1>" . ("maxima.info-1" 213098 4336 "Operadores definidos por el usuario"))
("infix <2>" . ("maxima.info-1" 213124 4310 "Operadores definidos por el usuario"))
("inflag" . ("maxima.info-1" 273778 758 "Funciones y variables para expresiones"))
("infolists" . ("maxima.info-1" 54806 2091 "Funciones y variables para la línea de comandos"))
("init_atensor" . ("maxima.info-4" 137374 2566 "Funciones y variables para atensor"))
("init_atensor <1>" . ("maxima.info-4" 137375 2565 "Funciones y variables para atensor"))
("init_ctensor" . ("maxima.info-4" 83457 344 "Funciones y variables para ctensor"))
("inm" . ("maxima.info-4" 53801 331 "Funciones y variables para itensor"))
("inmc1" . ("maxima.info-4" 54133 408 "Funciones y variables para itensor"))
("inmc2" . ("maxima.info-4" 54542 588 "Funciones y variables para itensor"))
("innerproduct" . ("maxima.info-3" 261553 456 "Funciones y variables para las matrices y el álgebra lineal"))
("inpart" . ("maxima.info-1" 274537 1231 "Funciones y variables para expresiones"))
("inprod" . ("maxima.info-3" 261554 455 "Funciones y variables para las matrices y el álgebra lineal"))
("inrt" . ("maxima.info-4" 205821 286 "Funciones y variables para teoría de números"))
("intanalysis" . ("maxima.info-3" 99493 1544 "Funciones y variables para integración"))
("integer" . ("maxima.info-2" 104252 480 "Funciones y variables para las propiedades"))
("integer_partitions" . ("maxima.info-5" 27161 1916 "Funciones y variables para los conjuntos"))
("integer_partitions <1>" . ("maxima.info-5" 27162 1915 "Funciones y variables para los conjuntos"))
("integerp" . ("maxima.info-1" 103215 967 "Funciones y variables para los números"))
("integervalued" . ("maxima.info-2" 104733 523 "Funciones y variables para las propiedades"))
("integrate" . ("maxima.info-3" 101038 8911 "Funciones y variables para integración"))
("integrate <1>" . ("maxima.info-3" 101039 8910 "Funciones y variables para integración"))
("integrate_use_rootsof" . ("maxima.info-3" 112215 2437 "Funciones y variables para integración"))
("integration_constant" . ("maxima.info-3" 109950 888 "Funciones y variables para integración"))
("integration_constant_counter" . ("maxima.info-3" 110839 1375 "Funciones y variables para integración"))
("interpolate_color" . ("maxima.info-6" 164539 2723 "Funciones y variables para draw"))
("intersect" . ("maxima.info-5" 29078 101 "Funciones y variables para los conjuntos"))
("intersection" . ("maxima.info-5" 29180 958 "Funciones y variables para los conjuntos"))
("intervalp" . ("maxima.info-7" 299070 114 "Funciones y variables para polinomios ortogonales"))
("intfaclim" . ("maxima.info-2" 244481 906 "Funciones y variables para polinomios"))
("intopois" . ("maxima.info-4" 188976 73 "Funciones y variables para series de Poisson"))
("intosum" . ("maxima.info-4" 144705 1189 "Funciones y variables para sumas y productos"))
("inv_mod" . ("maxima.info-4" 206108 410 "Funciones y variables para teoría de números"))
("invariant1" . ("maxima.info-4" 109834 202 "Funciones y variables para ctensor"))
("inverse_fft" . ("maxima.info-3" 201499 2371 "Funciones y variables para la transformada rápida de Fourier"))
("inverse_jacobi_cd" . ("maxima.info-3" 55181 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_cn" . ("maxima.info-3" 54495 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_cs" . ("maxima.info-3" 55083 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_dc" . ("maxima.info-3" 55475 99 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_dn" . ("maxima.info-3" 54593 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_ds" . ("maxima.info-3" 55377 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_nc" . ("maxima.info-3" 54985 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_nd" . ("maxima.info-3" 55279 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_ns" . ("maxima.info-3" 54691 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_sc" . ("maxima.info-3" 54789 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_sd" . ("maxima.info-3" 54887 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_sn" . ("maxima.info-3" 54397 97 "Funciones y variables para funciones elípticas"))
("invert" . ("maxima.info-3" 262010 616 "Funciones y variables para las matrices y el álgebra lineal"))
("invert_by_lu" . ("maxima.info-7" 213048 161 "Funciones y variables para linearalgebra"))
("ip_grid" . ("maxima.info-6" 167263 239 "Funciones y variables para draw"))
("ip_grid_in" . ("maxima.info-6" 167503 244 "Funciones y variables para draw"))
("irr" . ("maxima.info-7" 67656 487 "Funciones y Variables para finance"))
("irrational" . ("maxima.info-2" 109375 230 "Funciones y variables para las propiedades"))
("is" . ("maxima.info-2" 124397 1946 "Funciones y variables para los hechos"))
("is_biconnected" . ("maxima.info-7" 98646 362 "Funciones y variables para graphs"))
("is_bipartite" . ("maxima.info-7" 99009 369 "Funciones y variables para graphs"))
("is_connected" . ("maxima.info-7" 99379 288 "Funciones y variables para graphs"))
("is_digraph" . ("maxima.info-7" 99668 364 "Funciones y variables para graphs"))
("is_edge_in_graph" . ("maxima.info-7" 100033 633 "Funciones y variables para graphs"))
("is_graph" . ("maxima.info-7" 100667 333 "Funciones y variables para graphs"))
("is_graph_or_digraph" . ("maxima.info-7" 101001 388 "Funciones y variables para graphs"))
("is_isomorphic" . ("maxima.info-7" 101390 400 "Funciones y variables para graphs"))
("is_planar" . ("maxima.info-7" 101791 542 "Funciones y variables para graphs"))
("is_sconnected" . ("maxima.info-7" 102334 396 "Funciones y variables para graphs"))
("is_tree" . ("maxima.info-7" 103139 359 "Funciones y variables para graphs"))
("is_vertex_in_graph" . ("maxima.info-7" 102731 407 "Funciones y variables para graphs"))
("ishow" . ("maxima.info-4" 1878 370 "Funciones y variables para itensor"))
("isolate" . ("maxima.info-1" 275769 901 "Funciones y variables para expresiones"))
("isolate_wrt_times" . ("maxima.info-1" 276671 858 "Funciones y variables para expresiones"))
("isomorphism" . ("maxima.info-7" 97762 495 "Funciones y variables para graphs"))
("isqrt" . ("maxima.info-4" 206519 125 "Funciones y variables para teoría de números"))
("isreal_p" . ("maxima.info-8" 143133 1280 "Funciones y variables para to_poly_solve"))
("items_inference" . ("maxima.info-8" 48692 780 "Funciones y variables para inference_result"))
("itr" . ("maxima.info-4" 55926 2701 "Funciones y variables para itensor"))
("jacobi" . ("maxima.info-4" 206645 258 "Funciones y variables para teoría de números"))
("jacobi_cd" . ("maxima.info-3" 54027 93 "Funciones y variables para funciones elípticas"))
("jacobi_cn" . ("maxima.info-3" 53417 75 "Funciones y variables para funciones elípticas"))
("jacobi_cs" . ("maxima.info-3" 53933 93 "Funciones y variables para funciones elípticas"))
("jacobi_dc" . ("maxima.info-3" 54303 93 "Funciones y variables para funciones elípticas"))
("jacobi_dn" . ("maxima.info-3" 53493 75 "Funciones y variables para funciones elípticas"))
("jacobi_ds" . ("maxima.info-3" 54209 93 "Funciones y variables para funciones elípticas"))
("jacobi_nc" . ("maxima.info-3" 53845 87 "Funciones y variables para funciones elípticas"))
("jacobi_nd" . ("maxima.info-3" 54121 87 "Funciones y variables para funciones elípticas"))
("jacobi_ns" . ("maxima.info-3" 53569 87 "Funciones y variables para funciones elípticas"))
("jacobi_p" . ("maxima.info-7" 299185 316 "Funciones y variables para polinomios ortogonales"))
("jacobi_sc" . ("maxima.info-3" 53657 93 "Funciones y variables para funciones elípticas"))
("jacobi_sd" . ("maxima.info-3" 53751 93 "Funciones y variables para funciones elípticas"))
("jacobi_sn" . ("maxima.info-3" 53341 75 "Funciones y variables para funciones elípticas"))
("jacobian" . ("maxima.info-7" 213210 948 "Funciones y variables para linearalgebra"))
("join" . ("maxima.info-1" 130222 768 "Funciones y variables para listas"))
("jordan" . ("maxima.info-6" 23961 1692 "Funciones y variables para diag"))
("julia_parameter" . ("maxima.info-7" 72865 296 "Definiciones para fractales complejos"))
("julia_set" . ("maxima.info-7" 72330 534 "Definiciones para fractales complejos"))
("julia_sin" . ("maxima.info-7" 73162 726 "Definiciones para fractales complejos"))
("kdels" . ("maxima.info-4" 17573 872 "Funciones y variables para itensor"))
("kdelta" . ("maxima.info-4" 16843 729 "Funciones y variables para itensor"))
("keepfloat" . ("maxima.info-2" 245388 960 "Funciones y variables para polinomios"))
("key" . ("maxima.info-6" 167748 758 "Funciones y variables para draw"))
("key_pos" . ("maxima.info-6" 168507 1122 "Funciones y variables para draw"))
("kill" . ("maxima.info-1" 56898 3305 "Funciones y variables para la línea de comandos"))
("kill <1>" . ("maxima.info-1" 56899 3304 "Funciones y variables para la línea de comandos"))
("kill <2>" . ("maxima.info-1" 56937 3266 "Funciones y variables para la línea de comandos"))
("kill <3>" . ("maxima.info-1" 56964 3239 "Funciones y variables para la línea de comandos"))
("kill <4>" . ("maxima.info-1" 57016 3187 "Funciones y variables para la línea de comandos"))
("kill <5>" . ("maxima.info-1" 57040 3163 "Funciones y variables para la línea de comandos"))
("kill <6>" . ("maxima.info-1" 57071 3132 "Funciones y variables para la línea de comandos"))
("kill <7>" . ("maxima.info-1" 57122 3081 "Funciones y variables para la línea de comandos"))
("killcontext" . ("maxima.info-2" 126344 736 "Funciones y variables para los hechos"))
("kinvariant" . ("maxima.info-4" 129394 113 "Funciones y variables para ctensor"))
("km" . ("maxima.info-5" 260089 1529 "Funciones y variables de parámetros descriptivos"))
("km <1>" . ("maxima.info-5" 260090 1528 "Funciones y variables de parámetros descriptivos"))
("kostka" . ("maxima.info-4" 241124 259 "Funciones y variables para simetrías"))
("kron_delta" . ("maxima.info-5" 30139 920 "Funciones y variables para los conjuntos"))
("kronecker_product" . ("maxima.info-7" 214159 108 "Funciones y variables para linearalgebra"))
("kt" . ("maxima.info-4" 130414 112 "Funciones y variables para ctensor"))
("kummer_m" . ("maxima.info-5" 219317 453 "Funciones y variables para contrib_ode"))
("kummer_u" . ("maxima.info-5" 219771 195 "Funciones y variables para contrib_ode"))
("kurtosis" . ("maxima.info-5" 256657 856 "Funciones y variables de parámetros descriptivos"))
("kurtosis <1>" . ("maxima.info-5" 256658 855 "Funciones y variables de parámetros descriptivos"))
("kurtosis_bernoulli" . ("maxima.info-6" 108749 438 "Funciones y variables para distribuciones discretas"))
("kurtosis_beta" . ("maxima.info-6" 76608 204 "Funciones y variables para distribuciones continuas"))
("kurtosis_binomial" . ("maxima.info-6" 103136 250 "Funciones y variables para distribuciones discretas"))
("kurtosis_chi2" . ("maxima.info-6" 59840 385 "Funciones y variables para distribuciones continuas"))
("kurtosis_continuous_uniform" . ("maxima.info-6" 79058 226 "Funciones y variables para distribuciones continuas"))
("kurtosis_discrete_uniform" . ("maxima.info-6" 113684 235 "Funciones y variables para distribuciones discretas"))
("kurtosis_exp" . ("maxima.info-6" 68773 310 "Funciones y variables para distribuciones continuas"))
("kurtosis_f" . ("maxima.info-6" 65024 198 "Funciones y variables para distribuciones continuas"))
("kurtosis_gamma" . ("maxima.info-6" 73918 203 "Funciones y variables para distribuciones continuas"))
("kurtosis_general_finite_discrete" . ("maxima.info-6" 100180 242 "Funciones y variables para distribuciones discretas"))
("kurtosis_geometric" . ("maxima.info-6" 111265 216 "Funciones y variables para distribuciones discretas"))
("kurtosis_gumbel" . ("maxima.info-6" 96375 209 "Funciones y variables para distribuciones continuas"))
("kurtosis_laplace" . ("maxima.info-6" 92543 208 "Funciones y variables para distribuciones continuas"))
("kurtosis_logistic" . ("maxima.info-6" 81365 208 "Funciones y variables para distribuciones continuas"))
("kurtosis_lognormal" . ("maxima.info-6" 71455 209 "Funciones y variables para distribuciones continuas"))
("kurtosis_negative_binomial" . ("maxima.info-6" 119487 252 "Funciones y variables para distribuciones discretas"))
("kurtosis_noncentral_chi2" . ("maxima.info-6" 62435 212 "Funciones y variables para distribuciones continuas"))
("kurtosis_noncentral_student_t" . ("maxima.info-6" 55775 302 "Funciones y variables para distribuciones continuas"))
("kurtosis_normal" . ("maxima.info-6" 48202 232 "Funciones y variables para distribuciones continuas"))
("kurtosis_pareto" . ("maxima.info-6" 83577 213 "Funciones y variables para distribuciones continuas"))
("kurtosis_poisson" . ("maxima.info-6" 105616 205 "Funciones y variables para distribuciones discretas"))
("kurtosis_rayleigh" . ("maxima.info-6" 89918 638 "Funciones y variables para distribuciones continuas"))
("kurtosis_student_t" . ("maxima.info-6" 51060 206 "Funciones y variables para distribuciones continuas"))
("kurtosis_weibull" . ("maxima.info-6" 85796 210 "Funciones y variables para distribuciones continuas"))
("label" . ("maxima.info-6" 232403 1359 "Funciones y variables para draw"))
("label <1>" . ("maxima.info-6" 232404 1358 "Funciones y variables para draw"))
("label_alignment" . ("maxima.info-6" 169630 959 "Funciones y variables para draw"))
("label_alignment <1>" . ("maxima.info-7" 124592 218 "Funciones y variables para graphs"))
("label_orientation" . ("maxima.info-6" 170590 1023 "Funciones y variables para draw"))
("labels" . ("maxima.info-1" 60204 2057 "Funciones y variables para la línea de comandos"))
("labels <1>" . ("maxima.info-1" 60205 2056 "Funciones y variables para la línea de comandos"))
("lagrange" . ("maxima.info-7" 146705 4208 "Funciones y variables para interpol"))
("lagrange <1>" . ("maxima.info-7" 146706 4207 "Funciones y variables para interpol"))
("laguerre" . ("maxima.info-7" 299502 147 "Funciones y variables para polinomios ortogonales"))
("lambda" . ("maxima.info-5" 105418 6333 "Funciones y variables para la definición de funciones"))
("lambda <1>" . ("maxima.info-5" 105419 6332 "Funciones y variables para la definición de funciones"))
("lambda <2>" . ("maxima.info-5" 105486 6265 "Funciones y variables para la definición de funciones"))
("lambert_w" . ("maxima.info-3" 47941 143 "Funciones y variables para las funciones especiales"))
("laplace" . ("maxima.info-3" 86297 4010 "Funciones y variables para la diferenciación"))
("laplacian_matrix" . ("maxima.info-7" 103499 767 "Funciones y variables para graphs"))
("lassociative" . ("maxima.info-2" 18980 243 "Funciones y variables para simplificación"))
("last" . ("maxima.info-1" 130991 107 "Funciones y variables para listas"))
("lbfgs" . ("maxima.info-7" 181634 14837 "Funciones y variables para lbfgs"))
("lbfgs <1>" . ("maxima.info-7" 181635 14836 "Funciones y variables para lbfgs"))
("lbfgs_ncorrections" . ("maxima.info-7" 196779 236 "Funciones y variables para lbfgs"))
("lbfgs_nfeval_max" . ("maxima.info-7" 196472 306 "Funciones y variables para lbfgs"))
("lc2kdt" . ("maxima.info-4" 18702 3452 "Funciones y variables para itensor"))
("lc_l" . ("maxima.info-4" 22155 1083 "Funciones y variables para itensor"))
("lc_u" . ("maxima.info-4" 23239 289 "Funciones y variables para itensor"))
("lcharp" . ("maxima.info-8" 109508 152 "Funciones y variables para caracteres"))
("lcm" . ("maxima.info-4" 206904 259 "Funciones y variables para teoría de números"))
("ldefint" . ("maxima.info-3" 114653 750 "Funciones y variables para integración"))
("ldisp" . ("maxima.info-1" 86368 1074 "Funciones y variables para la impresión"))
("ldisplay" . ("maxima.info-1" 87443 1321 "Funciones y variables para la impresión"))
("legend" . ("maxima.info-2" 161580 528 "Opciones gráficas"))
("legend <1>" . ("maxima.info-2" 161581 527 "Opciones gráficas"))
("legendre_p" . ("maxima.info-7" 299650 168 "Funciones y variables para polinomios ortogonales"))
("legendre_q" . ("maxima.info-7" 299819 147 "Funciones y variables para polinomios ortogonales"))
("leinstein" . ("maxima.info-4" 87349 435 "Funciones y variables para ctensor"))
("length" . ("maxima.info-1" 131099 648 "Funciones y variables para listas"))
("let" . ("maxima.info-4" 301836 4310 "Funciones y variables sobre reglas y patrones"))
("let <1>" . ("maxima.info-4" 301837 4309 "Funciones y variables sobre reglas y patrones"))
("let_rule_packages" . ("maxima.info-4" 308354 279 "Funciones y variables sobre reglas y patrones"))
("letrat" . ("maxima.info-4" 306147 836 "Funciones y variables sobre reglas y patrones"))
("letrules" . ("maxima.info-4" 306984 626 "Funciones y variables sobre reglas y patrones"))
("letrules <1>" . ("maxima.info-4" 306985 625 "Funciones y variables sobre reglas y patrones"))
("letsimp" . ("maxima.info-4" 307611 742 "Funciones y variables sobre reglas y patrones"))
("letsimp <1>" . ("maxima.info-4" 307612 741 "Funciones y variables sobre reglas y patrones"))
("letsimp <2>" . ("maxima.info-4" 307642 711 "Funciones y variables sobre reglas y patrones"))
("levi_civita" . ("maxima.info-4" 18446 255 "Funciones y variables para itensor"))
("lfg" . ("maxima.info-4" 127832 252 "Funciones y variables para ctensor"))
("lfreeof" . ("maxima.info-1" 279209 231 "Funciones y variables para expresiones"))
("lg" . ("maxima.info-4" 128940 159 "Funciones y variables para ctensor"))
("lgtreillis" . ("maxima.info-4" 241384 246 "Funciones y variables para simetrías"))
("lhospitallim" . ("maxima.info-3" 57840 258 "Funciones y variables para límites"))
("lhs" . ("maxima.info-3" 165720 1636 "Funciones y variable para las ecuaciones"))
("li" . ("maxima.info-2" 59806 2525 "Funciones radicales exponenciales y logarítmicas"))
("liediff" . ("maxima.info-4" 30430 795 "Funciones y variables para itensor"))
("limit" . ("maxima.info-3" 58099 2336 "Funciones y variables para límites"))
("limit <1>" . ("maxima.info-3" 58100 2335 "Funciones y variables para límites"))
("limit <2>" . ("maxima.info-3" 58147 2288 "Funciones y variables para límites"))
("limsubst" . ("maxima.info-3" 60436 339 "Funciones y variables para límites"))
("line_graph" . ("maxima.info-7" 84420 77 "Funciones y variables para graphs"))
("line_type" . ("maxima.info-6" 171614 933 "Funciones y variables para draw"))
("line_width" . ("maxima.info-6" 172548 714 "Funciones y variables para draw"))
("linear" . ("maxima.info-2" 19224 1355 "Funciones y variables para simplificación"))
("linear <1>" . ("maxima.info-8" 21244 754 "Paquete functs"))
("linear_program" . ("maxima.info-8" 8660 1126 "Funciones y variables para simplex"))
("linear_regression" . ("maxima.info-8" 83572 3923 "Funciones y variables para stats"))
("linear_regression <1>" . ("maxima.info-8" 83573 3922 "Funciones y variables para stats"))
("linear_solver" . ("maxima.info-8" 198817 232 "Funciones y variables para zeilberger"))
("linearinterpol" . ("maxima.info-7" 151056 2604 "Funciones y variables para interpol"))
("linearinterpol <1>" . ("maxima.info-7" 151057 2603 "Funciones y variables para interpol"))
("linechar" . ("maxima.info-1" 62262 804 "Funciones y variables para la línea de comandos"))
("linel" . ("maxima.info-1" 88765 511 "Funciones y variables para la impresión"))
("linenum" . ("maxima.info-1" 63067 119 "Funciones y variables para la línea de comandos"))
("linewidth" . ("maxima.info-7" 10107 189 "Visualización usando VTK"))
("linsolve" . ("maxima.info-3" 167357 2921 "Funciones y variable para las ecuaciones"))
("linsolve_params" . ("maxima.info-3" 170455 392 "Funciones y variable para las ecuaciones"))
("linsolvewarn" . ("maxima.info-3" 170279 175 "Funciones y variable para las ecuaciones"))
("lispdisp" . ("maxima.info-1" 89277 492 "Funciones y variables para la impresión"))
("list_correlations" . ("maxima.info-5" 269245 3098 "Funciones y variables de parámetros descriptivos"))
("list_correlations <1>" . ("maxima.info-5" 269246 3097 "Funciones y variables de parámetros descriptivos"))
("list_matrix_entries" . ("maxima.info-3" 262627 228 "Funciones y variables para las matrices y el álgebra lineal"))
("list_nc_monomials" . ("maxima.info-3" 291733 295 "Funciones y variables para Afines"))
("list_nc_monomials <1>" . ("maxima.info-3" 291734 294 "Funciones y variables para Afines"))
("listarith" . ("maxima.info-1" 131748 442 "Funciones y variables para listas"))
("listarray" . ("maxima.info-1" 161555 3723 "Funciones y variables para los arrays"))
("listconstvars" . ("maxima.info-1" 277530 319 "Funciones y variables para expresiones"))
("listdummyvars" . ("maxima.info-1" 277850 735 "Funciones y variables para expresiones"))
("listify" . ("maxima.info-5" 31060 459 "Funciones y variables para los conjuntos"))
("listoftens" . ("maxima.info-4" 1305 572 "Funciones y variables para itensor"))
("listofvars" . ("maxima.info-1" 278586 622 "Funciones y variables para expresiones"))
("listp" . ("maxima.info-1" 132191 117 "Funciones y variables para listas"))
("listp <1>" . ("maxima.info-7" 214268 382 "Funciones y variables para linearalgebra"))
("listp <2>" . ("maxima.info-7" 214269 381 "Funciones y variables para linearalgebra"))
("lmax" . ("maxima.info-2" 42624 172 "Funciones para los números"))
("lmin" . ("maxima.info-2" 42797 172 "Funciones para los números"))
("lmxchar" . ("maxima.info-3" 262856 537 "Funciones y variables para las matrices y el álgebra lineal"))
("load" . ("maxima.info-2" 189469 1541 "Funciones y variables para lectura y escritura"))
("load_pathname" . ("maxima.info-2" 191011 877 "Funciones y variables para lectura y escritura"))
("loadfile" . ("maxima.info-2" 191889 418 "Funciones y variables para lectura y escritura"))
("loadprint" . ("maxima.info-2" 192308 575 "Funciones y variables para lectura y escritura"))
("local" . ("maxima.info-5" 111752 1476 "Funciones y variables para la definición de funciones"))
("locate_matrix_entry" . ("maxima.info-7" 214651 1047 "Funciones y variables para linearalgebra"))
("log" . ("maxima.info-2" 62332 1459 "Funciones radicales exponenciales y logarítmicas"))
("log_gamma" . ("maxima.info-3" 6365 73 "Funciones Gamma y factorial"))
("logabs" . ("maxima.info-2" 63792 510 "Funciones radicales exponenciales y logarítmicas"))
("logarc" . ("maxima.info-2" 64303 451 "Funciones radicales exponenciales y logarítmicas"))
("logarc <1>" . ("maxima.info-2" 64304 450 "Funciones radicales exponenciales y logarítmicas"))
("logcb" . ("maxima.info-6" 173263 736 "Funciones y variables para draw"))
("logconcoeffp" . ("maxima.info-2" 64755 457 "Funciones radicales exponenciales y logarítmicas"))
("logcontract" . ("maxima.info-2" 65213 1011 "Funciones radicales exponenciales y logarítmicas"))
("logexpand" . ("maxima.info-2" 66225 509 "Funciones radicales exponenciales y logarítmicas"))
("lognegint" . ("maxima.info-2" 66735 182 "Funciones radicales exponenciales y logarítmicas"))
("logsimp" . ("maxima.info-2" 66918 169 "Funciones radicales exponenciales y logarítmicas"))
("logx" . ("maxima.info-2" 162109 165 "Opciones gráficas"))
("logx <1>" . ("maxima.info-6" 174000 470 "Funciones y variables para draw"))
("logx_secondary" . ("maxima.info-6" 174471 824 "Funciones y variables para draw"))
("logy" . ("maxima.info-2" 162275 166 "Opciones gráficas"))
("logy <1>" . ("maxima.info-6" 175296 467 "Funciones y variables para draw"))
("logy_secondary" . ("maxima.info-6" 175764 824 "Funciones y variables para draw"))
("logz" . ("maxima.info-6" 176589 447 "Funciones y variables para draw"))
("lopow" . ("maxima.info-2" 246349 214 "Funciones y variables para polinomios"))
("lorentz_gauge" . ("maxima.info-4" 43961 206 "Funciones y variables para itensor"))
("lowercasep" . ("maxima.info-8" 109661 93 "Funciones y variables para caracteres"))
("lpart" . ("maxima.info-1" 279441 222 "Funciones y variables para expresiones"))
("lratsubst" . ("maxima.info-2" 246564 1055 "Funciones y variables para polinomios"))
("lreduce" . ("maxima.info-5" 31520 1419 "Funciones y variables para los conjuntos"))
("lreduce <1>" . ("maxima.info-5" 31521 1418 "Funciones y variables para los conjuntos"))
("lriem" . ("maxima.info-4" 128548 102 "Funciones y variables para ctensor"))
("lriemann" . ("maxima.info-4" 89098 538 "Funciones y variables para ctensor"))
("lsquares_estimates" . ("maxima.info-7" 231064 4838 "Funciones y variables para lsquares"))
("lsquares_estimates <1>" . ("maxima.info-7" 231065 4837 "Funciones y variables para lsquares"))
("lsquares_estimates_approximate" . ("maxima.info-7" 238229 3161 "Funciones y variables para lsquares"))
("lsquares_estimates_exact" . ("maxima.info-7" 235903 2325 "Funciones y variables para lsquares"))
("lsquares_mse" . ("maxima.info-7" 241391 2948 "Funciones y variables para lsquares"))
("lsquares_residual_mse" . ("maxima.info-7" 246688 2154 "Funciones y variables para lsquares"))
("lsquares_residuals" . ("maxima.info-7" 244340 2347 "Funciones y variables para lsquares"))
("lsum" . ("maxima.info-4" 144024 680 "Funciones y variables para sumas y productos"))
("ltreillis" . ("maxima.info-4" 241631 273 "Funciones y variables para simetrías"))
("lu_backsub" . ("maxima.info-7" 215699 176 "Funciones y variables para linearalgebra"))
("lu_factor" . ("maxima.info-7" 215876 5732 "Funciones y variables para linearalgebra"))
("lucas" . ("maxima.info-4" 207164 905 "Funciones y variables para teoría de números"))
("m1pbranch" . ("maxima.info-1" 104183 613 "Funciones y variables para los números"))
("macroexpand" . ("maxima.info-5" 73076 1250 "Macros"))
("macroexpand1" . ("maxima.info-5" 74327 1159 "Macros"))
("macroexpansion" . ("maxima.info-5" 113229 5980 "Funciones y variables para la definición de funciones"))
("macros" . ("maxima.info-5" 75487 378 "Macros"))
("mainvar" . ("maxima.info-1" 279664 724 "Funciones y variables para expresiones"))
("makeOrders" . ("maxima.info-7" 253518 811 "Funciones y variables para makeOrders"))
("make_array" . ("maxima.info-1" 165279 2101 "Funciones y variables para los arrays"))
("make_graph" . ("maxima.info-7" 84498 1131 "Funciones y variables para graphs"))
("make_graph <1>" . ("maxima.info-7" 84499 1130 "Funciones y variables para graphs"))
("make_level_picture" . ("maxima.info-6" 253335 1351 "Funciones y variables para picture"))
("make_level_picture <1>" . ("maxima.info-6" 253336 1350 "Funciones y variables para picture"))
("make_poly_continent" . ("maxima.info-6" 258864 504 "Funciones y variables para worldmap"))
("make_poly_continent <1>" . ("maxima.info-6" 258865 503 "Funciones y variables para worldmap"))
("make_poly_country" . ("maxima.info-6" 259369 324 "Funciones y variables para worldmap"))
("make_polygon" . ("maxima.info-6" 259694 3187 "Funciones y variables para worldmap"))
("make_random_state" . ("maxima.info-2" 84140 913 "Números aleatorios"))
("make_random_state <1>" . ("maxima.info-2" 84141 912 "Números aleatorios"))
("make_random_state <2>" . ("maxima.info-2" 84178 875 "Números aleatorios"))
("make_random_state <3>" . ("maxima.info-2" 84215 838 "Números aleatorios"))
("make_rgb_picture" . ("maxima.info-6" 254687 1246 "Funciones y variables para picture"))
("make_string_input_stream" . ("maxima.info-8" 98665 816 "Funciones y variables para entrada y salida"))
("make_string_input_stream <1>" . ("maxima.info-8" 98666 815 "Funciones y variables para entrada y salida"))
("make_string_input_stream <2>" . ("maxima.info-8" 98715 766 "Funciones y variables para entrada y salida"))
("make_string_output_stream" . ("maxima.info-8" 99482 690 "Funciones y variables para entrada y salida"))
("make_transform" . ("maxima.info-2" 143357 643 "Funciones y variables para gráficos"))
("makebox" . ("maxima.info-4" 35750 267 "Funciones y variables para itensor"))
("makefact" . ("maxima.info-3" 33471 177 "Funciones Gamma y factorial"))
("makegamma" . ("maxima.info-3" 10175 177 "Funciones Gamma y factorial"))
("makelist" . ("maxima.info-1" 132309 1894 "Funciones y variables para listas"))
("makelist <1>" . ("maxima.info-1" 132310 1893 "Funciones y variables para listas"))
("makelist <2>" . ("maxima.info-1" 132335 1868 "Funciones y variables para listas"))
("makelist <3>" . ("maxima.info-1" 132371 1832 "Funciones y variables para listas"))
("makelist <4>" . ("maxima.info-1" 132416 1787 "Funciones y variables para listas"))
("makelist <5>" . ("maxima.info-1" 132468 1735 "Funciones y variables para listas"))
("makeset" . ("maxima.info-5" 32940 1683 "Funciones y variables para los conjuntos"))
("mandelbrot_set" . ("maxima.info-7" 71818 511 "Definiciones para fractales complejos"))
("manual_demo" . ("maxima.info-1" 39376 210 "Funciones y variables para la ayuda"))
("map" . ("maxima.info-5" 159495 1463 "Funciones y variables para la programación"))
("mapatom" . ("maxima.info-5" 160959 124 "Funciones y variables para la programación"))
("maperror" . ("maxima.info-5" 161084 612 "Funciones y variables para la programación"))
("maplist" . ("maxima.info-5" 162021 476 "Funciones y variables para la programación"))
("mapprint" . ("maxima.info-5" 161697 323 "Funciones y variables para la programación"))
("mat_cond" . ("maxima.info-7" 221609 545 "Funciones y variables para linearalgebra"))
("mat_cond <1>" . ("maxima.info-7" 221610 544 "Funciones y variables para linearalgebra"))
("mat_fullunblocker" . ("maxima.info-7" 222972 261 "Funciones y variables para linearalgebra"))
("mat_function" . ("maxima.info-6" 28875 10207 "Funciones y variables para diag"))
("mat_norm" . ("maxima.info-7" 222155 295 "Funciones y variables para linearalgebra"))
("mat_norm <1>" . ("maxima.info-7" 222156 294 "Funciones y variables para linearalgebra"))
("mat_norm <2>" . ("maxima.info-7" 222187 263 "Funciones y variables para linearalgebra"))
("mat_trace" . ("maxima.info-7" 223234 246 "Funciones y variables para linearalgebra"))
("mat_unblocker" . ("maxima.info-7" 223481 1488 "Funciones y variables para linearalgebra"))
("matchdeclare" . ("maxima.info-4" 308634 6308 "Funciones y variables sobre reglas y patrones"))
("matchfix" . ("maxima.info-1" 217435 4757 "Operadores definidos por el usuario"))
("matchfix <1>" . ("maxima.info-1" 217436 4756 "Operadores definidos por el usuario"))
("matrix" . ("maxima.info-3" 263394 5957 "Funciones y variables para las matrices y el álgebra lineal"))
("matrix_element_add" . ("maxima.info-3" 269624 1362 "Funciones y variables para las matrices y el álgebra lineal"))
("matrix_element_mult" . ("maxima.info-3" 270987 2049 "Funciones y variables para las matrices y el álgebra lineal"))
("matrix_element_transpose" . ("maxima.info-3" 273037 2326 "Funciones y variables para las matrices y el álgebra lineal"))
("matrix_size" . ("maxima.info-7" 222858 113 "Funciones y variables para linearalgebra"))
("matrixmap" . ("maxima.info-3" 269352 167 "Funciones y variables para las matrices y el álgebra lineal"))
("matrixp" . ("maxima.info-3" 269520 103 "Funciones y variables para las matrices y el álgebra lineal"))
("matrixp <1>" . ("maxima.info-7" 222451 406 "Funciones y variables para linearalgebra"))
("matrixp <2>" . ("maxima.info-7" 222452 405 "Funciones y variables para linearalgebra"))
("mattrace" . ("maxima.info-3" 275364 244 "Funciones y variables para las matrices y el álgebra lineal"))
("max" . ("maxima.info-2" 42970 518 "Funciones para los números"))
("max_clique" . ("maxima.info-7" 104267 258 "Funciones y variables para graphs"))
("max_degree" . ("maxima.info-7" 104526 379 "Funciones y variables para graphs"))
("max_flow" . ("maxima.info-7" 104906 1028 "Funciones y variables para graphs"))
("max_independent_set" . ("maxima.info-7" 105935 361 "Funciones y variables para graphs"))
("max_matching" . ("maxima.info-7" 106297 432 "Funciones y variables para graphs"))
("max_ord" . ("maxima.info-8" 198456 150 "Funciones y variables para zeilberger"))
("maxapplydepth" . ("maxima.info-4" 314943 180 "Funciones y variables sobre reglas y patrones"))
("maxapplyheight" . ("maxima.info-4" 315124 178 "Funciones y variables sobre reglas y patrones"))
("maxima_tempdir" . ("maxima.info-4" 274674 611 "Funciones y variables para el entorno de ejecución"))
("maxima_userdir" . ("maxima.info-4" 275286 864 "Funciones y variables para el entorno de ejecución"))
("maximize_lp" . ("maxima.info-8" 9787 272 "Funciones y variables para simplex"))
("maxnegex" . ("maxima.info-2" 20580 188 "Funciones y variables para simplificación"))
("maxposex" . ("maxima.info-2" 20769 178 "Funciones y variables para simplificación"))
("maxpsifracdenom" . ("maxima.info-3" 33220 250 "Funciones Gamma y factorial"))
("maxpsifracnum" . ("maxima.info-3" 32971 248 "Funciones Gamma y factorial"))
("maxpsinegint" . ("maxima.info-3" 32649 321 "Funciones Gamma y factorial"))
("maxpsiposint" . ("maxima.info-3" 32456 192 "Funciones Gamma y factorial"))
("maxtayorder" . ("maxima.info-4" 159522 268 "Funciones y variables para las series"))
("maybe" . ("maxima.info-2" 127081 787 "Funciones y variables para los hechos"))
("md5sum" . ("maxima.info-8" 111878 597 "Funciones y variables para cadenas"))
("mean" . ("maxima.info-5" 243054 856 "Funciones y variables de parámetros descriptivos"))
("mean <1>" . ("maxima.info-5" 243055 855 "Funciones y variables de parámetros descriptivos"))
("mean_bernoulli" . ("maxima.info-6" 107327 309 "Funciones y variables para distribuciones discretas"))
("mean_beta" . ("maxima.info-6" 75846 176 "Funciones y variables para distribuciones continuas"))
("mean_binomial" . ("maxima.info-6" 102216 220 "Funciones y variables para distribuciones discretas"))
("mean_chi2" . ("maxima.info-6" 58539 270 "Funciones y variables para distribuciones continuas"))
("mean_continuous_uniform" . ("maxima.info-6" 78190 207 "Funciones y variables para distribuciones continuas"))
("mean_deviation" . ("maxima.info-5" 253046 941 "Funciones y variables de parámetros descriptivos"))
("mean_deviation <1>" . ("maxima.info-5" 253047 940 "Funciones y variables de parámetros descriptivos"))
("mean_discrete_uniform" . ("maxima.info-6" 112792 213 "Funciones y variables para distribuciones discretas"))
("mean_exp" . ("maxima.info-6" 67277 375 "Funciones y variables para distribuciones continuas"))
("mean_f" . ("maxima.info-6" 64274 173 "Funciones y variables para distribuciones continuas"))
("mean_gamma" . ("maxima.info-6" 73148 178 "Funciones y variables para distribuciones continuas"))
("mean_general_finite_discrete" . ("maxima.info-6" 99260 220 "Funciones y variables para distribuciones discretas"))
("mean_geometric" . ("maxima.info-6" 110449 194 "Funciones y variables para distribuciones discretas"))
("mean_gumbel" . ("maxima.info-6" 95122 336 "Funciones y variables para distribuciones continuas"))
("mean_hypergeometric" . ("maxima.info-6" 115745 258 "Funciones y variables para distribuciones discretas"))
("mean_laplace" . ("maxima.info-6" 91747 189 "Funciones y variables para distribuciones continuas"))
("mean_logistic" . ("maxima.info-6" 80569 189 "Funciones y variables para distribuciones continuas"))
("mean_lognormal" . ("maxima.info-6" 70655 190 "Funciones y variables para distribuciones continuas"))
("mean_negative_binomial" . ("maxima.info-6" 118527 230 "Funciones y variables para distribuciones discretas"))
("mean_noncentral_chi2" . ("maxima.info-6" 61623 193 "Funciones y variables para distribuciones continuas"))
("mean_noncentral_student_t" . ("maxima.info-6" 54239 656 "Funciones y variables para distribuciones continuas"))
("mean_normal" . ("maxima.info-6" 47429 198 "Funciones y variables para distribuciones continuas"))
("mean_pareto" . ("maxima.info-6" 82773 191 "Funciones y variables para distribuciones continuas"))
("mean_poisson" . ("maxima.info-6" 104850 177 "Funciones y variables para distribuciones discretas"))
("mean_rayleigh" . ("maxima.info-6" 87835 391 "Funciones y variables para distribuciones continuas"))
("mean_student_t" . ("maxima.info-6" 50102 207 "Funciones y variables para distribuciones continuas"))
("mean_weibull" . ("maxima.info-6" 84992 191 "Funciones y variables para distribuciones continuas"))
("median" . ("maxima.info-5" 251594 727 "Funciones y variables de parámetros descriptivos"))
("median <1>" . ("maxima.info-5" 251595 726 "Funciones y variables de parámetros descriptivos"))
("median_deviation" . ("maxima.info-5" 253988 864 "Funciones y variables de parámetros descriptivos"))
("median_deviation <1>" . ("maxima.info-5" 253989 863 "Funciones y variables de parámetros descriptivos"))
("member" . ("maxima.info-1" 134204 1406 "Funciones y variables para listas"))
("mesh" . ("maxima.info-6" 233763 1487 "Funciones y variables para draw"))
("mesh_lines_color" . ("maxima.info-2" 162442 338 "Opciones gráficas"))
("metricexpandall" . ("maxima.info-8" 191082 1039 "Funciones y variables para units"))
("min" . ("maxima.info-2" 43489 518 "Funciones para los números"))
("min_degree" . ("maxima.info-7" 106730 390 "Funciones y variables para graphs"))
("min_edge_cut" . ("maxima.info-7" 107121 233 "Funciones y variables para graphs"))
("min_vertex_cover" . ("maxima.info-7" 107355 91 "Funciones y variables para graphs"))
("min_vertex_cut" . ("maxima.info-7" 107447 129 "Funciones y variables para graphs"))
("minf" . ("maxima.info-1" 117186 81 "Funciones y variables para Constantes"))
("minfactorial" . ("maxima.info-2" 56843 549 "Funciones combinatorias"))
("minimalPoly" . ("maxima.info-6" 26723 682 "Funciones y variables para diag"))
("minimize_lp" . ("maxima.info-8" 10060 1979 "Funciones y variables para simplex"))
("minimum_spanning_tree" . ("maxima.info-7" 107577 313 "Funciones y variables para graphs"))
("minor" . ("maxima.info-3" 275609 174 "Funciones y variables para las matrices y el álgebra lineal"))
;; build_index.pl: something seems wrong for this next item
("minpack_lsquares" . ("" -1 828 "Funciones y variables para minpack"))
;; build_index.pl: something seems wrong for this next item
("minpack_solve" . ("" -1 191 "Funciones y variables para minpack"))
("mnewton" . ("maxima.info-7" 260232 1623 "Funciones y variables para mnewton"))
("mod" . ("maxima.info-4" 208070 1326 "Funciones y variables para teoría de números"))
("mod_big_prime" . ("maxima.info-8" 200360 181 "Funciones y variables para zeilberger"))
("mod_test" . ("maxima.info-8" 199697 192 "Funciones y variables para zeilberger"))
("mod_threshold" . ("maxima.info-8" 200542 168 "Funciones y variables para zeilberger"))
("mode_check_errorp" . ("maxima.info-5" 119395 154 "Funciones y variables para la definición de funciones"))
("mode_check_warnp" . ("maxima.info-5" 119550 154 "Funciones y variables para la definición de funciones"))
("mode_checkp" . ("maxima.info-5" 119210 184 "Funciones y variables para la definición de funciones"))
("mode_declare" . ("maxima.info-5" 119705 1336 "Funciones y variables para la definición de funciones"))
("mode_identity" . ("maxima.info-5" 121042 174 "Funciones y variables para la definición de funciones"))
("modular_linear_solver" . ("maxima.info-8" 199890 256 "Funciones y variables para zeilberger"))
("modulus" . ("maxima.info-2" 247620 1685 "Funciones y variables para polinomios"))
("moebius" . ("maxima.info-5" 34624 1235 "Funciones y variables para los conjuntos"))
("mon2schur" . ("maxima.info-4" 241905 1352 "Funciones y variables para simetrías"))
("mono" . ("maxima.info-3" 290970 159 "Funciones y variables para Afines"))
("monomial_dimensions" . ("maxima.info-3" 291130 182 "Funciones y variables para Afines"))
("multi_elem" . ("maxima.info-4" 243258 886 "Funciones y variables para simetrías"))
("multi_orbit" . ("maxima.info-4" 244145 684 "Funciones y variables para simetrías"))
("multi_pui" . ("maxima.info-4" 244830 440 "Funciones y variables para simetrías"))
("multibernstein_poly" . ("maxima.info-5" 197377 424 "Funciones y variables para Bernstein"))
("multinomial" . ("maxima.info-4" 245271 308 "Funciones y variables para simetrías"))
("multinomial_coeff" . ("maxima.info-5" 35860 1273 "Funciones y variables para los conjuntos"))
("multinomial_coeff <1>" . ("maxima.info-5" 35861 1272 "Funciones y variables para los conjuntos"))
("multiplicative" . ("maxima.info-2" 20948 998 "Funciones y variables para simplificación"))
("multiplicities" . ("maxima.info-3" 170848 202 "Funciones y variable para las ecuaciones"))
("multiplot_mode" . ("maxima.info-6" 126161 1255 "Funciones y variables para draw"))
("multsym" . ("maxima.info-4" 245580 854 "Funciones y variables para simetrías"))
("multthru" . ("maxima.info-2" 21947 2364 "Funciones y variables para simplificación"))
("multthru <1>" . ("maxima.info-2" 21948 2363 "Funciones y variables para simplificación"))
("mycielski_graph" . ("maxima.info-7" 85630 86 "Funciones y variables para graphs"))
("myoptions" . ("maxima.info-1" 63187 238 "Funciones y variables para la línea de comandos"))
("nary" . ("maxima.info-2" 24312 703 "Funciones y variables para simplificación"))
("nary <1>" . ("maxima.info-1" 222193 472 "Operadores definidos por el usuario"))
("nary <2>" . ("maxima.info-1" 222194 471 "Operadores definidos por el usuario"))
("natural_unit" . ("maxima.info-7" 54393 248 "Funciones y variables para ezunits"))
("nc_degree" . ("maxima.info-3" 289452 182 "Funciones y variables para Afines"))
("ncexpt" . ("maxima.info-1" 78617 329 "Funciones y variables para la impresión"))
("ncharpoly" . ("maxima.info-3" 275784 829 "Funciones y variables para las matrices y el álgebra lineal"))
("negative_picture" . ("maxima.info-6" 255934 133 "Funciones y variables para picture"))
("negdistrib" . ("maxima.info-2" 25016 390 "Funciones y variables para simplificación"))
("negsumdispflag" . ("maxima.info-1" 89770 424 "Funciones y variables para la impresión"))
("neighbors" . ("maxima.info-7" 107891 277 "Funciones y variables para graphs"))
("new" . ("maxima.info-1" 173308 868 "Funciones y variables para las estructuras"))
("new <1>" . ("maxima.info-1" 173309 867 "Funciones y variables para las estructuras"))
("new_graph" . ("maxima.info-7" 85717 74 "Funciones y variables para graphs"))
("new_variable" . ("maxima.info-8" 144414 1689 "Funciones y variables para to_poly_solve"))
("newcontext" . ("maxima.info-2" 127869 284 "Funciones y variables para los hechos"))
("newdet" . ("maxima.info-3" 276614 192 "Funciones y variables para las matrices y el álgebra lineal"))
("newline" . ("maxima.info-8" 109755 55 "Funciones y variables para caracteres"))
("newline <1>" . ("maxima.info-8" 100173 269 "Funciones y variables para entrada y salida"))
("newline <2>" . ("maxima.info-8" 100174 268 "Funciones y variables para entrada y salida"))
("newton" . ("maxima.info-3" 213464 1416 "Funciones para la resolución numérica de ecuaciones"))
("newtonepsilon" . ("maxima.info-7" 259631 359 "Funciones y variables para mnewton"))
("newtonmaxiter" . ("maxima.info-7" 259991 240 "Funciones y variables para mnewton"))
("next_prime" . ("maxima.info-4" 209397 158 "Funciones y variables para teoría de números"))
("nextlayerfactor" . ("maxima.info-8" 17185 779 "Paquete facexp"))
("nicedummies" . ("maxima.info-8" 146104 616 "Funciones y variables para to_poly_solve"))
("niceindices" . ("maxima.info-4" 159791 1500 "Funciones y variables para las series"))
("niceindicespref" . ("maxima.info-4" 161292 1084 "Funciones y variables para las series"))
("ninth" . ("maxima.info-1" 135611 133 "Funciones y variables para listas"))
("nm" . ("maxima.info-4" 130527 120 "Funciones y variables para ctensor"))
("nmc" . ("maxima.info-4" 130648 127 "Funciones y variables para ctensor"))
("noeval" . ("maxima.info-1" 253433 234 "Funciones y variables para la evaluación"))
("nofix" . ("maxima.info-1" 222666 456 "Operadores definidos por el usuario"))
("nofix <1>" . ("maxima.info-1" 222667 455 "Operadores definidos por el usuario"))
("nolabels" . ("maxima.info-1" 63426 952 "Funciones y variables para la línea de comandos"))
("nonarray" . ("maxima.info-2" 105257 608 "Funciones y variables para las propiedades"))
("noncentral_moment" . ("maxima.info-5" 246532 930 "Funciones y variables de parámetros descriptivos"))
("noncentral_moment <1>" . ("maxima.info-5" 246533 929 "Funciones y variables de parámetros descriptivos"))
("nonegative_lp" . ("maxima.info-8" 12040 244 "Funciones y variables para simplex"))
("noninteger" . ("maxima.info-2" 104253 479 "Funciones y variables para las propiedades"))
("nonmetricity" . ("maxima.info-4" 103681 127 "Funciones y variables para ctensor"))
("nonnegintegerp" . ("maxima.info-1" 104797 103 "Funciones y variables para los números"))
("nonscalar" . ("maxima.info-2" 105866 196 "Funciones y variables para las propiedades"))
("nonscalarp" . ("maxima.info-2" 106063 162 "Funciones y variables para las propiedades"))
("nonzeroandfreeof" . ("maxima.info-8" 21001 242 "Paquete functs"))
("not" . ("maxima.info-1" 195881 623 "Operadores lógicos"))
("notequal" . ("maxima.info-2" 134808 932 "Funciones y variables para los predicados"))
("noun" . ("maxima.info-1" 280389 206 "Funciones y variables para expresiones"))
("noundisp" . ("maxima.info-1" 280596 243 "Funciones y variables para expresiones"))
("nounify" . ("maxima.info-1" 280840 445 "Funciones y variables para expresiones"))
("nouns" . ("maxima.info-1" 253668 329 "Funciones y variables para la evaluación"))
("np" . ("maxima.info-4" 129508 110 "Funciones y variables para ctensor"))
("npi" . ("maxima.info-4" 129619 652 "Funciones y variables para ctensor"))
("nptetrad" . ("maxima.info-4" 97467 715 "Funciones y variables para ctensor"))
("npv" . ("maxima.info-7" 67300 355 "Funciones y Variables para finance"))
("nroots" . ("maxima.info-3" 171051 444 "Funciones y variable para las ecuaciones"))
("nterms" . ("maxima.info-1" 281286 428 "Funciones y variables para expresiones"))
("ntermst" . ("maxima.info-4" 110307 84 "Funciones y variables para ctensor"))
("nthroot" . ("maxima.info-3" 171496 354 "Funciones y variable para las ecuaciones"))
("nticks" . ("maxima.info-2" 162781 370 "Opciones gráficas"))
("nticks <1>" . ("maxima.info-6" 177037 656 "Funciones y variables para draw"))
("ntrig" . ("maxima.info-2" 76012 309 "Funciones y variables para trigonometría"))
("nullity" . ("maxima.info-7" 225197 107 "Funciones y variables para linearalgebra"))
("nullspace" . ("maxima.info-7" 224970 226 "Funciones y variables para linearalgebra"))
("num" . ("maxima.info-2" 249306 192 "Funciones y variables para polinomios"))
("num_distinct_partitions" . ("maxima.info-5" 37134 880 "Funciones y variables para los conjuntos"))
("num_distinct_partitions <1>" . ("maxima.info-5" 37135 879 "Funciones y variables para los conjuntos"))
("num_partitions" . ("maxima.info-5" 38015 925 "Funciones y variables para los conjuntos"))
("num_partitions <1>" . ("maxima.info-5" 38016 924 "Funciones y variables para los conjuntos"))
("numbered_boundaries" . ("maxima.info-6" 258254 609 "Funciones y variables para worldmap"))
("numberp" . ("maxima.info-1" 104901 1300 "Funciones y variables para los números"))
("numer" . ("maxima.info-1" 106202 757 "Funciones y variables para los números"))
("numer_pbranch" . ("maxima.info-1" 106960 908 "Funciones y variables para los números"))
("numerval" . ("maxima.info-1" 107869 443 "Funciones y variables para los números"))
("numfactor" . ("maxima.info-3" 33649 480 "Funciones Gamma y factorial"))
("nusum" . ("maxima.info-4" 162377 2060 "Funciones y variables para las series"))
("nzeta" . ("maxima.info-3" 48462 121 "Funciones y variables para las funciones especiales"))
("nzetai" . ("maxima.info-3" 48647 64 "Funciones y variables para las funciones especiales"))
("nzetar" . ("maxima.info-3" 48584 62 "Funciones y variables para las funciones especiales"))
("obase" . ("maxima.info-1" 90195 1272 "Funciones y variables para la impresión"))
("odd" . ("maxima.info-2" 100225 647 "Funciones y variables para las propiedades"))
("odd_girth" . ("maxima.info-7" 108169 368 "Funciones y variables para graphs"))
("oddfun" . ("maxima.info-2" 11061 682 "Funciones y variables para simplificación"))
("oddp" . ("maxima.info-1" 108313 111 "Funciones y variables para los números"))
("ode2" . ("maxima.info-3" 194785 3968 "Funciones y variables para ecuaciones diferenciales"))
("ode_check" . ("maxima.info-5" 216862 1183 "Funciones y variables para contrib_ode"))
("odelin" . ("maxima.info-5" 215521 1340 "Funciones y variables para contrib_ode"))
("op" . ("maxima.info-1" 281715 1898 "Funciones y variables para expresiones"))
("opacity" . ("maxima.info-7" 10297 260 "Visualización usando VTK"))
("opena" . ("maxima.info-8" 100443 167 "Funciones y variables para entrada y salida"))
("opena_binary" . ("maxima.info-7" 273482 152 "Funciones y variables para entrada y salida en formato binario"))
("openr" . ("maxima.info-8" 100611 130 "Funciones y variables para entrada y salida"))
("openr_binary" . ("maxima.info-7" 273174 151 "Funciones y variables para entrada y salida en formato binario"))
("openw" . ("maxima.info-8" 100742 223 "Funciones y variables para entrada y salida"))
("openw_binary" . ("maxima.info-7" 273326 155 "Funciones y variables para entrada y salida en formato binario"))
("operatorp" . ("maxima.info-1" 283614 355 "Funciones y variables para expresiones"))
("operatorp <1>" . ("maxima.info-1" 283615 354 "Funciones y variables para expresiones"))
("opproperties" . ("maxima.info-2" 25407 355 "Funciones y variables para simplificación"))
("opsubst" . ("maxima.info-1" 283970 237 "Funciones y variables para expresiones"))
("opsubst <1>" . ("maxima.info-7" 276908 2329 "Funciones y variables para opsubst"))
("opsubst <2>" . ("maxima.info-7" 276909 2328 "Funciones y variables para opsubst"))
("opsubst <3>" . ("maxima.info-7" 276944 2293 "Funciones y variables para opsubst"))
("optimize" . ("maxima.info-1" 284208 410 "Funciones y variables para expresiones"))
("optimprefix" . ("maxima.info-1" 284619 183 "Funciones y variables para expresiones"))
("optionset" . ("maxima.info-1" 64379 704 "Funciones y variables para la línea de comandos"))
("or" . ("maxima.info-1" 196505 1079 "Operadores lógicos"))
("orbit" . ("maxima.info-4" 246435 564 "Funciones y variables para simetrías"))
("orbits" . ("maxima.info-6" 285891 1289 "Análisis gráfico de sistemas dinámicos discretos"))
("ordergreat" . ("maxima.info-1" 284803 900 "Funciones y variables para expresiones"))
("ordergreatp" . ("maxima.info-1" 285704 3955 "Funciones y variables para expresiones"))
("orderless" . ("maxima.info-1" 284804 899 "Funciones y variables para expresiones"))
("orderlessp" . ("maxima.info-1" 285705 3954 "Funciones y variables para expresiones"))
("orientation" . ("maxima.info-7" 10558 396 "Visualización usando VTK"))
("origin" . ("maxima.info-7" 10955 276 "Visualización usando VTK"))
("orthogonal_complement" . ("maxima.info-7" 225305 239 "Funciones y variables para linearalgebra"))
("orthopoly_recur" . ("maxima.info-7" 299967 1189 "Funciones y variables para polinomios ortogonales"))
("orthopoly_returns_intervals" . ("maxima.info-7" 301157 397 "Funciones y variables para polinomios ortogonales"))
("orthopoly_weight" . ("maxima.info-7" 301555 785 "Funciones y variables para polinomios ortogonales"))
("out_neighbors" . ("maxima.info-7" 108538 389 "Funciones y variables para graphs"))
("outative" . ("maxima.info-2" 25763 1360 "Funciones y variables para simplificación"))
("outchar" . ("maxima.info-1" 65084 964 "Funciones y variables para la línea de comandos"))
("outermap" . ("maxima.info-5" 165158 7020 "Funciones y variables para la programación"))
("outofpois" . ("maxima.info-4" 189050 347 "Funciones y variables para series de Poisson"))
("packagefile" . ("maxima.info-2" 192884 365 "Funciones y variables para lectura y escritura"))
("packagefile <1>" . ("maxima.info-4" 285984 359 "Funciones y variables para la miscelánea de opciones"))
("pade" . ("maxima.info-4" 164438 2961 "Funciones y variables para las series"))
("palette" . ("maxima.info-2" 163152 1416 "Opciones gráficas"))
("palette <1>" . ("maxima.info-2" 163213 1355 "Opciones gráficas"))
("palette <2>" . ("maxima.info-6" 177694 3472 "Funciones y variables para draw"))
("parGosper" . ("maxima.info-8" 197309 361 "Funciones y variables para zeilberger"))
("parabolic_cylinder_d" . ("maxima.info-3" 41105 131 "Funciones de cilindro parabólico"))
("parametric" . ("maxima.info-6" 235251 1474 "Funciones y variables para draw"))
("parametric <1>" . ("maxima.info-6" 235252 1473 "Funciones y variables para draw"))
("parametric_surface" . ("maxima.info-6" 236726 1254 "Funciones y variables para draw"))
("parg" . ("maxima.info-8" 146721 1207 "Funciones y variables para to_poly_solve"))
("parse_string" . ("maxima.info-8" 112476 710 "Funciones y variables para cadenas"))
("part" . ("maxima.info-1" 289660 1626 "Funciones y variables para expresiones"))
("part2cont" . ("maxima.info-4" 247000 512 "Funciones y variables para simetrías"))
("partfrac" . ("maxima.info-4" 209556 1246 "Funciones y variables para teoría de números"))
("partition" . ("maxima.info-1" 291287 594 "Funciones y variables para expresiones"))
("partition_set" . ("maxima.info-5" 38941 827 "Funciones y variables para los conjuntos"))
("partpol" . ("maxima.info-4" 247513 408 "Funciones y variables para simetrías"))
("partswitch" . ("maxima.info-1" 291882 243 "Funciones y variables para expresiones"))
("path_digraph" . ("maxima.info-7" 85792 82 "Funciones y variables para graphs"))
("path_graph" . ("maxima.info-7" 85875 71 "Funciones y variables para graphs"))
("pathname_directory" . ("maxima.info-2" 193250 564 "Funciones y variables para lectura y escritura"))
("pathname_name" . ("maxima.info-2" 193251 563 "Funciones y variables para lectura y escritura"))
("pathname_type" . ("maxima.info-2" 193296 518 "Funciones y variables para lectura y escritura"))
("pdf_bernoulli" . ("maxima.info-6" 106369 370 "Funciones y variables para distribuciones discretas"))
("pdf_beta" . ("maxima.info-6" 74864 233 "Funciones y variables para distribuciones continuas"))
("pdf_binomial" . ("maxima.info-6" 101099 280 "Funciones y variables para distribuciones discretas"))
("pdf_cauchy" . ("maxima.info-6" 93146 238 "Funciones y variables para distribuciones continuas"))
("pdf_chi2" . ("maxima.info-6" 57027 589 "Funciones y variables para distribuciones continuas"))
("pdf_continuous_uniform" . ("maxima.info-6" 77313 258 "Funciones y variables para distribuciones continuas"))
("pdf_discrete_uniform" . ("maxima.info-6" 111890 273 "Funciones y variables para distribuciones discretas"))
("pdf_exp" . ("maxima.info-6" 65864 386 "Funciones y variables para distribuciones continuas"))
("pdf_f" . ("maxima.info-6" 63048 227 "Funciones y variables para distribuciones continuas"))
("pdf_gamma" . ("maxima.info-6" 72118 235 "Funciones y variables para distribuciones continuas"))
("pdf_general_finite_discrete" . ("maxima.info-6" 97245 882 "Funciones y variables para distribuciones discretas"))
("pdf_geometric" . ("maxima.info-6" 109617 254 "Funciones y variables para distribuciones discretas"))
("pdf_gumbel" . ("maxima.info-6" 94329 238 "Funciones y variables para distribuciones continuas"))
("pdf_hypergeometric" . ("maxima.info-6" 114399 566 "Funciones y variables para distribuciones discretas"))
("pdf_laplace" . ("maxima.info-6" 90946 240 "Funciones y variables para distribuciones continuas"))
("pdf_logistic" . ("maxima.info-6" 79768 240 "Funciones y variables para distribuciones continuas"))
("pdf_lognormal" . ("maxima.info-6" 69462 241 "Funciones y variables para distribuciones continuas"))
("pdf_negative_binomial" . ("maxima.info-6" 117440 288 "Funciones y variables para distribuciones discretas"))
("pdf_noncentral_chi2" . ("maxima.info-6" 60644 316 "Funciones y variables para distribuciones continuas"))
("pdf_noncentral_student_t" . ("maxima.info-6" 52111 1159 "Funciones y variables para distribuciones continuas"))
("pdf_normal" . ("maxima.info-6" 45955 234 "Funciones y variables para distribuciones continuas"))
("pdf_pareto" . ("maxima.info-6" 81973 240 "Funciones y variables para distribuciones continuas"))
("pdf_poisson" . ("maxima.info-6" 103938 237 "Funciones y variables para distribuciones discretas"))
("pdf_rank_sum" . ("maxima.info-8" 88306 288 "Funciones y variables para distribuciones especiales"))
("pdf_rayleigh" . ("maxima.info-6" 86403 447 "Funciones y variables para distribuciones continuas"))
("pdf_signed_rank" . ("maxima.info-8" 87744 281 "Funciones y variables para distribuciones especiales"))
("pdf_student_t" . ("maxima.info-6" 48944 238 "Funciones y variables para distribuciones continuas"))
("pdf_weibull" . ("maxima.info-6" 84185 242 "Funciones y variables para distribuciones continuas"))
("pearson_skewness" . ("maxima.info-5" 258385 807 "Funciones y variables de parámetros descriptivos"))
("pearson_skewness <1>" . ("maxima.info-5" 258386 806 "Funciones y variables de parámetros descriptivos"))
("permanent" . ("maxima.info-3" 276807 297 "Funciones y variables para las matrices y el álgebra lineal"))
("permut" . ("maxima.info-4" 247922 83 "Funciones y variables para simetrías"))
("permutation" . ("maxima.info-8" 26048 190 "Paquete functs"))
("permutations" . ("maxima.info-5" 39769 643 "Funciones y variables para los conjuntos"))
("petersen_graph" . ("maxima.info-7" 85947 180 "Funciones y variables para graphs"))
("petersen_graph <1>" . ("maxima.info-7" 85948 179 "Funciones y variables para graphs"))
("petrov" . ("maxima.info-4" 98448 3983 "Funciones y variables para ctensor"))
("pfeformat" . ("maxima.info-1" 91468 734 "Funciones y variables para la impresión"))
("phi" . ("maxima.info-1" 117269 2469 "Funciones y variables para Constantes"))
("phiresolution" . ("maxima.info-7" 11232 322 "Visualización usando VTK"))
("pi" . ("maxima.info-1" 119740 209 "Funciones y variables para Constantes"))
("pickapart" . ("maxima.info-1" 292126 3042 "Funciones y variables para expresiones"))
("picture_equalp" . ("maxima.info-6" 256068 139 "Funciones y variables para picture"))
("picturep" . ("maxima.info-6" 256208 127 "Funciones y variables para picture"))
("piece" . ("maxima.info-1" 295169 103 "Funciones y variables para expresiones"))
("piechart" . ("maxima.info-6" 13899 1966 "Funciones y variables para gráficos estadísticos"))
("piechart <1>" . ("maxima.info-6" 13900 1965 "Funciones y variables para gráficos estadísticos"))
("piechart <2>" . ("maxima.info-6" 13931 1934 "Funciones y variables para gráficos estadísticos"))
("piechart <3>" . ("maxima.info-6" 13991 1874 "Funciones y variables para gráficos estadísticos"))
("piechart <4>" . ("maxima.info-6" 14035 1830 "Funciones y variables para gráficos estadísticos"))
("piechart <5>" . ("maxima.info-6" 14108 1757 "Funciones y variables para gráficos estadísticos"))
("piechart_description" . ("maxima.info-6" 14149 1716 "Funciones y variables para gráficos estadísticos"))
("planar_embedding" . ("maxima.info-7" 108928 532 "Funciones y variables para graphs"))
("playback" . ("maxima.info-1" 66049 2264 "Funciones y variables para la línea de comandos"))
("playback <1>" . ("maxima.info-1" 66050 2263 "Funciones y variables para la línea de comandos"))
("playback <2>" . ("maxima.info-1" 66075 2238 "Funciones y variables para la línea de comandos"))
("playback <3>" . ("maxima.info-1" 66103 2210 "Funciones y variables para la línea de comandos"))
("playback <4>" . ("maxima.info-1" 66138 2175 "Funciones y variables para la línea de comandos"))
("playback <5>" . ("maxima.info-1" 66168 2145 "Funciones y variables para la línea de comandos"))
("playback <6>" . ("maxima.info-1" 66198 2115 "Funciones y variables para la línea de comandos"))
("playback <7>" . ("maxima.info-1" 66227 2086 "Funciones y variables para la línea de comandos"))
("plog" . ("maxima.info-2" 67088 131 "Funciones radicales exponenciales y logarítmicas"))
("plot2d" . ("maxima.info-2" 144264 6550 "Funciones y variables para gráficos"))
("plot2d <1>" . ("maxima.info-2" 144265 6549 "Funciones y variables para gráficos"))
("plot2d <2>" . ("maxima.info-2" 144326 6488 "Funciones y variables para gráficos"))
("plot3d" . ("maxima.info-2" 150815 4474 "Funciones y variables para gráficos"))
("plot3d <1>" . ("maxima.info-2" 150816 4473 "Funciones y variables para gráficos"))
("plot_format" . ("maxima.info-2" 164569 313 "Opciones gráficas"))
("plot_options" . ("maxima.info-2" 155290 799 "Funciones y variables para gráficos"))
("plot_realpart" . ("maxima.info-2" 164883 593 "Opciones gráficas"))
("plotdf" . ("maxima.info-3" 216648 10080 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("plotdf <1>" . ("maxima.info-3" 216649 10079 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("plotdf <2>" . ("maxima.info-3" 216693 10035 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("plotdf <3>" . ("maxima.info-3" 216752 9976 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("ploteq" . ("maxima.info-3" 226729 1276 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("plsquares" . ("maxima.info-7" 248843 4322 "Funciones y variables para lsquares"))
("plsquares <1>" . ("maxima.info-7" 248844 4321 "Funciones y variables para lsquares"))
("plsquares <2>" . ("maxima.info-7" 248895 4270 "Funciones y variables para lsquares"))
("pochhammer" . ("maxima.info-7" 302341 1313 "Funciones y variables para polinomios ortogonales"))
("pochhammer_max_index" . ("maxima.info-7" 303655 526 "Funciones y variables para polinomios ortogonales"))
("point_size" . ("maxima.info-6" 181167 637 "Funciones y variables para draw"))
("point_type" . ("maxima.info-2" 165477 703 "Opciones gráficas"))
("point_type <1>" . ("maxima.info-6" 181805 1720 "Funciones y variables para draw"))
("points" . ("maxima.info-7" 11555 288 "Visualización usando VTK"))
("points <10>" . ("maxima.info-6" 238444 6199 "Funciones y variables para draw"))
("points <11>" . ("maxima.info-6" 238514 6129 "Funciones y variables para draw"))
("points <1>" . ("maxima.info-6" 237981 6662 "Funciones y variables para draw"))
("points <2>" . ("maxima.info-6" 237982 6661 "Funciones y variables para draw"))
("points <3>" . ("maxima.info-6" 238042 6601 "Funciones y variables para draw"))
("points <4>" . ("maxima.info-6" 238104 6539 "Funciones y variables para draw"))
("points <5>" . ("maxima.info-6" 238149 6494 "Funciones y variables para draw"))
("points <6>" . ("maxima.info-6" 238281 6362 "Funciones y variables para draw"))
("points <7>" . ("maxima.info-6" 238308 6335 "Funciones y variables para draw"))
("points <8>" . ("maxima.info-6" 238346 6297 "Funciones y variables para draw"))
("points <9>" . ("maxima.info-6" 238388 6255 "Funciones y variables para draw"))
("points_joined" . ("maxima.info-6" 183526 1205 "Funciones y variables para draw"))
("pointsize" . ("maxima.info-7" 11844 186 "Visualización usando VTK"))
("poisdiff" . ("maxima.info-4" 189398 173 "Funciones y variables para series de Poisson"))
("poisexpt" . ("maxima.info-4" 189572 123 "Funciones y variables para series de Poisson"))
("poisint" . ("maxima.info-4" 189696 93 "Funciones y variables para series de Poisson"))
("poislim" . ("maxima.info-4" 189790 327 "Funciones y variables para series de Poisson"))
("poismap" . ("maxima.info-4" 190118 348 "Funciones y variables para series de Poisson"))
("poisplus" . ("maxima.info-4" 190467 69 "Funciones y variables para series de Poisson"))
("poissimp" . ("maxima.info-4" 190537 115 "Funciones y variables para series de Poisson"))
("poisson" . ("maxima.info-4" 190653 144 "Funciones y variables para series de Poisson"))
("poissubst" . ("maxima.info-4" 190798 416 "Funciones y variables para series de Poisson"))
("poistimes" . ("maxima.info-4" 191215 72 "Funciones y variables para series de Poisson"))
("polar" . ("maxima.info-6" 244644 899 "Funciones y variables para draw"))
("polar_to_xy" . ("maxima.info-2" 144001 262 "Funciones y variables para gráficos"))
("polarform" . ("maxima.info-2" 49573 147 "Funciones para los números complejos"))
("polartorect" . ("maxima.info-3" 200028 696 "Funciones y variables para la transformada rápida de Fourier"))
("poly_add" . ("maxima.info-7" 134176 246 "Funciones y variables para grobner"))
("poly_buchberger" . ("maxima.info-7" 138729 192 "Funciones y variables para grobner"))
("poly_buchberger_criterion" . ("maxima.info-7" 138406 322 "Funciones y variables para grobner"))
("poly_coefficient_ring" . ("maxima.info-7" 131729 355 "Funciones y variables para grobner"))
("poly_colon_ideal" . ("maxima.info-7" 141048 214 "Funciones y variables para grobner"))
("poly_content" . ("maxima.info-7" 137124 201 "Funciones y variables para grobner"))
("poly_depends_p" . ("maxima.info-7" 140635 136 "Funciones y variables para grobner"))
("poly_elimination_ideal" . ("maxima.info-7" 140772 275 "Funciones y variables para grobner"))
("poly_elimination_order" . ("maxima.info-7" 132602 383 "Funciones y variables para grobner"))
("poly_exact_divide" . ("maxima.info-7" 138000 214 "Funciones y variables para grobner"))
("poly_expand" . ("maxima.info-7" 135596 1203 "Funciones y variables para grobner"))
("poly_expt" . ("maxima.info-7" 136800 323 "Funciones y variables para grobner"))
("poly_gcd" . ("maxima.info-7" 141518 604 "Funciones y variables para grobner"))
("poly_grobner" . ("maxima.info-7" 140233 189 "Funciones y variables para grobner"))
("poly_grobner_algorithm" . ("maxima.info-7" 133410 276 "Funciones y variables para grobner"))
("poly_grobner_debug" . ("maxima.info-7" 133266 143 "Funciones y variables para grobner"))
("poly_grobner_equal" . ("maxima.info-7" 142123 680 "Funciones y variables para grobner"))
("poly_grobner_member" . ("maxima.info-7" 143103 264 "Funciones y variables para grobner"))
("poly_grobner_subsetp" . ("maxima.info-7" 142804 298 "Funciones y variables para grobner"))
("poly_ideal_intersection" . ("maxima.info-7" 141263 158 "Funciones y variables para grobner"))
("poly_ideal_polysaturation" . ("maxima.info-7" 144560 311 "Funciones y variables para grobner"))
("poly_ideal_polysaturation1" . ("maxima.info-7" 144132 427 "Funciones y variables para grobner"))
("poly_ideal_saturation" . ("maxima.info-7" 143740 391 "Funciones y variables para grobner"))
("poly_ideal_saturation1" . ("maxima.info-7" 143368 371 "Funciones y variables para grobner"))
("poly_lcm" . ("maxima.info-7" 141422 95 "Funciones y variables para grobner"))
("poly_minimization" . ("maxima.info-7" 139677 320 "Funciones y variables para grobner"))
("poly_monomial_order" . ("maxima.info-7" 131494 234 "Funciones y variables para grobner"))
("poly_multiply" . ("maxima.info-7" 134684 221 "Funciones y variables para grobner"))
("poly_normal_form" . ("maxima.info-7" 138215 190 "Funciones y variables para grobner"))
("poly_normalize" . ("maxima.info-7" 135307 242 "Funciones y variables para grobner"))
("poly_normalize_list" . ("maxima.info-7" 139998 234 "Funciones y variables para grobner"))
("poly_polysaturation_extension" . ("maxima.info-7" 145032 100 "Funciones y variables para grobner"))
("poly_primary_elimination_order" . ("maxima.info-7" 132085 258 "Funciones y variables para grobner"))
("poly_primitive_part" . ("maxima.info-7" 135062 244 "Funciones y variables para grobner"))
("poly_pseudo_divide" . ("maxima.info-7" 137326 673 "Funciones y variables para grobner"))
("poly_reduced_grobner" . ("maxima.info-7" 140423 211 "Funciones y variables para grobner"))
("poly_reduction" . ("maxima.info-7" 139463 213 "Funciones y variables para grobner"))
("poly_return_term_list" . ("maxima.info-7" 132986 279 "Funciones y variables para grobner"))
("poly_s_polynomial" . ("maxima.info-7" 134906 155 "Funciones y variables para grobner"))
("poly_saturation_extension" . ("maxima.info-7" 144872 159 "Funciones y variables para grobner"))
("poly_secondary_elimination_order" . ("maxima.info-7" 132344 257 "Funciones y variables para grobner"))
("poly_subtract" . ("maxima.info-7" 134423 260 "Funciones y variables para grobner"))
("poly_top_reduction_only" . ("maxima.info-7" 133687 199 "Funciones y variables para grobner"))
("polydecomp" . ("maxima.info-2" 249499 2007 "Funciones y variables para polinomios"))
("polyfactor" . ("maxima.info-3" 171851 333 "Funciones y variable para las ecuaciones"))
("polygon" . ("maxima.info-6" 245544 839 "Funciones y variables para draw"))
("polygon <1>" . ("maxima.info-6" 245545 838 "Funciones y variables para draw"))
("polymod" . ("maxima.info-2" 251507 364 "Funciones y variables para polinomios"))
("polymod <1>" . ("maxima.info-2" 251508 363 "Funciones y variables para polinomios"))
("polynome2ele" . ("maxima.info-4" 248006 596 "Funciones y variables para simetrías"))
("polynomialp" . ("maxima.info-7" 225545 1766 "Funciones y variables para linearalgebra"))
("polynomialp <1>" . ("maxima.info-7" 225546 1765 "Funciones y variables para linearalgebra"))
("polynomialp <2>" . ("maxima.info-7" 225602 1709 "Funciones y variables para linearalgebra"))
("polytocompanion" . ("maxima.info-7" 227312 304 "Funciones y variables para linearalgebra"))
("pop" . ("maxima.info-1" 135745 335 "Funciones y variables para listas"))
("posfun" . ("maxima.info-2" 106226 186 "Funciones y variables para las propiedades"))
("position" . ("maxima.info-7" 12031 218 "Visualización usando VTK"))
("postfix" . ("maxima.info-1" 223123 269 "Operadores definidos por el usuario"))
("postfix <1>" . ("maxima.info-1" 223124 268 "Operadores definidos por el usuario"))
("power_mod" . ("maxima.info-4" 210803 559 "Funciones y variables para teoría de números"))
("powerdisp" . ("maxima.info-1" 92203 857 "Funciones y variables para la impresión"))
("powers" . ("maxima.info-2" 251872 144 "Funciones y variables para polinomios"))
("powerseries" . ("maxima.info-4" 167400 1979 "Funciones y variables para las series"))
("powerset" . ("maxima.info-5" 40413 1180 "Funciones y variables para los conjuntos"))
("powerset <1>" . ("maxima.info-5" 40414 1179 "Funciones y variables para los conjuntos"))
("pred" . ("maxima.info-1" 253998 389 "Funciones y variables para la evaluación"))
("prederror" . ("maxima.info-5" 162498 576 "Funciones y variables para la programación"))
("prefix" . ("maxima.info-1" 223393 274 "Operadores definidos por el usuario"))
("prefix <1>" . ("maxima.info-1" 223394 273 "Operadores definidos por el usuario"))
("prev_prime" . ("maxima.info-4" 212402 158 "Funciones y variables para teoría de números"))
("primep" . ("maxima.info-4" 211363 899 "Funciones y variables para teoría de números"))
("primep_number_of_tests" . ("maxima.info-4" 212263 138 "Funciones y variables para teoría de números"))
("principal_components" . ("maxima.info-5" 272344 4385 "Funciones y variables de parámetros descriptivos"))
("principal_components <1>" . ("maxima.info-5" 272345 4384 "Funciones y variables de parámetros descriptivos"))
("print" . ("maxima.info-1" 93061 1373 "Funciones y variables para la impresión"))
("print_graph" . ("maxima.info-7" 109461 706 "Funciones y variables para graphs"))
("printf" . ("maxima.info-8" 100966 4811 "Funciones y variables para entrada y salida"))
("printf <1>" . ("maxima.info-8" 100967 4810 "Funciones y variables para entrada y salida"))
("printfile" . ("maxima.info-2" 193815 568 "Funciones y variables para lectura y escritura"))
("printpois" . ("maxima.info-4" 191288 202 "Funciones y variables para series de Poisson"))
("printprops" . ("maxima.info-2" 106413 559 "Funciones y variables para las propiedades"))
("printprops <1>" . ("maxima.info-2" 106414 558 "Funciones y variables para las propiedades"))
("printprops <2>" . ("maxima.info-2" 106449 523 "Funciones y variables para las propiedades"))
("prodrac" . ("maxima.info-4" 248603 254 "Funciones y variables para simetrías"))
("product" . ("maxima.info-4" 145895 2672 "Funciones y variables para sumas y productos"))
("product_use_gamma" . ("maxima.info-8" 42669 281 "Funciones y variables para solve_rec"))
("program" . ("maxima.info-7" 128178 465 "Funciones y variables para graphs"))
("programmode" . ("maxima.info-3" 172185 1037 "Funciones y variable para las ecuaciones"))
("prompt" . ("maxima.info-1" 68314 227 "Funciones y variables para la línea de comandos"))
("properties" . ("maxima.info-2" 106973 124 "Funciones y variables para las propiedades"))
("proportional_axes" . ("maxima.info-6" 184732 1146 "Funciones y variables para draw"))
("props" . ("maxima.info-2" 107098 275 "Funciones y variables para las propiedades"))
("propvars" . ("maxima.info-2" 107374 262 "Funciones y variables para las propiedades"))
("psexpand" . ("maxima.info-4" 169380 433 "Funciones y variables para las series"))
("psfile" . ("maxima.info-2" 166181 326 "Opciones gráficas"))
("psi" . ("maxima.info-3" 31460 995 "Funciones Gamma y factorial"))
("psi <1>" . ("maxima.info-4" 98183 264 "Funciones y variables para ctensor"))
("psubst" . ("maxima.info-1" 295273 778 "Funciones y variables para expresiones"))
("psubst <1>" . ("maxima.info-1" 295274 777 "Funciones y variables para expresiones"))
("ptriangularize" . ("maxima.info-7" 227617 467 "Funciones y variables para linearalgebra"))
("pui" . ("maxima.info-4" 248858 1573 "Funciones y variables para simetrías"))
("pui2comp" . ("maxima.info-4" 250432 1504 "Funciones y variables para simetrías"))
("pui2ele" . ("maxima.info-4" 251937 526 "Funciones y variables para simetrías"))
("pui2polynome" . ("maxima.info-4" 252464 670 "Funciones y variables para simetrías"))
("pui_direct" . ("maxima.info-4" 253135 2183 "Funciones y variables para simetrías"))
("puireduc" . ("maxima.info-4" 255319 717 "Funciones y variables para simetrías"))
("push" . ("maxima.info-1" 136081 1385 "Funciones y variables para listas"))
("put" . ("maxima.info-2" 107637 904 "Funciones y variables para las propiedades"))
("pv" . ("maxima.info-7" 58487 338 "Funciones y Variables para finance"))
("qput" . ("maxima.info-2" 108542 831 "Funciones y variables para las propiedades"))
("qrange" . ("maxima.info-5" 252322 723 "Funciones y variables de parámetros descriptivos"))
("qrange <1>" . ("maxima.info-5" 252323 722 "Funciones y variables de parámetros descriptivos"))
("qty" . ("maxima.info-7" 40979 1709 "Funciones y variables para ezunits"))
("quad_control" . ("maxima.info-3" 145627 661 "Funciones y variables para QUADPACK"))
("quad_qag" . ("maxima.info-3" 121879 3185 "Funciones y variables para QUADPACK"))
("quad_qag <1>" . ("maxima.info-3" 121944 3120 "Funciones y variables para QUADPACK"))
("quad_qagi" . ("maxima.info-3" 127524 2762 "Funciones y variables para QUADPACK"))
("quad_qagi <1>" . ("maxima.info-3" 127593 2693 "Funciones y variables para QUADPACK"))
("quad_qagp" . ("maxima.info-3" 142862 2764 "Funciones y variables para QUADPACK"))
("quad_qagp <1>" . ("maxima.info-3" 142930 2696 "Funciones y variables para QUADPACK"))
("quad_qags" . ("maxima.info-3" 125065 2458 "Funciones y variables para QUADPACK"))
("quad_qags <1>" . ("maxima.info-3" 125134 2389 "Funciones y variables para QUADPACK"))
("quad_qawc" . ("maxima.info-3" 130287 3413 "Funciones y variables para QUADPACK"))
("quad_qawc <1>" . ("maxima.info-3" 130361 3339 "Funciones y variables para QUADPACK"))
("quad_qawf" . ("maxima.info-3" 133701 2740 "Funciones y variables para QUADPACK"))
("quad_qawf <1>" . ("maxima.info-3" 133772 2669 "Funciones y variables para QUADPACK"))
("quad_qawo" . ("maxima.info-3" 136442 3182 "Funciones y variables para QUADPACK"))
("quad_qawo <1>" . ("maxima.info-3" 136507 3117 "Funciones y variables para QUADPACK"))
("quad_qaws" . ("maxima.info-3" 139625 3236 "Funciones y variables para QUADPACK"))
("quad_qaws <1>" . ("maxima.info-3" 139698 3163 "Funciones y variables para QUADPACK"))
("quadrilateral" . ("maxima.info-6" 246384 1044 "Funciones y variables para draw"))
("quantile" . ("maxima.info-5" 250705 888 "Funciones y variables de parámetros descriptivos"))
("quantile <1>" . ("maxima.info-5" 250706 887 "Funciones y variables de parámetros descriptivos"))
("quantile_bernoulli" . ("maxima.info-6" 107000 326 "Funciones y variables para distribuciones discretas"))
("quantile_beta" . ("maxima.info-6" 75540 305 "Funciones y variables para distribuciones continuas"))
("quantile_binomial" . ("maxima.info-6" 101863 352 "Funciones y variables para distribuciones discretas"))
("quantile_cauchy" . ("maxima.info-6" 93627 311 "Funciones y variables para distribuciones continuas"))
("quantile_chi2" . ("maxima.info-6" 58109 429 "Funciones y variables para distribuciones continuas"))
("quantile_continuous_uniform" . ("maxima.info-6" 77840 349 "Funciones y variables para distribuciones continuas"))
("quantile_discrete_uniform" . ("maxima.info-6" 112438 353 "Funciones y variables para distribuciones discretas"))
("quantile_exp" . ("maxima.info-6" 66644 632 "Funciones y variables para distribuciones continuas"))
("quantile_f" . ("maxima.info-6" 63755 518 "Funciones y variables para distribuciones continuas"))
("quantile_gamma" . ("maxima.info-6" 72840 307 "Funciones y variables para distribuciones continuas"))
("quantile_general_finite_discrete" . ("maxima.info-6" 99025 234 "Funciones y variables para distribuciones discretas"))
("quantile_geometric" . ("maxima.info-6" 110127 321 "Funciones y variables para distribuciones discretas"))
("quantile_gumbel" . ("maxima.info-6" 94810 311 "Funciones y variables para distribuciones continuas"))
("quantile_hypergeometric" . ("maxima.info-6" 115348 396 "Funciones y variables para distribuciones discretas"))
("quantile_laplace" . ("maxima.info-6" 91431 315 "Funciones y variables para distribuciones continuas"))
("quantile_logistic" . ("maxima.info-6" 80253 315 "Funciones y variables para distribuciones continuas"))
("quantile_lognormal" . ("maxima.info-6" 70337 317 "Funciones y variables para distribuciones continuas"))
("quantile_negative_binomial" . ("maxima.info-6" 118157 369 "Funciones y variables para distribuciones discretas"))
("quantile_noncentral_chi2" . ("maxima.info-6" 61210 412 "Funciones y variables para distribuciones continuas"))
("quantile_noncentral_student_t" . ("maxima.info-6" 53814 424 "Funciones y variables para distribuciones continuas"))
("quantile_normal" . ("maxima.info-6" 46823 605 "Funciones y variables para distribuciones continuas"))
("quantile_pareto" . ("maxima.info-6" 82458 314 "Funciones y variables para distribuciones continuas"))
("quantile_poisson" . ("maxima.info-6" 104540 309 "Funciones y variables para distribuciones discretas"))
("quantile_rayleigh" . ("maxima.info-6" 87300 534 "Funciones y variables para distribuciones continuas"))
("quantile_student_t" . ("maxima.info-6" 49787 314 "Funciones y variables para distribuciones continuas"))
("quantile_weibull" . ("maxima.info-6" 84674 317 "Funciones y variables para distribuciones continuas"))
("quartile_skewness" . ("maxima.info-5" 259193 895 "Funciones y variables de parámetros descriptivos"))
("quartile_skewness <1>" . ("maxima.info-5" 259194 894 "Funciones y variables de parámetros descriptivos"))
("quit" . ("maxima.info-1" 68542 397 "Funciones y variables para la línea de comandos"))
("qunit" . ("maxima.info-4" 212561 345 "Funciones y variables para teoría de números"))
("quotient" . ("maxima.info-2" 252017 344 "Funciones y variables para polinomios"))
("quotient <1>" . ("maxima.info-2" 252018 343 "Funciones y variables para polinomios"))
("radcan" . ("maxima.info-2" 27124 1255 "Funciones y variables para simplificación"))
("radexpand" . ("maxima.info-2" 28380 882 "Funciones y variables para simplificación"))
("radius" . ("maxima.info-7" 12250 234 "Visualización usando VTK"))
("radius <1>" . ("maxima.info-7" 110168 209 "Funciones y variables para graphs"))
("radsubstflag" . ("maxima.info-2" 268058 181 "Funciones y variables para polinomios"))
("random" . ("maxima.info-2" 85225 1692 "Números aleatorios"))
("random_bernoulli" . ("maxima.info-6" 109188 428 "Funciones y variables para distribuciones discretas"))
("random_bernoulli <1>" . ("maxima.info-6" 109189 427 "Funciones y variables para distribuciones discretas"))
("random_beta" . ("maxima.info-6" 76813 499 "Funciones y variables para distribuciones continuas"))
("random_beta <1>" . ("maxima.info-6" 76814 498 "Funciones y variables para distribuciones continuas"))
("random_binomial" . ("maxima.info-6" 103387 550 "Funciones y variables para distribuciones discretas"))
("random_binomial <1>" . ("maxima.info-6" 103388 549 "Funciones y variables para distribuciones discretas"))
("random_bipartite_graph" . ("maxima.info-7" 86128 177 "Funciones y variables para graphs"))
("random_cauchy" . ("maxima.info-6" 93939 389 "Funciones y variables para distribuciones continuas"))
("random_cauchy <1>" . ("maxima.info-6" 93940 388 "Funciones y variables para distribuciones continuas"))
("random_chi2" . ("maxima.info-6" 60226 417 "Funciones y variables para distribuciones continuas"))
("random_chi2 <1>" . ("maxima.info-6" 60227 416 "Funciones y variables para distribuciones continuas"))
("random_continuous_uniform" . ("maxima.info-6" 79285 482 "Funciones y variables para distribuciones continuas"))
("random_continuous_uniform <1>" . ("maxima.info-6" 79286 481 "Funciones y variables para distribuciones continuas"))
("random_digraph" . ("maxima.info-7" 86306 152 "Funciones y variables para graphs"))
("random_discrete_uniform" . ("maxima.info-6" 113920 478 "Funciones y variables para distribuciones discretas"))
("random_discrete_uniform <1>" . ("maxima.info-6" 113921 477 "Funciones y variables para distribuciones discretas"))
("random_exp" . ("maxima.info-6" 69084 377 "Funciones y variables para distribuciones continuas"))
("random_exp <1>" . ("maxima.info-6" 69085 376 "Funciones y variables para distribuciones continuas"))
("random_f" . ("maxima.info-6" 65223 640 "Funciones y variables para distribuciones continuas"))
("random_f <1>" . ("maxima.info-6" 65224 639 "Funciones y variables para distribuciones continuas"))
("random_gamma" . ("maxima.info-6" 74122 741 "Funciones y variables para distribuciones continuas"))
("random_gamma <1>" . ("maxima.info-6" 74123 740 "Funciones y variables para distribuciones continuas"))
("random_general_finite_discrete" . ("maxima.info-6" 100423 675 "Funciones y variables para distribuciones discretas"))
("random_general_finite_discrete <1>" . ("maxima.info-6" 100424 674 "Funciones y variables para distribuciones discretas"))
("random_geometric" . ("maxima.info-6" 111482 407 "Funciones y variables para distribuciones discretas"))
("random_geometric <1>" . ("maxima.info-6" 111483 406 "Funciones y variables para distribuciones discretas"))
("random_graph" . ("maxima.info-7" 86653 141 "Funciones y variables para graphs"))
("random_graph1" . ("maxima.info-7" 86795 111 "Funciones y variables para graphs"))
("random_gumbel" . ("maxima.info-6" 96585 391 "Funciones y variables para distribuciones continuas"))
("random_gumbel <1>" . ("maxima.info-6" 96586 390 "Funciones y variables para distribuciones continuas"))
("random_hypergeometric" . ("maxima.info-6" 116817 622 "Funciones y variables para distribuciones discretas"))
("random_hypergeometric <1>" . ("maxima.info-6" 116818 621 "Funciones y variables para distribuciones discretas"))
("random_laplace" . ("maxima.info-6" 92752 393 "Funciones y variables para distribuciones continuas"))
("random_laplace <1>" . ("maxima.info-6" 92753 392 "Funciones y variables para distribuciones continuas"))
("random_logistic" . ("maxima.info-6" 81574 398 "Funciones y variables para distribuciones continuas"))
("random_logistic <1>" . ("maxima.info-6" 81575 397 "Funciones y variables para distribuciones continuas"))
("random_lognormal" . ("maxima.info-6" 71665 452 "Funciones y variables para distribuciones continuas"))
("random_lognormal <1>" . ("maxima.info-6" 71666 451 "Funciones y variables para distribuciones continuas"))
("random_negative_binomial" . ("maxima.info-6" 119740 515 "Funciones y variables para distribuciones discretas"))
("random_negative_binomial <1>" . ("maxima.info-6" 119741 514 "Funciones y variables para distribuciones discretas"))
("random_network" . ("maxima.info-7" 86907 528 "Funciones y variables para graphs"))
("random_noncentral_chi2" . ("maxima.info-6" 62648 399 "Funciones y variables para distribuciones continuas"))
("random_noncentral_chi2 <1>" . ("maxima.info-6" 62649 398 "Funciones y variables para distribuciones continuas"))
("random_noncentral_student_t" . ("maxima.info-6" 56078 948 "Funciones y variables para distribuciones continuas"))
("random_noncentral_student_t <1>" . ("maxima.info-6" 56079 947 "Funciones y variables para distribuciones continuas"))
("random_normal" . ("maxima.info-6" 48435 508 "Funciones y variables para distribuciones continuas"))
("random_normal <1>" . ("maxima.info-6" 48436 507 "Funciones y variables para distribuciones continuas"))
("random_pareto" . ("maxima.info-6" 83791 393 "Funciones y variables para distribuciones continuas"))
("random_pareto <1>" . ("maxima.info-6" 83792 392 "Funciones y variables para distribuciones continuas"))
("random_permutation" . ("maxima.info-5" 41594 787 "Funciones y variables para los conjuntos"))
("random_poisson" . ("maxima.info-6" 105822 546 "Funciones y variables para distribuciones discretas"))
("random_poisson <1>" . ("maxima.info-6" 105823 545 "Funciones y variables para distribuciones discretas"))
("random_rayleigh" . ("maxima.info-6" 90557 388 "Funciones y variables para distribuciones continuas"))
("random_rayleigh <1>" . ("maxima.info-6" 90558 387 "Funciones y variables para distribuciones continuas"))
("random_regular_graph" . ("maxima.info-7" 86459 193 "Funciones y variables para graphs"))
("random_regular_graph <1>" . ("maxima.info-7" 86460 192 "Funciones y variables para graphs"))
("random_student_t" . ("maxima.info-6" 51267 843 "Funciones y variables para distribuciones continuas"))
("random_student_t <1>" . ("maxima.info-6" 51268 842 "Funciones y variables para distribuciones continuas"))
("random_tournament" . ("maxima.info-7" 87436 88 "Funciones y variables para graphs"))
("random_tree" . ("maxima.info-7" 87525 81 "Funciones y variables para graphs"))
("random_weibull" . ("maxima.info-6" 86007 395 "Funciones y variables para distribuciones continuas"))
("random_weibull <1>" . ("maxima.info-6" 86008 394 "Funciones y variables para distribuciones continuas"))
("range" . ("maxima.info-5" 250257 447 "Funciones y variables de parámetros descriptivos"))
("range <1>" . ("maxima.info-5" 250258 446 "Funciones y variables de parámetros descriptivos"))
("rank" . ("maxima.info-3" 277105 270 "Funciones y variables para las matrices y el álgebra lineal"))
("rank <1>" . ("maxima.info-7" 228344 349 "Funciones y variables para linearalgebra"))
("rassociative" . ("maxima.info-2" 29263 232 "Funciones y variables para simplificación"))
("rat" . ("maxima.info-2" 252362 2070 "Funciones y variables para polinomios"))
("rat <1>" . ("maxima.info-2" 252363 2069 "Funciones y variables para polinomios"))
("ratalgdenom" . ("maxima.info-2" 254433 320 "Funciones y variables para polinomios"))
("ratchristof" . ("maxima.info-4" 126837 117 "Funciones y variables para ctensor"))
("ratcoef" . ("maxima.info-2" 254754 1127 "Funciones y variables para polinomios"))
("ratcoef <1>" . ("maxima.info-2" 254755 1126 "Funciones y variables para polinomios"))
("ratdenom" . ("maxima.info-2" 255882 730 "Funciones y variables para polinomios"))
("ratdenomdivide" . ("maxima.info-2" 256613 1841 "Funciones y variables para polinomios"))
("ratdiff" . ("maxima.info-2" 258455 1979 "Funciones y variables para polinomios"))
("ratdisrep" . ("maxima.info-2" 260435 481 "Funciones y variables para polinomios"))
("rateinstein" . ("maxima.info-4" 126955 275 "Funciones y variables para ctensor"))
("ratepsilon" . ("maxima.info-1" 108425 214 "Funciones y variables para los números"))
("ratexpand" . ("maxima.info-2" 260917 2243 "Funciones y variables para polinomios"))
("ratexpand <1>" . ("maxima.info-2" 260918 2242 "Funciones y variables para polinomios"))
("ratfac" . ("maxima.info-2" 263161 1138 "Funciones y variables para polinomios"))
("ratinterpol" . ("maxima.info-7" 158327 1419 "Funciones y variables para interpol"))
("ratinterpol <1>" . ("maxima.info-7" 158328 1418 "Funciones y variables para interpol"))
("rational" . ("maxima.info-2" 109374 231 "Funciones y variables para las propiedades"))
("rational <1>" . ("maxima.info-8" 20588 412 "Paquete functs"))
("rationalize" . ("maxima.info-1" 108640 1477 "Funciones y variables para los números"))
("ratmx" . ("maxima.info-3" 277376 632 "Funciones y variables para las matrices y el álgebra lineal"))
("ratnumer" . ("maxima.info-2" 264300 731 "Funciones y variables para polinomios"))
("ratnump" . ("maxima.info-1" 110118 162 "Funciones y variables para los números"))
("ratp" . ("maxima.info-2" 265032 341 "Funciones y variables para polinomios"))
("ratprint" . ("maxima.info-2" 265374 227 "Funciones y variables para polinomios"))
("ratriemann" . ("maxima.info-4" 127231 318 "Funciones y variables para ctensor"))
("ratsimp" . ("maxima.info-2" 265602 2259 "Funciones y variables para polinomios"))
("ratsimp <1>" . ("maxima.info-2" 265603 2258 "Funciones y variables para polinomios"))
("ratsimpexpons" . ("maxima.info-2" 267862 195 "Funciones y variables para polinomios"))
("ratsubst" . ("maxima.info-2" 268240 1622 "Funciones y variables para polinomios"))
("ratvars" . ("maxima.info-2" 269863 1061 "Funciones y variables para polinomios"))
("ratvars <1>" . ("maxima.info-2" 269864 1060 "Funciones y variables para polinomios"))
("ratvars <2>" . ("maxima.info-2" 269905 1019 "Funciones y variables para polinomios"))
("ratvarswitch" . ("maxima.info-2" 270925 1610 "Funciones y variables para polinomios"))
("ratweight" . ("maxima.info-2" 272536 1249 "Funciones y variables para polinomios"))
("ratweight <1>" . ("maxima.info-2" 272537 1248 "Funciones y variables para polinomios"))
("ratweights" . ("maxima.info-2" 273786 285 "Funciones y variables para polinomios"))
("ratweyl" . ("maxima.info-4" 127550 281 "Funciones y variables para ctensor"))
("ratwtlvl" . ("maxima.info-2" 274072 305 "Funciones y variables para polinomios"))
("read" . ("maxima.info-1" 68940 553 "Funciones y variables para la línea de comandos"))
("read_array" . ("maxima.info-7" 268114 631 "Funciones y variables para entrada y salida en formato texto"))
("read_array <1>" . ("maxima.info-7" 268115 630 "Funciones y variables para entrada y salida en formato texto"))
("read_binary_array" . ("maxima.info-7" 274113 541 "Funciones y variables para entrada y salida en formato binario"))
("read_binary_list" . ("maxima.info-7" 274655 737 "Funciones y variables para entrada y salida en formato binario"))
("read_binary_list <1>" . ("maxima.info-7" 274656 736 "Funciones y variables para entrada y salida en formato binario"))
("read_binary_matrix" . ("maxima.info-7" 273635 477 "Funciones y variables para entrada y salida en formato binario"))
("read_hashed_array" . ("maxima.info-7" 268746 805 "Funciones y variables para entrada y salida en formato texto"))
("read_hashed_array <1>" . ("maxima.info-7" 268747 804 "Funciones y variables para entrada y salida en formato texto"))
("read_list" . ("maxima.info-7" 270285 692 "Funciones y variables para entrada y salida en formato texto"))
("read_list <1>" . ("maxima.info-7" 270286 691 "Funciones y variables para entrada y salida en formato texto"))
("read_list <2>" . ("maxima.info-7" 270315 662 "Funciones y variables para entrada y salida en formato texto"))
("read_list <3>" . ("maxima.info-7" 270349 628 "Funciones y variables para entrada y salida en formato texto"))
("read_matrix" . ("maxima.info-7" 267043 1070 "Funciones y variables para entrada y salida en formato texto"))
("read_matrix <1>" . ("maxima.info-7" 267044 1069 "Funciones y variables para entrada y salida en formato texto"))
("read_matrix <2>" . ("maxima.info-7" 267075 1038 "Funciones y variables para entrada y salida en formato texto"))
("read_matrix <3>" . ("maxima.info-7" 267111 1002 "Funciones y variables para entrada y salida en formato texto"))
("read_nested_list" . ("maxima.info-7" 269552 732 "Funciones y variables para entrada y salida en formato texto"))
("read_nested_list <1>" . ("maxima.info-7" 269553 731 "Funciones y variables para entrada y salida en formato texto"))
("read_xpm" . ("maxima.info-6" 256336 115 "Funciones y variables para picture"))
("readchar" . ("maxima.info-8" 105778 222 "Funciones y variables para entrada y salida"))
("readline" . ("maxima.info-8" 106001 221 "Funciones y variables para entrada y salida"))
("readonly" . ("maxima.info-1" 69494 658 "Funciones y variables para la línea de comandos"))
("real" . ("maxima.info-2" 109606 289 "Funciones y variables para las propiedades"))
("real_imagpart_to_conjugate" . ("maxima.info-8" 147929 614 "Funciones y variables para to_poly_solve"))
("realonly" . ("maxima.info-3" 173223 175 "Funciones y variable para las ecuaciones"))
("realpart" . ("maxima.info-2" 49721 253 "Funciones para los números complejos"))
("realroots" . ("maxima.info-3" 173399 2327 "Funciones y variable para las ecuaciones"))
("realroots <1>" . ("maxima.info-3" 173400 2326 "Funciones y variable para las ecuaciones"))
("realroots <2>" . ("maxima.info-3" 173441 2285 "Funciones y variable para las ecuaciones"))
("realroots <3>" . ("maxima.info-3" 173481 2245 "Funciones y variable para las ecuaciones"))
("rearray" . ("maxima.info-1" 167381 376 "Funciones y variables para los arrays"))
("rectangle" . ("maxima.info-6" 247429 986 "Funciones y variables para draw"))
("rectform" . ("maxima.info-2" 49975 133 "Funciones para los números complejos"))
("rectform_log_if_constant" . ("maxima.info-8" 148544 821 "Funciones y variables para to_poly_solve"))
("recttopolar" . ("maxima.info-3" 200725 773 "Funciones y variables para la transformada rápida de Fourier"))
("rediff" . ("maxima.info-4" 31226 126 "Funciones y variables para itensor"))
("redraw" . ("maxima.info-7" 127294 226 "Funciones y variables para graphs"))
("reduce_consts" . ("maxima.info-8" 30805 1483 "Paquete rducon"))
("reduce_order" . ("maxima.info-8" 35791 1817 "Funciones y variables para solve_rec"))
("refcheck" . ("maxima.info-5" 179757 202 "Funciones y variables para depurado"))
("region" . ("maxima.info-6" 248416 746 "Funciones y variables para draw"))
("region_boundaries" . ("maxima.info-6" 262882 575 "Funciones y variables para worldmap"))
("region_boundaries_plus" . ("maxima.info-6" 263458 579 "Funciones y variables para worldmap"))
("rem" . ("maxima.info-2" 109896 313 "Funciones y variables para las propiedades"))
("remainder" . ("maxima.info-2" 274378 337 "Funciones y variables para polinomios"))
("remainder <1>" . ("maxima.info-2" 274379 336 "Funciones y variables para polinomios"))
("remarray" . ("maxima.info-1" 167758 532 "Funciones y variables para los arrays"))
("remarray <1>" . ("maxima.info-1" 167759 531 "Funciones y variables para los arrays"))
("rembox" . ("maxima.info-1" 296052 3193 "Funciones y variables para expresiones"))
("rembox <1>" . ("maxima.info-1" 296053 3192 "Funciones y variables para expresiones"))
("rembox <2>" . ("maxima.info-1" 296094 3151 "Funciones y variables para expresiones"))
("remcomps" . ("maxima.info-4" 13623 132 "Funciones y variables para itensor"))
("remcon" . ("maxima.info-4" 8555 269 "Funciones y variables para itensor"))
("remcon <1>" . ("maxima.info-4" 8556 268 "Funciones y variables para itensor"))
("remcoord" . ("maxima.info-4" 35444 305 "Funciones y variables para itensor"))
("remcoord <1>" . ("maxima.info-4" 35445 304 "Funciones y variables para itensor"))
("remfun" . ("maxima.info-4" 185833 353 "Funciones y variables para series de Fourier"))
("remfun <1>" . ("maxima.info-4" 185834 352 "Funciones y variables para series de Fourier"))
("remfunction" . ("maxima.info-5" 121217 836 "Funciones y variables para la definición de funciones"))
("remfunction <1>" . ("maxima.info-5" 121218 835 "Funciones y variables para la definición de funciones"))
("remlet" . ("maxima.info-4" 315303 1083 "Funciones y variables sobre reglas y patrones"))
("remlet <1>" . ("maxima.info-4" 315304 1082 "Funciones y variables sobre reglas y patrones"))
("remlet <2>" . ("maxima.info-4" 315343 1043 "Funciones y variables sobre reglas y patrones"))
("remlet <3>" . ("maxima.info-4" 315366 1020 "Funciones y variables sobre reglas y patrones"))
("remove" . ("maxima.info-2" 110210 1353 "Funciones y variables para las propiedades"))
("remove <1>" . ("maxima.info-2" 110211 1352 "Funciones y variables para las propiedades"))
("remove <2>" . ("maxima.info-2" 110265 1298 "Funciones y variables para las propiedades"))
("remove <3>" . ("maxima.info-2" 110333 1230 "Funciones y variables para las propiedades"))
("remove <4>" . ("maxima.info-2" 110371 1192 "Funciones y variables para las propiedades"))
("remove_constvalue" . ("maxima.info-7" 37039 1511 "Funciones y variables para ezunits"))
("remove_dimensions" . ("maxima.info-7" 45869 1138 "Funciones y variables para ezunits"))
("remove_edge" . ("maxima.info-7" 118789 364 "Funciones y variables para graphs"))
("remove_fundamental_dimensions" . ("maxima.info-7" 47009 1392 "Funciones y variables para ezunits"))
("remove_fundamental_units" . ("maxima.info-7" 48403 1684 "Funciones y variables para ezunits"))
("remove_vertex" . ("maxima.info-7" 119154 83 "Funciones y variables para graphs"))
("rempart" . ("maxima.info-8" 19658 258 "Paquete functs"))
("remrule" . ("maxima.info-4" 316387 2176 "Funciones y variables sobre reglas y patrones"))
("remrule <1>" . ("maxima.info-4" 316388 2175 "Funciones y variables sobre reglas y patrones"))
("remsym" . ("maxima.info-4" 27298 162 "Funciones y variables para itensor"))
("remvalue" . ("maxima.info-4" 286344 494 "Funciones y variables para la miscelánea de opciones"))
("remvalue <1>" . ("maxima.info-4" 286345 493 "Funciones y variables para la miscelánea de opciones"))
("rename" . ("maxima.info-4" 3281 3294 "Funciones y variables para itensor"))
("rename <1>" . ("maxima.info-4" 3282 3293 "Funciones y variables para itensor"))
("reset" . ("maxima.info-1" 70153 514 "Funciones y variables para la línea de comandos"))
("residue" . ("maxima.info-3" 115404 670 "Funciones y variables para integración"))
("resolution" . ("maxima.info-7" 12485 232 "Visualización usando VTK"))
("resolvante" . ("maxima.info-4" 256037 6968 "Funciones y variables para simetrías"))
("resolvante_alternee1" . ("maxima.info-4" 263006 372 "Funciones y variables para simetrías"))
("resolvante_bipartite" . ("maxima.info-4" 263379 561 "Funciones y variables para simetrías"))
("resolvante_diedrale" . ("maxima.info-4" 263941 689 "Funciones y variables para simetrías"))
("resolvante_klein" . ("maxima.info-4" 264631 324 "Funciones y variables para simetrías"))
("resolvante_klein3" . ("maxima.info-4" 264956 324 "Funciones y variables para simetrías"))
("resolvante_produit_sym" . ("maxima.info-4" 265281 1078 "Funciones y variables para simetrías"))
("resolvante_unitaire" . ("maxima.info-4" 266360 321 "Funciones y variables para simetrías"))
("resolvante_vierer" . ("maxima.info-4" 266682 324 "Funciones y variables para simetrías"))
("rest" . ("maxima.info-1" 137467 335 "Funciones y variables para listas"))
("rest <1>" . ("maxima.info-1" 137468 334 "Funciones y variables para listas"))
("restart" . ("maxima.info-7" 4008 268 "Visualización usando VTK"))
("resultant" . ("maxima.info-2" 276189 699 "Funciones y variables para polinomios"))
("resultant <1>" . ("maxima.info-2" 274716 1472 "Funciones y variables para polinomios"))
("return" . ("maxima.info-5" 163075 142 "Funciones y variables para la programación"))
("reveal" . ("maxima.info-1" 299246 2590 "Funciones y variables para expresiones"))
("reverse" . ("maxima.info-1" 137803 226 "Funciones y variables para listas"))
("revert" . ("maxima.info-4" 169814 1975 "Funciones y variables para las series"))
("revert2" . ("maxima.info-4" 169815 1974 "Funciones y variables para las series"))
("rgb2level" . ("maxima.info-6" 256452 129 "Funciones y variables para picture"))
("rhs" . ("maxima.info-3" 175727 1632 "Funciones y variable para las ecuaciones"))
("ric" . ("maxima.info-4" 128758 79 "Funciones y variables para ctensor"))
("ricci" . ("maxima.info-4" 85782 269 "Funciones y variables para ctensor"))
("riem" . ("maxima.info-4" 128249 298 "Funciones y variables para ctensor"))
("riemann" . ("maxima.info-4" 87785 1312 "Funciones y variables para ctensor"))
("rinvariant" . ("maxima.info-4" 89833 260 "Funciones y variables para ctensor"))
("risch" . ("maxima.info-3" 116075 1084 "Funciones y variables para integración"))
("rk" . ("maxima.info-3" 228006 2644 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("rk <1>" . ("maxima.info-3" 228007 2643 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("rmxchar" . ("maxima.info-3" 278141 183 "Funciones y variables para las matrices y el álgebra lineal"))
("rncombine" . ("maxima.info-4" 286839 703 "Funciones y variables para la miscelánea de opciones"))
("romberg" . ("maxima.info-8" 392 5180 "Funciones y variables para romberg"))
("romberg <1>" . ("maxima.info-8" 393 5179 "Funciones y variables para romberg"))
("rombergabs" . ("maxima.info-8" 5573 590 "Funciones y variables para romberg"))
("rombergit" . ("maxima.info-8" 6164 540 "Funciones y variables para romberg"))
("rombergmin" . ("maxima.info-8" 6705 308 "Funciones y variables para romberg"))
("rombergtol" . ("maxima.info-8" 7014 593 "Funciones y variables para romberg"))
("room" . ("maxima.info-4" 276151 398 "Funciones y variables para el entorno de ejecución"))
("room <1>" . ("maxima.info-4" 276152 397 "Funciones y variables para el entorno de ejecución"))
("room <2>" . ("maxima.info-4" 276173 376 "Funciones y variables para el entorno de ejecución"))
("rootsconmode" . ("maxima.info-3" 177360 214 "Funciones y variable para las ecuaciones"))
("rootscontract" . ("maxima.info-3" 177575 2720 "Funciones y variable para las ecuaciones"))
("rootsepsilon" . ("maxima.info-3" 180296 222 "Funciones y variable para las ecuaciones"))
("round" . ("maxima.info-2" 44008 233 "Funciones para los números"))
("row" . ("maxima.info-3" 278009 131 "Funciones y variables para las matrices y el álgebra lineal"))
("rowop" . ("maxima.info-7" 228085 258 "Funciones y variables para linearalgebra"))
("rowswap" . ("maxima.info-7" 228694 164 "Funciones y variables para linearalgebra"))
("rreduce" . ("maxima.info-5" 42382 1526 "Funciones y variables para los conjuntos"))
("rreduce <1>" . ("maxima.info-5" 42383 1525 "Funciones y variables para los conjuntos"))
("run_testsuite" . ("maxima.info-1" 23894 1994 "Funciones y variables para la detección e informe de fallos"))
("run_viewer" . ("maxima.info-2" 166508 174 "Opciones gráficas"))
("save" . ("maxima.info-2" 194384 3037 "Funciones y variables para lectura y escritura"))
("save <1>" . ("maxima.info-2" 194385 3036 "Funciones y variables para lectura y escritura"))
("save <2>" . ("maxima.info-2" 194451 2970 "Funciones y variables para lectura y escritura"))
("save <3>" . ("maxima.info-2" 194514 2907 "Funciones y variables para lectura y escritura"))
("save <4>" . ("maxima.info-2" 194557 2864 "Funciones y variables para lectura y escritura"))
("save <5>" . ("maxima.info-2" 194612 2809 "Funciones y variables para lectura y escritura"))
("savedef" . ("maxima.info-5" 122054 400 "Funciones y variables para la definición de funciones"))
("savefactors" . ("maxima.info-2" 276889 294 "Funciones y variables para polinomios"))
("saving" . ("maxima.info-7" 65877 1422 "Funciones y Variables para finance"))
("scalar" . ("maxima.info-2" 111564 150 "Funciones y variables para las propiedades"))
("scalarmatrixp" . ("maxima.info-3" 278325 460 "Funciones y variables para las matrices y el álgebra lineal"))
("scalarp" . ("maxima.info-2" 111715 263 "Funciones y variables para las propiedades"))
("scale" . ("maxima.info-7" 12718 500 "Visualización usando VTK"))
("scaled_bessel_i" . ("maxima.info-2" 288817 472 "Funciones de Bessel"))
("scaled_bessel_i0" . ("maxima.info-2" 289290 77 "Funciones de Bessel"))
("scaled_bessel_i1" . ("maxima.info-2" 289368 77 "Funciones de Bessel"))
("scanmap" . ("maxima.info-5" 163218 1669 "Funciones y variables para la programación"))
("scanmap <1>" . ("maxima.info-5" 163219 1668 "Funciones y variables para la programación"))
("scatterplot" . ("maxima.info-6" 15866 2230 "Funciones y variables para gráficos estadísticos"))
("scatterplot <1>" . ("maxima.info-6" 15867 2229 "Funciones y variables para gráficos estadísticos"))
("scatterplot <2>" . ("maxima.info-6" 15901 2195 "Funciones y variables para gráficos estadísticos"))
("scatterplot <3>" . ("maxima.info-6" 15964 2132 "Funciones y variables para gráficos estadísticos"))
("scatterplot_description" . ("maxima.info-6" 16000 2096 "Funciones y variables para gráficos estadísticos"))
("scene" . ("maxima.info-7" 563 2296 "Visualización usando VTK"))
("schur2comp" . ("maxima.info-4" 267007 710 "Funciones y variables para simetrías"))
("sconcat" . ("maxima.info-1" 114488 330 "Funciones y variables para las cadenas de texto"))
("scopy" . ("maxima.info-8" 113187 83 "Funciones y variables para cadenas"))
("scsimp" . ("maxima.info-2" 29496 499 "Funciones y variables para simplificación"))
("scurvature" . ("maxima.info-4" 86576 165 "Funciones y variables para ctensor"))
("sdowncase" . ("maxima.info-8" 113271 210 "Funciones y variables para cadenas"))
("sdowncase <1>" . ("maxima.info-8" 113272 209 "Funciones y variables para cadenas"))
("sdowncase <2>" . ("maxima.info-8" 113306 175 "Funciones y variables para cadenas"))
("sec" . ("maxima.info-2" 76322 37 "Funciones y variables para trigonometría"))
("sech" . ("maxima.info-2" 76360 50 "Funciones y variables para trigonometría"))
("second" . ("maxima.info-1" 138030 135 "Funciones y variables para listas"))
("sequal" . ("maxima.info-8" 113482 137 "Funciones y variables para cadenas"))
("sequalignore" . ("maxima.info-8" 113620 131 "Funciones y variables para cadenas"))
("set_draw_defaults" . ("maxima.info-6" 127417 878 "Funciones y variables para draw"))
("set_edge_weight" . ("maxima.info-7" 110378 494 "Funciones y variables para graphs"))
("set_partitions" . ("maxima.info-5" 46621 1715 "Funciones y variables para los conjuntos"))
("set_partitions <1>" . ("maxima.info-5" 46622 1714 "Funciones y variables para los conjuntos"))
("set_plot_option" . ("maxima.info-2" 156090 1396 "Funciones y variables para gráficos"))
("set_random_state" . ("maxima.info-2" 85054 170 "Números aleatorios"))
("set_tex_environment" . ("maxima.info-2" 210153 1089 "Funciones y variables para salida TeX"))
("set_tex_environment_default" . ("maxima.info-2" 211244 1204 "Funciones y variables para salida TeX"))
("set_up_dot_simplifications" . ("maxima.info-3" 288603 571 "Funciones y variables para Afines"))
("set_up_dot_simplifications <1>" . ("maxima.info-3" 288604 570 "Funciones y variables para Afines"))
("set_vertex_label" . ("maxima.info-7" 110873 502 "Funciones y variables para graphs"))
("setcheck" . ("maxima.info-5" 179960 1386 "Funciones y variables para depurado"))
("setcheckbreak" . ("maxima.info-5" 181347 505 "Funciones y variables para depurado"))
("setdifference" . ("maxima.info-5" 43909 890 "Funciones y variables para los conjuntos"))
("setelmx" . ("maxima.info-3" 278786 246 "Funciones y variables para las matrices y el álgebra lineal"))
("setequalp" . ("maxima.info-5" 44800 653 "Funciones y variables para los conjuntos"))
("setify" . ("maxima.info-5" 45454 610 "Funciones y variables para los conjuntos"))
("setp" . ("maxima.info-5" 46065 555 "Funciones y variables para los conjuntos"))
("setunits" . ("maxima.info-8" 172387 5026 "Funciones y variables para units"))
("setup_autoload" . ("maxima.info-4" 287543 1496 "Funciones y variables para la miscelánea de opciones"))
("setval" . ("maxima.info-5" 181853 275 "Funciones y variables para depurado"))
("seventh" . ("maxima.info-1" 138166 136 "Funciones y variables para listas"))
("sexplode" . ("maxima.info-8" 113752 101 "Funciones y variables para cadenas"))
("sf" . ("maxima.info-4" 141042 291 "Funciones y variables para atensor"))
("shortest_path" . ("maxima.info-7" 111376 375 "Funciones y variables para graphs"))
("shortest_weighted_path" . ("maxima.info-7" 111752 647 "Funciones y variables para graphs"))
("show" . ("maxima.info-4" 6576 271 "Funciones y variables para itensor"))
("show_edge_color" . ("maxima.info-7" 126588 105 "Funciones y variables para graphs"))
("show_edge_type" . ("maxima.info-7" 126781 157 "Funciones y variables para graphs"))
("show_edge_width" . ("maxima.info-7" 126694 86 "Funciones y variables para graphs"))
("show_edges" . ("maxima.info-7" 126481 106 "Funciones y variables para graphs"))
("show_id" . ("maxima.info-7" 124262 168 "Funciones y variables para graphs"))
("show_label" . ("maxima.info-7" 124431 160 "Funciones y variables para graphs"))
("show_vertex_color" . ("maxima.info-7" 125699 111 "Funciones y variables para graphs"))
("show_vertex_size" . ("maxima.info-7" 125606 92 "Funciones y variables para graphs"))
("show_vertex_type" . ("maxima.info-7" 125441 164 "Funciones y variables para graphs"))
("show_vertices" . ("maxima.info-7" 125292 148 "Funciones y variables para graphs"))
("show_weight" . ("maxima.info-7" 124811 158 "Funciones y variables para graphs"))
("showcomps" . ("maxima.info-4" 13756 2410 "Funciones y variables para itensor"))
("showratvars" . ("maxima.info-2" 277184 177 "Funciones y variables para polinomios"))
("showtime" . ("maxima.info-1" 70668 415 "Funciones y variables para la línea de comandos"))
("sierpinskiale" . ("maxima.info-7" 70455 427 "Definiciones para IFS fractals"))
("sierpinskimap" . ("maxima.info-7" 75424 270 "Definiciones para curvas de Peano"))
("sign" . ("maxima.info-2" 128154 403 "Funciones y variables para los hechos"))
("signum" . ("maxima.info-2" 44242 720 "Funciones para los números"))
("similaritytransform" . ("maxima.info-3" 279033 1232 "Funciones y variables para las matrices y el álgebra lineal"))
("simp" . ("maxima.info-2" 29996 1568 "Funciones y variables para simplificación"))
("simp_inequality" . ("maxima.info-8" 149366 678 "Funciones y variables para to_poly_solve"))
("simplified_output" . ("maxima.info-8" 198607 209 "Funciones y variables para zeilberger"))
("simplify_products" . ("maxima.info-8" 37609 215 "Funciones y variables para solve_rec"))
("simplify_sum" . ("maxima.info-8" 37825 876 "Funciones y variables para solve_rec"))
("simplode" . ("maxima.info-8" 113854 704 "Funciones y variables para cadenas"))
("simplode <1>" . ("maxima.info-8" 113855 703 "Funciones y variables para cadenas"))
("simpmetderiv" . ("maxima.info-4" 36878 3004 "Funciones y variables para itensor"))
("simpmetderiv <1>" . ("maxima.info-4" 36879 3003 "Funciones y variables para itensor"))
("simpsum" . ("maxima.info-4" 148568 398 "Funciones y variables para sumas y productos"))
("simtran" . ("maxima.info-3" 279034 1231 "Funciones y variables para las matrices y el álgebra lineal"))
("sin" . ("maxima.info-2" 76411 34 "Funciones y variables para trigonometría"))
("sinh" . ("maxima.info-2" 76446 47 "Funciones y variables para trigonometría"))
("sinnpiflag" . ("maxima.info-4" 187445 89 "Funciones y variables para series de Fourier"))
("sinsert" . ("maxima.info-8" 114559 503 "Funciones y variables para cadenas"))
("sinvertcase" . ("maxima.info-8" 115063 556 "Funciones y variables para cadenas"))
("sinvertcase <1>" . ("maxima.info-8" 115064 555 "Funciones y variables para cadenas"))
("sinvertcase <2>" . ("maxima.info-8" 115100 519 "Funciones y variables para cadenas"))
("sixth" . ("maxima.info-1" 138303 132 "Funciones y variables para listas"))
("skewness" . ("maxima.info-5" 257514 870 "Funciones y variables de parámetros descriptivos"))
("skewness <1>" . ("maxima.info-5" 257515 869 "Funciones y variables de parámetros descriptivos"))
("skewness_bernoulli" . ("maxima.info-6" 108286 462 "Funciones y variables para distribuciones discretas"))
("skewness_beta" . ("maxima.info-6" 76402 205 "Funciones y variables para distribuciones continuas"))
("skewness_binomial" . ("maxima.info-6" 102892 243 "Funciones y variables para distribuciones discretas"))
("skewness_chi2" . ("maxima.info-6" 59376 463 "Funciones y variables para distribuciones continuas"))
("skewness_continuous_uniform" . ("maxima.info-6" 78827 230 "Funciones y variables para distribuciones continuas"))
("skewness_discrete_uniform" . ("maxima.info-6" 113447 236 "Funciones y variables para distribuciones discretas"))
("skewness_exp" . ("maxima.info-6" 68458 314 "Funciones y variables para distribuciones continuas"))
("skewness_f" . ("maxima.info-6" 64821 202 "Funciones y variables para distribuciones continuas"))
("skewness_gamma" . ("maxima.info-6" 73710 207 "Funciones y variables para distribuciones continuas"))
("skewness_general_finite_discrete" . ("maxima.info-6" 99936 243 "Funciones y variables para distribuciones discretas"))
("skewness_geometric" . ("maxima.info-6" 111047 217 "Funciones y variables para distribuciones discretas"))
("skewness_gumbel" . ("maxima.info-6" 95848 526 "Funciones y variables para distribuciones continuas"))
("skewness_hypergeometric" . ("maxima.info-6" 116535 281 "Funciones y variables para distribuciones discretas"))
("skewness_laplace" . ("maxima.info-6" 92330 212 "Funciones y variables para distribuciones continuas"))
("skewness_logistic" . ("maxima.info-6" 81152 212 "Funciones y variables para distribuciones continuas"))
("skewness_lognormal" . ("maxima.info-6" 71241 213 "Funciones y variables para distribuciones continuas"))
("skewness_negative_binomial" . ("maxima.info-6" 119233 253 "Funciones y variables para distribuciones discretas"))
("skewness_noncentral_chi2" . ("maxima.info-6" 62218 216 "Funciones y variables para distribuciones continuas"))
("skewness_noncentral_student_t" . ("maxima.info-6" 55471 303 "Funciones y variables para distribuciones continuas"))
("skewness_normal" . ("maxima.info-6" 47968 233 "Funciones y variables para distribuciones continuas"))
("skewness_pareto" . ("maxima.info-6" 83362 214 "Funciones y variables para distribuciones continuas"))
("skewness_poisson" . ("maxima.info-6" 105409 206 "Funciones y variables para distribuciones discretas"))
("skewness_rayleigh" . ("maxima.info-6" 89255 662 "Funciones y variables para distribuciones continuas"))
("skewness_student_t" . ("maxima.info-6" 50829 230 "Funciones y variables para distribuciones continuas"))
("skewness_weibull" . ("maxima.info-6" 85581 214 "Funciones y variables para distribuciones continuas"))
("slength" . ("maxima.info-8" 115620 83 "Funciones y variables para cadenas"))
("smake" . ("maxima.info-8" 115704 203 "Funciones y variables para cadenas"))
("small_rhombicosidodecahedron_graph" . ("maxima.info-7" 87607 105 "Funciones y variables para graphs"))
("small_rhombicuboctahedron_graph" . ("maxima.info-7" 87713 104 "Funciones y variables para graphs"))
("smax" . ("maxima.info-5" 249638 618 "Funciones y variables de parámetros descriptivos"))
("smax <1>" . ("maxima.info-5" 249639 617 "Funciones y variables de parámetros descriptivos"))
("smin" . ("maxima.info-5" 249022 615 "Funciones y variables de parámetros descriptivos"))
("smin <1>" . ("maxima.info-5" 249023 614 "Funciones y variables de parámetros descriptivos"))
("smismatch" . ("maxima.info-8" 115908 546 "Funciones y variables para cadenas"))
("smismatch <1>" . ("maxima.info-8" 115909 545 "Funciones y variables para cadenas"))
("snowmap" . ("maxima.info-7" 74118 750 "Definiciones para cops de Koch"))
("snub_cube_graph" . ("maxima.info-7" 87818 72 "Funciones y variables para graphs"))
("snub_dodecahedron_graph" . ("maxima.info-7" 87891 86 "Funciones y variables para graphs"))
("solve" . ("maxima.info-3" 180519 7479 "Funciones y variable para las ecuaciones"))
("solve <1>" . ("maxima.info-3" 180520 7478 "Funciones y variable para las ecuaciones"))
("solve <2>" . ("maxima.info-3" 180553 7445 "Funciones y variable para las ecuaciones"))
("solve_rec" . ("maxima.info-8" 38702 3072 "Funciones y variables para solve_rec"))
("solve_rec_rat" . ("maxima.info-8" 41775 893 "Funciones y variables para solve_rec"))
("solvedecomposes" . ("maxima.info-3" 187999 203 "Funciones y variable para las ecuaciones"))
("solveexplicit" . ("maxima.info-3" 188203 242 "Funciones y variable para las ecuaciones"))
("solvefactors" . ("maxima.info-3" 188446 256 "Funciones y variable para las ecuaciones"))
("solvenullwarn" . ("maxima.info-3" 188703 323 "Funciones y variable para las ecuaciones"))
("solveradcan" . ("maxima.info-3" 189027 292 "Funciones y variable para las ecuaciones"))
("solvetrigwarn" . ("maxima.info-3" 189320 310 "Funciones y variable para las ecuaciones"))
("some" . ("maxima.info-5" 48337 3129 "Funciones y variables para los conjuntos"))
("some <1>" . ("maxima.info-5" 48338 3128 "Funciones y variables para los conjuntos"))
("somrac" . ("maxima.info-4" 267718 260 "Funciones y variables para simetrías"))
("sort" . ("maxima.info-1" 138436 6208 "Funciones y variables para listas"))
("sort <1>" . ("maxima.info-1" 138437 6207 "Funciones y variables para listas"))
("space" . ("maxima.info-8" 109811 49 "Funciones y variables para caracteres"))
("sparse" . ("maxima.info-3" 280266 219 "Funciones y variables para las matrices y el álgebra lineal"))
("sparse6_decode" . ("maxima.info-7" 120599 109 "Funciones y variables para graphs"))
("sparse6_encode" . ("maxima.info-7" 120709 106 "Funciones y variables para graphs"))
("sparse6_export" . ("maxima.info-7" 120816 133 "Funciones y variables para graphs"))
("sparse6_import" . ("maxima.info-7" 120950 120 "Funciones y variables para graphs"))
("specint" . ("maxima.info-3" 41501 4226 "Funciones y variables para las funciones especiales"))
("sphere" . ("maxima.info-7" 7218 120 "Visualización usando VTK"))
("spherical" . ("maxima.info-6" 249163 742 "Funciones y variables para draw"))
("spherical_bessel_j" . ("maxima.info-7" 304182 192 "Funciones y variables para polinomios ortogonales"))
("spherical_bessel_y" . ("maxima.info-7" 304375 192 "Funciones y variables para polinomios ortogonales"))
("spherical_hankel1" . ("maxima.info-7" 304568 163 "Funciones y variables para polinomios ortogonales"))
("spherical_hankel2" . ("maxima.info-7" 304732 163 "Funciones y variables para polinomios ortogonales"))
("spherical_harmonic" . ("maxima.info-7" 304896 120 "Funciones y variables para polinomios ortogonales"))
("spherical_to_xyz" . ("maxima.info-2" 157487 258 "Funciones y variables para gráficos"))
("splice" . ("maxima.info-5" 75866 1659 "Macros"))
("split" . ("maxima.info-8" 116455 927 "Funciones y variables para cadenas"))
("split <1>" . ("maxima.info-8" 116456 926 "Funciones y variables para cadenas"))
("split <2>" . ("maxima.info-8" 116486 896 "Funciones y variables para cadenas"))
("sposition" . ("maxima.info-8" 117383 295 "Funciones y variables para cadenas"))
("spring_embedding_depth" . ("maxima.info-7" 127789 139 "Funciones y variables para graphs"))
("sprint" . ("maxima.info-8" 106223 658 "Funciones y variables para entrada y salida"))
("sqrt" . ("maxima.info-2" 67220 394 "Funciones radicales exponenciales y logarítmicas"))
("sqrtdenest" . ("maxima.info-8" 33573 1062 "Paquete sqdnst"))
("sqrtdispflag" . ("maxima.info-1" 94435 159 "Funciones y variables para la impresión"))
("sremove" . ("maxima.info-8" 117679 808 "Funciones y variables para cadenas"))
("sremove <1>" . ("maxima.info-8" 117680 807 "Funciones y variables para cadenas"))
("sremove <2>" . ("maxima.info-8" 117719 768 "Funciones y variables para cadenas"))
("sremove <3>" . ("maxima.info-8" 117766 721 "Funciones y variables para cadenas"))
("sremovefirst" . ("maxima.info-8" 118488 344 "Funciones y variables para cadenas"))
("sremovefirst <1>" . ("maxima.info-8" 118489 343 "Funciones y variables para cadenas"))
("sremovefirst <2>" . ("maxima.info-8" 118533 299 "Funciones y variables para cadenas"))
("sremovefirst <3>" . ("maxima.info-8" 118585 247 "Funciones y variables para cadenas"))
("sreverse" . ("maxima.info-8" 118833 118 "Funciones y variables para cadenas"))
("ssearch" . ("maxima.info-8" 118952 729 "Funciones y variables para cadenas"))
("ssearch <1>" . ("maxima.info-8" 118953 728 "Funciones y variables para cadenas"))
("ssearch <2>" . ("maxima.info-8" 118992 689 "Funciones y variables para cadenas"))
("ssearch <3>" . ("maxima.info-8" 119039 642 "Funciones y variables para cadenas"))
("ssort" . ("maxima.info-8" 119682 727 "Funciones y variables para cadenas"))
("ssort <1>" . ("maxima.info-8" 119683 726 "Funciones y variables para cadenas"))
("sstatus" . ("maxima.info-4" 276550 535 "Funciones y variables para el entorno de ejecución"))
("ssubst" . ("maxima.info-8" 120410 1046 "Funciones y variables para cadenas"))
("ssubst <1>" . ("maxima.info-8" 120411 1045 "Funciones y variables para cadenas"))
("ssubst <2>" . ("maxima.info-8" 120456 1000 "Funciones y variables para cadenas"))
("ssubst <3>" . ("maxima.info-8" 120509 947 "Funciones y variables para cadenas"))
("ssubstfirst" . ("maxima.info-8" 121457 382 "Funciones y variables para cadenas"))
("ssubstfirst <1>" . ("maxima.info-8" 121458 381 "Funciones y variables para cadenas"))
("ssubstfirst <2>" . ("maxima.info-8" 121508 331 "Funciones y variables para cadenas"))
("ssubstfirst <3>" . ("maxima.info-8" 121566 273 "Funciones y variables para cadenas"))
("staircase" . ("maxima.info-6" 287181 944 "Análisis gráfico de sistemas dinámicos discretos"))
("standardize" . ("maxima.info-5" 234463 360 "Funciones y variables para el tratamiento de datos"))
("standardize <1>" . ("maxima.info-5" 234464 359 "Funciones y variables para el tratamiento de datos"))
("standardize_inverse_trig" . ("maxima.info-8" 150045 330 "Funciones y variables para to_poly_solve"))
("stardisp" . ("maxima.info-1" 94595 173 "Funciones y variables para la impresión"))
("starplot" . ("maxima.info-6" 18097 2782 "Funciones y variables para gráficos estadísticos"))
("starplot_description" . ("maxima.info-6" 18168 2711 "Funciones y variables para gráficos estadísticos"))
("startphi" . ("maxima.info-7" 13219 462 "Visualización usando VTK"))
("starttheta" . ("maxima.info-7" 13682 465 "Visualización usando VTK"))
("stats_numer" . ("maxima.info-8" 51644 306 "Funciones y variables para stats"))
("status" . ("maxima.info-4" 277086 1083 "Funciones y variables para el entorno de ejecución"))
("status <1>" . ("maxima.info-4" 277087 1082 "Funciones y variables para el entorno de ejecución"))
("std" . ("maxima.info-5" 245336 593 "Funciones y variables de parámetros descriptivos"))
("std <1>" . ("maxima.info-5" 245337 592 "Funciones y variables de parámetros descriptivos"))
("std1" . ("maxima.info-5" 245930 601 "Funciones y variables de parámetros descriptivos"))
("std1 <1>" . ("maxima.info-5" 245931 600 "Funciones y variables de parámetros descriptivos"))
("std_bernoulli" . ("maxima.info-6" 107952 333 "Funciones y variables para distribuciones discretas"))
("std_beta" . ("maxima.info-6" 76208 193 "Funciones y variables para distribuciones continuas"))
("std_binomial" . ("maxima.info-6" 102660 231 "Funciones y variables para distribuciones discretas"))
("std_chi2" . ("maxima.info-6" 59083 292 "Funciones y variables para distribuciones continuas"))
("std_continuous_uniform" . ("maxima.info-6" 78608 218 "Funciones y variables para distribuciones continuas"))
("std_discrete_uniform" . ("maxima.info-6" 113222 224 "Funciones y variables para distribuciones discretas"))
("std_exp" . ("maxima.info-6" 68072 385 "Funciones y variables para distribuciones continuas"))
("std_f" . ("maxima.info-6" 64630 190 "Funciones y variables para distribuciones continuas"))
("std_gamma" . ("maxima.info-6" 73514 195 "Funciones y variables para distribuciones continuas"))
("std_general_finite_discrete" . ("maxima.info-6" 99704 231 "Funciones y variables para distribuciones discretas"))
("std_geometric" . ("maxima.info-6" 110841 205 "Funciones y variables para distribuciones discretas"))
("std_gumbel" . ("maxima.info-6" 95649 198 "Funciones y variables para distribuciones continuas"))
("std_hypergeometric" . ("maxima.info-6" 116265 269 "Funciones y variables para distribuciones discretas"))
("std_laplace" . ("maxima.info-6" 92129 200 "Funciones y variables para distribuciones continuas"))
("std_logistic" . ("maxima.info-6" 80951 200 "Funciones y variables para distribuciones continuas"))
("std_lognormal" . ("maxima.info-6" 71039 201 "Funciones y variables para distribuciones continuas"))
("std_negative_binomial" . ("maxima.info-6" 118991 241 "Funciones y variables para distribuciones discretas"))
("std_noncentral_chi2" . ("maxima.info-6" 62013 204 "Funciones y variables para distribuciones continuas"))
("std_noncentral_student_t" . ("maxima.info-6" 55179 291 "Funciones y variables para distribuciones continuas"))
("std_normal" . ("maxima.info-6" 47758 209 "Funciones y variables para distribuciones continuas"))
("std_pareto" . ("maxima.info-6" 83159 202 "Funciones y variables para distribuciones continuas"))
("std_poisson" . ("maxima.info-6" 105214 194 "Funciones y variables para distribuciones discretas"))
("std_rayleigh" . ("maxima.info-6" 88753 501 "Funciones y variables para distribuciones continuas"))
("std_student_t" . ("maxima.info-6" 50630 198 "Funciones y variables para distribuciones continuas"))
("std_weibull" . ("maxima.info-6" 85378 202 "Funciones y variables para distribuciones continuas"))
("stemplot" . ("maxima.info-6" 20880 997 "Funciones y variables para gráficos estadísticos"))
("stemplot <1>" . ("maxima.info-6" 20881 996 "Funciones y variables para gráficos estadísticos"))
("stirling" . ("maxima.info-8" 89218 2537 "Funciones y variables para stirling"))
("stirling <1>" . ("maxima.info-8" 89219 2536 "Funciones y variables para stirling"))
("stirling1" . ("maxima.info-5" 51467 2053 "Funciones y variables para los conjuntos"))
("stirling2" . ("maxima.info-5" 53521 2393 "Funciones y variables para los conjuntos"))
("strim" . ("maxima.info-8" 121840 328 "Funciones y variables para cadenas"))
("striml" . ("maxima.info-8" 122169 133 "Funciones y variables para cadenas"))
("strimr" . ("maxima.info-8" 122303 135 "Funciones y variables para cadenas"))
("string" . ("maxima.info-1" 114819 234 "Funciones y variables para las cadenas de texto"))
("stringdisp" . ("maxima.info-1" 115054 826 "Funciones y variables para las cadenas de texto"))
("stringout" . ("maxima.info-2" 197422 2026 "Funciones y variables para lectura y escritura"))
("stringout <1>" . ("maxima.info-2" 197423 2025 "Funciones y variables para lectura y escritura"))
("stringout <2>" . ("maxima.info-2" 197494 1954 "Funciones y variables para lectura y escritura"))
("stringout <3>" . ("maxima.info-2" 197542 1906 "Funciones y variables para lectura y escritura"))
("stringout <4>" . ("maxima.info-2" 197585 1863 "Funciones y variables para lectura y escritura"))
("stringp" . ("maxima.info-8" 122439 117 "Funciones y variables para cadenas"))
("strong_components" . ("maxima.info-7" 112400 381 "Funciones y variables para graphs"))
("structures" . ("maxima.info-1" 171485 139 "Funciones y variables para las estructuras"))
("struve_h" . ("maxima.info-3" 37877 102 "Funciones de Struve"))
("struve_l" . ("maxima.info-3" 37980 120 "Funciones de Struve"))
("style" . ("maxima.info-2" 166683 2138 "Opciones gráficas"))
("style <1>" . ("maxima.info-2" 166684 2137 "Opciones gráficas"))
("sublis" . ("maxima.info-1" 301837 481 "Funciones y variables para expresiones"))
("sublis_apply_lambda" . ("maxima.info-1" 302319 323 "Funciones y variables para expresiones"))
("sublist" . ("maxima.info-1" 144645 326 "Funciones y variables para listas"))
("sublist_indices" . ("maxima.info-1" 144972 1089 "Funciones y variables para listas"))
("submatrix" . ("maxima.info-3" 280486 323 "Funciones y variables para las matrices y el álgebra lineal"))
("submatrix <1>" . ("maxima.info-3" 280487 322 "Funciones y variables para las matrices y el álgebra lineal"))
("submatrix <2>" . ("maxima.info-3" 280554 255 "Funciones y variables para las matrices y el álgebra lineal"))
("subnumsimp" . ("maxima.info-1" 302643 600 "Funciones y variables para expresiones"))
("subsample" . ("maxima.info-5" 234824 6294 "Funciones y variables para el tratamiento de datos"))
("subsample <1>" . ("maxima.info-5" 234825 6293 "Funciones y variables para el tratamiento de datos"))
("subset" . ("maxima.info-5" 55915 699 "Funciones y variables para los conjuntos"))
("subsetp" . ("maxima.info-5" 56615 447 "Funciones y variables para los conjuntos"))
("subst" . ("maxima.info-1" 303244 2434 "Funciones y variables para expresiones"))
("subst_parallel" . ("maxima.info-8" 150376 2003 "Funciones y variables para to_poly_solve"))
("substinpart" . ("maxima.info-1" 305679 2420 "Funciones y variables para expresiones"))
("substpart" . ("maxima.info-1" 308100 1198 "Funciones y variables para expresiones"))
("substring" . ("maxima.info-8" 122557 589 "Funciones y variables para cadenas"))
("substring <1>" . ("maxima.info-8" 122558 588 "Funciones y variables para cadenas"))
("subvar" . ("maxima.info-1" 168291 733 "Funciones y variables para los arrays"))
("subvarp" . ("maxima.info-1" 169025 103 "Funciones y variables para los arrays"))
("sum" . ("maxima.info-4" 148967 4687 "Funciones y variables para sumas y productos"))
("sumcontract" . ("maxima.info-4" 153655 1374 "Funciones y variables para sumas y productos"))
("sumexpand" . ("maxima.info-4" 155030 983 "Funciones y variables para sumas y productos"))
("summand_to_rec" . ("maxima.info-8" 42951 1981 "Funciones y variables para solve_rec"))
("sumsplitfact" . ("maxima.info-2" 57393 160 "Funciones combinatorias"))
("supcase" . ("maxima.info-8" 123147 449 "Funciones y variables para cadenas"))
("supcase <1>" . ("maxima.info-8" 123148 448 "Funciones y variables para cadenas"))
("supcase <2>" . ("maxima.info-8" 123180 416 "Funciones y variables para cadenas"))
("supcontext" . ("maxima.info-2" 128558 277 "Funciones y variables para los hechos"))
("supcontext <1>" . ("maxima.info-2" 128559 276 "Funciones y variables para los hechos"))
("surface" . ("maxima.info-7" 14148 283 "Visualización usando VTK"))
("surface_hide" . ("maxima.info-6" 185879 577 "Funciones y variables para draw"))
("symbolp" . ("maxima.info-1" 309299 226 "Funciones y variables para expresiones"))
("symmdifference" . ("maxima.info-5" 57063 1093 "Funciones y variables para los conjuntos"))
("symmetric" . ("maxima.info-2" 31565 264 "Funciones y variables para simplificación"))
("symmetricp" . ("maxima.info-4" 110190 116 "Funciones y variables para ctensor"))
("system" . ("maxima.info-4" 278170 865 "Funciones y variables para el entorno de ejecución"))
("tab" . ("maxima.info-8" 109861 50 "Funciones y variables para caracteres"))
("take_channel" . ("maxima.info-6" 256582 955 "Funciones y variables para picture"))
("take_inference" . ("maxima.info-8" 49473 1910 "Funciones y variables para inference_result"))
("take_inference <1>" . ("maxima.info-8" 49474 1909 "Funciones y variables para inference_result"))
("take_inference <2>" . ("maxima.info-8" 49515 1868 "Funciones y variables para inference_result"))
("tan" . ("maxima.info-2" 76494 38 "Funciones y variables para trigonometría"))
("tanh" . ("maxima.info-2" 76533 51 "Funciones y variables para trigonometría"))
("taylor" . ("maxima.info-4" 171790 8179 "Funciones y variables para las series"))
("taylor <1>" . ("maxima.info-4" 171791 8178 "Funciones y variables para las series"))
("taylor <2>" . ("maxima.info-4" 171835 8134 "Funciones y variables para las series"))
("taylor <3>" . ("maxima.info-4" 171895 8074 "Funciones y variables para las series"))
("taylor <4>" . ("maxima.info-4" 172020 7949 "Funciones y variables para las series"))
("taylor_logexpand" . ("maxima.info-4" 181330 495 "Funciones y variables para las series"))
("taylor_order_coefficients" . ("maxima.info-4" 181826 332 "Funciones y variables para las series"))
("taylor_simplifier" . ("maxima.info-4" 182159 163 "Funciones y variables para las series"))
("taylor_truncate_polynomials" . ("maxima.info-4" 182323 362 "Funciones y variables para las series"))
("taylordepth" . ("maxima.info-4" 179970 291 "Funciones y variables para las series"))
("taylorinfo" . ("maxima.info-4" 180262 944 "Funciones y variables para las series"))
("taylorp" . ("maxima.info-4" 181207 122 "Funciones y variables para las series"))
("taytorat" . ("maxima.info-4" 182686 216 "Funciones y variables para las series"))
("tcl_output" . ("maxima.info-4" 289040 1247 "Funciones y variables para la miscelánea de opciones"))
("tcl_output <1>" . ("maxima.info-4" 289041 1246 "Funciones y variables para la miscelánea de opciones"))
("tcl_output <2>" . ("maxima.info-4" 289088 1199 "Funciones y variables para la miscelánea de opciones"))
("tcontract" . ("maxima.info-4" 267979 386 "Funciones y variables para simetrías"))
("tellrat" . ("maxima.info-2" 277362 2296 "Funciones y variables para polinomios"))
("tellrat <1>" . ("maxima.info-2" 277363 2295 "Funciones y variables para polinomios"))
("tellsimp" . ("maxima.info-4" 318564 2555 "Funciones y variables sobre reglas y patrones"))
("tellsimpafter" . ("maxima.info-4" 321120 9923 "Funciones y variables sobre reglas y patrones"))
("tensorkill" . ("maxima.info-4" 130776 174 "Funciones y variables para ctensor"))
("tentex" . ("maxima.info-4" 67205 1273 "Funciones y variables para itensor"))
("tenth" . ("maxima.info-1" 146490 133 "Funciones y variables para listas"))
("terminal" . ("maxima.info-6" 186457 3272 "Funciones y variables para draw"))
("terminal <1>" . ("maxima.info-7" 127929 131 "Funciones y variables para graphs"))
("test_mean" . ("maxima.info-8" 51951 4556 "Funciones y variables para stats"))
("test_mean <1>" . ("maxima.info-8" 51952 4555 "Funciones y variables para stats"))
("test_means_difference" . ("maxima.info-8" 56508 5041 "Funciones y variables para stats"))
("test_means_difference <1>" . ("maxima.info-8" 56509 5040 "Funciones y variables para stats"))
("test_normality" . ("maxima.info-8" 82447 1124 "Funciones y variables para stats"))
("test_proportion" . ("maxima.info-8" 67025 3564 "Funciones y variables para stats"))
("test_proportion <1>" . ("maxima.info-8" 67026 3563 "Funciones y variables para stats"))
("test_proportions_difference" . ("maxima.info-8" 70590 3801 "Funciones y variables para stats"))
("test_proportions_difference <1>" . ("maxima.info-8" 70591 3800 "Funciones y variables para stats"))
("test_rank_sum" . ("maxima.info-8" 79635 2811 "Funciones y variables para stats"))
("test_rank_sum <1>" . ("maxima.info-8" 79636 2810 "Funciones y variables para stats"))
("test_sign" . ("maxima.info-8" 74392 2028 "Funciones y variables para stats"))
("test_sign <1>" . ("maxima.info-8" 74393 2027 "Funciones y variables para stats"))
("test_signed_rank" . ("maxima.info-8" 76421 3213 "Funciones y variables para stats"))
("test_signed_rank <1>" . ("maxima.info-8" 76422 3212 "Funciones y variables para stats"))
("test_variance" . ("maxima.info-8" 61550 2617 "Funciones y variables para stats"))
("test_variance <1>" . ("maxima.info-8" 61551 2616 "Funciones y variables para stats"))
("test_variance_ratio" . ("maxima.info-8" 64168 2856 "Funciones y variables para stats"))
("test_variance_ratio <1>" . ("maxima.info-8" 64169 2855 "Funciones y variables para stats"))
("testsuite_files" . ("maxima.info-1" 25889 749 "Funciones y variables para la detección e informe de fallos"))
("tex" . ("maxima.info-2" 202243 2539 "Funciones y variables para salida TeX"))
("tex <1>" . ("maxima.info-2" 202244 2538 "Funciones y variables para salida TeX"))
("tex <2>" . ("maxima.info-2" 202270 2512 "Funciones y variables para salida TeX"))
("tex <3>" . ("maxima.info-2" 202311 2471 "Funciones y variables para salida TeX"))
("tex <4>" . ("maxima.info-2" 202344 2438 "Funciones y variables para salida TeX"))
("tex <5>" . ("maxima.info-2" 202371 2411 "Funciones y variables para salida TeX"))
("tex1" . ("maxima.info-2" 204783 304 "Funciones y variables para salida TeX"))
("texput" . ("maxima.info-2" 205088 5063 "Funciones y variables para salida TeX"))
("texput <1>" . ("maxima.info-2" 205089 5062 "Funciones y variables para salida TeX"))
("texput <2>" . ("maxima.info-2" 205120 5031 "Funciones y variables para salida TeX"))
("texput <3>" . ("maxima.info-2" 205151 5000 "Funciones y variables para salida TeX"))
("texput <4>" . ("maxima.info-2" 205199 4952 "Funciones y variables para salida TeX"))
("thetaresolution" . ("maxima.info-7" 14432 337 "Visualización usando VTK"))
("third" . ("maxima.info-1" 146624 135 "Funciones y variables para listas"))
("throw" . ("maxima.info-5" 164888 191 "Funciones y variables para la programación"))
("time" . ("maxima.info-4" 279036 575 "Funciones y variables para el entorno de ejecución"))
("timedate" . ("maxima.info-4" 279612 1279 "Funciones y variables para el entorno de ejecución"))
("timedate <1>" . ("maxima.info-4" 279613 1278 "Funciones y variables para el entorno de ejecución"))
("timer" . ("maxima.info-5" 182129 1377 "Funciones y variables para depurado"))
("timer <1>" . ("maxima.info-5" 182130 1376 "Funciones y variables para depurado"))
("timer <2>" . ("maxima.info-5" 182169 1337 "Funciones y variables para depurado"))
("timer_devalue" . ("maxima.info-5" 184182 623 "Funciones y variables para depurado"))
("timer_info" . ("maxima.info-5" 184806 976 "Funciones y variables para depurado"))
("timer_info <1>" . ("maxima.info-5" 184807 975 "Funciones y variables para depurado"))
("title" . ("maxima.info-6" 189730 460 "Funciones y variables para draw"))
("tldefint" . ("maxima.info-3" 117160 107 "Funciones y variables para integración"))
("tlimit" . ("maxima.info-3" 60776 251 "Funciones y variables para límites"))
("tlimit <1>" . ("maxima.info-3" 60777 250 "Funciones y variables para límites"))
("tlimit <2>" . ("maxima.info-3" 60825 202 "Funciones y variables para límites"))
("tlimswitch" . ("maxima.info-3" 61028 475 "Funciones y variables para límites"))
("to_lisp" . ("maxima.info-1" 71084 1130 "Funciones y variables para la línea de comandos"))
("to_poly" . ("maxima.info-8" 152380 2469 "Funciones y variables para to_poly_solve"))
("to_poly_solve" . ("maxima.info-8" 154850 13841 "Funciones y variables para to_poly_solve"))
("todd_coxeter" . ("maxima.info-4" 269877 1805 "Funciones y variables para grupos"))
("todd_coxeter <1>" . ("maxima.info-4" 269878 1804 "Funciones y variables para grupos"))
("toeplitz" . ("maxima.info-7" 228859 841 "Funciones y variables para linearalgebra"))
("toeplitz <1>" . ("maxima.info-7" 228860 840 "Funciones y variables para linearalgebra"))
("tokens" . ("maxima.info-8" 123597 747 "Funciones y variables para cadenas"))
("tokens <1>" . ("maxima.info-8" 123598 746 "Funciones y variables para cadenas"))
("topological_sort" . ("maxima.info-7" 112782 546 "Funciones y variables para graphs"))
("totaldisrep" . ("maxima.info-2" 279659 409 "Funciones y variables para polinomios"))
("totalfourier" . ("maxima.info-4" 188188 128 "Funciones y variables para series de Fourier"))
("totient" . ("maxima.info-4" 212907 125 "Funciones y variables para teoría de números"))
("tpartpol" . ("maxima.info-4" 268366 388 "Funciones y variables para simetrías"))
("tr" . ("maxima.info-4" 130272 141 "Funciones y variables para ctensor"))
("tr_array_as_ref" . ("maxima.info-5" 128294 358 "Funciones y variables para la definición de funciones"))
("tr_bound_function_applyp" . ("maxima.info-5" 128653 448 "Funciones y variables para la definición de funciones"))
("tr_file_tty_messagesp" . ("maxima.info-5" 129102 385 "Funciones y variables para la definición de funciones"))
("tr_float_can_branch_complex" . ("maxima.info-5" 129488 216 "Funciones y variables para la definición de funciones"))
("tr_function_call_default" . ("maxima.info-5" 129705 838 "Funciones y variables para la definición de funciones"))
("tr_numer" . ("maxima.info-5" 130544 189 "Funciones y variables para la definición de funciones"))
("tr_optimize_max_loop" . ("maxima.info-5" 130734 251 "Funciones y variables para la definición de funciones"))
("tr_semicompile" . ("maxima.info-5" 130986 250 "Funciones y variables para la definición de funciones"))
("tr_state_vars" . ("maxima.info-5" 131237 638 "Funciones y variables para la definición de funciones"))
("tr_warn_bad_function_calls" . ("maxima.info-5" 131973 251 "Funciones y variables para la definición de funciones"))
("tr_warn_fexpr" . ("maxima.info-5" 132225 190 "Funciones y variables para la definición de funciones"))
("tr_warn_meval" . ("maxima.info-5" 132416 216 "Funciones y variables para la definición de funciones"))
("tr_warn_mode" . ("maxima.info-5" 132633 168 "Funciones y variables para la definición de funciones"))
("tr_warn_undeclared" . ("maxima.info-5" 132802 146 "Funciones y variables para la definición de funciones"))
("tr_warn_undefined_variable" . ("maxima.info-5" 132949 161 "Funciones y variables para la definición de funciones"))
("tr_warnings_get" . ("maxima.info-5" 131876 96 "Funciones y variables para la definición de funciones"))
("trace" . ("maxima.info-5" 185783 1192 "Funciones y variables para depurado"))
("trace <1>" . ("maxima.info-5" 185784 1191 "Funciones y variables para depurado"))
("trace <2>" . ("maxima.info-5" 185823 1152 "Funciones y variables para depurado"))
("trace_options" . ("maxima.info-5" 186976 2618 "Funciones y variables para depurado"))
("trace_options <1>" . ("maxima.info-5" 186977 2617 "Funciones y variables para depurado"))
("tracematrix" . ("maxima.info-8" 20409 178 "Paquete functs"))
("track" . ("maxima.info-7" 14770 1074 "Visualización usando VTK"))
("transcompile" . ("maxima.info-5" 122455 297 "Funciones y variables para la definición de funciones"))
("transform" . ("maxima.info-6" 190191 1518 "Funciones y variables para draw"))
("transform_sample" . ("maxima.info-5" 241119 1615 "Funciones y variables para el tratamiento de datos"))
("transform_xy" . ("maxima.info-2" 168822 359 "Opciones gráficas"))
("translate" . ("maxima.info-5" 122753 2573 "Funciones y variables para la definición de funciones"))
("translate <1>" . ("maxima.info-5" 122754 2572 "Funciones y variables para la definición de funciones"))
("translate <2>" . ("maxima.info-5" 122797 2529 "Funciones y variables para la definición de funciones"))
("translate_file" . ("maxima.info-5" 125327 2746 "Funciones y variables para la definición de funciones"))
("translate_file <1>" . ("maxima.info-5" 125328 2745 "Funciones y variables para la definición de funciones"))
("transparent" . ("maxima.info-6" 191710 542 "Funciones y variables para draw"))
("transpose" . ("maxima.info-3" 280810 396 "Funciones y variables para las matrices y el álgebra lineal"))
("transrun" . ("maxima.info-5" 128074 219 "Funciones y variables para la definición de funciones"))
("tree_reduce" . ("maxima.info-5" 58157 1387 "Funciones y variables para los conjuntos"))
("tree_reduce <1>" . ("maxima.info-5" 58158 1386 "Funciones y variables para los conjuntos"))
("treefale" . ("maxima.info-7" 70883 326 "Definiciones para IFS fractals"))
("treillis" . ("maxima.info-4" 268755 235 "Funciones y variables para simetrías"))
("treinat" . ("maxima.info-4" 268991 555 "Funciones y variables para simetrías"))
("triangle" . ("maxima.info-6" 249906 911 "Funciones y variables para draw"))
("triangularize" . ("maxima.info-3" 281207 1054 "Funciones y variables para las matrices y el álgebra lineal"))
("trigexpand" . ("maxima.info-2" 76585 1441 "Funciones y variables para trigonometría"))
("trigexpandplus" . ("maxima.info-2" 78027 347 "Funciones y variables para trigonometría"))
("trigexpandtimes" . ("maxima.info-2" 78375 356 "Funciones y variables para trigonometría"))
("triginverses" . ("maxima.info-2" 78732 516 "Funciones y variables para trigonometría"))
("trigrat" . ("maxima.info-2" 81084 2898 "Funciones y variables para trigonometría"))
("trigreduce" . ("maxima.info-2" 79249 1100 "Funciones y variables para trigonometría"))
("trigreduce <1>" . ("maxima.info-2" 79250 1099 "Funciones y variables para trigonometría"))
("trigsign" . ("maxima.info-2" 80350 278 "Funciones y variables para trigonometría"))
("trigsimp" . ("maxima.info-2" 80629 454 "Funciones y variables para trigonometría"))
("trivial_solutions" . ("maxima.info-8" 199522 174 "Funciones y variables para zeilberger"))
("true" . ("maxima.info-1" 119950 140 "Funciones y variables para Constantes"))
("trunc" . ("maxima.info-4" 182903 578 "Funciones y variables para las series"))
("truncated_cube_graph" . ("maxima.info-7" 87978 77 "Funciones y variables para graphs"))
("truncated_dodecahedron_graph" . ("maxima.info-7" 88056 91 "Funciones y variables para graphs"))
("truncated_icosahedron_graph" . ("maxima.info-7" 88148 89 "Funciones y variables para graphs"))
("truncated_tetrahedron_graph" . ("maxima.info-7" 88238 91 "Funciones y variables para graphs"))
("tstep" . ("maxima.info-7" 4277 229 "Visualización usando VTK"))
("ttyoff" . ("maxima.info-1" 94769 400 "Funciones y variables para la impresión"))
("tube" . ("maxima.info-6" 250818 786 "Funciones y variables para draw"))
("tutte_graph" . ("maxima.info-7" 88330 61 "Funciones y variables para graphs"))
("ueivects" . ("maxima.info-3" 282263 1252 "Funciones y variables para las matrices y el álgebra lineal"))
("ufg" . ("maxima.info-4" 128085 163 "Funciones y variables para ctensor"))
("uforget" . ("maxima.info-8" 177414 1448 "Funciones y variables para units"))
("ug" . ("maxima.info-4" 129100 91 "Funciones y variables para ctensor"))
("ultraspherical" . ("maxima.info-7" 305348 161 "Funciones y variables para polinomios ortogonales"))
("und" . ("maxima.info-1" 120091 199 "Funciones y variables para Constantes"))
("underlying_graph" . ("maxima.info-7" 88392 92 "Funciones y variables para graphs"))
("undiff" . ("maxima.info-4" 31353 200 "Funciones y variables para itensor"))
("union" . ("maxima.info-5" 59545 1043 "Funciones y variables para los conjuntos"))
("unique" . ("maxima.info-1" 146062 427 "Funciones y variables para listas"))
("unit_step" . ("maxima.info-7" 305017 330 "Funciones y variables para polinomios ortogonales"))
("unit_vectors" . ("maxima.info-6" 192253 728 "Funciones y variables para draw"))
("uniteigenvectors" . ("maxima.info-3" 282262 1253 "Funciones y variables para las matrices y el álgebra lineal"))
("unitp" . ("maxima.info-7" 42689 1256 "Funciones y variables para ezunits"))
("units" . ("maxima.info-7" 38551 2427 "Funciones y variables para ezunits"))
("unitvector" . ("maxima.info-3" 283516 339 "Funciones y variables para las matrices y el álgebra lineal"))
("unknown" . ("maxima.info-2" 135741 153 "Funciones y variables para los predicados"))
("unless" . ("maxima.info-5" 165081 76 "Funciones y variables para la programación"))
("unorder" . ("maxima.info-1" 309526 1436 "Funciones y variables para expresiones"))
("unsum" . ("maxima.info-4" 183482 1197 "Funciones y variables para las series"))
("untellrat" . ("maxima.info-2" 280069 128 "Funciones y variables para polinomios"))
("untimer" . ("maxima.info-5" 183507 674 "Funciones y variables para depurado"))
("untimer <1>" . ("maxima.info-5" 183508 673 "Funciones y variables para depurado"))
("untrace" . ("maxima.info-5" 189595 401 "Funciones y variables para depurado"))
("untrace <1>" . ("maxima.info-5" 189596 400 "Funciones y variables para depurado"))
("uppercasep" . ("maxima.info-8" 109912 95 "Funciones y variables para caracteres"))
("uric" . ("maxima.info-4" 128838 101 "Funciones y variables para ctensor"))
("uricci" . ("maxima.info-4" 86052 523 "Funciones y variables para ctensor"))
("uriem" . ("maxima.info-4" 128651 106 "Funciones y variables para ctensor"))
("uriemann" . ("maxima.info-4" 89637 195 "Funciones y variables para ctensor"))
("use_fast_arrays" . ("maxima.info-1" 169129 169 "Funciones y variables para los arrays"))
("user_preamble" . ("maxima.info-6" 192982 880 "Funciones y variables para draw"))
("usersetunits" . ("maxima.info-8" 182278 8803 "Funciones y variables para units"))
("uvect" . ("maxima.info-3" 283517 338 "Funciones y variables para las matrices y el álgebra lineal"))
("values" . ("maxima.info-1" 72215 1248 "Funciones y variables para la línea de comandos"))
("vandermonde_matrix" . ("maxima.info-7" 229701 163 "Funciones y variables para linearalgebra"))
("var" . ("maxima.info-5" 243911 589 "Funciones y variables de parámetros descriptivos"))
("var <1>" . ("maxima.info-5" 243912 588 "Funciones y variables de parámetros descriptivos"))
("var1" . ("maxima.info-5" 244501 834 "Funciones y variables de parámetros descriptivos"))
("var1 <1>" . ("maxima.info-5" 244502 833 "Funciones y variables de parámetros descriptivos"))
("var_bernoulli" . ("maxima.info-6" 107637 314 "Funciones y variables para distribuciones discretas"))
("var_beta" . ("maxima.info-6" 76023 184 "Funciones y variables para distribuciones continuas"))
("var_binomial" . ("maxima.info-6" 102437 222 "Funciones y variables para distribuciones discretas"))
("var_chi2" . ("maxima.info-6" 58810 272 "Funciones y variables para distribuciones continuas"))
("var_continuous_uniform" . ("maxima.info-6" 78398 209 "Funciones y variables para distribuciones continuas"))
("var_discrete_uniform" . ("maxima.info-6" 113006 215 "Funciones y variables para distribuciones discretas"))
("var_exp" . ("maxima.info-6" 67653 418 "Funciones y variables para distribuciones continuas"))
("var_f" . ("maxima.info-6" 64448 181 "Funciones y variables para distribuciones continuas"))
("var_gamma" . ("maxima.info-6" 73327 186 "Funciones y variables para distribuciones continuas"))
("var_general_finite_discrete" . ("maxima.info-6" 99481 222 "Funciones y variables para distribuciones discretas"))
("var_geometric" . ("maxima.info-6" 110644 196 "Funciones y variables para distribuciones discretas"))
("var_gumbel" . ("maxima.info-6" 95459 189 "Funciones y variables para distribuciones continuas"))
("var_hypergeometric" . ("maxima.info-6" 116004 260 "Funciones y variables para distribuciones discretas"))
("var_laplace" . ("maxima.info-6" 91937 191 "Funciones y variables para distribuciones continuas"))
("var_logistic" . ("maxima.info-6" 80759 191 "Funciones y variables para distribuciones continuas"))
("var_lognormal" . ("maxima.info-6" 70846 192 "Funciones y variables para distribuciones continuas"))
("var_negative_binomial" . ("maxima.info-6" 118758 232 "Funciones y variables para distribuciones discretas"))
("var_noncentral_chi2" . ("maxima.info-6" 61817 195 "Funciones y variables para distribuciones continuas"))
("var_noncentral_student_t" . ("maxima.info-6" 54896 282 "Funciones y variables para distribuciones continuas"))
("var_normal" . ("maxima.info-6" 47628 129 "Funciones y variables para distribuciones continuas"))
("var_pareto" . ("maxima.info-6" 82965 193 "Funciones y variables para distribuciones continuas"))
("var_poisson" . ("maxima.info-6" 105028 185 "Funciones y variables para distribuciones discretas"))
("var_rayleigh" . ("maxima.info-6" 88227 525 "Funciones y variables para distribuciones continuas"))
("var_student_t" . ("maxima.info-6" 50310 319 "Funciones y variables para distribuciones continuas"))
("var_weibull" . ("maxima.info-6" 85184 193 "Funciones y variables para distribuciones continuas"))
("vector" . ("maxima.info-6" 251605 1319 "Funciones y variables para draw"))
("vector <1>" . ("maxima.info-6" 251606 1318 "Funciones y variables para draw"))
("vectorpotential" . ("maxima.info-3" 283856 320 "Funciones y variables para las matrices y el álgebra lineal"))
("vectorsimp" . ("maxima.info-3" 284177 1369 "Funciones y variables para las matrices y el álgebra lineal"))
("verbify" . ("maxima.info-1" 310963 385 "Funciones y variables para expresiones"))
("verbose" . ("maxima.info-4" 184680 173 "Funciones y variables para las series"))
("vers" . ("maxima.info-8" 25393 115 "Paquete functs"))
("vertex_color" . ("maxima.info-7" 125213 78 "Funciones y variables para graphs"))
("vertex_coloring" . ("maxima.info-7" 126027 175 "Funciones y variables para graphs"))
("vertex_coloring <1>" . ("maxima.info-7" 115450 481 "Funciones y variables para graphs"))
("vertex_coloring <2>" . ("maxima.info-7" 119238 480 "Funciones y variables para graphs"))
("vertex_connectivity" . ("maxima.info-7" 113329 138 "Funciones y variables para graphs"))
("vertex_degree" . ("maxima.info-7" 113468 94 "Funciones y variables para graphs"))
("vertex_distance" . ("maxima.info-7" 113563 419 "Funciones y variables para graphs"))
("vertex_eccentricity" . ("maxima.info-7" 113983 276 "Funciones y variables para graphs"))
("vertex_in_degree" . ("maxima.info-7" 114260 580 "Funciones y variables para graphs"))
("vertex_out_degree" . ("maxima.info-7" 114841 381 "Funciones y variables para graphs"))
("vertex_partition" . ("maxima.info-7" 125811 215 "Funciones y variables para graphs"))
("vertex_size" . ("maxima.info-7" 125145 67 "Funciones y variables para graphs"))
("vertex_type" . ("maxima.info-7" 124970 174 "Funciones y variables para graphs"))
("vertices" . ("maxima.info-7" 115223 226 "Funciones y variables para graphs"))
("vertices_to_cycle" . ("maxima.info-7" 128993 161 "Funciones y variables para graphs"))
("vertices_to_path" . ("maxima.info-7" 128833 159 "Funciones y variables para graphs"))
("view" . ("maxima.info-6" 193863 943 "Funciones y variables para draw"))
("warnings" . ("maxima.info-8" 199050 187 "Funciones y variables para zeilberger"))
("weyl" . ("maxima.info-4" 129192 78 "Funciones y variables para ctensor"))
("weyl <1>" . ("maxima.info-4" 90094 433 "Funciones y variables para ctensor"))
("wheel_graph" . ("maxima.info-7" 88485 83 "Funciones y variables para graphs"))
("while" . ("maxima.info-5" 165080 77 "Funciones y variables para la programación"))
("width" . ("maxima.info-7" 4507 178 "Visualización usando VTK"))
("wiener_index" . ("maxima.info-7" 115932 197 "Funciones y variables para graphs"))
("windowname" . ("maxima.info-7" 4686 338 "Visualización usando VTK"))
("windowtitle" . ("maxima.info-7" 5025 226 "Visualización usando VTK"))
("wired_surface" . ("maxima.info-6" 194807 511 "Funciones y variables para draw"))
("wireframe" . ("maxima.info-7" 16538 293 "Visualización usando VTK"))
("with_stdout" . ("maxima.info-2" 199449 1130 "Funciones y variables para lectura y escritura"))
("with_stdout <1>" . ("maxima.info-2" 199450 1129 "Funciones y variables para lectura y escritura"))
("write_binary_data" . ("maxima.info-7" 275393 1184 "Funciones y variables para entrada y salida en formato binario"))
("write_data" . ("maxima.info-7" 270978 1345 "Funciones y variables para entrada y salida en formato texto"))
("write_data <1>" . ("maxima.info-7" 270979 1344 "Funciones y variables para entrada y salida en formato texto"))
("writefile" . ("maxima.info-2" 200580 1379 "Funciones y variables para lectura y escritura"))
("wronskian" . ("maxima.info-8" 19917 491 "Paquete functs"))
("x" . ("maxima.info-2" 169182 447 "Opciones gráficas"))
("x_voxel" . ("maxima.info-6" 195319 285 "Funciones y variables para draw"))
("xaxis" . ("maxima.info-6" 195605 477 "Funciones y variables para draw"))
("xaxis_color" . ("maxima.info-6" 196083 539 "Funciones y variables para draw"))
("xaxis_secondary" . ("maxima.info-6" 196623 836 "Funciones y variables para draw"))
("xaxis_type" . ("maxima.info-6" 197460 528 "Funciones y variables para draw"))
("xaxis_width" . ("maxima.info-6" 197989 503 "Funciones y variables para draw"))
("xlabel" . ("maxima.info-2" 169630 442 "Opciones gráficas"))
("xlabel <1>" . ("maxima.info-6" 198493 566 "Funciones y variables para draw"))
("xlabel_secondary" . ("maxima.info-6" 199060 742 "Funciones y variables para draw"))
("xlength" . ("maxima.info-7" 15845 230 "Visualización usando VTK"))
("xrange" . ("maxima.info-6" 199803 595 "Funciones y variables para draw"))
("xrange_secondary" . ("maxima.info-6" 200399 550 "Funciones y variables para draw"))
("xreduce" . ("maxima.info-5" 60589 1714 "Funciones y variables para los conjuntos"))
("xreduce <1>" . ("maxima.info-5" 60590 1713 "Funciones y variables para los conjuntos"))
("xthru" . ("maxima.info-2" 31830 1017 "Funciones y variables para simplificación"))
("xtics" . ("maxima.info-6" 200950 2082 "Funciones y variables para draw"))
("xtics_axis" . ("maxima.info-6" 203033 366 "Funciones y variables para draw"))
("xtics_rotate" . ("maxima.info-6" 203400 273 "Funciones y variables para draw"))
("xtics_rotate_secondary" . ("maxima.info-6" 203674 304 "Funciones y variables para draw"))
("xtics_secondary" . ("maxima.info-6" 203979 225 "Funciones y variables para draw"))
("xtics_secondary_axis" . ("maxima.info-6" 204205 397 "Funciones y variables para draw"))
("xu_grid" . ("maxima.info-6" 204603 554 "Funciones y variables para draw"))
("xy_file" . ("maxima.info-6" 205158 429 "Funciones y variables para draw"))
("xyplane" . ("maxima.info-6" 205588 582 "Funciones y variables para draw"))
("y" . ("maxima.info-2" 170073 292 "Opciones gráficas"))
("y_voxel" . ("maxima.info-6" 206171 285 "Funciones y variables para draw"))
("yaxis" . ("maxima.info-6" 206457 477 "Funciones y variables para draw"))
("yaxis_color" . ("maxima.info-6" 206935 539 "Funciones y variables para draw"))
("yaxis_secondary" . ("maxima.info-6" 207475 737 "Funciones y variables para draw"))
("yaxis_type" . ("maxima.info-6" 208213 528 "Funciones y variables para draw"))
("yaxis_width" . ("maxima.info-6" 208742 503 "Funciones y variables para draw"))
("ylabel" . ("maxima.info-2" 170366 408 "Opciones gráficas"))
("ylabel <1>" . ("maxima.info-6" 209246 567 "Funciones y variables para draw"))
("ylabel_secondary" . ("maxima.info-6" 209814 836 "Funciones y variables para draw"))
("ylength" . ("maxima.info-7" 16076 230 "Visualización usando VTK"))
("yrange" . ("maxima.info-6" 210651 635 "Funciones y variables para draw"))
("yrange_secondary" . ("maxima.info-6" 211287 903 "Funciones y variables para draw"))
("ytics" . ("maxima.info-6" 212191 204 "Funciones y variables para draw"))
("ytics_axis" . ("maxima.info-6" 212396 366 "Funciones y variables para draw"))
("ytics_rotate" . ("maxima.info-6" 212763 273 "Funciones y variables para draw"))
("ytics_rotate_secondary" . ("maxima.info-6" 213037 304 "Funciones y variables para draw"))
("ytics_secondary" . ("maxima.info-6" 213342 225 "Funciones y variables para draw"))
("ytics_secondary_axis" . ("maxima.info-6" 213568 397 "Funciones y variables para draw"))
("yv_grid" . ("maxima.info-6" 213966 554 "Funciones y variables para draw"))
("z" . ("maxima.info-2" 170775 144 "Opciones gráficas"))
("z_voxel" . ("maxima.info-6" 214521 219 "Funciones y variables para draw"))
("zaxis" . ("maxima.info-6" 214741 601 "Funciones y variables para draw"))
("zaxis_color" . ("maxima.info-6" 215343 650 "Funciones y variables para draw"))
("zaxis_type" . ("maxima.info-6" 215994 595 "Funciones y variables para draw"))
("zaxis_width" . ("maxima.info-6" 216590 609 "Funciones y variables para draw"))
("zeroa" . ("maxima.info-1" 120291 481 "Funciones y variables para Constantes"))
("zerob" . ("maxima.info-1" 120773 235 "Funciones y variables para Constantes"))
("zerobern" . ("maxima.info-4" 213033 235 "Funciones y variables para teoría de números"))
("zeroequiv" . ("maxima.info-2" 135895 1076 "Funciones y variables para los predicados"))
("zerofor" . ("maxima.info-7" 229865 551 "Funciones y variables para linearalgebra"))
("zerofor <1>" . ("maxima.info-7" 229866 550 "Funciones y variables para linearalgebra"))
("zeromatrix" . ("maxima.info-3" 285547 129 "Funciones y variables para las matrices y el álgebra lineal"))
("zeromatrixp" . ("maxima.info-7" 230417 304 "Funciones y variables para linearalgebra"))
("zeta" . ("maxima.info-4" 213269 1393 "Funciones y variables para teoría de números"))
("zeta%pi" . ("maxima.info-4" 214663 633 "Funciones y variables para teoría de números"))
("zlabel" . ("maxima.info-2" 170920 371 "Opciones gráficas"))
("zlabel <1>" . ("maxima.info-6" 217200 587 "Funciones y variables para draw"))
("zlange" . ("maxima.info-7" 173922 789 "Funciones y variables para lapack"))
("zlength" . ("maxima.info-7" 16307 230 "Visualización usando VTK"))
("zn_add_table" . ("maxima.info-4" 215297 158 "Funciones y variables para teoría de números"))
("zn_determinant" . ("maxima.info-4" 215456 1112 "Funciones y variables para teoría de números"))
("zn_invert_by_lu" . ("maxima.info-4" 216569 1052 "Funciones y variables para teoría de números"))
("zn_log" . ("maxima.info-4" 217622 2466 "Funciones y variables para teoría de números"))
("zn_log <1>" . ("maxima.info-4" 217623 2465 "Funciones y variables para teoría de números"))
("zn_mult_table" . ("maxima.info-4" 220089 917 "Funciones y variables para teoría de números"))
("zn_mult_table <1>" . ("maxima.info-4" 220090 916 "Funciones y variables para teoría de números"))
("zn_order" . ("maxima.info-4" 221007 2322 "Funciones y variables para teoría de números"))
("zn_order <1>" . ("maxima.info-4" 221008 2321 "Funciones y variables para teoría de números"))
("zn_power_table" . ("maxima.info-4" 223330 1416 "Funciones y variables para teoría de números"))
("zn_power_table <1>" . ("maxima.info-4" 223331 1415 "Funciones y variables para teoría de números"))
("zn_primroot" . ("maxima.info-4" 224747 2522 "Funciones y variables para teoría de números"))
("zn_primroot <1>" . ("maxima.info-4" 224748 2521 "Funciones y variables para teoría de números"))
("zn_primroot_limit" . ("maxima.info-4" 227270 328 "Funciones y variables para teoría de números"))
("zn_primroot_p" . ("maxima.info-4" 227599 1793 "Funciones y variables para teoría de números"))
("zn_primroot_p <1>" . ("maxima.info-4" 227600 1792 "Funciones y variables para teoría de números"))
("zn_primroot_pretest" . ("maxima.info-4" 229393 512 "Funciones y variables para teoría de números"))
("zn_primroot_verbose" . ("maxima.info-4" 229906 167 "Funciones y variables para teoría de números"))
("zrange" . ("maxima.info-6" 217788 686 "Funciones y variables para draw"))
("ztics" . ("maxima.info-6" 218475 204 "Funciones y variables para draw"))
("ztics_axis" . ("maxima.info-6" 218680 366 "Funciones y variables para draw"))
("ztics_rotate" . ("maxima.info-6" 219047 274 "Funciones y variables para draw"))
("|" . ("maxima.info-4" 62029 1323 "Funciones y variables para itensor"))
("~" . ("maxima.info-4" 60418 1610 "Funciones y variables para itensor"))
))
(section-pairs '(
; CONTENT: (<NODE NAME> . (<FILENAME> <BYTE OFFSET> <LENGTH IN CHARACTERS>))
("Análisis gráfico de sistemas dinámicos discretos" . ("maxima.info-6" 281668 6457))
("Archivos" . ("maxima.info-2" 178209 1118))
("Arrays" . ("maxima.info-1" 146865 105))
("Cadenas de texto" . ("maxima.info-1" 110396 145))
("Claves de depuración" . ("maxima.info-5" 176713 2373))
("Comentarios" . ("maxima.info-2" 176915 1155))
("Constantes" . ("maxima.info-1" 115995 86))
("Definiciones para IFS fractals" . ("maxima.info-7" 69852 1725))
("Definiciones para cops de Koch" . ("maxima.info-7" 74046 822))
("Definiciones para curvas de Peano" . ("maxima.info-7" 74985 709))
("Definiciones para fractales complejos" . ("maxima.info-7" 71732 2156))
("Depuración del código fuente" . ("maxima.info-5" 172511 4056))
("Desigualdades" . ("maxima.info-1" 261235 193))
("Documentación" . ("maxima.info-1" 29006 959))
("Ejemplos para cobyla" . ("maxima.info-5" 208623 617))
("El paquete dynamics" . ("maxima.info-6" 280926 588))
("Estructuras" . ("maxima.info-1" 169389 139))
("Formatos gráficos" . ("maxima.info-2" 139098 2721))
("Funciones" . ("maxima.info-5" 63026 5335))
("Funciones Gamma y factorial" . ("maxima.info-3" 1758 32371))
("Funciones combinatorias" . ("maxima.info-2" 50291 7262))
("Funciones de Airy" . ("maxima.info-3" 211 1408))
("Funciones de Bessel" . ("maxima.info-2" 282993 6552))
("Funciones de Struve" . ("maxima.info-3" 37713 387))
("Funciones de cilindro parabólico" . ("maxima.info-3" 40787 449))
("Funciones hipergeométricas" . ("maxima.info-3" 38252 2351))
("Funciones para el formato Gnuplot_pipes" . ("maxima.info-2" 175444 1039))
("Funciones para la resolución numérica de ecuaciones" . ("maxima.info-3" 208058 6822))
("Funciones para la resolución numérica de ecuaciones diferenciales" . ("maxima.info-3" 216506 14144))
("Funciones para los números" . ("maxima.info-2" 33389 11573))
("Funciones para los números complejos" . ("maxima.info-2" 45123 4985))
("Funciones radicales, exponenciales y logarítmicas" . ("maxima.info-2" 57725 9889))
("Funciones trigonométricas" . ("maxima.info-2" 67781 157))
("Funciones y Variables para finance" . ("maxima.info-7" 57750 11096))
("Funciones y variable para las ecuaciones" . ("maxima.info-3" 146581 43049))
("Funciones y variables de parámetros descriptivos" . ("maxima.info-5" 242946 33783))
("Funciones y variables para Afines" . ("maxima.info-3" 287135 5263))
("Funciones y variables para Bernstein" . ("maxima.info-5" 194692 4863))
("Funciones y variables para QUADPACK" . ("maxima.info-3" 121797 24491))
("Funciones y variables para atensor" . ("maxima.info-4" 137294 5618))
("Funciones y variables para augmented_lagrangian" . ("maxima.info-5" 190335 4088))
("Funciones y variables para bode" . ("maxima.info-5" 199780 3176))
("Funciones y variables para cadenas" . ("maxima.info-8" 110134 14210))
("Funciones y variables para caracteres" . ("maxima.info-8" 107060 2947))
("Funciones y variables para cobyla" . ("maxima.info-5" 205588 2930))
("Funciones y variables para contrib_ode" . ("maxima.info-5" 215235 4929))
("Funciones y variables para ctensor" . ("maxima.info-4" 76328 56613))
("Funciones y variables para depurado" . ("maxima.info-5" 179195 10801))
("Funciones y variables para diag" . ("maxima.info-6" 22105 16977))
("Funciones y variables para distribuciones continuas" . ("maxima.info-6" 45841 51135))
("Funciones y variables para distribuciones discretas" . ("maxima.info-6" 97131 23124))
("Funciones y variables para distribuciones especiales" . ("maxima.info-8" 87628 1255))
("Funciones y variables para draw" . ("maxima.info-6" 121188 131736))
("Funciones y variables para drawdf" . ("maxima.info-6" 272151 8417))
("Funciones y variables para ecuaciones diferenciales." . ("maxima.info-3" 190732 8021))
("Funciones y variables para el entorno de ejecución" . ("maxima.info-4" 274562 7879))
("Funciones y variables para el tratamiento de datos" . ("maxima.info-5" 229439 13295))
("Funciones y variables para entrada y salida" . ("maxima.info-8" 96700 10181))
("Funciones y variables para entrada y salida en formato binario" . ("maxima.info-7" 272502 4075))
("Funciones y variables para entrada y salida en formato texto" . ("maxima.info-7" 266911 5412))
("Funciones y variables para expresiones" . ("maxima.info-1" 261536 49812))
("Funciones y variables para ezunits" . ("maxima.info-7" 25985 28656))
("Funciones y variables para f90" . ("maxima.info-7" 54858 2000))
("Funciones y variables para funciones elípticas" . ("maxima.info-3" 53237 2337))
("Funciones y variables para ggf" . ("maxima.info-7" 75911 1823))
("Funciones y variables para graphs" . ("maxima.info-7" 79145 50009))
("Funciones y variables para grobner" . ("maxima.info-7" 131357 13775))
("Funciones y variables para grupos" . ("maxima.info-4" 269799 1883))
("Funciones y variables para gráficos" . ("maxima.info-2" 141951 15794))
("Funciones y variables para gráficos estadísticos" . ("maxima.info-6" 226 21651))
("Funciones y variables para impdiff" . ("maxima.info-7" 145378 549))
("Funciones y variables para inference_result" . ("maxima.info-8" 46562 4821))
("Funciones y variables para integración" . ("maxima.info-3" 91781 25486))
("Funciones y variables para integrales elípticas" . ("maxima.info-3" 55732 1758))
("Funciones y variables para interpol" . ("maxima.info-7" 146623 13123))
("Funciones y variables para itensor" . ("maxima.info-4" 182 71216))
("Funciones y variables para la ayuda" . ("maxima.info-1" 30064 9522))
("Funciones y variables para la definición de funciones" . ("maxima.info-5" 77653 57328))
("Funciones y variables para la detección e informe de fallos" . ("maxima.info-1" 23766 4933))
("Funciones y variables para la diferenciación" . ("maxima.info-3" 61809 28498))
("Funciones y variables para la evaluación" . ("maxima.info-1" 227823 26564))
("Funciones y variables para la impresión" . ("maxima.info-1" 73612 21557))
("Funciones y variables para la línea de comandos" . ("maxima.info-1" 45733 27730))
("Funciones y variables para la miscelánea de opciones" . ("maxima.info-4" 284061 6226))
("Funciones y variables para la programación" . ("maxima.info-5" 140992 31186))
("Funciones y variables para la transformada rápida de Fourier" . ("maxima.info-3" 199896 7914))
("Funciones y variables para lapack" . ("maxima.info-7" 160319 19900))
("Funciones y variables para las funciones especiales" . ("maxima.info-3" 41385 7326))
("Funciones y variables para las matrices y el álgebra lineal" . ("maxima.info-3" 235035 51838))
("Funciones y variables para las propiedades" . ("maxima.info-2" 90908 21070))
("Funciones y variables para las series" . ("maxima.info-4" 156835 28018))
("Funciones y variables para lbfgs" . ("maxima.info-7" 181558 15457))
("Funciones y variables para lectura y escritura" . ("maxima.info-2" 179492 22467))
("Funciones y variables para lindstedt" . ("maxima.info-7" 197278 1187))
("Funciones y variables para linearalgebra" . ("maxima.info-7" 203338 27383))
("Funciones y variables para los conjuntos" . ("maxima.info-5" 11174 51129))
("Funciones y variables para los hechos" . ("maxima.info-2" 112176 16659))
("Funciones y variables para los predicados" . ("maxima.info-2" 128982 7989))
("Funciones y variables para lsquares" . ("maxima.info-7" 230982 22183))
("Funciones y variables para límites" . ("maxima.info-3" 57760 3743))
("Funciones y variables para makeOrders" . ("maxima.info-7" 253432 897))
("Funciones y variables para minpack" . ("maxima.info-7" 254819 4483))
("Funciones y variables para mnewton" . ("maxima.info-7" 259551 2304))
("Funciones y variables para opsubst" . ("maxima.info-7" 276828 2409))
("Funciones y variables para picture" . ("maxima.info-6" 253083 4454))
("Funciones y variables para polinomios" . ("maxima.info-2" 219039 61158))
("Funciones y variables para polinomios ortogonales" . ("maxima.info-7" 297900 7609))
("Funciones y variables para romberg" . ("maxima.info-8" 312 7295))
("Funciones y variables para salida Fortran" . ("maxima.info-2" 212591 2806))
("Funciones y variables para salida TeX" . ("maxima.info-2" 202157 10291))
("Funciones y variables para series de Fourier" . ("maxima.info-4" 185528 3190))
("Funciones y variables para series de Poisson" . ("maxima.info-4" 188876 2614))
("Funciones y variables para simetrías" . ("maxima.info-4" 231526 38020))
("Funciones y variables para simplex" . ("maxima.info-8" 8393 3891))
("Funciones y variables para simplificación" . ("maxima.info-2" 4987 27860))
("Funciones y variables para solve_rec" . ("maxima.info-8" 35707 9225))
("Funciones y variables para stats" . ("maxima.info-8" 51568 35927))
("Funciones y variables para stirling" . ("maxima.info-8" 89136 2619))
("Funciones y variables para sumas y productos" . ("maxima.info-4" 143517 12496))
("Funciones y variables para teoría de números" . ("maxima.info-4" 191826 38247))
("Funciones y variables para to_poly_solve" . ("maxima.info-8" 124631 44060))
("Funciones y variables para units" . ("maxima.info-8" 172311 19969))
("Funciones y variables para worldmap" . ("maxima.info-6" 257656 12377))
("Funciones y variables para zeilberger" . ("maxima.info-8" 194726 5984))
("Funciones y variables sobre reglas y patrones" . ("maxima.info-4" 291793 39436))
("Función de error" . ("maxima.info-3" 35778 1799))
("Identificadores" . ("maxima.info-1" 258948 2155))
("Integral exponencial" . ("maxima.info-3" 34267 1381))
("Interrupciones" . ("maxima.info-4" 273397 1035))
("Introducción a QUADPACK" . ("maxima.info-3" 117429 4253))
("Introducción a atensor" . ("maxima.info-4" 133261 3924))
("Introducción a cobyla" . ("maxima.info-5" 203278 2175))
("Introducción a contrib_ode" . ("maxima.info-5" 209689 5386))
("Introducción a ctensor" . ("maxima.info-4" 71711 4508))
("Introducción a descriptive" . ("maxima.info-5" 224169 5080))
("Introducción a distrib" . ("maxima.info-6" 39469 6186))
("Introducción a draw" . ("maxima.info-6" 120609 436))
("Introducción a drawdf" . ("maxima.info-6" 270325 1720))
("Introducción a ezunits" . ("maxima.info-7" 17168 3785))
("Introducción a finance" . ("maxima.info-7" 57158 483))
("Introducción a fractals" . ("maxima.info-7" 69259 440))
("Introducción a graphs" . ("maxima.info-7" 78024 1015))
("Introducción a grobner" . ("maxima.info-7" 129448 1804))
("Introducción a interpol" . ("maxima.info-7" 146238 273))
("Introducción a itensor" . ("maxima.info-3" 292700 12131))
("Introducción a la base de datos de Maxima" . ("maxima.info-2" 87457 3253))
("Introducción a la definición de funciones" . ("maxima.info-5" 62797 95))
("Introducción a la evaluación" . ("maxima.info-1" 224014 3685))
("Introducción a la integración" . ("maxima.info-3" 90730 895))
("Introducción a la línea de comandos" . ("maxima.info-1" 40052 5488))
("Introducción a la miscelánea de opciones" . ("maxima.info-4" 282881 288))
("Introducción a la programación" . ("maxima.info-5" 140674 188))
("Introducción a la resolución numérica de ecuaciones diferenciales" . ("maxima.info-3" 215133 1180))
("Introducción a la simplificación" . ("maxima.info-2" 463 4391))
("Introducción a la transformada rápida de Fourier" . ("maxima.info-3" 199441 224))
("Introducción a lapack" . ("maxima.info-7" 160039 174))
("Introducción a las ecuaciones diferenciales" . ("maxima.info-3" 190109 459))
("Introducción a las expresiones" . ("maxima.info-1" 254779 1487))
("Introducción a las funciones e integrales elípticas" . ("maxima.info-3" 49221 3798))
("Introducción a las funciones especiales" . ("maxima.info-2" 280832 2011))
("Introducción a las matrices y el álgebra lineal" . ("maxima.info-3" 231152 213))
("Introducción a las series" . ("maxima.info-4" 156198 460))
("Introducción a las series de Fourier" . ("maxima.info-4" 185049 276))
("Introducción a lbfgs" . ("maxima.info-7" 180501 956))
("Introducción a linearalgebra" . ("maxima.info-7" 198825 4386))
("Introducción a los conjuntos" . ("maxima.info-5" 450 10601))
("Introducción a los gráficos" . ("maxima.info-2" 137415 1541))
("Introducción a los operadores" . ("maxima.info-1" 176403 5821))
("Introducción a los polinomios" . ("maxima.info-2" 215764 3153))
("Introducción a minpack" . ("maxima.info-7" 254562 193))
("Introducción a numericalio" . ("maxima.info-7" 262305 4392))
("Introducción a physical_constants" . ("maxima.info-7" 21104 4761))
("Introducción a polinomios ortogonales" . ("maxima.info-7" 279610 18149))
("Introducción a reglas y patrones" . ("maxima.info-4" 290703 950))
("Introducción a simplex" . ("maxima.info-8" 7917 367))
("Introducción a simplification" . ("maxima.info-8" 12710 274))
("Introducción a solve_rec" . ("maxima.info-8" 34961 631))
("Introducción a stats" . ("maxima.info-8" 45332 1077))
("Introducción a units" . ("maxima.info-8" 168994 3215))
("Introducción a zeilberger" . ("maxima.info-8" 192631 1977))
("Introducción al entorno de ejecución" . ("maxima.info-4" 272112 1110))
("Introducción al procesamiento de cadenas" . ("maxima.info-8" 92215 4300))
("Lisp y Maxima" . ("maxima.info-5" 135366 4229))
("Listas" . ("maxima.info-1" 121113 115))
("Macros" . ("maxima.info-5" 68507 9018))
("Nombres y verbos" . ("maxima.info-1" 256393 2445))
("Números" . ("maxima.info-1" 95575 109))
("Números aleatorios" . ("maxima.info-2" 84092 2825))
("Opciones gráficas" . ("maxima.info-2" 157881 13410))
("Opciones para Gnuplot" . ("maxima.info-2" 171431 3899))
("Operadores aritméticos" . ("maxima.info-1" 182363 9539))
("Operadores de asignación" . ("maxima.info-1" 202884 10018))
("Operadores definidos por el usuario" . ("maxima.info-1" 213017 10650))
("Operadores lógicos" . ("maxima.info-1" 194749 2835))
("Operadores para ecuaciones" . ("maxima.info-1" 197717 5017))
("Operadores relacionales" . ("maxima.info-1" 192030 2587))
("Paquete absimp" . ("maxima.info-8" 13110 972))
("Paquete facexp" . ("maxima.info-8" 14193 5316))
("Paquete functs" . ("maxima.info-8" 19618 6620))
("Paquete ineq" . ("maxima.info-8" 26347 4309))
("Paquete rducon" . ("maxima.info-8" 30765 1523))
("Paquete scifac" . ("maxima.info-8" 32399 1046))
("Paquete sqdnst" . ("maxima.info-8" 33533 1102))
("Posibles mejoras a contrib_ode" . ("maxima.info-5" 220332 974))
("Pruebas realizadas con contrib_ode" . ("maxima.info-5" 221464 684))
("Recolector de basura" . ("maxima.info-5" 139725 790))
("Referencias para contrib_ode" . ("maxima.info-5" 222267 1432))
("Share" . ("maxima.info-4" 283343 593))
("Visualización usando VTK" . ("maxima.info-7" 199 16632))
)))
(load-info-hashtables (maxima::maxima-load-pathname-directory) deffn-defvr-pairs section-pairs))
