(in-package :cl-info)
(let (
(deffn-defvr-pairs '(
; CONTENT: (<INDEX TOPIC> . (<FILENAME> <BYTE OFFSET> <LENGTH IN CHARACTERS> <NODE NAME>))
("!" . ("maxima.info-2" 52859 3782 "Funciones combinatorias"))
("!!" . ("maxima.info-2" 50821 731 "Funciones combinatorias"))
("#" . ("maxima.info-1" 199442 773 "Operadores para ecuaciones"))
("%" . ("maxima.info-1" 49885 730 "Funciones y variables para la línea de comandos"))
("%%" . ("maxima.info-1" 50626 1685 "Funciones y variables para la línea de comandos"))
("%and" . ("maxima.info-8" 126170 2725 "Funciones y variables para to_poly_solve"))
("%e" . ("maxima.info-1" 117340 224 "Funciones y variables para Constantes"))
("%e_to_numlog" . ("maxima.info-2" 58360 410 "Funciones radicales exponenciales y logarítmicas"))
("%edispflag" . ("maxima.info-1" 74407 621 "Funciones y variables para la impresión"))
("%emode" . ("maxima.info-2" 58780 606 "Funciones radicales exponenciales y logarítmicas"))
("%enumer" . ("maxima.info-2" 59391 345 "Funciones radicales exponenciales y logarítmicas"))
("%f" . ("maxima.info-3" 39060 183 "Funciones hipergeométricas"))
("%gamma" . ("maxima.info-1" 117799 88 "Funciones y variables para Constantes"))
("%i" . ("maxima.info-1" 117570 83 "Funciones y variables para Constantes"))
("%iargs" . ("maxima.info-2" 71867 1080 "Funciones y variables para trigonometría"))
("%if" . ("maxima.info-8" 128925 1425 "Funciones y variables para to_poly_solve"))
("%m" . ("maxima.info-3" 38844 134 "Funciones hipergeométricas"))
("%or" . ("maxima.info-8" 130367 2563 "Funciones y variables para to_poly_solve"))
("%phi" . ("maxima.info-1" 118349 2470 "Funciones y variables para Constantes"))
("%pi" . ("maxima.info-1" 120831 210 "Funciones y variables para Constantes"))
("%piargs" . ("maxima.info-2" 69867 1996 "Funciones y variables para trigonometría"))
("%rnum_list" . ("maxima.info-3" 147999 968 "Funciones y variable para las ecuaciones"))
("%s" . ("maxima.info-2" 292063 99 "Funciones de Bessel"))
("%th" . ("maxima.info-1" 52318 1126 "Funciones y variables para la línea de comandos"))
("%unitexpand" . ("maxima.info-8" 193648 158 "Funciones y variables para units"))
("%w" . ("maxima.info-3" 38981 76 "Funciones hipergeométricas"))
("'" . ("maxima.info-1" 229848 3782 "Funciones y variables para la evaluación"))
("'%c'" . ("maxima.info-5" 220001 88 "Funciones y variables para contrib_ode"))
("'%k1'" . ("maxima.info-5" 220091 104 "Funciones y variables para contrib_ode"))
("'%k2'" . ("maxima.info-5" 220197 104 "Funciones y variables para contrib_ode"))
("''" . ("maxima.info-1" 233665 5858 "Funciones y variables para la evaluación"))
("'method'" . ("maxima.info-5" 219905 94 "Funciones y variables para contrib_ode"))
("*" . ("maxima.info-1" 183959 7400 "Operadores aritméticos"))
("**" . ("maxima.info-1" 191426 831 "Operadores aritméticos"))
("+" . ("maxima.info-1" 183942 7417 "Operadores aritméticos"))
("-" . ("maxima.info-1" 183943 7416 "Operadores aritméticos"))
("." . ("maxima.info-1" 193080 425 "Operadores aritméticos"))
("/" . ("maxima.info-1" 183975 7384 "Operadores aritméticos"))
(":" . ("maxima.info-1" 204645 2849 "Operadores de asignación"))
("::" . ("maxima.info-1" 207515 746 "Operadores de asignación"))
("::=" . ("maxima.info-1" 208264 3164 "Operadores de asignación"))
(":=" . ("maxima.info-1" 211457 3198 "Operadores de asignación"))
("<" . ("maxima.info-1" 193697 2531 "Operadores relacionales"))
("<=" . ("maxima.info-1" 193698 2530 "Operadores relacionales"))
("=" . ("maxima.info-1" 200221 4181 "Operadores para ecuaciones"))
(">" . ("maxima.info-1" 193731 2497 "Operadores relacionales"))
(">=" . ("maxima.info-1" 193714 2514 "Operadores relacionales"))
("?" . ("maxima.info-1" 53459 565 "Funciones y variables para la línea de comandos"))
("??" . ("maxima.info-1" 54038 313 "Funciones y variables para la línea de comandos"))
("@" . ("maxima.info-1" 175647 1745 "Funciones y variables para las estructuras"))
("Acotador de listas" . ("maxima.info-1" 123459 1104 "Funciones y variables para listas"))
("AntiDifference" . ("maxima.info-8" 196372 177 "Funciones y variables para zeilberger"))
("Ayuda" . ("maxima.info-1" 34848 2961 "Funciones y variables para la ayuda"))
("Base de los logaritmos naturales" . ("maxima.info-1" 117341 223 "Funciones y variables para Constantes"))
("Busca coincidencias inexactas en la documentaci&oacute;n" . ("maxima.info-1" 54039 312 "Funciones y variables para la línea de comandos"))
("Busca en la documentaci&oacute;n" . ("maxima.info-1" 53460 564 "Funciones y variables para la línea de comandos"))
("Conjunción lógica" . ("maxima.info-1" 196423 1084 "Operadores lógicos"))
("Conjunción lógica <1>" . ("maxima.info-8" 126171 2724 "Funciones y variables para to_poly_solve"))
("Constante de Euler-Mascheroni" . ("maxima.info-1" 117800 87 "Funciones y variables para Constantes"))
("Contracción con vector" . ("maxima.info-4" 62550 1322 "Funciones y variables para itensor"))
("Delimitadores de listas" . ("maxima.info-3" 288135 1171 "Funciones y variables para las matrices y el álgebra lineal"))
("Disyunción lógica" . ("maxima.info-1" 198154 1078 "Operadores lógicos"))
("Disyunción lógica <1>" . ("maxima.info-8" 130368 2562 "Funciones y variables para to_poly_solve"))
("División" . ("maxima.info-1" 183991 7368 "Operadores aritméticos"))
("Doble factorial" . ("maxima.info-2" 50822 730 "Funciones combinatorias"))
("Entrada anterior" . ("maxima.info-1" 48250 1620 "Funciones y variables para la línea de comandos"))
("Evaluación condicionada" . ("maxima.info-8" 128926 1424 "Funciones y variables para to_poly_solve"))
("Exponenciación" . ("maxima.info-1" 184007 7352 "Operadores aritméticos"))
("Exponenciación no conmutativa" . ("maxima.info-1" 192263 808 "Operadores aritméticos"))
("Expresión actual de entrada" . ("maxima.info-1" 46284 1949 "Funciones y variables para la línea de comandos"))
("GGFCFMAX" . ("maxima.info-7" 77175 573 "Funciones y variables para ggf"))
("GGFINFINITY" . ("maxima.info-7" 76547 612 "Funciones y variables para ggf"))
("Gosper" . ("maxima.info-8" 196551 264 "Funciones y variables para zeilberger"))
("GosperSum" . ("maxima.info-8" 196818 2053 "Funciones y variables para zeilberger"))
("Gosper_in_Zeilberger" . ("maxima.info-8" 200824 283 "Funciones y variables para zeilberger"))
("Igualdad sintáctica" . ("maxima.info-1" 200222 4180 "Operadores para ecuaciones"))
("Indefinido" . ("maxima.info-1" 121190 198 "Funciones y variables para Constantes"))
("Indeterminado" . ("maxima.info-1" 117889 207 "Funciones y variables para Constantes"))
("Infinito complejo" . ("maxima.info-1" 118181 83 "Funciones y variables para Constantes"))
("Infinito negativo" . ("maxima.info-1" 118267 80 "Funciones y variables para Constantes"))
("JF" . ("maxima.info-6" 23306 837 "Funciones y variables para diag"))
("Lindstedt" . ("maxima.info-7" 198757 1103 "Funciones y variables para lindstedt"))
("Mayor o igual que" . ("maxima.info-1" 193731 2497 "Operadores relacionales"))
("Mayor que" . ("maxima.info-1" 193748 2480 "Operadores relacionales"))
("Menor o igual que" . ("maxima.info-1" 193714 2514 "Operadores relacionales"))
("Menor que" . ("maxima.info-1" 193698 2530 "Operadores relacionales"))
("Menos infinito" . ("maxima.info-1" 118267 80 "Funciones y variables para Constantes"))
("ModeMatrix" . ("maxima.info-6" 27617 1468 "Funciones y variables para diag"))
("Multiplicación" . ("maxima.info-1" 183975 7384 "Operadores aritméticos"))
("Multiplicación no conmutativa" . ("maxima.info-1" 193081 424 "Operadores aritméticos"))
("Más infinito" . ("maxima.info-1" 118100 78 "Funciones y variables para Constantes"))
("Negación de la igualdad sintáctica" . ("maxima.info-1" 199443 772 "Operadores para ecuaciones"))
("Negación lógica" . ("maxima.info-1" 197523 622 "Operadores lógicos"))
("Número de Euler" . ("maxima.info-1" 117341 223 "Funciones y variables para Constantes"))
("Operador comilla" . ("maxima.info-1" 229849 3781 "Funciones y variables para la evaluación"))
("Operador comilla-comilla" . ("maxima.info-1" 233666 5857 "Funciones y variables para la evaluación"))
("Operador de asignación" . ("maxima.info-1" 204646 2848 "Operadores de asignación"))
("Operador de asignación (evalúa el miembro izquierdo)" . ("maxima.info-1" 207516 745 "Operadores de asignación"))
("Operador de definición de funciones" . ("maxima.info-1" 211458 3197 "Operadores de asignación"))
("Operador de definición de funciones macro" . ("maxima.info-1" 208265 3163 "Operadores de asignación"))
("Operador de ecuación" . ("maxima.info-1" 200222 4180 "Operadores para ecuaciones"))
("Operador de subíndice" . ("maxima.info-3" 288135 1171 "Funciones y variables para las matrices y el álgebra lineal"))
("Operador para subíndices" . ("maxima.info-1" 123459 1104 "Funciones y variables para listas"))
("Producto exterior" . ("maxima.info-4" 60936 1609 "Funciones y variables para itensor"))
("Razón áurea" . ("maxima.info-1" 118350 2469 "Funciones y variables para Constantes"))
("Resta" . ("maxima.info-1" 183959 7400 "Operadores aritméticos"))
("Resultado anterior en una sentencia compuesta" . ("maxima.info-1" 50627 1684 "Funciones y variables para la línea de comandos"))
("Salida anterior" . ("maxima.info-1" 49886 729 "Funciones y variables para la línea de comandos"))
("Suma" . ("maxima.info-1" 183943 7416 "Operadores aritméticos"))
("Unidad imaginaria" . ("maxima.info-1" 117571 82 "Funciones y variables para Constantes"))
("Zeilberger" . ("maxima.info-8" 199239 784 "Funciones y variables para zeilberger"))
("[" . ("maxima.info-1" 123442 1121 "Funciones y variables para listas"))
("[ <1>" . ("maxima.info-3" 288109 1196 "Funciones y variables para las matrices y el álgebra lineal"))
("[ <2>" . ("maxima.info-3" 288110 1195 "Funciones y variables para las matrices y el álgebra lineal"))
("]" . ("maxima.info-1" 123443 1120 "Funciones y variables para listas"))
("^" . ("maxima.info-1" 183991 7368 "Operadores aritméticos"))
("^^" . ("maxima.info-1" 192262 809 "Operadores aritméticos"))
("_" . ("maxima.info-1" 48249 1621 "Funciones y variables para la línea de comandos"))
("__" . ("maxima.info-1" 46283 1950 "Funciones y variables para la línea de comandos"))
("`" . ("maxima.info-7" 26394 3299 "Funciones y variables para ezunits"))
("` <1>" . ("maxima.info-7" 29702 7642 "Funciones y variables para ezunits"))
("abasep" . ("maxima.info-4" 143696 261 "Funciones y variables para atensor"))
("abs" . ("maxima.info-2" 33781 3472 "Funciones para los números"))
("absboxchar" . ("maxima.info-1" 75031 358 "Funciones y variables para la impresión"))
("absint" . ("maxima.info-4" 187826 606 "Funciones y variables para series de Fourier"))
("absint <1>" . ("maxima.info-4" 187827 605 "Funciones y variables para series de Fourier"))
("absint <2>" . ("maxima.info-4" 187872 561 "Funciones y variables para series de Fourier"))
("absolute_real_time" . ("maxima.info-4" 282945 472 "Funciones y variables para el entorno de ejecución"))
("acos" . ("maxima.info-2" 72956 43 "Funciones y variables para trigonometría"))
("acosh" . ("maxima.info-2" 73000 55 "Funciones y variables para trigonometría"))
("acot" . ("maxima.info-2" 73058 46 "Funciones y variables para trigonometría"))
("acoth" . ("maxima.info-2" 73106 59 "Funciones y variables para trigonometría"))
("acsc" . ("maxima.info-2" 73168 45 "Funciones y variables para trigonometría"))
("acsch" . ("maxima.info-2" 73215 58 "Funciones y variables para trigonometría"))
("activate" . ("maxima.info-2" 113294 409 "Funciones y variables para los hechos"))
("activecontexts" . ("maxima.info-2" 113708 266 "Funciones y variables para los hechos"))
("adapt_depth" . ("maxima.info-2" 159723 190 "Opciones gráficas"))
("adapt_depth <1>" . ("maxima.info-6" 129696 246 "Funciones y variables para draw"))
("add_edge" . ("maxima.info-7" 117092 412 "Funciones y variables para graphs"))
("add_edges" . ("maxima.info-7" 117507 386 "Funciones y variables para graphs"))
("add_vertex" . ("maxima.info-7" 117896 346 "Funciones y variables para graphs"))
("add_vertices" . ("maxima.info-7" 118246 103 "Funciones y variables para graphs"))
("addcol" . ("maxima.info-3" 237321 138 "Funciones y variables para las matrices y el álgebra lineal"))
("additive" . ("maxima.info-2" 5157 928 "Funciones y variables para simplificación"))
("addmatrices" . ("maxima.info-7" 204835 570 "Funciones y variables para linearalgebra"))
("addrow" . ("maxima.info-3" 237462 135 "Funciones y variables para las matrices y el álgebra lineal"))
("adim" . ("maxima.info-4" 141563 225 "Funciones y variables para atensor"))
("adjacency_matrix" . ("maxima.info-7" 89372 612 "Funciones y variables para graphs"))
("adjoin" . ("maxima.info-5" 11337 535 "Funciones y variables para los conjuntos"))
("adjoint" . ("maxima.info-3" 237600 147 "Funciones y variables para las matrices y el álgebra lineal"))
("af" . ("maxima.info-4" 142369 295 "Funciones y variables para atensor"))
("aform" . ("maxima.info-4" 141793 188 "Funciones y variables para atensor"))
("agd" . ("maxima.info-8" 25474 170 "Paquete functs"))
("airy_ai" . ("maxima.info-3" 657 177 "Funciones de Airy"))
("airy_bi" . ("maxima.info-3" 953 551 "Funciones de Airy"))
("airy_dai" . ("maxima.info-3" 838 111 "Funciones de Airy"))
("airy_dbi" . ("maxima.info-3" 1515 126 "Funciones de Airy"))
("alg_type" . ("maxima.info-4" 141407 152 "Funciones y variables para atensor"))
("algebraic" . ("maxima.info-2" 221220 184 "Funciones y variables para polinomios"))
("algepsilon" . ("maxima.info-3" 2242 117 "Funciones Gamma y factorial"))
("algepsilon <1>" . ("maxima.info-3" 148972 121 "Funciones y variable para las ecuaciones"))
("algexact" . ("maxima.info-3" 149094 650 "Funciones y variable para las ecuaciones"))
("algsys" . ("maxima.info-3" 149755 4733 "Funciones y variable para las ecuaciones"))
("algsys <1>" . ("maxima.info-3" 149756 4732 "Funciones y variable para las ecuaciones"))
("alias" . ("maxima.info-1" 263804 281 "Funciones y variables para expresiones"))
("aliases" . ("maxima.info-1" 264089 307 "Funciones y variables para expresiones"))
("all_dotsimp_denoms" . ("maxima.info-3" 294544 369 "Funciones y variables para Afines"))
("allbut" . ("maxima.info-1" 264399 1028 "Funciones y variables para expresiones"))
("allocation" . ("maxima.info-6" 129950 1483 "Funciones y variables para draw"))
("allroots" . ("maxima.info-3" 154531 2281 "Funciones y variable para las ecuaciones"))
("allroots <1>" . ("maxima.info-3" 154532 2280 "Funciones y variable para las ecuaciones"))
("allsym" . ("maxima.info-4" 24525 436 "Funciones y variables para itensor"))
("alphabetic" . ("maxima.info-2" 91843 678 "Funciones y variables para las propiedades"))
("alphacharp" . ("maxima.info-8" 107966 92 "Funciones y variables para caracteres"))
("alphanumericp" . ("maxima.info-8" 108062 107 "Funciones y variables para caracteres"))
("amortization" . ("maxima.info-7" 61508 1395 "Funciones y Variables para finance"))
("and" . ("maxima.info-1" 196422 1085 "Operadores lógicos"))
("animation" . ("maxima.info-7" 7512 660 "Visualización usando VTK"))
("annuity_fv" . ("maxima.info-7" 60128 410 "Funciones y Variables para finance"))
("annuity_pv" . ("maxima.info-7" 59723 401 "Funciones y Variables para finance"))
("antid" . ("maxima.info-3" 62500 2515 "Funciones y variables para la diferenciación"))
("antidiff" . ("maxima.info-3" 65029 2369 "Funciones y variables para la diferenciación"))
("antisymmetric" . ("maxima.info-2" 6092 1046 "Funciones y variables para simplificación"))
("append" . ("maxima.info-1" 124566 368 "Funciones y variables para listas"))
("appendfile" . ("maxima.info-2" 181304 341 "Funciones y variables para lectura y escritura"))
("apply" . ("maxima.info-5" 78335 2377 "Funciones y variables para la definición de funciones"))
("apply1" . ("maxima.info-4" 294078 739 "Funciones y variables sobre reglas y patrones"))
("apply2" . ("maxima.info-4" 294826 586 "Funciones y variables sobre reglas y patrones"))
("applyb1" . ("maxima.info-4" 295424 801 "Funciones y variables sobre reglas y patrones"))
("apropos" . ("maxima.info-1" 30469 1293 "Funciones y variables para la ayuda"))
("args" . ("maxima.info-1" 265431 551 "Funciones y variables para expresiones"))
("arit_amortization" . ("maxima.info-7" 62907 1709 "Funciones y Variables para finance"))
("arithmetic" . ("maxima.info-8" 23924 227 "Paquete functs"))
("arithsum" . ("maxima.info-8" 24647 189 "Paquete functs"))
("array" . ("maxima.info-1" 151519 1639 "Funciones y variables para los arrays"))
("array <1>" . ("maxima.info-1" 151520 1638 "Funciones y variables para los arrays"))
("array <2>" . ("maxima.info-1" 151574 1585 "Funciones y variables para los arrays"))
("arrayapply" . ("maxima.info-1" 153178 268 "Funciones y variables para los arrays"))
("arrayinfo" . ("maxima.info-1" 153450 4023 "Funciones y variables para los arrays"))
("arraymake" . ("maxima.info-1" 157498 1817 "Funciones y variables para los arrays"))
("arrays" . ("maxima.info-1" 159320 1156 "Funciones y variables para los arrays"))
("arraysetapply" . ("maxima.info-1" 160480 203 "Funciones y variables para los arrays"))
("ascii" . ("maxima.info-8" 108174 396 "Funciones y variables para caracteres"))
("asec" . ("maxima.info-2" 73276 43 "Funciones y variables para trigonometría"))
("asech" . ("maxima.info-2" 73321 56 "Funciones y variables para trigonometría"))
("asin" . ("maxima.info-2" 73380 40 "Funciones y variables para trigonometría"))
("asinh" . ("maxima.info-2" 73422 53 "Funciones y variables para trigonometría"))
("askexp" . ("maxima.info-4" 286277 133 "Funciones y variables para la miscelánea de opciones"))
("askinteger" . ("maxima.info-2" 113976 710 "Funciones y variables para los hechos"))
("askinteger <1>" . ("maxima.info-2" 113977 709 "Funciones y variables para los hechos"))
("askinteger <2>" . ("maxima.info-2" 114020 667 "Funciones y variables para los hechos"))
("askinteger <3>" . ("maxima.info-2" 114054 634 "Funciones y variables para los hechos"))
("asksign" . ("maxima.info-2" 114695 412 "Funciones y variables para los hechos"))
("assoc" . ("maxima.info-1" 124939 688 "Funciones y variables para listas"))
("assoc <1>" . ("maxima.info-1" 124940 687 "Funciones y variables para listas"))
("assoc_legendre_p" . ("maxima.info-7" 300135 264 "Funciones y variables para polinomios ortogonales"))
("assoc_legendre_q" . ("maxima.info-7" 300406 195 "Funciones y variables para polinomios ortogonales"))
("assume" . ("maxima.info-2" 115114 2557 "Funciones y variables para los hechos"))
("assume_external_byte_order" . ("maxima.info-7" 274531 535 "Funciones y variables para entrada y salida en formato binario"))
("assume_pos" . ("maxima.info-2" 118694 1390 "Funciones y variables para los hechos"))
("assume_pos_pred" . ("maxima.info-2" 120094 2284 "Funciones y variables para los hechos"))
("assumescalar" . ("maxima.info-2" 117686 995 "Funciones y variables para los hechos"))
("asymbol" . ("maxima.info-4" 141982 88 "Funciones y variables para atensor"))
("at" . ("maxima.info-3" 67415 1921 "Funciones y variables para la diferenciación"))
("at <1>" . ("maxima.info-3" 67416 1920 "Funciones y variables para la diferenciación"))
("atan" . ("maxima.info-2" 73478 44 "Funciones y variables para trigonometría"))
("atan2" . ("maxima.info-2" 73524 111 "Funciones y variables para trigonometría"))
("atanh" . ("maxima.info-2" 73637 57 "Funciones y variables para trigonometría"))
("atensimp" . ("maxima.info-4" 140957 443 "Funciones y variables para atensor"))
("atom" . ("maxima.info-1" 265989 337 "Funciones y variables para expresiones"))
("atomgrad" . ("maxima.info-3" 69349 80 "Funciones y variables para la diferenciación"))
("atrig1" . ("maxima.info-2" 73697 464 "Funciones y variables para trigonometría"))
("atvalue" . ("maxima.info-3" 69430 2614 "Funciones y variables para la diferenciación"))
("atvalue <1>" . ("maxima.info-3" 69431 2613 "Funciones y variables para la diferenciación"))
("augcoefmatrix" . ("maxima.info-3" 237749 706 "Funciones y variables para las matrices y el álgebra lineal"))
("augmented_lagrangian_method" . ("maxima.info-5" 192122 3982 "Funciones y variables para augmented_lagrangian"))
("augmented_lagrangian_method <1>" . ("maxima.info-5" 192123 3981 "Funciones y variables para augmented_lagrangian"))
("augmented_lagrangian_method <2>" . ("maxima.info-5" 192257 3849 "Funciones y variables para augmented_lagrangian"))
("augmented_lagrangian_method <3>" . ("maxima.info-5" 192353 3754 "Funciones y variables para augmented_lagrangian"))
("av" . ("maxima.info-4" 142670 1020 "Funciones y variables para atensor"))
("average_degree" . ("maxima.info-7" 89986 345 "Funciones y variables para graphs"))
("axes" . ("maxima.info-2" 159919 377 "Opciones gráficas"))
("axis_3d" . ("maxima.info-6" 131449 529 "Funciones y variables para draw"))
("axis_bottom" . ("maxima.info-6" 131987 490 "Funciones y variables para draw"))
("axis_left" . ("maxima.info-6" 132486 487 "Funciones y variables para draw"))
("axis_right" . ("maxima.info-6" 132982 487 "Funciones y variables para draw"))
("axis_top" . ("maxima.info-6" 133478 489 "Funciones y variables para draw"))
("azimuth" . ("maxima.info-2" 160302 518 "Opciones gráficas"))
("azimuth <1>" . ("maxima.info-7" 2946 309 "Visualización usando VTK"))
("background" . ("maxima.info-7" 3265 268 "Visualización usando VTK"))
("background_color" . ("maxima.info-6" 133976 444 "Funciones y variables para draw"))
("backsubst" . ("maxima.info-3" 157292 749 "Funciones y variable para las ecuaciones"))
("backtrace" . ("maxima.info-5" 142264 1258 "Funciones y variables para la programación"))
("backtrace <1>" . ("maxima.info-5" 142265 1257 "Funciones y variables para la programación"))
("bars" . ("maxima.info-6" 221847 907 "Funciones y variables para draw"))
("barsplot" . ("maxima.info-6" 339 5803 "Funciones y variables para gráficos estadísticos"))
("barsplot_description" . ("maxima.info-6" 411 5732 "Funciones y variables para gráficos estadísticos"))
("base64" . ("maxima.info-8" 111095 347 "Funciones y variables para cadenas"))
("base64_decode" . ("maxima.info-8" 111445 200 "Funciones y variables para cadenas"))
("bashindices" . ("maxima.info-4" 144672 406 "Funciones y variables para sumas y productos"))
("batch" . ("maxima.info-2" 181652 2520 "Funciones y variables para lectura y escritura"))
("batch <1>" . ("maxima.info-2" 181653 2519 "Funciones y variables para lectura y escritura"))
("batchload" . ("maxima.info-2" 184203 653 "Funciones y variables para lectura y escritura"))
("bc2" . ("maxima.info-3" 192505 599 "Funciones y variables para ecuaciones diferenciales"))
("bdvac" . ("maxima.info-4" 110191 482 "Funciones y variables para ctensor"))
("belln" . ("maxima.info-5" 11880 967 "Funciones y variables para los conjuntos"))
("benefit_cost" . ("maxima.info-7" 68644 702 "Funciones y Variables para finance"))
("berlefact" . ("maxima.info-2" 221406 267 "Funciones y variables para polinomios"))
("bern" . ("maxima.info-4" 193279 780 "Funciones y variables para teoría de números"))
("bernpoly" . ("maxima.info-4" 194066 100 "Funciones y variables para teoría de números"))
("bernstein_approx" . ("maxima.info-5" 199517 1224 "Funciones y variables para Bernstein"))
("bernstein_expand" . ("maxima.info-5" 200746 528 "Funciones y variables para Bernstein"))
("bernstein_explicit" . ("maxima.info-5" 198417 669 "Funciones y variables para Bernstein"))
("bernstein_poly" . ("maxima.info-5" 196480 1930 "Funciones y variables para Bernstein"))
("bessel_i" . ("maxima.info-2" 286576 545 "Funciones de Bessel"))
("bessel_j" . ("maxima.info-2" 285595 525 "Funciones de Bessel"))
("bessel_k" . ("maxima.info-2" 287126 461 "Funciones de Bessel"))
("bessel_y" . ("maxima.info-2" 286125 446 "Funciones de Bessel"))
("besselexpand" . ("maxima.info-2" 290400 1016 "Funciones de Bessel"))
("beta" . ("maxima.info-3" 10459 4306 "Funciones Gamma y factorial"))
("beta_args_sum_to_integer" . ("maxima.info-3" 31439 225 "Funciones Gamma y factorial"))
("beta_expand" . ("maxima.info-3" 31210 227 "Funciones Gamma y factorial"))
("beta_incomplete" . ("maxima.info-3" 14798 6248 "Funciones Gamma y factorial"))
("beta_incomplete_generalized" . ("maxima.info-3" 25517 5674 "Funciones Gamma y factorial"))
("beta_incomplete_regularized" . ("maxima.info-3" 21079 4421 "Funciones Gamma y factorial"))
("bezout" . ("maxima.info-2" 221676 574 "Funciones y variables para polinomios"))
("bf_find_root" . ("maxima.info-3" 211223 4078 "Funciones para la resolución numérica de ecuaciones"))
("bf_find_root <1>" . ("maxima.info-3" 211285 4017 "Funciones para la resolución numérica de ecuaciones"))
("bf_fmin_cobyla" . ("maxima.info-5" 209905 399 "Funciones y variables para cobyla"))
("bf_fmin_cobyla <1>" . ("maxima.info-5" 209906 398 "Funciones y variables para cobyla"))
("bfallroots" . ("maxima.info-3" 156828 450 "Funciones y variable para las ecuaciones"))
("bfallroots <1>" . ("maxima.info-3" 156829 449 "Funciones y variable para las ecuaciones"))
("bffac" . ("maxima.info-3" 1995 241 "Funciones Gamma y factorial"))
("bfhzeta" . ("maxima.info-4" 194449 629 "Funciones y variables para teoría de números"))
("bfloat" . ("maxima.info-1" 100514 507 "Funciones y variables para los números"))
("bfloatp" . ("maxima.info-1" 101033 163 "Funciones y variables para los números"))
("bfpsi" . ("maxima.info-3" 2360 414 "Funciones Gamma y factorial"))
("bfpsi0" . ("maxima.info-3" 2361 413 "Funciones Gamma y factorial"))
("bftorat" . ("maxima.info-1" 101199 511 "Funciones y variables para los números"))
("bftrunc" . ("maxima.info-1" 101721 326 "Funciones y variables para los números"))
("bfzeta" . ("maxima.info-4" 194169 274 "Funciones y variables para teoría de números"))
("biconnected_components" . ("maxima.info-7" 90334 492 "Funciones y variables para graphs"))
("bindtest" . ("maxima.info-2" 92526 626 "Funciones y variables para las propiedades"))
("binomial" . ("maxima.info-2" 51557 989 "Funciones combinatorias"))
("bipartition" . ("maxima.info-7" 90829 405 "Funciones y variables para graphs"))
("block" . ("maxima.info-5" 80734 3235 "Funciones y variables para la definición de funciones"))
("block <1>" . ("maxima.info-5" 80735 3234 "Funciones y variables para la definición de funciones"))
("blockmatrixp" . ("maxima.info-7" 205413 138 "Funciones y variables para linearalgebra"))
("bode_gain" . ("maxima.info-5" 201576 1428 "Funciones y variables para bode"))
("bode_phase" . ("maxima.info-5" 203012 1673 "Funciones y variables para bode"))
("border" . ("maxima.info-6" 134430 617 "Funciones y variables para draw"))
("bothcoef" . ("maxima.info-2" 222253 629 "Funciones y variables para polinomios"))
("boundaries_array" . ("maxima.info-6" 260534 401 "Funciones y variables para worldmap"))
("box" . ("maxima.info-2" 160829 163 "Opciones gráficas"))
("box <1>" . ("maxima.info-1" 266334 2020 "Funciones y variables para expresiones"))
("box <2>" . ("maxima.info-1" 266335 2019 "Funciones y variables para expresiones"))
("boxchar" . ("maxima.info-1" 268368 332 "Funciones y variables para expresiones"))
("boxplot" . ("maxima.info-6" 6183 4129 "Funciones y variables para gráficos estadísticos"))
("boxplot <1>" . ("maxima.info-6" 6184 4128 "Funciones y variables para gráficos estadísticos"))
("boxplot_description" . ("maxima.info-6" 6215 4098 "Funciones y variables para gráficos estadísticos"))
("break" . ("maxima.info-5" 83998 281 "Funciones y variables para la definición de funciones"))
("breakup" . ("maxima.info-3" 158046 3315 "Funciones y variable para las ecuaciones"))
("bug_report" . ("maxima.info-1" 26940 490 "Funciones y variables para la detección e informe de fallos"))
("build_info" . ("maxima.info-1" 27438 1569 "Funciones y variables para la detección e informe de fallos"))
("build_sample" . ("maxima.info-5" 231499 2396 "Funciones y variables para el tratamiento de datos"))
("build_sample <1>" . ("maxima.info-5" 231500 2395 "Funciones y variables para el tratamiento de datos"))
("buildq" . ("maxima.info-5" 69021 4544 "Macros"))
("burn" . ("maxima.info-4" 195086 996 "Funciones y variables para teoría de números"))
("cabs" . ("maxima.info-2" 45634 2550 "Funciones para los números complejos"))
("canform" . ("maxima.info-4" 27720 988 "Funciones y variables para itensor"))
("canform <1>" . ("maxima.info-4" 27721 987 "Funciones y variables para itensor"))
("canten" . ("maxima.info-4" 23732 601 "Funciones y variables para itensor"))
("capping" . ("maxima.info-6" 135053 595 "Funciones y variables para draw"))
("capping <1>" . ("maxima.info-7" 8183 263 "Visualización usando VTK"))
("cardinality" . ("maxima.info-5" 12855 556 "Funciones y variables para los conjuntos"))
("carg" . ("maxima.info-2" 48206 1048 "Funciones para los números complejos"))
("cartan" . ("maxima.info-3" 72059 643 "Funciones y variables para la diferenciación"))
("cartesian_product" . ("maxima.info-5" 13417 785 "Funciones y variables para los conjuntos"))
("catch" . ("maxima.info-5" 84284 1159 "Funciones y variables para la definición de funciones"))
("cauchy_matrix" . ("maxima.info-3" 238460 1381 "Funciones y variables para las matrices y el álgebra lineal"))
("cauchy_matrix <1>" . ("maxima.info-3" 238531 1311 "Funciones y variables para las matrices y el álgebra lineal"))
("cauchysum" . ("maxima.info-4" 158083 1270 "Funciones y variables para las series"))
("cbffac" . ("maxima.info-3" 2780 173 "Funciones Gamma y factorial"))
("cbrange" . ("maxima.info-6" 135653 975 "Funciones y variables para draw"))
("cbtics" . ("maxima.info-6" 136641 734 "Funciones y variables para draw"))
("cdf_bernoulli" . ("maxima.info-6" 107795 259 "Funciones y variables para distribuciones discretas"))
("cdf_beta" . ("maxima.info-6" 75713 441 "Funciones y variables para distribuciones continuas"))
("cdf_binomial" . ("maxima.info-6" 102365 482 "Funciones y variables para distribuciones discretas"))
("cdf_cauchy" . ("maxima.info-6" 94271 241 "Funciones y variables para distribuciones continuas"))
("cdf_chi2" . ("maxima.info-6" 58027 491 "Funciones y variables para distribuciones continuas"))
("cdf_continuous_uniform" . ("maxima.info-6" 78223 267 "Funciones y variables para distribuciones continuas"))
("cdf_discrete_uniform" . ("maxima.info-6" 113298 273 "Funciones y variables para distribuciones discretas"))
("cdf_empirical" . ("maxima.info-5" 263764 1009 "Funciones y variables de parámetros descriptivos"))
("cdf_empirical <1>" . ("maxima.info-5" 263765 1008 "Funciones y variables de parámetros descriptivos"))
("cdf_exp" . ("maxima.info-6" 66759 392 "Funciones y variables para distribuciones continuas"))
("cdf_f" . ("maxima.info-6" 63751 478 "Funciones y variables para distribuciones continuas"))
("cdf_gamma" . ("maxima.info-6" 72930 485 "Funciones y variables para distribuciones continuas"))
("cdf_general_finite_discrete" . ("maxima.info-6" 99077 896 "Funciones y variables para distribuciones discretas"))
("cdf_geometric" . ("maxima.info-6" 110959 254 "Funciones y variables para distribuciones discretas"))
("cdf_gumbel" . ("maxima.info-6" 95476 241 "Funciones y variables para distribuciones continuas"))
("cdf_hypergeometric" . ("maxima.info-6" 116146 381 "Funciones y variables para distribuciones discretas"))
("cdf_laplace" . ("maxima.info-6" 92033 243 "Funciones y variables para distribuciones continuas"))
("cdf_logistic" . ("maxima.info-6" 80703 243 "Funciones y variables para distribuciones continuas"))
("cdf_lognormal" . ("maxima.info-6" 70238 632 "Funciones y variables para distribuciones continuas"))
("cdf_negative_binomial" . ("maxima.info-6" 118952 427 "Funciones y variables para distribuciones discretas"))
("cdf_noncentral_chi2" . ("maxima.info-6" 61402 248 "Funciones y variables para distribuciones continuas"))
("cdf_noncentral_student_t" . ("maxima.info-6" 53635 542 "Funciones y variables para distribuciones continuas"))
("cdf_normal" . ("maxima.info-6" 46480 632 "Funciones y variables para distribuciones continuas"))
("cdf_pareto" . ("maxima.info-6" 82956 243 "Funciones y variables para distribuciones continuas"))
("cdf_poisson" . ("maxima.info-6" 105197 363 "Funciones y variables para distribuciones discretas"))
("cdf_rank_sum" . ("maxima.info-8" 89274 288 "Funciones y variables para distribuciones especiales"))
("cdf_rayleigh" . ("maxima.info-6" 87671 448 "Funciones y variables para distribuciones continuas"))
("cdf_signed_rank" . ("maxima.info-8" 88691 279 "Funciones y variables para distribuciones especiales"))
("cdf_student_t" . ("maxima.info-6" 49512 603 "Funciones y variables para distribuciones continuas"))
("cdf_weibull" . ("maxima.info-6" 85210 245 "Funciones y variables para distribuciones continuas"))
("cdisplay" . ("maxima.info-4" 111243 13864 "Funciones y variables para ctensor"))
("ceiling" . ("maxima.info-2" 37279 3407 "Funciones para los números"))
("center" . ("maxima.info-7" 8450 425 "Visualización usando VTK"))
("central_moment" . ("maxima.info-5" 249517 928 "Funciones y variables de parámetros descriptivos"))
("central_moment <1>" . ("maxima.info-5" 249518 927 "Funciones y variables de parámetros descriptivos"))
("cequal" . ("maxima.info-8" 108574 108 "Funciones y variables para caracteres"))
("cequalignore" . ("maxima.info-8" 108685 132 "Funciones y variables para caracteres"))
("cf" . ("maxima.info-4" 197058 3176 "Funciones y variables para teoría de números"))
("cfdisrep" . ("maxima.info-4" 200277 700 "Funciones y variables para teoría de números"))
("cfexpand" . ("maxima.info-4" 200983 638 "Funciones y variables para teoría de números"))
("cflength" . ("maxima.info-4" 201626 654 "Funciones y variables para teoría de números"))
("cframe_flag" . ("maxima.info-4" 126170 127 "Funciones y variables para ctensor"))
("cgeodesic" . ("maxima.info-4" 109899 287 "Funciones y variables para ctensor"))
("cgreaterp" . ("maxima.info-8" 108822 130 "Funciones y variables para caracteres"))
("cgreaterpignore" . ("maxima.info-8" 108955 138 "Funciones y variables para caracteres"))
("changename" . ("maxima.info-4" 914 401 "Funciones y variables para itensor"))
("changevar" . ("maxima.info-3" 92678 2347 "Funciones y variables para integración"))
("chaosgame" . ("maxima.info-6" 284645 835 "Análisis gráfico de sistemas dinámicos discretos"))
("charat" . ("maxima.info-8" 111648 225 "Funciones y variables para cadenas"))
("charfun" . ("maxima.info-2" 130237 514 "Funciones y variables para los predicados"))
("charfun2" . ("maxima.info-7" 152100 141 "Funciones y variables para interpol"))
("charlist" . ("maxima.info-8" 111878 245 "Funciones y variables para cadenas"))
("charp" . ("maxima.info-8" 109098 83 "Funciones y variables para caracteres"))
("charpoly" . ("maxima.info-3" 239846 1622 "Funciones y variables para las matrices y el álgebra lineal"))
("chebyshev_t" . ("maxima.info-7" 300604 151 "Funciones y variables para polinomios ortogonales"))
("chebyshev_u" . ("maxima.info-7" 300760 151 "Funciones y variables para polinomios ortogonales"))
("check_overlaps" . ("maxima.info-3" 292816 642 "Funciones y variables para Afines"))
("checkdiv" . ("maxima.info-4" 109615 281 "Funciones y variables para ctensor"))
("chinese" . ("maxima.info-4" 196100 955 "Funciones y variables para teoría de números"))
("cholesky" . ("maxima.info-7" 207056 309 "Funciones y variables para linearalgebra"))
("cholesky <1>" . ("maxima.info-7" 207057 308 "Funciones y variables para linearalgebra"))
("christof" . ("maxima.info-4" 85601 856 "Funciones y variables para ctensor"))
("chromatic_index" . ("maxima.info-7" 91239 258 "Funciones y variables para graphs"))
("chromatic_number" . ("maxima.info-7" 91501 336 "Funciones y variables para graphs"))
("cint" . ("maxima.info-8" 109184 68 "Funciones y variables para caracteres"))
("circulant_graph" . ("maxima.info-7" 81842 575 "Funciones y variables para graphs"))
("clear_edge_weight" . ("maxima.info-7" 91841 441 "Funciones y variables para graphs"))
("clear_rules" . ("maxima.info-4" 333548 185 "Funciones y variables sobre reglas y patrones"))
("clear_vertex_label" . ("maxima.info-7" 92284 493 "Funciones y variables para graphs"))
("clebsch_graph" . ("maxima.info-7" 82420 65 "Funciones y variables para graphs"))
("clessp" . ("maxima.info-8" 109255 127 "Funciones y variables para caracteres"))
("clesspignore" . ("maxima.info-8" 109385 132 "Funciones y variables para caracteres"))
("close" . ("maxima.info-8" 98206 124 "Funciones y variables para entrada y salida"))
("closefile" . ("maxima.info-2" 184867 120 "Funciones y variables para lectura y escritura"))
("cmetric" . ("maxima.info-4" 77299 944 "Funciones y variables para ctensor"))
("cmetric <1>" . ("maxima.info-4" 77300 943 "Funciones y variables para ctensor"))
("cnonmet_flag" . ("maxima.info-4" 126585 311 "Funciones y variables para ctensor"))
("coeff" . ("maxima.info-2" 222886 3702 "Funciones y variables para polinomios"))
("coeff <1>" . ("maxima.info-2" 222887 3701 "Funciones y variables para polinomios"))
("coefmatrix" . ("maxima.info-3" 241471 451 "Funciones y variables para las matrices y el álgebra lineal"))
("cograd" . ("maxima.info-4" 107304 251 "Funciones y variables para ctensor"))
("col" . ("maxima.info-3" 241924 135 "Funciones y variables para las matrices y el álgebra lineal"))
("collapse" . ("maxima.info-1" 268705 546 "Funciones y variables para expresiones"))
("collectterms" . ("maxima.info-8" 18874 813 "Paquete facexp"))
("color" . ("maxima.info-2" 160996 928 "Opciones gráficas"))
("color <1>" . ("maxima.info-6" 137385 1983 "Funciones y variables para draw"))
("color <2>" . ("maxima.info-7" 8884 248 "Visualización usando VTK"))
("colorbox" . ("maxima.info-2" 161933 410 "Opciones gráficas"))
("colorbox <1>" . ("maxima.info-6" 139378 1078 "Funciones y variables para draw"))
("columnop" . ("maxima.info-7" 205553 259 "Funciones y variables para linearalgebra"))
("columns" . ("maxima.info-6" 140469 660 "Funciones y variables para draw"))
("columnspace" . ("maxima.info-7" 206014 195 "Funciones y variables para linearalgebra"))
("columnswap" . ("maxima.info-7" 205815 197 "Funciones y variables para linearalgebra"))
("columnvector" . ("maxima.info-3" 242062 859 "Funciones y variables para las matrices y el álgebra lineal"))
("combination" . ("maxima.info-8" 26129 181 "Paquete functs"))
("combine" . ("maxima.info-2" 7142 145 "Funciones y variables para simplificación"))
("commutative" . ("maxima.info-2" 7293 234 "Funciones y variables para simplificación"))
("comp2pui" . ("maxima.info-4" 233291 583 "Funciones y variables para simetrías"))
("compare" . ("maxima.info-2" 130753 968 "Funciones y variables para los predicados"))
("compfile" . ("maxima.info-5" 85461 837 "Funciones y variables para la definición de funciones"))
("compfile <1>" . ("maxima.info-5" 85462 836 "Funciones y variables para la definición de funciones"))
("compfile <2>" . ("maxima.info-5" 85517 781 "Funciones y variables para la definición de funciones"))
("compile" . ("maxima.info-5" 86307 596 "Funciones y variables para la definición de funciones"))
("compile <1>" . ("maxima.info-5" 86308 595 "Funciones y variables para la definición de funciones"))
("compile <2>" . ("maxima.info-5" 86350 554 "Funciones y variables para la definición de funciones"))
("compile_file" . ("maxima.info-5" 134168 1410 "Funciones y variables para la definición de funciones"))
("compile_file <1>" . ("maxima.info-5" 134169 1409 "Funciones y variables para la definición de funciones"))
("compile_file <2>" . ("maxima.info-5" 134212 1367 "Funciones y variables para la definición de funciones"))
("complement_graph" . ("maxima.info-7" 82487 80 "Funciones y variables para graphs"))
("complete_bipartite_graph" . ("maxima.info-7" 82569 110 "Funciones y variables para graphs"))
("complete_graph" . ("maxima.info-7" 82682 83 "Funciones y variables para graphs"))
("complex" . ("maxima.info-2" 110628 268 "Funciones y variables para las propiedades"))
("complex_number_p" . ("maxima.info-8" 132959 795 "Funciones y variables para to_poly_solve"))
("components" . ("maxima.info-4" 9654 4082 "Funciones y variables para itensor"))
("compose_functions" . ("maxima.info-8" 133762 1951 "Funciones y variables para to_poly_solve"))
("concan" . ("maxima.info-4" 24349 112 "Funciones y variables para itensor"))
("concat" . ("maxima.info-1" 114205 1327 "Funciones y variables para las cadenas de texto"))
("cone" . ("maxima.info-7" 5377 882 "Visualización usando VTK"))
("conjugate" . ("maxima.info-2" 49261 536 "Funciones para los números complejos"))
("conmetderiv" . ("maxima.info-4" 36360 859 "Funciones y variables para itensor"))
("connect_vertices" . ("maxima.info-7" 118353 567 "Funciones y variables para graphs"))
("connected_components" . ("maxima.info-7" 92780 311 "Funciones y variables para graphs"))
("cons" . ("maxima.info-1" 125633 281 "Funciones y variables para listas"))
("constant" . ("maxima.info-2" 93154 552 "Funciones y variables para las propiedades"))
("constantp" . ("maxima.info-2" 93711 1189 "Funciones y variables para las propiedades"))
("constituent" . ("maxima.info-8" 109522 602 "Funciones y variables para caracteres"))
("constvalue" . ("maxima.info-7" 37358 1542 "Funciones y variables para ezunits"))
("cont2part" . ("maxima.info-4" 233879 598 "Funciones y variables para simetrías"))
("content" . ("maxima.info-2" 226600 467 "Funciones y variables para polinomios"))
("context" . ("maxima.info-2" 122395 587 "Funciones y variables para los hechos"))
("contexts" . ("maxima.info-2" 122989 1495 "Funciones y variables para los hechos"))
("continuous_freq" . ("maxima.info-5" 233901 1882 "Funciones y variables para el tratamiento de datos"))
("continuous_freq <1>" . ("maxima.info-5" 233902 1881 "Funciones y variables para el tratamiento de datos"))
("contortion" . ("maxima.info-4" 104376 115 "Funciones y variables para ctensor"))
("contour" . ("maxima.info-6" 141141 1092 "Funciones y variables para draw"))
("contour_levels" . ("maxima.info-6" 142251 2108 "Funciones y variables para draw"))
("contour_plot" . ("maxima.info-2" 143317 933 "Funciones y variables para gráficos"))
("contract" . ("maxima.info-4" 8926 475 "Funciones y variables para itensor"))
("contract <1>" . ("maxima.info-4" 234481 993 "Funciones y variables para simetrías"))
("contract_edge" . ("maxima.info-7" 118926 779 "Funciones y variables para graphs"))
("contragrad" . ("maxima.info-4" 107561 1162 "Funciones y variables para ctensor"))
("contrib_ode" . ("maxima.info-5" 217170 197 "Funciones y variables para contrib_ode"))
("convert" . ("maxima.info-8" 180348 3414 "Funciones y variables para units"))
("coord" . ("maxima.info-4" 35340 437 "Funciones y variables para itensor"))
("copy" . ("maxima.info-7" 206211 836 "Funciones y variables para linearalgebra"))
("copy_graph" . ("maxima.info-7" 81771 69 "Funciones y variables para graphs"))
("copylist" . ("maxima.info-1" 125919 72 "Funciones y variables para listas"))
("copymatrix" . ("maxima.info-3" 242926 503 "Funciones y variables para las matrices y el álgebra lineal"))
("cor" . ("maxima.info-5" 269210 2215 "Funciones y variables de parámetros descriptivos"))
("cor <1>" . ("maxima.info-5" 269211 2214 "Funciones y variables de parámetros descriptivos"))
("cos" . ("maxima.info-2" 74170 36 "Funciones y variables para trigonometría"))
("cosh" . ("maxima.info-2" 74208 49 "Funciones y variables para trigonometría"))
("cosnpiflag" . ("maxima.info-4" 188831 89 "Funciones y variables para series de Fourier"))
("cot" . ("maxima.info-2" 74260 40 "Funciones y variables para trigonometría"))
("coth" . ("maxima.info-2" 74302 53 "Funciones y variables para trigonometría"))
("cov" . ("maxima.info-5" 264784 1277 "Funciones y variables de parámetros descriptivos"))
("cov1" . ("maxima.info-5" 266066 1280 "Funciones y variables de parámetros descriptivos"))
("covdiff" . ("maxima.info-4" 42458 1900 "Funciones y variables para itensor"))
("covect" . ("maxima.info-3" 242063 858 "Funciones y variables para las matrices y el álgebra lineal"))
("covers" . ("maxima.info-8" 25766 119 "Paquete functs"))
("create_graph" . ("maxima.info-7" 79900 1858 "Funciones y variables para graphs"))
("create_graph <1>" . ("maxima.info-7" 79901 1857 "Funciones y variables para graphs"))
("create_graph <2>" . ("maxima.info-7" 79949 1810 "Funciones y variables para graphs"))
("create_list" . ("maxima.info-1" 125993 1306 "Funciones y variables para listas"))
("csc" . ("maxima.info-2" 74358 39 "Funciones y variables para trigonometría"))
("csch" . ("maxima.info-2" 74399 52 "Funciones y variables para trigonometría"))
("csetup" . ("maxima.info-4" 77092 201 "Funciones y variables para ctensor"))
("cspline" . ("maxima.info-7" 154860 4665 "Funciones y variables para interpol"))
("cspline <1>" . ("maxima.info-7" 154861 4664 "Funciones y variables para interpol"))
("ct_coords" . ("maxima.info-4" 131880 384 "Funciones y variables para ctensor"))
("ct_coordsys" . ("maxima.info-4" 78261 5828 "Funciones y variables para ctensor"))
("ct_coordsys <1>" . ("maxima.info-4" 78262 5827 "Funciones y variables para ctensor"))
("ctaylor" . ("maxima.info-4" 91913 5445 "Funciones y variables para ctensor"))
("ctaypov" . ("maxima.info-4" 127313 131 "Funciones y variables para ctensor"))
("ctaypt" . ("maxima.info-4" 127446 134 "Funciones y variables para ctensor"))
("ctayswitch" . ("maxima.info-4" 126900 276 "Funciones y variables para ctensor"))
("ctayvar" . ("maxima.info-4" 127179 133 "Funciones y variables para ctensor"))
("ctorsion_flag" . ("maxima.info-4" 126300 279 "Funciones y variables para ctensor"))
("ctransform" . ("maxima.info-4" 104682 262 "Funciones y variables para ctensor"))
("ctranspose" . ("maxima.info-7" 207372 211 "Funciones y variables para linearalgebra"))
("ctrgsimp" . ("maxima.info-4" 125924 241 "Funciones y variables para ctensor"))
("cube" . ("maxima.info-7" 6273 351 "Visualización usando VTK"))
("cube_graph" . ("maxima.info-7" 83008 72 "Funciones y variables para graphs"))
("cuboctahedron_graph" . ("maxima.info-7" 82927 78 "Funciones y variables para graphs"))
("cunlisp" . ("maxima.info-8" 110133 155 "Funciones y variables para caracteres"))
("current_let_rule_package" . ("maxima.info-4" 296240 647 "Funciones y variables sobre reglas y patrones"))
("cv" . ("maxima.info-5" 250450 629 "Funciones y variables de parámetros descriptivos"))
("cv <1>" . ("maxima.info-5" 250451 628 "Funciones y variables de parámetros descriptivos"))
("cycle_digraph" . ("maxima.info-7" 82768 82 "Funciones y variables para graphs"))
("cycle_graph" . ("maxima.info-7" 82853 71 "Funciones y variables para graphs"))
("cylinder" . ("maxima.info-7" 6628 682 "Visualización usando VTK"))
("cylindrical" . ("maxima.info-6" 222757 730 "Funciones y variables para draw"))
("data_file_name" . ("maxima.info-6" 144390 457 "Funciones y variables para draw"))
("days360" . ("maxima.info-7" 58287 298 "Funciones y Variables para finance"))
("dblint" . ("maxima.info-3" 95039 2360 "Funciones y variables para integración"))
("deactivate" . ("maxima.info-2" 124501 139 "Funciones y variables para los hechos"))
("debugmode" . ("maxima.info-5" 180815 479 "Funciones y variables para depurado"))
("declare" . ("maxima.info-2" 94909 5548 "Funciones y variables para las propiedades"))
("declare_constvalue" . ("maxima.info-7" 37359 1541 "Funciones y variables para ezunits"))
("declare_dimensions" . ("maxima.info-7" 46263 1139 "Funciones y variables para ezunits"))
("declare_fundamental_dimensions" . ("maxima.info-7" 47410 1393 "Funciones y variables para ezunits"))
("declare_fundamental_units" . ("maxima.info-7" 48811 1685 "Funciones y variables para ezunits"))
("declare_qty" . ("maxima.info-7" 41346 1708 "Funciones y variables para ezunits"))
("declare_translated" . ("maxima.info-5" 135598 459 "Funciones y variables para la definición de funciones"))
("declare_unit_conversion" . ("maxima.info-7" 44326 1921 "Funciones y variables para ezunits"))
("declare_units" . ("maxima.info-7" 38910 2426 "Funciones y variables para ezunits"))
("declare_weights" . ("maxima.info-3" 291645 276 "Funciones y variables para Afines"))
("decreasing" . ("maxima.info-2" 100515 621 "Funciones y variables para las propiedades"))
("decsym" . ("maxima.info-4" 24969 2558 "Funciones y variables para itensor"))
("default_let_rule_package" . ("maxima.info-4" 296892 318 "Funciones y variables sobre reglas y patrones"))
("defcon" . ("maxima.info-4" 7588 1044 "Funciones y variables para itensor"))
("defcon <1>" . ("maxima.info-4" 7589 1043 "Funciones y variables para itensor"))
("define" . ("maxima.info-5" 86914 4182 "Funciones y variables para la definición de funciones"))
("define <1>" . ("maxima.info-5" 86915 4181 "Funciones y variables para la definición de funciones"))
("define <2>" . ("maxima.info-5" 86969 4128 "Funciones y variables para la definición de funciones"))
("define <3>" . ("maxima.info-5" 87023 4075 "Funciones y variables para la definición de funciones"))
("define <4>" . ("maxima.info-5" 87089 4010 "Funciones y variables para la definición de funciones"))
("define_variable" . ("maxima.info-5" 91144 4013 "Funciones y variables para la definición de funciones"))
("defint" . ("maxima.info-3" 97434 569 "Funciones y variables para integración"))
("defmatch" . ("maxima.info-4" 297212 4752 "Funciones y variables sobre reglas y patrones"))
("defmatch <1>" . ("maxima.info-4" 297213 4751 "Funciones y variables sobre reglas y patrones"))
("defrule" . ("maxima.info-4" 302003 659 "Funciones y variables sobre reglas y patrones"))
("defstruct" . ("maxima.info-1" 173083 1682 "Funciones y variables para las estructuras"))
("defstruct <1>" . ("maxima.info-1" 173084 1681 "Funciones y variables para las estructuras"))
("deftaylor" . ("maxima.info-4" 159358 1329 "Funciones y variables para las series"))
("degree_sequence" . ("maxima.info-7" 94073 268 "Funciones y variables para graphs"))
("del" . ("maxima.info-3" 72711 722 "Funciones y variables para la diferenciación"))
("delay" . ("maxima.info-6" 144860 922 "Funciones y variables para draw"))
("delete" . ("maxima.info-1" 127317 2481 "Funciones y variables para listas"))
("delete <1>" . ("maxima.info-1" 127318 2480 "Funciones y variables para listas"))
("deleten" . ("maxima.info-4" 125111 114 "Funciones y variables para ctensor"))
("delta" . ("maxima.info-3" 73438 391 "Funciones y variables para la diferenciación"))
("demo" . ("maxima.info-1" 31770 2994 "Funciones y variables para la ayuda"))
("demoivre" . ("maxima.info-2" 7530 732 "Funciones y variables para simplificación"))
("demoivre <1>" . ("maxima.info-2" 7531 731 "Funciones y variables para simplificación"))
("denom" . ("maxima.info-2" 227072 90 "Funciones y variables para polinomios"))
("dependencies" . ("maxima.info-3" 73834 361 "Funciones y variables para la diferenciación"))
("depends" . ("maxima.info-3" 74200 2643 "Funciones y variables para la diferenciación"))
("derivabbrev" . ("maxima.info-3" 76864 278 "Funciones y variables para la diferenciación"))
("derivdegree" . ("maxima.info-3" 77146 606 "Funciones y variables para la diferenciación"))
("derivlist" . ("maxima.info-3" 77754 164 "Funciones y variables para la diferenciación"))
("derivsubst" . ("maxima.info-3" 77921 220 "Funciones y variables para la diferenciación"))
("describe" . ("maxima.info-1" 34772 3035 "Funciones y variables para la ayuda"))
("describe <1>" . ("maxima.info-1" 34773 3034 "Funciones y variables para la ayuda"))
("describe <2>" . ("maxima.info-1" 34807 3001 "Funciones y variables para la ayuda"))
("desolve" . ("maxima.info-3" 193110 2241 "Funciones y variables para ecuaciones diferenciales"))
("desolve <1>" . ("maxima.info-3" 193111 2240 "Funciones y variables para ecuaciones diferenciales"))
("determinant" . ("maxima.info-3" 243438 379 "Funciones y variables para las matrices y el álgebra lineal"))
("detout" . ("maxima.info-3" 243822 972 "Funciones y variables para las matrices y el álgebra lineal"))
("dfloat" . ("maxima.info-8" 135728 840 "Funciones y variables para to_poly_solve"))
("dgauss_a" . ("maxima.info-5" 220982 104 "Funciones y variables para contrib_ode"))
("dgauss_b" . ("maxima.info-5" 221088 104 "Funciones y variables para contrib_ode"))
("dgeev" . ("maxima.info-7" 161634 3071 "Funciones y variables para lapack"))
("dgeev <1>" . ("maxima.info-7" 161635 3070 "Funciones y variables para lapack"))
("dgemm" . ("maxima.info-7" 176011 5507 "Funciones y variables para lapack"))
("dgemm <1>" . ("maxima.info-7" 176012 5506 "Funciones y variables para lapack"))
("dgeqrf" . ("maxima.info-7" 164720 1433 "Funciones y variables para lapack"))
("dgesv" . ("maxima.info-7" 166161 3644 "Funciones y variables para lapack"))
("dgesvd" . ("maxima.info-7" 169827 5372 "Funciones y variables para lapack"))
("dgesvd <1>" . ("maxima.info-7" 169828 5371 "Funciones y variables para lapack"))
("diag" . ("maxima.info-6" 22359 943 "Funciones y variables para diag"))
("diag_matrix" . ("maxima.info-7" 207586 984 "Funciones y variables para linearalgebra"))
("diagmatrix" . ("maxima.info-3" 244797 525 "Funciones y variables para las matrices y el álgebra lineal"))
("diagmatrixp" . ("maxima.info-4" 110919 116 "Funciones y variables para ctensor"))
("diagmetric" . ("maxima.info-4" 125521 393 "Funciones y variables para ctensor"))
("diameter" . ("maxima.info-7" 93093 216 "Funciones y variables para graphs"))
("diff" . ("maxima.info-3" 81488 190 "Funciones y variables para la diferenciación"))
("diff <1>" . ("maxima.info-3" 78144 3319 "Funciones y variables para la diferenciación"))
("diff <2>" . ("maxima.info-3" 78145 3318 "Funciones y variables para la diferenciación"))
("diff <3>" . ("maxima.info-3" 78206 3258 "Funciones y variables para la diferenciación"))
("diff <4>" . ("maxima.info-3" 78244 3221 "Funciones y variables para la diferenciación"))
("diff <5>" . ("maxima.info-4" 28804 1389 "Funciones y variables para itensor"))
("digitcharp" . ("maxima.info-8" 110293 78 "Funciones y variables para caracteres"))
("dim" . ("maxima.info-4" 125312 203 "Funciones y variables para ctensor"))
("dimacs_export" . ("maxima.info-7" 120725 236 "Funciones y variables para graphs"))
("dimacs_export <1>" . ("maxima.info-7" 120726 235 "Funciones y variables para graphs"))
("dimacs_import" . ("maxima.info-7" 120966 101 "Funciones y variables para graphs"))
("dimension" . ("maxima.info-3" 161367 247 "Funciones y variable para las ecuaciones"))
("dimension <1>" . ("maxima.info-3" 161368 246 "Funciones y variable para las ecuaciones"))
("dimensionless" . ("maxima.info-7" 53701 1116 "Funciones y variables para ezunits"))
("dimensions" . ("maxima.info-6" 145797 1647 "Funciones y variables para draw"))
("dimensions <1>" . ("maxima.info-7" 50501 1785 "Funciones y variables para ezunits"))
("dimensions_as_list" . ("maxima.info-7" 50502 1784 "Funciones y variables para ezunits"))
("direct" . ("maxima.info-4" 235485 3720 "Funciones y variables para simetrías"))
("discrete_freq" . ("maxima.info-5" 235802 631 "Funciones y variables para el tratamiento de datos"))
("disjoin" . ("maxima.info-5" 14205 759 "Funciones y variables para los conjuntos"))
("disjointp" . ("maxima.info-5" 14970 417 "Funciones y variables para los conjuntos"))
("disolate" . ("maxima.info-1" 269257 477 "Funciones y variables para expresiones"))
("disp" . ("maxima.info-1" 75394 696 "Funciones y variables para la impresión"))
("dispJordan" . ("maxima.info-6" 25848 1068 "Funciones y variables para diag"))
("dispcon" . ("maxima.info-4" 344 278 "Funciones y variables para itensor"))
("dispcon <1>" . ("maxima.info-4" 345 277 "Funciones y variables para itensor"))
("dispflag" . ("maxima.info-3" 161621 373 "Funciones y variable para las ecuaciones"))
("dispform" . ("maxima.info-1" 269745 1845 "Funciones y variables para expresiones"))
("dispform <1>" . ("maxima.info-1" 269746 1844 "Funciones y variables para expresiones"))
("dispfun" . ("maxima.info-5" 95178 3834 "Funciones y variables para la definición de funciones"))
("dispfun <1>" . ("maxima.info-5" 95179 3833 "Funciones y variables para la definición de funciones"))
("display" . ("maxima.info-1" 76097 868 "Funciones y variables para la impresión"))
("display2d" . ("maxima.info-1" 76972 614 "Funciones y variables para la impresión"))
("display_format_internal" . ("maxima.info-1" 77592 695 "Funciones y variables para la impresión"))
("disprule" . ("maxima.info-4" 302676 1431 "Funciones y variables sobre reglas y patrones"))
("disprule <1>" . ("maxima.info-4" 302677 1430 "Funciones y variables sobre reglas y patrones"))
("dispterms" . ("maxima.info-1" 78293 1057 "Funciones y variables para la impresión"))
("distrib" . ("maxima.info-2" 8271 868 "Funciones y variables para simplificación"))
("distribute_over" . ("maxima.info-2" 9145 1846 "Funciones y variables para simplificación"))
("divide" . ("maxima.info-2" 227165 609 "Funciones y variables para polinomios"))
("divisors" . ("maxima.info-5" 15391 1669 "Funciones y variables para los conjuntos"))
("divsum" . ("maxima.info-4" 202290 580 "Funciones y variables para teoría de números"))
("divsum <1>" . ("maxima.info-4" 202291 579 "Funciones y variables para teoría de números"))
("dkummer_m" . ("maxima.info-5" 221858 97 "Funciones y variables para contrib_ode"))
("dkummer_u" . ("maxima.info-5" 221957 99 "Funciones y variables para contrib_ode"))
("dlange" . ("maxima.info-7" 175212 790 "Funciones y variables para lapack"))
("do" . ("maxima.info-5" 143528 8632 "Funciones y variables para la programación"))
("doallmxops" . ("maxima.info-3" 245327 277 "Funciones y variables para las matrices y el álgebra lineal"))
("dodecahedron_graph" . ("maxima.info-7" 83082 74 "Funciones y variables para graphs"))
("domain" . ("maxima.info-2" 11005 174 "Funciones y variables para simplificación"))
("domxexpt" . ("maxima.info-3" 245605 1329 "Funciones y variables para las matrices y el álgebra lineal"))
("domxmxops" . ("maxima.info-3" 246937 308 "Funciones y variables para las matrices y el álgebra lineal"))
("domxnctimes" . ("maxima.info-3" 247246 170 "Funciones y variables para las matrices y el álgebra lineal"))
("dontfactor" . ("maxima.info-3" 247417 227 "Funciones y variables para las matrices y el álgebra lineal"))
("doscmxops" . ("maxima.info-3" 247648 164 "Funciones y variables para las matrices y el álgebra lineal"))
("doscmxplus" . ("maxima.info-3" 247813 184 "Funciones y variables para las matrices y el álgebra lineal"))
("dot0nscsimp" . ("maxima.info-3" 247998 137 "Funciones y variables para las matrices y el álgebra lineal"))
("dotassoc" . ("maxima.info-3" 248139 162 "Funciones y variables para las matrices y el álgebra lineal"))
("dotconstrules" . ("maxima.info-3" 248303 223 "Funciones y variables para las matrices y el álgebra lineal"))
("dotdistrib" . ("maxima.info-3" 248528 171 "Funciones y variables para las matrices y el álgebra lineal"))
("dotexptsimp" . ("maxima.info-3" 248701 161 "Funciones y variables para las matrices y el álgebra lineal"))
("dotident" . ("maxima.info-3" 248864 141 "Funciones y variables para las matrices y el álgebra lineal"))
("dotproduct" . ("maxima.info-7" 208574 210 "Funciones y variables para linearalgebra"))
("dotscrules" . ("maxima.info-3" 249006 197 "Funciones y variables para las matrices y el álgebra lineal"))
("dotsimp" . ("maxima.info-3" 292113 267 "Funciones y variables para Afines"))
("dpart" . ("maxima.info-1" 271602 583 "Funciones y variables para expresiones"))
("draw" . ("maxima.info-6" 124456 1871 "Funciones y variables para draw"))
("draw2d" . ("maxima.info-6" 126339 316 "Funciones y variables para draw"))
("draw3d" . ("maxima.info-6" 126663 316 "Funciones y variables para draw"))
("draw_file" . ("maxima.info-6" 126987 484 "Funciones y variables para draw"))
("draw_graph" . ("maxima.info-7" 122055 2962 "Funciones y variables para graphs"))
("draw_graph <1>" . ("maxima.info-7" 122056 2961 "Funciones y variables para graphs"))
("draw_graph_program" . ("maxima.info-7" 125024 185 "Funciones y variables para graphs"))
("draw_realpart" . ("maxima.info-6" 147462 826 "Funciones y variables para draw"))
("drawdf" . ("maxima.info-6" 275047 8304 "Funciones y variables para drawdf"))
("drawdf <1>" . ("maxima.info-6" 275048 8303 "Funciones y variables para drawdf"))
("drawdf <2>" . ("maxima.info-6" 275105 8247 "Funciones y variables para drawdf"))
("drawdf <3>" . ("maxima.info-6" 275231 8123 "Funciones y variables para drawdf"))
("drawdf <4>" . ("maxima.info-6" 275292 8062 "Funciones y variables para drawdf"))
("drawdf <5>" . ("maxima.info-6" 275433 7923 "Funciones y variables para drawdf"))
("dscalar" . ("maxima.info-4" 108726 886 "Funciones y variables para ctensor"))
("e" . ("maxima.info-1" 117341 223 "Funciones y variables para Constantes"))
("echelon" . ("maxima.info-3" 249205 1561 "Funciones y variables para las matrices y el álgebra lineal"))
("edge_color" . ("maxima.info-7" 127195 75 "Funciones y variables para graphs"))
("edge_coloring" . ("maxima.info-7" 128133 170 "Funciones y variables para graphs"))
("edge_coloring <1>" . ("maxima.info-7" 93312 754 "Funciones y variables para graphs"))
("edge_connectivity" . ("maxima.info-7" 94344 135 "Funciones y variables para graphs"))
("edge_partition" . ("maxima.info-7" 127946 183 "Funciones y variables para graphs"))
("edge_type" . ("maxima.info-7" 127338 136 "Funciones y variables para graphs"))
("edge_width" . ("maxima.info-7" 127272 64 "Funciones y variables para graphs"))
("edges" . ("maxima.info-7" 94483 261 "Funciones y variables para graphs"))
("eigens_by_jacobi" . ("maxima.info-7" 208786 3211 "Funciones y variables para linearalgebra"))
("eigens_by_jacobi <1>" . ("maxima.info-7" 208787 3210 "Funciones y variables para linearalgebra"))
("eigenvalues" . ("maxima.info-3" 250773 1094 "Funciones y variables para las matrices y el álgebra lineal"))
("eigenvectors" . ("maxima.info-3" 251884 4898 "Funciones y variables para las matrices y el álgebra lineal"))
("eighth" . ("maxima.info-1" 129815 134 "Funciones y variables para listas"))
("einstein" . ("maxima.info-4" 87447 606 "Funciones y variables para ctensor"))
("eivals" . ("maxima.info-3" 250774 1093 "Funciones y variables para las matrices y el álgebra lineal"))
("eivects" . ("maxima.info-3" 251885 4897 "Funciones y variables para las matrices y el álgebra lineal"))
("elapsed_real_time" . ("maxima.info-4" 283423 514 "Funciones y variables para el entorno de ejecución"))
("elapsed_run_time" . ("maxima.info-4" 283944 561 "Funciones y variables para el entorno de ejecución"))
("ele2comp" . ("maxima.info-4" 239225 356 "Funciones y variables para simetrías"))
("ele2polynome" . ("maxima.info-4" 239584 840 "Funciones y variables para simetrías"))
("ele2pui" . ("maxima.info-4" 240435 357 "Funciones y variables para simetrías"))
("elem" . ("maxima.info-4" 240795 1652 "Funciones y variables para simetrías"))
("elementp" . ("maxima.info-5" 17068 424 "Funciones y variables para los conjuntos"))
("elevation" . ("maxima.info-2" 162348 522 "Opciones gráficas"))
("elevation <1>" . ("maxima.info-7" 3537 338 "Visualización usando VTK"))
("elevation_grid" . ("maxima.info-6" 223493 1049 "Funciones y variables para draw"))
("elim" . ("maxima.info-8" 136582 2601 "Funciones y variables para to_poly_solve"))
("elim_allbut" . ("maxima.info-8" 139208 574 "Funciones y variables para to_poly_solve"))
("eliminate" . ("maxima.info-2" 227778 1317 "Funciones y variables para polinomios"))
("eliminate_using" . ("maxima.info-8" 139789 1253 "Funciones y variables para to_poly_solve"))
("ellipse" . ("maxima.info-6" 224547 1198 "Funciones y variables para draw"))
("elliptic_e" . ("maxima.info-3" 56541 226 "Funciones y variables para integrales elípticas"))
("elliptic_ec" . ("maxima.info-3" 57689 313 "Funciones y variables para integrales elípticas"))
("elliptic_eu" . ("maxima.info-3" 56772 357 "Funciones y variables para integrales elípticas"))
("elliptic_f" . ("maxima.info-3" 56329 207 "Funciones y variables para integrales elípticas"))
("elliptic_kc" . ("maxima.info-3" 57369 313 "Funciones y variables para integrales elípticas"))
("elliptic_pi" . ("maxima.info-3" 57134 231 "Funciones y variables para integrales elípticas"))
("ematrix" . ("maxima.info-3" 256806 202 "Funciones y variables para las matrices y el álgebra lineal"))
("empty_graph" . ("maxima.info-7" 83158 77 "Funciones y variables para graphs"))
("emptyp" . ("maxima.info-5" 17496 318 "Funciones y variables para los conjuntos"))
("endcons" . ("maxima.info-1" 129954 265 "Funciones y variables para listas"))
("endphi" . ("maxima.info-7" 9136 461 "Visualización usando VTK"))
("endtheta" . ("maxima.info-7" 9605 466 "Visualización usando VTK"))
("enhanced3d" . ("maxima.info-6" 148296 5866 "Funciones y variables para draw"))
("entermatrix" . ("maxima.info-3" 257011 1505 "Funciones y variables para las matrices y el álgebra lineal"))
("entertensor" . ("maxima.info-4" 625 280 "Funciones y variables para itensor"))
("entier" . ("maxima.info-2" 40704 209 "Funciones para los números"))
("epsilon_lp" . ("maxima.info-8" 8548 186 "Funciones y variables para simplex"))
("equal" . ("maxima.info-2" 131730 4247 "Funciones y variables para los predicados"))
("equalp" . ("maxima.info-4" 186907 204 "Funciones y variables para series de Fourier"))
("equiv_classes" . ("maxima.info-5" 17819 1388 "Funciones y variables para los conjuntos"))
("erf" . ("maxima.info-3" 36198 95 "Función de error"))
("erf_generalized" . ("maxima.info-3" 36505 87 "Función de error"))
("erf_representation" . ("maxima.info-3" 37485 176 "Función de error"))
("erfc" . ("maxima.info-3" 36298 111 "Función de error"))
("erfflag" . ("maxima.info-3" 98014 203 "Funciones y variables para integración"))
("erfi" . ("maxima.info-3" 36412 90 "Función de error"))
("errcatch" . ("maxima.info-5" 152262 598 "Funciones y variables para la programación"))
("error" . ("maxima.info-5" 152874 648 "Funciones y variables para la programación"))
("error <1>" . ("maxima.info-5" 152875 647 "Funciones y variables para la programación"))
("error_size" . ("maxima.info-5" 153530 1578 "Funciones y variables para la programación"))
("error_syms" . ("maxima.info-5" 155125 685 "Funciones y variables para la programación"))
("error_type" . ("maxima.info-6" 154193 331 "Funciones y variables para draw"))
("errormsg" . ("maxima.info-5" 155980 1311 "Funciones y variables para la programación"))
("errormsg <1>" . ("maxima.info-5" 155822 155 "Funciones y variables para la programación"))
("errors" . ("maxima.info-6" 225751 1390 "Funciones y variables para draw"))
("euler" . ("maxima.info-4" 202874 541 "Funciones y variables para teoría de números"))
("ev" . ("maxima.info-1" 239545 9464 "Funciones y variables para la evaluación"))
("ev_point" . ("maxima.info-8" 201739 212 "Funciones y variables para zeilberger"))
("eval" . ("maxima.info-1" 249107 566 "Funciones y variables para la evaluación"))
("eval_string" . ("maxima.info-8" 112125 642 "Funciones y variables para cadenas"))
("even" . ("maxima.info-2" 101142 648 "Funciones y variables para las propiedades"))
("evenfun" . ("maxima.info-2" 11181 683 "Funciones y variables para simplificación"))
("evenp" . ("maxima.info-1" 102052 115 "Funciones y variables para los números"))
("every" . ("maxima.info-5" 19222 3019 "Funciones y variables para los conjuntos"))
("every <1>" . ("maxima.info-5" 19223 3018 "Funciones y variables para los conjuntos"))
("evflag" . ("maxima.info-1" 249678 2516 "Funciones y variables para la evaluación"))
("evfun" . ("maxima.info-1" 252198 2925 "Funciones y variables para la evaluación"))
("evolution" . ("maxima.info-6" 285490 739 "Análisis gráfico de sistemas dinámicos discretos"))
("evolution2d" . ("maxima.info-6" 286241 987 "Análisis gráfico de sistemas dinámicos discretos"))
("evundiff" . ("maxima.info-4" 31874 2376 "Funciones y variables para itensor"))
("example" . ("maxima.info-1" 37825 1906 "Funciones y variables para la ayuda"))
("example <1>" . ("maxima.info-1" 37826 1905 "Funciones y variables para la ayuda"))
("exp" . ("maxima.info-2" 59742 606 "Funciones radicales exponenciales y logarítmicas"))
("expand" . ("maxima.info-2" 11866 4470 "Funciones y variables para simplificación"))
("expand <1>" . ("maxima.info-2" 11867 4469 "Funciones y variables para simplificación"))
("expandwrt" . ("maxima.info-2" 16373 634 "Funciones y variables para simplificación"))
("expandwrt_denom" . ("maxima.info-2" 17013 407 "Funciones y variables para simplificación"))
("expandwrt_factored" . ("maxima.info-2" 17425 386 "Funciones y variables para simplificación"))
("expintegral_chi" . ("maxima.info-3" 35282 83 "Integral exponencial"))
("expintegral_ci" . ("maxima.info-3" 35114 81 "Integral exponencial"))
("expintegral_e" . ("maxima.info-3" 34945 84 "Integral exponencial"))
("expintegral_e1" . ("maxima.info-3" 34696 81 "Integral exponencial"))
("expintegral_ei" . ("maxima.info-3" 34779 81 "Integral exponencial"))
("expintegral_li" . ("maxima.info-3" 34862 81 "Integral exponencial"))
("expintegral_shi" . ("maxima.info-3" 35197 83 "Integral exponencial"))
("expintegral_si" . ("maxima.info-3" 35031 81 "Integral exponencial"))
("expintexpand" . ("maxima.info-3" 35651 242 "Integral exponencial"))
("expintrep" . ("maxima.info-3" 35367 281 "Integral exponencial"))
("explicit" . ("maxima.info-6" 227149 1830 "Funciones y variables para draw"))
("explicit <1>" . ("maxima.info-6" 227150 1829 "Funciones y variables para draw"))
("explose" . ("maxima.info-4" 242464 406 "Funciones y variables para simetrías"))
("expon" . ("maxima.info-2" 17817 286 "Funciones y variables para simplificación"))
("exponentialize" . ("maxima.info-2" 18108 636 "Funciones y variables para simplificación"))
("exponentialize <1>" . ("maxima.info-2" 18109 635 "Funciones y variables para simplificación"))
("expop" . ("maxima.info-2" 18754 410 "Funciones y variables para simplificación"))
("express" . ("maxima.info-3" 81682 2912 "Funciones y variables para la diferenciación"))
("expt" . ("maxima.info-1" 79359 330 "Funciones y variables para la impresión"))
("exptdispflag" . ("maxima.info-1" 79694 701 "Funciones y variables para la impresión"))
("exptisolate" . ("maxima.info-1" 272192 90 "Funciones y variables para expresiones"))
("exptsubst" . ("maxima.info-1" 272284 156 "Funciones y variables para expresiones"))
("exsec" . ("maxima.info-8" 25888 118 "Paquete functs"))
("extdiff" . ("maxima.info-4" 63889 1496 "Funciones y variables para itensor"))
("extract_linear_equations" . ("maxima.info-3" 293820 419 "Funciones y variables para Afines"))
("extremal_subset" . ("maxima.info-5" 22260 765 "Funciones y variables para los conjuntos"))
("extremal_subset <1>" . ("maxima.info-5" 22261 764 "Funciones y variables para los conjuntos"))
("ezgcd" . ("maxima.info-2" 229102 1217 "Funciones y variables para polinomios"))
("f90" . ("maxima.info-7" 55364 1928 "Funciones y variables para f90"))
("facexpand" . ("maxima.info-2" 230328 232 "Funciones y variables para polinomios"))
("facsum" . ("maxima.info-8" 15496 1843 "Paquete facexp"))
("facsum_combine" . ("maxima.info-8" 18138 387 "Paquete facexp"))
("factcomb" . ("maxima.info-2" 52549 305 "Funciones combinatorias"))
("factlim" . ("maxima.info-2" 56672 224 "Funciones combinatorias"))
("factor" . ("maxima.info-2" 230562 6357 "Funciones y variables para polinomios"))
("factor <1>" . ("maxima.info-2" 230563 6356 "Funciones y variables para polinomios"))
("factorfacsum" . ("maxima.info-8" 18528 342 "Paquete facexp"))
("factorflag" . ("maxima.info-2" 236947 179 "Funciones y variables para polinomios"))
("factorial" . ("maxima.info-2" 52858 3783 "Funciones combinatorias"))
("factorial_expand" . ("maxima.info-2" 56900 230 "Funciones combinatorias"))
("factorout" . ("maxima.info-2" 237128 689 "Funciones y variables para polinomios"))
("factors_only" . ("maxima.info-4" 203421 399 "Funciones y variables para teoría de números"))
("factorsum" . ("maxima.info-2" 237822 884 "Funciones y variables para polinomios"))
("facts" . ("maxima.info-2" 124642 502 "Funciones y variables para los hechos"))
("facts <1>" . ("maxima.info-2" 124643 501 "Funciones y variables para los hechos"))
("false" . ("maxima.info-1" 117655 141 "Funciones y variables para Constantes"))
("fast_central_elements" . ("maxima.info-3" 292388 423 "Funciones y variables para Afines"))
("fast_linsolve" . ("maxima.info-3" 289653 757 "Funciones y variables para Afines"))
("fasttimes" . ("maxima.info-2" 238712 422 "Funciones y variables para polinomios"))
("fb" . ("maxima.info-4" 130191 122 "Funciones y variables para ctensor"))
("feature" . ("maxima.info-2" 101794 381 "Funciones y variables para las propiedades"))
("featurep" . ("maxima.info-2" 102182 553 "Funciones y variables para las propiedades"))
("features" . ("maxima.info-2" 102742 1397 "Funciones y variables para las propiedades"))
("fernfale" . ("maxima.info-7" 71733 367 "Definiciones para IFS fractals"))
("fft" . ("maxima.info-3" 205671 3939 "Funciones y variables para la transformada rápida de Fourier"))
("fib" . ("maxima.info-4" 203824 493 "Funciones y variables para teoría de números"))
("fibtophi" . ("maxima.info-4" 204324 1203 "Funciones y variables para teoría de números"))
("fifth" . ("maxima.info-1" 130223 133 "Funciones y variables para listas"))
("file_name" . ("maxima.info-6" 154532 646 "Funciones y variables para draw"))
("file_name <1>" . ("maxima.info-7" 129084 116 "Funciones y variables para graphs"))
("file_output_append" . ("maxima.info-2" 184990 994 "Funciones y variables para lectura y escritura"))
("file_search" . ("maxima.info-2" 186385 2000 "Funciones y variables para lectura y escritura"))
("file_search <1>" . ("maxima.info-2" 186386 1999 "Funciones y variables para lectura y escritura"))
("file_search_demo" . ("maxima.info-2" 188451 1598 "Funciones y variables para lectura y escritura"))
("file_search_lisp" . ("maxima.info-2" 188409 1640 "Funciones y variables para lectura y escritura"))
("file_search_maxima" . ("maxima.info-2" 188408 1641 "Funciones y variables para lectura y escritura"))
("file_search_tests" . ("maxima.info-2" 188531 1518 "Funciones y variables para lectura y escritura"))
("file_search_usage" . ("maxima.info-2" 188491 1558 "Funciones y variables para lectura y escritura"))
("file_type" . ("maxima.info-2" 190058 731 "Funciones y variables para lectura y escritura"))
("file_type_lisp" . ("maxima.info-2" 190799 221 "Funciones y variables para lectura y escritura"))
("file_type_maxima" . ("maxima.info-2" 191023 253 "Funciones y variables para lectura y escritura"))
("filename_merge" . ("maxima.info-2" 185995 388 "Funciones y variables para lectura y escritura"))
("fill_color" . ("maxima.info-6" 155191 240 "Funciones y variables para draw"))
("fill_density" . ("maxima.info-6" 155440 246 "Funciones y variables para draw"))
("fillarray" . ("maxima.info-1" 160686 2262 "Funciones y variables para los arrays"))
("filled_func" . ("maxima.info-6" 155691 1498 "Funciones y variables para draw"))
("find_root" . ("maxima.info-3" 211152 4148 "Funciones para la resolución numérica de ecuaciones"))
("find_root <1>" . ("maxima.info-3" 211153 4147 "Funciones para la resolución numérica de ecuaciones"))
("find_root_abs" . ("maxima.info-3" 211423 3881 "Funciones para la resolución numérica de ecuaciones"))
("find_root_error" . ("maxima.info-3" 211358 3945 "Funciones para la resolución numérica de ecuaciones"))
("find_root_rel" . ("maxima.info-3" 211462 3842 "Funciones para la resolución numérica de ecuaciones"))
("findde" . ("maxima.info-4" 104950 2348 "Funciones y variables para ctensor"))
("first" . ("maxima.info-1" 130361 903 "Funciones y variables para listas"))
("fix" . ("maxima.info-2" 42952 62 "Funciones para los números"))
("fixed_vertices" . ("maxima.info-7" 129674 188 "Funciones y variables para graphs"))
("flatten" . ("maxima.info-5" 23033 2661 "Funciones y variables para los conjuntos"))
("flength" . ("maxima.info-8" 98333 148 "Funciones y variables para entrada y salida"))
("flipflag" . ("maxima.info-4" 6914 661 "Funciones y variables para itensor"))
("float" . ("maxima.info-1" 102169 238 "Funciones y variables para los números"))
("float2bf" . ("maxima.info-1" 102415 290 "Funciones y variables para los números"))
("floatnump" . ("maxima.info-1" 102711 134 "Funciones y variables para los números"))
("floor" . ("maxima.info-2" 40918 2016 "Funciones para los números"))
("flower_snark" . ("maxima.info-7" 83239 254 "Funciones y variables para graphs"))
("flush" . ("maxima.info-4" 34256 177 "Funciones y variables para itensor"))
("flush1deriv" . ("maxima.info-4" 40251 157 "Funciones y variables para itensor"))
("flushd" . ("maxima.info-4" 34437 174 "Funciones y variables para itensor"))
("flushnd" . ("maxima.info-4" 34615 721 "Funciones y variables para itensor"))
("fmin_cobyla" . ("maxima.info-5" 207424 2452 "Funciones y variables para cobyla"))
("fmin_cobyla <1>" . ("maxima.info-5" 207425 2451 "Funciones y variables para cobyla"))
("font" . ("maxima.info-6" 157217 3569 "Funciones y variables para draw"))
("font_size" . ("maxima.info-6" 160819 473 "Funciones y variables para draw"))
("for" . ("maxima.info-5" 157293 140 "Funciones y variables para la programación"))
("forget" . ("maxima.info-2" 125148 376 "Funciones y variables para los hechos"))
("forget <1>" . ("maxima.info-2" 125149 375 "Funciones y variables para los hechos"))
("fortindent" . ("maxima.info-2" 214703 322 "Funciones y variables para salida Fortran"))
("fortran" . ("maxima.info-2" 215029 2222 "Funciones y variables para salida Fortran"))
("fortspaces" . ("maxima.info-2" 217276 166 "Funciones y variables para salida Fortran"))
("fourcos" . ("maxima.info-4" 189213 137 "Funciones y variables para series de Fourier"))
("fourexpand" . ("maxima.info-4" 188922 288 "Funciones y variables para series de Fourier"))
("fourier" . ("maxima.info-4" 188437 148 "Funciones y variables para series de Fourier"))
("fourier_elim" . ("maxima.info-8" 141049 3299 "Funciones y variables para to_poly_solve"))
("fourint" . ("maxima.info-4" 189619 153 "Funciones y variables para series de Fourier"))
("fourintcos" . ("maxima.info-4" 189774 123 "Funciones y variables para series de Fourier"))
("fourintsin" . ("maxima.info-4" 189899 123 "Funciones y variables para series de Fourier"))
("foursimp" . ("maxima.info-4" 188587 151 "Funciones y variables para series de Fourier"))
("foursin" . ("maxima.info-4" 189352 135 "Funciones y variables para series de Fourier"))
("fourth" . ("maxima.info-1" 131274 134 "Funciones y variables para listas"))
("fposition" . ("maxima.info-8" 98484 412 "Funciones y variables para entrada y salida"))
("fposition <1>" . ("maxima.info-8" 98485 411 "Funciones y variables para entrada y salida"))
("fpprec" . ("maxima.info-1" 102848 368 "Funciones y variables para los números"))
("fpprintprec" . ("maxima.info-1" 103225 944 "Funciones y variables para los números"))
("frame_bracket" . ("maxima.info-4" 97590 355 "Funciones y variables para ctensor"))
("freeof" . ("maxima.info-1" 272442 3589 "Funciones y variables para expresiones"))
("freshline" . ("maxima.info-8" 98902 217 "Funciones y variables para entrada y salida"))
("freshline <1>" . ("maxima.info-8" 98903 216 "Funciones y variables para entrada y salida"))
("fresnel_c" . ("maxima.info-3" 36595 442 "Función de error"))
("fresnel_s" . ("maxima.info-3" 37043 436 "Función de error"))
("from_adjacency_matrix" . ("maxima.info-7" 83496 106 "Funciones y variables para graphs"))
("frucht_graph" . ("maxima.info-7" 83604 63 "Funciones y variables para graphs"))
("full_listify" . ("maxima.info-5" 25712 617 "Funciones y variables para los conjuntos"))
("fullmap" . ("maxima.info-5" 99037 726 "Funciones y variables para la definición de funciones"))
("fullmapl" . ("maxima.info-5" 99770 252 "Funciones y variables para la definición de funciones"))
("fullratsimp" . ("maxima.info-2" 239140 1545 "Funciones y variables para polinomios"))
("fullratsubst" . ("maxima.info-2" 240692 2012 "Funciones y variables para polinomios"))
("fullsetify" . ("maxima.info-5" 26334 668 "Funciones y variables para los conjuntos"))
("funcsolve" . ("maxima.info-3" 161998 909 "Funciones y variable para las ecuaciones"))
("functions" . ("maxima.info-5" 100025 1959 "Funciones y variables para la definición de funciones"))
("fundamental_dimensions" . ("maxima.info-7" 47483 1321 "Funciones y variables para ezunits"))
("fundamental_units" . ("maxima.info-7" 52291 1401 "Funciones y variables para ezunits"))
("fundamental_units <1>" . ("maxima.info-7" 52292 1400 "Funciones y variables para ezunits"))
("fundef" . ("maxima.info-5" 101992 939 "Funciones y variables para la definición de funciones"))
("funmake" . ("maxima.info-5" 102952 3243 "Funciones y variables para la definición de funciones"))
("funp" . ("maxima.info-4" 187471 349 "Funciones y variables para series de Fourier"))
("funp <1>" . ("maxima.info-4" 187472 348 "Funciones y variables para series de Fourier"))
("fv" . ("maxima.info-7" 58590 357 "Funciones y Variables para finance"))
("gamma" . ("maxima.info-3" 2958 3447 "Funciones Gamma y factorial"))
("gamma_expand" . ("maxima.info-3" 8995 856 "Funciones Gamma y factorial"))
("gamma_incomplete" . ("maxima.info-3" 7056 682 "Funciones Gamma y factorial"))
("gamma_incomplete_generalized" . ("maxima.info-3" 8318 672 "Funciones Gamma y factorial"))
("gamma_incomplete_lower" . ("maxima.info-3" 6508 542 "Funciones Gamma y factorial"))
("gamma_incomplete_regularized" . ("maxima.info-3" 7747 562 "Funciones Gamma y factorial"))
("gammalim" . ("maxima.info-3" 9854 416 "Funciones Gamma y factorial"))
("gauss_a" . ("maxima.info-5" 220303 593 "Funciones y variables para contrib_ode"))
("gauss_b" . ("maxima.info-5" 220906 72 "Funciones y variables para contrib_ode"))
("gaussprob" . ("maxima.info-8" 25133 188 "Paquete functs"))
("gcd" . ("maxima.info-2" 242723 1206 "Funciones y variables para polinomios"))
("gcdex" . ("maxima.info-2" 243949 1016 "Funciones y variables para polinomios"))
("gcdex <1>" . ("maxima.info-2" 243950 1015 "Funciones y variables para polinomios"))
("gcdivide" . ("maxima.info-8" 22204 1710 "Paquete functs"))
("gcfac" . ("maxima.info-8" 32749 1006 "Paquete scifac"))
("gcfactor" . ("maxima.info-2" 244971 320 "Funciones y variables para polinomios"))
("gd" . ("maxima.info-8" 25325 145 "Paquete functs"))
("gdet" . ("maxima.info-4" 127581 142 "Funciones y variables para ctensor"))
("gen_laguerre" . ("maxima.info-7" 300916 165 "Funciones y variables para polinomios ortogonales"))
("generalized_lambert_w" . ("maxima.info-3" 48441 376 "Funciones y variables para las funciones especiales"))
("genfact" . ("maxima.info-2" 57133 225 "Funciones combinatorias"))
("genindex" . ("maxima.info-4" 286412 198 "Funciones y variables para la miscelánea de opciones"))
("genmatrix" . ("maxima.info-3" 258524 2735 "Funciones y variables para las matrices y el álgebra lineal"))
("genmatrix <1>" . ("maxima.info-3" 258525 2734 "Funciones y variables para las matrices y el álgebra lineal"))
("genmatrix <2>" . ("maxima.info-3" 258583 2677 "Funciones y variables para las matrices y el álgebra lineal"))
("gensumnum" . ("maxima.info-4" 286613 277 "Funciones y variables para la miscelánea de opciones"))
("gensym" . ("maxima.info-4" 286896 1195 "Funciones y variables para la miscelánea de opciones"))
("gensym <1>" . ("maxima.info-4" 286897 1194 "Funciones y variables para la miscelánea de opciones"))
("geo_amortization" . ("maxima.info-7" 64623 1744 "Funciones y Variables para finance"))
("geo_annuity_fv" . ("maxima.info-7" 61025 480 "Funciones y Variables para finance"))
("geo_annuity_pv" . ("maxima.info-7" 60542 480 "Funciones y Variables para finance"))
("geomap" . ("maxima.info-6" 266808 5947 "Funciones y variables para worldmap"))
("geomap <1>" . ("maxima.info-6" 266809 5946 "Funciones y variables para worldmap"))
("geometric" . ("maxima.info-8" 24158 220 "Paquete functs"))
("geometric_mean" . ("maxima.info-5" 257872 897 "Funciones y variables de parámetros descriptivos"))
("geometric_mean <1>" . ("maxima.info-5" 257873 896 "Funciones y variables de parámetros descriptivos"))
("geosum" . ("maxima.info-8" 24843 281 "Paquete functs"))
("get" . ("maxima.info-2" 104147 1044 "Funciones y variables para las propiedades"))
("get_edge_weight" . ("maxima.info-7" 94746 702 "Funciones y variables para graphs"))
("get_edge_weight <1>" . ("maxima.info-7" 94747 701 "Funciones y variables para graphs"))
("get_lu_factors" . ("maxima.info-7" 212009 314 "Funciones y variables para linearalgebra"))
("get_output_stream_string" . ("maxima.info-8" 99127 294 "Funciones y variables para entrada y salida"))
("get_pixel" . ("maxima.info-6" 255817 171 "Funciones y variables para picture"))
("get_plot_option" . ("maxima.info-2" 144264 389 "Funciones y variables para gráficos"))
("get_tex_environment" . ("maxima.info-2" 212158 1090 "Funciones y variables para salida TeX"))
("get_tex_environment_default" . ("maxima.info-2" 213256 1205 "Funciones y variables para salida TeX"))
("get_vertex_label" . ("maxima.info-7" 95453 307 "Funciones y variables para graphs"))
("gfactor" . ("maxima.info-2" 245294 365 "Funciones y variables para polinomios"))
("gfactorsum" . ("maxima.info-2" 245662 125 "Funciones y variables para polinomios"))
("ggf" . ("maxima.info-7" 77763 564 "Funciones y variables para ggf"))
("girth" . ("maxima.info-7" 97537 248 "Funciones y variables para graphs"))
("global_variances" . ("maxima.info-5" 267351 1840 "Funciones y variables de parámetros descriptivos"))
("global_variances <1>" . ("maxima.info-5" 267352 1839 "Funciones y variables de parámetros descriptivos"))
("globalsolve" . ("maxima.info-3" 162911 2136 "Funciones y variable para las ecuaciones"))
("gnuplot_close" . ("maxima.info-2" 177445 118 "Funciones para el formato Gnuplot_pipes"))
("gnuplot_curve_styles" . ("maxima.info-2" 176003 101 "Opciones para Gnuplot"))
("gnuplot_curve_titles" . ("maxima.info-2" 175898 102 "Opciones para Gnuplot"))
("gnuplot_default_term_command" . ("maxima.info-2" 176107 170 "Opciones para Gnuplot"))
("gnuplot_dumb_term_command" . ("maxima.info-2" 176279 246 "Opciones para Gnuplot"))
("gnuplot_file_name" . ("maxima.info-6" 161309 738 "Funciones y variables para draw"))
("gnuplot_out_file" . ("maxima.info-2" 174864 410 "Opciones para Gnuplot"))
("gnuplot_pm3d" . ("maxima.info-2" 175280 284 "Opciones para Gnuplot"))
("gnuplot_preamble" . ("maxima.info-2" 175570 322 "Opciones para Gnuplot"))
("gnuplot_ps_term_command" . ("maxima.info-2" 176527 480 "Opciones para Gnuplot"))
("gnuplot_replot" . ("maxima.info-2" 177714 256 "Funciones para el formato Gnuplot_pipes"))
("gnuplot_replot <1>" . ("maxima.info-2" 177715 255 "Funciones para el formato Gnuplot_pipes"))
("gnuplot_reset" . ("maxima.info-2" 177973 206 "Funciones para el formato Gnuplot_pipes"))
("gnuplot_restart" . ("maxima.info-2" 177567 143 "Funciones para el formato Gnuplot_pipes"))
("gnuplot_start" . ("maxima.info-2" 177219 222 "Funciones para el formato Gnuplot_pipes"))
("gnuplot_term" . ("maxima.info-2" 173337 1509 "Opciones para Gnuplot"))
("go" . ("maxima.info-5" 157438 632 "Funciones y variables para la programación"))
("gr2d" . ("maxima.info-6" 122553 906 "Funciones y variables para draw"))
("gr3d" . ("maxima.info-6" 123473 935 "Funciones y variables para draw"))
("gradef" . ("maxima.info-3" 84604 2118 "Funciones y variables para la diferenciación"))
("gradef <1>" . ("maxima.info-3" 84605 2117 "Funciones y variables para la diferenciación"))
("gradefs" . ("maxima.info-3" 86743 303 "Funciones y variables para la diferenciación"))
("gramschmidt" . ("maxima.info-3" 261269 2481 "Funciones y variables para las matrices y el álgebra lineal"))
("gramschmidt <1>" . ("maxima.info-3" 261270 2480 "Funciones y variables para las matrices y el álgebra lineal"))
("graph6_decode" . ("maxima.info-7" 121069 107 "Funciones y variables para graphs"))
("graph6_encode" . ("maxima.info-7" 121178 104 "Funciones y variables para graphs"))
("graph6_export" . ("maxima.info-7" 121284 131 "Funciones y variables para graphs"))
("graph6_import" . ("maxima.info-7" 121417 117 "Funciones y variables para graphs"))
("graph_center" . ("maxima.info-7" 96141 241 "Funciones y variables para graphs"))
("graph_charpoly" . ("maxima.info-7" 95763 375 "Funciones y variables para graphs"))
("graph_eigenvalues" . ("maxima.info-7" 96384 386 "Funciones y variables para graphs"))
("graph_flow" . ("maxima.info-7" 59293 424 "Funciones y Variables para finance"))
("graph_order" . ("maxima.info-7" 97282 251 "Funciones y variables para graphs"))
("graph_periphery" . ("maxima.info-7" 96774 255 "Funciones y variables para graphs"))
("graph_product" . ("maxima.info-7" 83669 251 "Funciones y variables para graphs"))
("graph_size" . ("maxima.info-7" 97031 248 "Funciones y variables para graphs"))
("graph_union" . ("maxima.info-7" 83922 95 "Funciones y variables para graphs"))
("great_rhombicosidodecahedron_graph" . ("maxima.info-7" 84091 102 "Funciones y variables para graphs"))
("great_rhombicuboctahedron_graph" . ("maxima.info-7" 84196 101 "Funciones y variables para graphs"))
("grid" . ("maxima.info-2" 162879 198 "Opciones gráficas"))
("grid <1>" . ("maxima.info-6" 162058 729 "Funciones y variables para draw"))
("grid_graph" . ("maxima.info-7" 84020 69 "Funciones y variables para graphs"))
("grind" . ("maxima.info-1" 80398 4690 "Funciones y variables para la impresión"))
("grind <1>" . ("maxima.info-1" 80399 4689 "Funciones y variables para la impresión"))
("grobner_basis" . ("maxima.info-3" 290424 631 "Funciones y variables para Afines"))
("grotzch_graph" . ("maxima.info-7" 84300 65 "Funciones y variables para graphs"))
("halfangles" . ("maxima.info-2" 74454 2256 "Funciones y variables para trigonometría"))
("hamilton_cycle" . ("maxima.info-7" 97788 380 "Funciones y variables para graphs"))
("hamilton_path" . ("maxima.info-7" 98171 382 "Funciones y variables para graphs"))
("hankel" . ("maxima.info-7" 212327 320 "Funciones y variables para linearalgebra"))
("hankel <1>" . ("maxima.info-7" 212328 319 "Funciones y variables para linearalgebra"))
("hankel_1" . ("maxima.info-2" 287592 2023 "Funciones de Bessel"))
("hankel_2" . ("maxima.info-2" 289634 751 "Funciones de Bessel"))
("harmonic" . ("maxima.info-8" 24385 255 "Paquete functs"))
("harmonic_mean" . ("maxima.info-5" 256961 905 "Funciones y variables de parámetros descriptivos"))
("harmonic_mean <1>" . ("maxima.info-5" 256962 904 "Funciones y variables de parámetros descriptivos"))
("hav" . ("maxima.info-8" 26009 117 "Paquete functs"))
("head_angle" . ("maxima.info-6" 162800 1228 "Funciones y variables para draw"))
("head_angle <1>" . ("maxima.info-7" 128535 131 "Funciones y variables para graphs"))
("head_both" . ("maxima.info-6" 164036 631 "Funciones y variables para draw"))
("head_length" . ("maxima.info-6" 164674 728 "Funciones y variables para draw"))
("head_length <1>" . ("maxima.info-7" 128669 135 "Funciones y variables para graphs"))
("head_type" . ("maxima.info-6" 165409 886 "Funciones y variables para draw"))
("heawood_graph" . ("maxima.info-7" 84367 65 "Funciones y variables para graphs"))
("height" . ("maxima.info-7" 3885 179 "Visualización usando VTK"))
("height <1>" . ("maxima.info-7" 10079 166 "Visualización usando VTK"))
("hermite" . ("maxima.info-7" 301085 128 "Funciones y variables para polinomios ortogonales"))
("hessian" . ("maxima.info-7" 212652 1123 "Funciones y variables para linearalgebra"))
("hgfred" . ("maxima.info-3" 46064 2212 "Funciones y variables para las funciones especiales"))
("hilbert_matrix" . ("maxima.info-7" 213778 149 "Funciones y variables para linearalgebra"))
("hilbertmap" . ("maxima.info-7" 75724 259 "Definiciones para curvas de Peano"))
("hipow" . ("maxima.info-2" 245790 924 "Funciones y variables para polinomios"))
("histogram" . ("maxima.info-6" 10345 3630 "Funciones y variables para gráficos estadísticos"))
("histogram <1>" . ("maxima.info-6" 10346 3629 "Funciones y variables para gráficos estadísticos"))
("histogram <2>" . ("maxima.info-6" 10379 3597 "Funciones y variables para gráficos estadísticos"))
("histogram <3>" . ("maxima.info-6" 10441 3536 "Funciones y variables para gráficos estadísticos"))
("histogram <4>" . ("maxima.info-6" 10557 3422 "Funciones y variables para gráficos estadísticos"))
("histogram <5>" . ("maxima.info-6" 10572 3407 "Funciones y variables para gráficos estadísticos"))
("histogram_description" . ("maxima.info-6" 10615 3365 "Funciones y variables para gráficos estadísticos"))
("hodge" . ("maxima.info-4" 65390 1493 "Funciones y variables para itensor"))
("horner" . ("maxima.info-3" 209996 1142 "Funciones para la resolución numérica de ecuaciones"))
("horner <1>" . ("maxima.info-3" 209997 1141 "Funciones para la resolución numérica de ecuaciones"))
("hypergeometric" . ("maxima.info-3" 39247 1648 "Funciones hipergeométricas"))
("hypergeometric_representation" . ("maxima.info-3" 37662 186 "Función de error"))
("i" . ("maxima.info-1" 117571 82 "Funciones y variables para Constantes"))
("i-ésima salida anterior" . ("maxima.info-1" 52319 1125 "Funciones y variables para la línea de comandos"))
("ibase" . ("maxima.info-1" 85106 2027 "Funciones y variables para la impresión"))
("ic1" . ("maxima.info-3" 195362 452 "Funciones y variables para ecuaciones diferenciales"))
("ic2" . ("maxima.info-3" 195823 641 "Funciones y variables para ecuaciones diferenciales"))
("ic_convert" . ("maxima.info-4" 69272 2072 "Funciones y variables para itensor"))
("icc1" . ("maxima.info-4" 51192 567 "Funciones y variables para itensor"))
("icc2" . ("maxima.info-4" 51768 615 "Funciones y variables para itensor"))
("ichr1" . ("maxima.info-4" 41146 399 "Funciones y variables para itensor"))
("ichr2" . ("maxima.info-4" 41552 290 "Funciones y variables para itensor"))
("icosahedron_graph" . ("maxima.info-7" 84434 70 "Funciones y variables para graphs"))
("icosidodecahedron_graph" . ("maxima.info-7" 84507 82 "Funciones y variables para graphs"))
("icounter" . ("maxima.info-4" 16764 231 "Funciones y variables para itensor"))
("icurvature" . ("maxima.info-4" 41846 607 "Funciones y variables para itensor"))
("ident" . ("maxima.info-3" 263760 73 "Funciones y variables para las matrices y el álgebra lineal"))
("identfor" . ("maxima.info-7" 213929 576 "Funciones y variables para linearalgebra"))
("identfor <1>" . ("maxima.info-7" 213930 575 "Funciones y variables para linearalgebra"))
("identity" . ("maxima.info-5" 27005 331 "Funciones y variables para los conjuntos"))
("idiff" . ("maxima.info-4" 30211 521 "Funciones y variables para itensor"))
("idim" . ("maxima.info-4" 40952 187 "Funciones y variables para itensor"))
("idummy" . ("maxima.info-4" 16311 323 "Funciones y variables para itensor"))
("idummyx" . ("maxima.info-4" 16642 119 "Funciones y variables para itensor"))
("ieqn" . ("maxima.info-3" 165053 1400 "Funciones y variable para las ecuaciones"))
("ieqnprint" . ("maxima.info-3" 166481 701 "Funciones y variable para las ecuaciones"))
("if" . ("maxima.info-5" 158075 2759 "Funciones y variables para la programación"))
("ifactors" . ("maxima.info-4" 205533 1074 "Funciones y variables para teoría de números"))
("ifb" . ("maxima.info-4" 50138 1047 "Funciones y variables para itensor"))
("ifc1" . ("maxima.info-4" 52390 490 "Funciones y variables para itensor"))
("ifc2" . ("maxima.info-4" 52886 329 "Funciones y variables para itensor"))
("ifg" . ("maxima.info-4" 53534 148 "Funciones y variables para itensor"))
("ifgi" . ("maxima.info-4" 53684 128 "Funciones y variables para itensor"))
("ifr" . ("maxima.info-4" 53219 167 "Funciones y variables para itensor"))
("iframe_bracket_form" . ("maxima.info-4" 53815 115 "Funciones y variables para itensor"))
("ifri" . ("maxima.info-4" 53388 145 "Funciones y variables para itensor"))
("ifs" . ("maxima.info-6" 287239 1550 "Análisis gráfico de sistemas dinámicos discretos"))
("igcdex" . ("maxima.info-4" 206620 686 "Funciones y variables para teoría de números"))
("igeodesic_coords" . ("maxima.info-4" 44574 2000 "Funciones y variables para itensor"))
("igeowedge_flag" . ("maxima.info-4" 66885 405 "Funciones y variables para itensor"))
("ikt1" . ("maxima.info-4" 55614 479 "Funciones y variables para itensor"))
("ikt2" . ("maxima.info-4" 56100 314 "Funciones y variables para itensor"))
("ilt" . ("maxima.info-3" 98221 2140 "Funciones y variables para integración"))
("image" . ("maxima.info-6" 228991 3940 "Funciones y variables para draw"))
("imaginary" . ("maxima.info-2" 110608 288 "Funciones y variables para las propiedades"))
("imagpart" . ("maxima.info-2" 49799 228 "Funciones para los números complejos"))
("imetric" . ("maxima.info-4" 40481 461 "Funciones y variables para itensor"))
("imetric <1>" . ("maxima.info-4" 40482 460 "Funciones y variables para itensor"))
("implicit" . ("maxima.info-6" 232966 1981 "Funciones y variables para draw"))
("implicit <1>" . ("maxima.info-6" 232967 1980 "Funciones y variables para draw"))
("implicit_derivative" . ("maxima.info-7" 146615 469 "Funciones y variables para impdiff"))
("in_neighbors" . ("maxima.info-7" 99054 387 "Funciones y variables para graphs"))
("inchar" . ("maxima.info-1" 54358 971 "Funciones y variables para la línea de comandos"))
("increasing" . ("maxima.info-2" 100516 620 "Funciones y variables para las propiedades"))
("ind" . ("maxima.info-1" 117888 208 "Funciones y variables para Constantes"))
("indexed_tensor" . ("maxima.info-4" 9412 238 "Funciones y variables para itensor"))
("indices" . ("maxima.info-4" 2276 1031 "Funciones y variables para itensor"))
("induced_subgraph" . ("maxima.info-7" 84592 499 "Funciones y variables para graphs"))
("inf" . ("maxima.info-1" 118099 79 "Funciones y variables para Constantes"))
("inference_result" . ("maxima.info-8" 47056 1888 "Funciones y variables para inference_result"))
("inferencep" . ("maxima.info-8" 48957 142 "Funciones y variables para inference_result"))
("infeval" . ("maxima.info-1" 255129 402 "Funciones y variables para la evaluación"))
("infinity" . ("maxima.info-1" 118180 84 "Funciones y variables para Constantes"))
("infix" . ("maxima.info-1" 214886 4337 "Operadores definidos por el usuario"))
("infix <1>" . ("maxima.info-1" 214887 4336 "Operadores definidos por el usuario"))
("infix <2>" . ("maxima.info-1" 214914 4310 "Operadores definidos por el usuario"))
("inflag" . ("maxima.info-1" 276051 758 "Funciones y variables para expresiones"))
("infolists" . ("maxima.info-1" 55337 2091 "Funciones y variables para la línea de comandos"))
("init_atensor" . ("maxima.info-4" 138344 2566 "Funciones y variables para atensor"))
("init_atensor <1>" . ("maxima.info-4" 138345 2565 "Funciones y variables para atensor"))
("init_ctensor" . ("maxima.info-4" 84126 344 "Funciones y variables para ctensor"))
("inm" . ("maxima.info-4" 54275 331 "Funciones y variables para itensor"))
("inmc1" . ("maxima.info-4" 54609 408 "Funciones y variables para itensor"))
("inmc2" . ("maxima.info-4" 55021 588 "Funciones y variables para itensor"))
("innerproduct" . ("maxima.info-3" 263835 456 "Funciones y variables para las matrices y el álgebra lineal"))
("inpart" . ("maxima.info-1" 276816 1231 "Funciones y variables para expresiones"))
("inprod" . ("maxima.info-3" 263836 455 "Funciones y variables para las matrices y el álgebra lineal"))
("inrt" . ("maxima.info-4" 207314 286 "Funciones y variables para teoría de números"))
("intanalysis" . ("maxima.info-3" 100368 1544 "Funciones y variables para integración"))
("integer" . ("maxima.info-2" 105198 480 "Funciones y variables para las propiedades"))
("integer_partitions" . ("maxima.info-5" 27341 1916 "Funciones y variables para los conjuntos"))
("integer_partitions <1>" . ("maxima.info-5" 27342 1915 "Funciones y variables para los conjuntos"))
("integerp" . ("maxima.info-1" 104183 967 "Funciones y variables para los números"))
("integervalued" . ("maxima.info-2" 105681 523 "Funciones y variables para las propiedades"))
("integrate" . ("maxima.info-3" 101920 8911 "Funciones y variables para integración"))
("integrate <1>" . ("maxima.info-3" 101921 8910 "Funciones y variables para integración"))
("integrate_use_rootsof" . ("maxima.info-3" 113203 2437 "Funciones y variables para integración"))
("integration_constant" . ("maxima.info-3" 110930 888 "Funciones y variables para integración"))
("integration_constant_counter" . ("maxima.info-3" 111823 1375 "Funciones y variables para integración"))
("interpolate_color" . ("maxima.info-6" 166305 2723 "Funciones y variables para draw"))
("intersect" . ("maxima.info-5" 29271 101 "Funciones y variables para los conjuntos"))
("intersection" . ("maxima.info-5" 29375 958 "Funciones y variables para los conjuntos"))
("intervalp" . ("maxima.info-7" 301217 114 "Funciones y variables para polinomios ortogonales"))
("intfaclim" . ("maxima.info-2" 246719 906 "Funciones y variables para polinomios"))
("intopois" . ("maxima.info-4" 190281 73 "Funciones y variables para series de Poisson"))
("intosum" . ("maxima.info-4" 145771 1189 "Funciones y variables para sumas y productos"))
("inv_mod" . ("maxima.info-4" 207604 410 "Funciones y variables para teoría de números"))
("invariant1" . ("maxima.info-4" 110679 202 "Funciones y variables para ctensor"))
("inverse_fft" . ("maxima.info-3" 203286 2371 "Funciones y variables para la transformada rápida de Fourier"))
("inverse_jacobi_cd" . ("maxima.info-3" 55656 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_cn" . ("maxima.info-3" 54949 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_cs" . ("maxima.info-3" 55555 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_dc" . ("maxima.info-3" 55959 99 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_dn" . ("maxima.info-3" 55050 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_ds" . ("maxima.info-3" 55858 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_nc" . ("maxima.info-3" 55454 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_nd" . ("maxima.info-3" 55757 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_ns" . ("maxima.info-3" 55151 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_sc" . ("maxima.info-3" 55252 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_sd" . ("maxima.info-3" 55353 97 "Funciones y variables para funciones elípticas"))
("inverse_jacobi_sn" . ("maxima.info-3" 54848 97 "Funciones y variables para funciones elípticas"))
("invert" . ("maxima.info-3" 264297 616 "Funciones y variables para las matrices y el álgebra lineal"))
("invert_by_lu" . ("maxima.info-7" 214510 161 "Funciones y variables para linearalgebra"))
("ip_grid" . ("maxima.info-6" 169055 239 "Funciones y variables para draw"))
("ip_grid_in" . ("maxima.info-6" 169301 244 "Funciones y variables para draw"))
("irr" . ("maxima.info-7" 68155 487 "Funciones y Variables para finance"))
("irrational" . ("maxima.info-2" 110374 230 "Funciones y variables para las propiedades"))
("is" . ("maxima.info-2" 125528 1946 "Funciones y variables para los hechos"))
("is_biconnected" . ("maxima.info-7" 99444 362 "Funciones y variables para graphs"))
("is_bipartite" . ("maxima.info-7" 99809 369 "Funciones y variables para graphs"))
("is_connected" . ("maxima.info-7" 100180 288 "Funciones y variables para graphs"))
("is_digraph" . ("maxima.info-7" 100470 364 "Funciones y variables para graphs"))
("is_edge_in_graph" . ("maxima.info-7" 100836 633 "Funciones y variables para graphs"))
("is_graph" . ("maxima.info-7" 101471 333 "Funciones y variables para graphs"))
("is_graph_or_digraph" . ("maxima.info-7" 101806 388 "Funciones y variables para graphs"))
("is_isomorphic" . ("maxima.info-7" 102196 400 "Funciones y variables para graphs"))
("is_planar" . ("maxima.info-7" 102600 542 "Funciones y variables para graphs"))
("is_sconnected" . ("maxima.info-7" 103145 396 "Funciones y variables para graphs"))
("is_tree" . ("maxima.info-7" 103953 359 "Funciones y variables para graphs"))
("is_vertex_in_graph" . ("maxima.info-7" 103543 407 "Funciones y variables para graphs"))
("ishow" . ("maxima.info-4" 1897 370 "Funciones y variables para itensor"))
("isolate" . ("maxima.info-1" 278056 901 "Funciones y variables para expresiones"))
("isolate_wrt_times" . ("maxima.info-1" 278971 858 "Funciones y variables para expresiones"))
("isomorphism" . ("maxima.info-7" 98556 495 "Funciones y variables para graphs"))
("isqrt" . ("maxima.info-4" 208018 125 "Funciones y variables para teoría de números"))
("isreal_p" . ("maxima.info-8" 144372 1280 "Funciones y variables para to_poly_solve"))
("items_inference" . ("maxima.info-8" 49101 780 "Funciones y variables para inference_result"))
("itr" . ("maxima.info-4" 56421 2701 "Funciones y variables para itensor"))
("jacobi" . ("maxima.info-4" 208146 258 "Funciones y variables para teoría de números"))
("jacobi_cd" . ("maxima.info-3" 54466 93 "Funciones y variables para funciones elípticas"))
("jacobi_cn" . ("maxima.info-3" 53835 75 "Funciones y variables para funciones elípticas"))
("jacobi_cs" . ("maxima.info-3" 54369 93 "Funciones y variables para funciones elípticas"))
("jacobi_dc" . ("maxima.info-3" 54751 93 "Funciones y variables para funciones elípticas"))
("jacobi_dn" . ("maxima.info-3" 53914 75 "Funciones y variables para funciones elípticas"))
("jacobi_ds" . ("maxima.info-3" 54654 93 "Funciones y variables para funciones elípticas"))
("jacobi_nc" . ("maxima.info-3" 54278 87 "Funciones y variables para funciones elípticas"))
("jacobi_nd" . ("maxima.info-3" 54563 87 "Funciones y variables para funciones elípticas"))
("jacobi_ns" . ("maxima.info-3" 53993 87 "Funciones y variables para funciones elípticas"))
("jacobi_p" . ("maxima.info-7" 301333 316 "Funciones y variables para polinomios ortogonales"))
("jacobi_sc" . ("maxima.info-3" 54084 93 "Funciones y variables para funciones elípticas"))
("jacobi_sd" . ("maxima.info-3" 54181 93 "Funciones y variables para funciones elípticas"))
("jacobi_sn" . ("maxima.info-3" 53756 75 "Funciones y variables para funciones elípticas"))
("jacobian" . ("maxima.info-7" 214674 948 "Funciones y variables para linearalgebra"))
("join" . ("maxima.info-1" 131413 768 "Funciones y variables para listas"))
("jordan" . ("maxima.info-6" 24148 1692 "Funciones y variables para diag"))
("julia_parameter" . ("maxima.info-7" 73401 296 "Definiciones para fractales complejos"))
("julia_set" . ("maxima.info-7" 72860 534 "Definiciones para fractales complejos"))
("julia_sin" . ("maxima.info-7" 73699 726 "Definiciones para fractales complejos"))
("kdels" . ("maxima.info-4" 17741 872 "Funciones y variables para itensor"))
("kdelta" . ("maxima.info-4" 17000 729 "Funciones y variables para itensor"))
("keepfloat" . ("maxima.info-2" 247634 960 "Funciones y variables para polinomios"))
("key" . ("maxima.info-6" 169552 758 "Funciones y variables para draw"))
("key_pos" . ("maxima.info-6" 170319 1122 "Funciones y variables para draw"))
("kill" . ("maxima.info-1" 57440 3305 "Funciones y variables para la línea de comandos"))
("kill <1>" . ("maxima.info-1" 57441 3304 "Funciones y variables para la línea de comandos"))
("kill <2>" . ("maxima.info-1" 57480 3266 "Funciones y variables para la línea de comandos"))
("kill <3>" . ("maxima.info-1" 57508 3239 "Funciones y variables para la línea de comandos"))
("kill <4>" . ("maxima.info-1" 57561 3187 "Funciones y variables para la línea de comandos"))
("kill <5>" . ("maxima.info-1" 57586 3163 "Funciones y variables para la línea de comandos"))
("kill <6>" . ("maxima.info-1" 57618 3132 "Funciones y variables para la línea de comandos"))
("kill <7>" . ("maxima.info-1" 57670 3081 "Funciones y variables para la línea de comandos"))
("killcontext" . ("maxima.info-2" 127482 736 "Funciones y variables para los hechos"))
("kinvariant" . ("maxima.info-4" 130314 113 "Funciones y variables para ctensor"))
("km" . ("maxima.info-5" 262227 1529 "Funciones y variables de parámetros descriptivos"))
("km <1>" . ("maxima.info-5" 262228 1528 "Funciones y variables de parámetros descriptivos"))
("kostka" . ("maxima.info-4" 242875 259 "Funciones y variables para simetrías"))
("kron_delta" . ("maxima.info-5" 30335 920 "Funciones y variables para los conjuntos"))
("kronecker_product" . ("maxima.info-7" 215625 108 "Funciones y variables para linearalgebra"))
("kt" . ("maxima.info-4" 131339 112 "Funciones y variables para ctensor"))
("kummer_m" . ("maxima.info-5" 221194 453 "Funciones y variables para contrib_ode"))
("kummer_u" . ("maxima.info-5" 221657 195 "Funciones y variables para contrib_ode"))
("kurtosis" . ("maxima.info-5" 258775 856 "Funciones y variables de parámetros descriptivos"))
("kurtosis <1>" . ("maxima.info-5" 258776 855 "Funciones y variables de parámetros descriptivos"))
("kurtosis_bernoulli" . ("maxima.info-6" 109820 438 "Funciones y variables para distribuciones discretas"))
("kurtosis_beta" . ("maxima.info-6" 77245 204 "Funciones y variables para distribuciones continuas"))
("kurtosis_binomial" . ("maxima.info-6" 104143 250 "Funciones y variables para distribuciones discretas"))
("kurtosis_chi2" . ("maxima.info-6" 60265 385 "Funciones y variables para distribuciones continuas"))
("kurtosis_continuous_uniform" . ("maxima.info-6" 79733 226 "Funciones y variables para distribuciones continuas"))
("kurtosis_discrete_uniform" . ("maxima.info-6" 114842 235 "Funciones y variables para distribuciones discretas"))
("kurtosis_exp" . ("maxima.info-6" 69293 310 "Funciones y variables para distribuciones continuas"))
("kurtosis_f" . ("maxima.info-6" 65519 198 "Funciones y variables para distribuciones continuas"))
("kurtosis_gamma" . ("maxima.info-6" 74516 203 "Funciones y variables para distribuciones continuas"))
("kurtosis_general_finite_discrete" . ("maxima.info-6" 101152 242 "Funciones y variables para distribuciones discretas"))
("kurtosis_geometric" . ("maxima.info-6" 112382 216 "Funciones y variables para distribuciones discretas"))
("kurtosis_gumbel" . ("maxima.info-6" 97307 209 "Funciones y variables para distribuciones continuas"))
("kurtosis_laplace" . ("maxima.info-6" 93413 208 "Funciones y variables para distribuciones continuas"))
("kurtosis_logistic" . ("maxima.info-6" 82089 208 "Funciones y variables para distribuciones continuas"))
("kurtosis_lognormal" . ("maxima.info-6" 72016 209 "Funciones y variables para distribuciones continuas"))
("kurtosis_negative_binomial" . ("maxima.info-6" 120732 252 "Funciones y variables para distribuciones discretas"))
("kurtosis_noncentral_chi2" . ("maxima.info-6" 62897 212 "Funciones y variables para distribuciones continuas"))
("kurtosis_noncentral_student_t" . ("maxima.info-6" 56172 302 "Funciones y variables para distribuciones continuas"))
("kurtosis_normal" . ("maxima.info-6" 48517 232 "Funciones y variables para distribuciones continuas"))
("kurtosis_pareto" . ("maxima.info-6" 84343 213 "Funciones y variables para distribuciones continuas"))
("kurtosis_poisson" . ("maxima.info-6" 106659 205 "Funciones y variables para distribuciones discretas"))
("kurtosis_rayleigh" . ("maxima.info-6" 90750 638 "Funciones y variables para distribuciones continuas"))
("kurtosis_student_t" . ("maxima.info-6" 51409 206 "Funciones y variables para distribuciones continuas"))
("kurtosis_weibull" . ("maxima.info-6" 86602 210 "Funciones y variables para distribuciones continuas"))
("label" . ("maxima.info-6" 234958 1359 "Funciones y variables para draw"))
("label <1>" . ("maxima.info-6" 234959 1358 "Funciones y variables para draw"))
("label_alignment" . ("maxima.info-6" 171457 959 "Funciones y variables para draw"))
("label_alignment <1>" . ("maxima.info-7" 125547 218 "Funciones y variables para graphs"))
("label_orientation" . ("maxima.info-6" 172426 1023 "Funciones y variables para draw"))
("labels" . ("maxima.info-1" 60785 2057 "Funciones y variables para la línea de comandos"))
("labels <1>" . ("maxima.info-1" 60786 2056 "Funciones y variables para la línea de comandos"))
("lagrange" . ("maxima.info-7" 147876 4208 "Funciones y variables para interpol"))
("lagrange <1>" . ("maxima.info-7" 147877 4207 "Funciones y variables para interpol"))
("laguerre" . ("maxima.info-7" 301654 147 "Funciones y variables para polinomios ortogonales"))
("lambda" . ("maxima.info-5" 106219 6333 "Funciones y variables para la definición de funciones"))
("lambda <1>" . ("maxima.info-5" 106220 6332 "Funciones y variables para la definición de funciones"))
("lambda <2>" . ("maxima.info-5" 106288 6265 "Funciones y variables para la definición de funciones"))
("lambert_w" . ("maxima.info-3" 48293 143 "Funciones y variables para las funciones especiales"))
("laplace" . ("maxima.info-3" 87048 4010 "Funciones y variables para la diferenciación"))
("laplacian_matrix" . ("maxima.info-7" 104315 767 "Funciones y variables para graphs"))
("lassociative" . ("maxima.info-2" 19173 243 "Funciones y variables para simplificación"))
("last" . ("maxima.info-1" 132184 107 "Funciones y variables para listas"))
("lbfgs" . ("maxima.info-7" 182957 14837 "Funciones y variables para lbfgs"))
("lbfgs <1>" . ("maxima.info-7" 182958 14836 "Funciones y variables para lbfgs"))
("lbfgs_ncorrections" . ("maxima.info-7" 198173 236 "Funciones y variables para lbfgs"))
("lbfgs_nfeval_max" . ("maxima.info-7" 197861 306 "Funciones y variables para lbfgs"))
("lc2kdt" . ("maxima.info-4" 18879 3452 "Funciones y variables para itensor"))
("lc_l" . ("maxima.info-4" 22347 1083 "Funciones y variables para itensor"))
("lc_u" . ("maxima.info-4" 23436 289 "Funciones y variables para itensor"))
("lcharp" . ("maxima.info-8" 110374 152 "Funciones y variables para caracteres"))
("lcm" . ("maxima.info-4" 208407 259 "Funciones y variables para teoría de números"))
("ldefint" . ("maxima.info-3" 115651 750 "Funciones y variables para integración"))
("ldisp" . ("maxima.info-1" 87150 1074 "Funciones y variables para la impresión"))
("ldisplay" . ("maxima.info-1" 88228 1321 "Funciones y variables para la impresión"))
("legend" . ("maxima.info-2" 163080 528 "Opciones gráficas"))
("legend <1>" . ("maxima.info-2" 163081 527 "Opciones gráficas"))
("legendre_p" . ("maxima.info-7" 301804 168 "Funciones y variables para polinomios ortogonales"))
("legendre_q" . ("maxima.info-7" 301975 147 "Funciones y variables para polinomios ortogonales"))
("leinstein" . ("maxima.info-4" 88065 435 "Funciones y variables para ctensor"))
("length" . ("maxima.info-1" 132295 648 "Funciones y variables para listas"))
("let" . ("maxima.info-4" 304116 4310 "Funciones y variables sobre reglas y patrones"))
("let <1>" . ("maxima.info-4" 304117 4309 "Funciones y variables sobre reglas y patrones"))
("let_rule_packages" . ("maxima.info-4" 310674 279 "Funciones y variables sobre reglas y patrones"))
("letrat" . ("maxima.info-4" 308457 836 "Funciones y variables sobre reglas y patrones"))
("letrules" . ("maxima.info-4" 309295 626 "Funciones y variables sobre reglas y patrones"))
("letrules <1>" . ("maxima.info-4" 309296 625 "Funciones y variables sobre reglas y patrones"))
("letsimp" . ("maxima.info-4" 309926 742 "Funciones y variables sobre reglas y patrones"))
("letsimp <1>" . ("maxima.info-4" 309927 741 "Funciones y variables sobre reglas y patrones"))
("letsimp <2>" . ("maxima.info-4" 309958 711 "Funciones y variables sobre reglas y patrones"))
("levi_civita" . ("maxima.info-4" 18617 255 "Funciones y variables para itensor"))
("lfg" . ("maxima.info-4" 128736 252 "Funciones y variables para ctensor"))
("lfreeof" . ("maxima.info-1" 281526 231 "Funciones y variables para expresiones"))
("lg" . ("maxima.info-4" 129855 159 "Funciones y variables para ctensor"))
("lgtreillis" . ("maxima.info-4" 243138 246 "Funciones y variables para simetrías"))
("lhospitallim" . ("maxima.info-3" 58367 258 "Funciones y variables para límites"))
("lhs" . ("maxima.info-3" 167196 1636 "Funciones y variable para las ecuaciones"))
("li" . ("maxima.info-2" 60355 2525 "Funciones radicales exponenciales y logarítmicas"))
("liediff" . ("maxima.info-4" 30741 795 "Funciones y variables para itensor"))
("limit" . ("maxima.info-3" 58629 2336 "Funciones y variables para límites"))
("limit <1>" . ("maxima.info-3" 58630 2335 "Funciones y variables para límites"))
("limit <2>" . ("maxima.info-3" 58678 2288 "Funciones y variables para límites"))
("limsubst" . ("maxima.info-3" 60998 339 "Funciones y variables para límites"))
("line_graph" . ("maxima.info-7" 85094 77 "Funciones y variables para graphs"))
("line_type" . ("maxima.info-6" 173459 933 "Funciones y variables para draw"))
("line_width" . ("maxima.info-6" 174403 714 "Funciones y variables para draw"))
("linear" . ("maxima.info-2" 19418 1355 "Funciones y variables para simplificación"))
("linear <1>" . ("maxima.info-8" 21445 754 "Paquete functs"))
("linear_program" . ("maxima.info-8" 8739 1126 "Funciones y variables para simplex"))
("linear_regression" . ("maxima.info-8" 84211 3923 "Funciones y variables para stats"))
("linear_regression <1>" . ("maxima.info-8" 84212 3922 "Funciones y variables para stats"))
("linear_solver" . ("maxima.info-8" 200400 232 "Funciones y variables para zeilberger"))
("linearinterpol" . ("maxima.info-7" 152244 2604 "Funciones y variables para interpol"))
("linearinterpol <1>" . ("maxima.info-7" 152245 2603 "Funciones y variables para interpol"))
("linechar" . ("maxima.info-1" 62863 804 "Funciones y variables para la línea de comandos"))
("linel" . ("maxima.info-1" 89558 511 "Funciones y variables para la impresión"))
("linenum" . ("maxima.info-1" 63675 119 "Funciones y variables para la línea de comandos"))
("linewidth" . ("maxima.info-7" 10248 189 "Visualización usando VTK"))
("linsolve" . ("maxima.info-3" 168841 2921 "Funciones y variable para las ecuaciones"))
("linsolve_params" . ("maxima.info-3" 171955 392 "Funciones y variable para las ecuaciones"))
("linsolvewarn" . ("maxima.info-3" 171778 175 "Funciones y variable para las ecuaciones"))
("lispdisp" . ("maxima.info-1" 90075 492 "Funciones y variables para la impresión"))
("list_correlations" . ("maxima.info-5" 271435 3098 "Funciones y variables de parámetros descriptivos"))
("list_correlations <1>" . ("maxima.info-5" 271436 3097 "Funciones y variables de parámetros descriptivos"))
("list_matrix_entries" . ("maxima.info-3" 264922 228 "Funciones y variables para las matrices y el álgebra lineal"))
("list_nc_monomials" . ("maxima.info-3" 294244 295 "Funciones y variables para Afines"))
("list_nc_monomials <1>" . ("maxima.info-3" 294245 294 "Funciones y variables para Afines"))
("listarith" . ("maxima.info-1" 132955 442 "Funciones y variables para listas"))
("listarray" . ("maxima.info-1" 162960 3723 "Funciones y variables para los arrays"))
("listconstvars" . ("maxima.info-1" 279833 319 "Funciones y variables para expresiones"))
("listdummyvars" . ("maxima.info-1" 280154 735 "Funciones y variables para expresiones"))
("listify" . ("maxima.info-5" 31259 459 "Funciones y variables para los conjuntos"))
("listoftens" . ("maxima.info-4" 1321 572 "Funciones y variables para itensor"))
("listofvars" . ("maxima.info-1" 280899 622 "Funciones y variables para expresiones"))
("listp" . ("maxima.info-1" 133402 117 "Funciones y variables para listas"))
("listp <1>" . ("maxima.info-7" 215735 382 "Funciones y variables para linearalgebra"))
("listp <2>" . ("maxima.info-7" 215736 381 "Funciones y variables para linearalgebra"))
("lmax" . ("maxima.info-2" 43017 172 "Funciones para los números"))
("lmin" . ("maxima.info-2" 43192 172 "Funciones para los números"))
("lmxchar" . ("maxima.info-3" 265152 537 "Funciones y variables para las matrices y el álgebra lineal"))
("load" . ("maxima.info-2" 191279 1541 "Funciones y variables para lectura y escritura"))
("load_pathname" . ("maxima.info-2" 192844 877 "Funciones y variables para lectura y escritura"))
("loadfile" . ("maxima.info-2" 193725 418 "Funciones y variables para lectura y escritura"))
("loadprint" . ("maxima.info-2" 194150 575 "Funciones y variables para lectura y escritura"))
("local" . ("maxima.info-5" 112601 1476 "Funciones y variables para la definición de funciones"))
("locate_matrix_entry" . ("maxima.info-7" 216120 1047 "Funciones y variables para linearalgebra"))
("log" . ("maxima.info-2" 62893 1459 "Funciones radicales exponenciales y logarítmicas"))
("log_gamma" . ("maxima.info-3" 6432 73 "Funciones Gamma y factorial"))
("logabs" . ("maxima.info-2" 64373 510 "Funciones radicales exponenciales y logarítmicas"))
("logarc" . ("maxima.info-2" 64888 451 "Funciones radicales exponenciales y logarítmicas"))
("logarc <1>" . ("maxima.info-2" 64889 450 "Funciones radicales exponenciales y logarítmicas"))
("logcb" . ("maxima.info-6" 175126 736 "Funciones y variables para draw"))
("logconcoeffp" . ("maxima.info-2" 65346 457 "Funciones radicales exponenciales y logarítmicas"))
("logcontract" . ("maxima.info-2" 65808 1011 "Funciones radicales exponenciales y logarítmicas"))
("logexpand" . ("maxima.info-2" 66827 509 "Funciones radicales exponenciales y logarítmicas"))
("lognegint" . ("maxima.info-2" 67343 182 "Funciones radicales exponenciales y logarítmicas"))
("logsimp" . ("maxima.info-2" 67526 169 "Funciones radicales exponenciales y logarítmicas"))
("logx" . ("maxima.info-2" 163619 165 "Opciones gráficas"))
("logx <1>" . ("maxima.info-6" 175873 470 "Funciones y variables para draw"))
("logx_secondary" . ("maxima.info-6" 176353 824 "Funciones y variables para draw"))
("logy" . ("maxima.info-2" 163788 166 "Opciones gráficas"))
("logy <1>" . ("maxima.info-6" 177187 467 "Funciones y variables para draw"))
("logy_secondary" . ("maxima.info-6" 177664 824 "Funciones y variables para draw"))
("logz" . ("maxima.info-6" 178498 447 "Funciones y variables para draw"))
("lopow" . ("maxima.info-2" 248601 214 "Funciones y variables para polinomios"))
("lorentz_gauge" . ("maxima.info-4" 44362 206 "Funciones y variables para itensor"))
("lowercasep" . ("maxima.info-8" 110531 93 "Funciones y variables para caracteres"))
("lpart" . ("maxima.info-1" 281760 222 "Funciones y variables para expresiones"))
("lratsubst" . ("maxima.info-2" 248818 1055 "Funciones y variables para polinomios"))
("lreduce" . ("maxima.info-5" 31721 1419 "Funciones y variables para los conjuntos"))
("lreduce <1>" . ("maxima.info-5" 31722 1418 "Funciones y variables para los conjuntos"))
("lriem" . ("maxima.info-4" 129459 102 "Funciones y variables para ctensor"))
("lriemann" . ("maxima.info-4" 89834 538 "Funciones y variables para ctensor"))
("lsquares_estimates" . ("maxima.info-7" 232644 4838 "Funciones y variables para lsquares"))
("lsquares_estimates <1>" . ("maxima.info-7" 232645 4837 "Funciones y variables para lsquares"))
("lsquares_estimates_approximate" . ("maxima.info-7" 239867 3161 "Funciones y variables para lsquares"))
("lsquares_estimates_exact" . ("maxima.info-7" 237525 2325 "Funciones y variables para lsquares"))
("lsquares_mse" . ("maxima.info-7" 243060 2948 "Funciones y variables para lsquares"))
("lsquares_residual_mse" . ("maxima.info-7" 248377 2154 "Funciones y variables para lsquares"))
("lsquares_residuals" . ("maxima.info-7" 246017 2347 "Funciones y variables para lsquares"))
("lsum" . ("maxima.info-4" 145088 680 "Funciones y variables para sumas y productos"))
("ltreillis" . ("maxima.info-4" 243388 273 "Funciones y variables para simetrías"))
("lu_backsub" . ("maxima.info-7" 217176 176 "Funciones y variables para linearalgebra"))
("lu_factor" . ("maxima.info-7" 217354 5732 "Funciones y variables para linearalgebra"))
("lucas" . ("maxima.info-4" 208672 905 "Funciones y variables para teoría de números"))
("m1pbranch" . ("maxima.info-1" 105156 613 "Funciones y variables para los números"))
("macroexpand" . ("maxima.info-5" 73596 1250 "Macros"))
("macroexpand1" . ("maxima.info-5" 74860 1159 "Macros"))
("macroexpansion" . ("maxima.info-5" 114095 5980 "Funciones y variables para la definición de funciones"))
("macros" . ("maxima.info-5" 76032 378 "Macros"))
("mainvar" . ("maxima.info-1" 281985 724 "Funciones y variables para expresiones"))
("makeOrders" . ("maxima.info-7" 255242 811 "Funciones y variables para makeOrders"))
("make_array" . ("maxima.info-1" 166696 2101 "Funciones y variables para los arrays"))
("make_graph" . ("maxima.info-7" 85174 1131 "Funciones y variables para graphs"))
("make_graph <1>" . ("maxima.info-7" 85175 1130 "Funciones y variables para graphs"))
("make_level_picture" . ("maxima.info-6" 255990 1351 "Funciones y variables para picture"))
("make_level_picture <1>" . ("maxima.info-6" 255991 1350 "Funciones y variables para picture"))
("make_poly_continent" . ("maxima.info-6" 261558 504 "Funciones y variables para worldmap"))
("make_poly_continent <1>" . ("maxima.info-6" 261559 503 "Funciones y variables para worldmap"))
("make_poly_country" . ("maxima.info-6" 262067 324 "Funciones y variables para worldmap"))
("make_polygon" . ("maxima.info-6" 262398 3187 "Funciones y variables para worldmap"))
("make_random_state" . ("maxima.info-2" 84927 913 "Números aleatorios"))
("make_random_state <1>" . ("maxima.info-2" 84928 912 "Números aleatorios"))
("make_random_state <2>" . ("maxima.info-2" 84966 875 "Números aleatorios"))
("make_random_state <3>" . ("maxima.info-2" 85004 838 "Números aleatorios"))
("make_rgb_picture" . ("maxima.info-6" 257352 1246 "Funciones y variables para picture"))
("make_string_input_stream" . ("maxima.info-8" 99424 816 "Funciones y variables para entrada y salida"))
("make_string_input_stream <1>" . ("maxima.info-8" 99425 815 "Funciones y variables para entrada y salida"))
("make_string_input_stream <2>" . ("maxima.info-8" 99475 766 "Funciones y variables para entrada y salida"))
("make_string_output_stream" . ("maxima.info-8" 100249 690 "Funciones y variables para entrada y salida"))
("make_transform" . ("maxima.info-2" 144663 643 "Funciones y variables para gráficos"))
("makebox" . ("maxima.info-4" 36090 267 "Funciones y variables para itensor"))
("makefact" . ("maxima.info-3" 33695 177 "Funciones Gamma y factorial"))
("makegamma" . ("maxima.info-3" 10278 177 "Funciones Gamma y factorial"))
("makelist" . ("maxima.info-1" 133521 1894 "Funciones y variables para listas"))
("makelist <1>" . ("maxima.info-1" 133522 1893 "Funciones y variables para listas"))
("makelist <2>" . ("maxima.info-1" 133548 1868 "Funciones y variables para listas"))
("makelist <3>" . ("maxima.info-1" 133585 1832 "Funciones y variables para listas"))
("makelist <4>" . ("maxima.info-1" 133631 1787 "Funciones y variables para listas"))
("makelist <5>" . ("maxima.info-1" 133684 1735 "Funciones y variables para listas"))
("makeset" . ("maxima.info-5" 33155 1683 "Funciones y variables para los conjuntos"))
("mandelbrot_set" . ("maxima.info-7" 72344 511 "Definiciones para fractales complejos"))
("manual_demo" . ("maxima.info-1" 39740 210 "Funciones y variables para la ayuda"))
("map" . ("maxima.info-5" 160858 1463 "Funciones y variables para la programación"))
("mapatom" . ("maxima.info-5" 162334 124 "Funciones y variables para la programación"))
("maperror" . ("maxima.info-5" 162462 612 "Funciones y variables para la programación"))
("maplist" . ("maxima.info-5" 163403 476 "Funciones y variables para la programación"))
("mapprint" . ("maxima.info-5" 163077 323 "Funciones y variables para la programación"))
("mat_cond" . ("maxima.info-7" 223129 545 "Funciones y variables para linearalgebra"))
("mat_cond <1>" . ("maxima.info-7" 223130 544 "Funciones y variables para linearalgebra"))
("mat_fullunblocker" . ("maxima.info-7" 224512 261 "Funciones y variables para linearalgebra"))
("mat_function" . ("maxima.info-6" 29093 10207 "Funciones y variables para diag"))
("mat_norm" . ("maxima.info-7" 223686 295 "Funciones y variables para linearalgebra"))
("mat_norm <1>" . ("maxima.info-7" 223687 294 "Funciones y variables para linearalgebra"))
("mat_norm <2>" . ("maxima.info-7" 223719 263 "Funciones y variables para linearalgebra"))
("mat_trace" . ("maxima.info-7" 224776 246 "Funciones y variables para linearalgebra"))
("mat_unblocker" . ("maxima.info-7" 225024 1488 "Funciones y variables para linearalgebra"))
("matchdeclare" . ("maxima.info-4" 310954 6308 "Funciones y variables sobre reglas y patrones"))
("matchfix" . ("maxima.info-1" 219243 4757 "Operadores definidos por el usuario"))
("matchfix <1>" . ("maxima.info-1" 219244 4756 "Operadores definidos por el usuario"))
("matrix" . ("maxima.info-3" 265693 5957 "Funciones y variables para las matrices y el álgebra lineal"))
("matrix_element_add" . ("maxima.info-3" 271947 1362 "Funciones y variables para las matrices y el álgebra lineal"))
("matrix_element_mult" . ("maxima.info-3" 273318 2049 "Funciones y variables para las matrices y el álgebra lineal"))
("matrix_element_transpose" . ("maxima.info-3" 275377 2326 "Funciones y variables para las matrices y el álgebra lineal"))
("matrix_size" . ("maxima.info-7" 224396 113 "Funciones y variables para linearalgebra"))
("matrixmap" . ("maxima.info-3" 271671 167 "Funciones y variables para las matrices y el álgebra lineal"))
("matrixp" . ("maxima.info-3" 271842 103 "Funciones y variables para las matrices y el álgebra lineal"))
("matrixp <1>" . ("maxima.info-7" 223985 406 "Funciones y variables para linearalgebra"))
("matrixp <2>" . ("maxima.info-7" 223986 405 "Funciones y variables para linearalgebra"))
("mattrace" . ("maxima.info-3" 277719 244 "Funciones y variables para las matrices y el álgebra lineal"))
("max" . ("maxima.info-2" 43367 518 "Funciones para los números"))
("max_clique" . ("maxima.info-7" 105084 258 "Funciones y variables para graphs"))
("max_degree" . ("maxima.info-7" 105345 379 "Funciones y variables para graphs"))
("max_flow" . ("maxima.info-7" 105730 1028 "Funciones y variables para graphs"))
("max_independent_set" . ("maxima.info-7" 106762 361 "Funciones y variables para graphs"))
("max_matching" . ("maxima.info-7" 107126 432 "Funciones y variables para graphs"))
("max_ord" . ("maxima.info-8" 200036 150 "Funciones y variables para zeilberger"))
("maxapplydepth" . ("maxima.info-4" 317328 180 "Funciones y variables sobre reglas y patrones"))
("maxapplyheight" . ("maxima.info-4" 317510 178 "Funciones y variables sobre reglas y patrones"))
("maxima_tempdir" . ("maxima.info-4" 276670 611 "Funciones y variables para el entorno de ejecución"))
("maxima_userdir" . ("maxima.info-4" 277284 864 "Funciones y variables para el entorno de ejecución"))
("maximize_lp" . ("maxima.info-8" 9879 272 "Funciones y variables para simplex"))
("maxnegex" . ("maxima.info-2" 20780 188 "Funciones y variables para simplificación"))
("maxposex" . ("maxima.info-2" 20973 178 "Funciones y variables para simplificación"))
("maxpsifracdenom" . ("maxima.info-3" 33442 250 "Funciones Gamma y factorial"))
("maxpsifracnum" . ("maxima.info-3" 33191 248 "Funciones Gamma y factorial"))
("maxpsinegint" . ("maxima.info-3" 32866 321 "Funciones Gamma y factorial"))
("maxpsiposint" . ("maxima.info-3" 32672 192 "Funciones Gamma y factorial"))
("maxtayorder" . ("maxima.info-4" 160696 268 "Funciones y variables para las series"))
("maybe" . ("maxima.info-2" 128228 787 "Funciones y variables para los hechos"))
("md5sum" . ("maxima.info-8" 112780 597 "Funciones y variables para cadenas"))
("mean" . ("maxima.info-5" 245080 856 "Funciones y variables de parámetros descriptivos"))
("mean <1>" . ("maxima.info-5" 245081 855 "Funciones y variables de parámetros descriptivos"))
("mean_bernoulli" . ("maxima.info-6" 108391 309 "Funciones y variables para distribuciones discretas"))
("mean_beta" . ("maxima.info-6" 76468 176 "Funciones y variables para distribuciones continuas"))
("mean_binomial" . ("maxima.info-6" 103208 220 "Funciones y variables para distribuciones discretas"))
("mean_chi2" . ("maxima.info-6" 58957 270 "Funciones y variables para distribuciones continuas"))
("mean_continuous_uniform" . ("maxima.info-6" 78850 207 "Funciones y variables para distribuciones continuas"))
("mean_deviation" . ("maxima.info-5" 255144 941 "Funciones y variables de parámetros descriptivos"))
("mean_deviation <1>" . ("maxima.info-5" 255145 940 "Funciones y variables de parámetros descriptivos"))
("mean_discrete_uniform" . ("maxima.info-6" 113935 213 "Funciones y variables para distribuciones discretas"))
("mean_exp" . ("maxima.info-6" 67790 375 "Funciones y variables para distribuciones continuas"))
("mean_f" . ("maxima.info-6" 64754 173 "Funciones y variables para distribuciones continuas"))
("mean_gamma" . ("maxima.info-6" 73731 178 "Funciones y variables para distribuciones continuas"))
("mean_general_finite_discrete" . ("maxima.info-6" 100217 220 "Funciones y variables para distribuciones discretas"))
("mean_geometric" . ("maxima.info-6" 111547 194 "Funciones y variables para distribuciones discretas"))
("mean_gumbel" . ("maxima.info-6" 96039 336 "Funciones y variables para distribuciones continuas"))
("mean_hypergeometric" . ("maxima.info-6" 116939 258 "Funciones y variables para distribuciones discretas"))
("mean_laplace" . ("maxima.info-6" 92602 189 "Funciones y variables para distribuciones continuas"))
("mean_logistic" . ("maxima.info-6" 81274 189 "Funciones y variables para distribuciones continuas"))
("mean_lognormal" . ("maxima.info-6" 71201 190 "Funciones y variables para distribuciones continuas"))
("mean_negative_binomial" . ("maxima.info-6" 119757 230 "Funciones y variables para distribuciones discretas"))
("mean_noncentral_chi2" . ("maxima.info-6" 62074 193 "Funciones y variables para distribuciones continuas"))
("mean_noncentral_student_t" . ("maxima.info-6" 54617 656 "Funciones y variables para distribuciones continuas"))
("mean_normal" . ("maxima.info-6" 47731 198 "Funciones y variables para distribuciones continuas"))
("mean_pareto" . ("maxima.info-6" 83524 191 "Funciones y variables para distribuciones continuas"))
("mean_poisson" . ("maxima.info-6" 105878 177 "Funciones y variables para distribuciones discretas"))
("mean_rayleigh" . ("maxima.info-6" 88660 391 "Funciones y variables para distribuciones continuas"))
("mean_student_t" . ("maxima.info-6" 50438 207 "Funciones y variables para distribuciones continuas"))
("mean_weibull" . ("maxima.info-6" 85783 191 "Funciones y variables para distribuciones continuas"))
("median" . ("maxima.info-5" 253681 727 "Funciones y variables de parámetros descriptivos"))
("median <1>" . ("maxima.info-5" 253682 726 "Funciones y variables de parámetros descriptivos"))
("median_deviation" . ("maxima.info-5" 256091 864 "Funciones y variables de parámetros descriptivos"))
("median_deviation <1>" . ("maxima.info-5" 256092 863 "Funciones y variables de parámetros descriptivos"))
("member" . ("maxima.info-1" 135430 1406 "Funciones y variables para listas"))
("mesh" . ("maxima.info-6" 236322 1487 "Funciones y variables para draw"))
("mesh_lines_color" . ("maxima.info-2" 163958 338 "Opciones gráficas"))
("metricexpandall" . ("maxima.info-8" 192596 1039 "Funciones y variables para units"))
("min" . ("maxima.info-2" 43891 518 "Funciones para los números"))
("min_degree" . ("maxima.info-7" 107560 390 "Funciones y variables para graphs"))
("min_edge_cut" . ("maxima.info-7" 107956 233 "Funciones y variables para graphs"))
("min_vertex_cover" . ("maxima.info-7" 108196 91 "Funciones y variables para graphs"))
("min_vertex_cut" . ("maxima.info-7" 108290 129 "Funciones y variables para graphs"))
("minf" . ("maxima.info-1" 118266 81 "Funciones y variables para Constantes"))
("minfactorial" . ("maxima.info-2" 57361 549 "Funciones combinatorias"))
("minimalPoly" . ("maxima.info-6" 26925 682 "Funciones y variables para diag"))
("minimize_lp" . ("maxima.info-8" 10157 1979 "Funciones y variables para simplex"))
("minimum_spanning_tree" . ("maxima.info-7" 108424 313 "Funciones y variables para graphs"))
("minor" . ("maxima.info-3" 277966 174 "Funciones y variables para las matrices y el álgebra lineal"))
;; build_index.pl: something seems wrong for this next item
("minpack_lsquares" . ("" -1 828 "Funciones y variables para minpack"))
;; build_index.pl: something seems wrong for this next item
("minpack_solve" . ("" -1 191 "Funciones y variables para minpack"))
("mnewton" . ("maxima.info-7" 262014 1623 "Funciones y variables para mnewton"))
("mod" . ("maxima.info-4" 209585 1326 "Funciones y variables para teoría de números"))
("mod_big_prime" . ("maxima.info-8" 201953 181 "Funciones y variables para zeilberger"))
("mod_test" . ("maxima.info-8" 201286 192 "Funciones y variables para zeilberger"))
("mod_threshold" . ("maxima.info-8" 202136 168 "Funciones y variables para zeilberger"))
("mode_check_errorp" . ("maxima.info-5" 120294 154 "Funciones y variables para la definición de funciones"))
("mode_check_warnp" . ("maxima.info-5" 120449 154 "Funciones y variables para la definición de funciones"))
("mode_checkp" . ("maxima.info-5" 120109 184 "Funciones y variables para la definición de funciones"))
("mode_declare" . ("maxima.info-5" 120604 1336 "Funciones y variables para la definición de funciones"))
("mode_identity" . ("maxima.info-5" 121958 174 "Funciones y variables para la definición de funciones"))
("modular_linear_solver" . ("maxima.info-8" 201480 256 "Funciones y variables para zeilberger"))
("modulus" . ("maxima.info-2" 249879 1685 "Funciones y variables para polinomios"))
("moebius" . ("maxima.info-5" 34849 1235 "Funciones y variables para los conjuntos"))
("mon2schur" . ("maxima.info-4" 243665 1352 "Funciones y variables para simetrías"))
("mono" . ("maxima.info-3" 293471 159 "Funciones y variables para Afines"))
("monomial_dimensions" . ("maxima.info-3" 293634 182 "Funciones y variables para Afines"))
("multi_elem" . ("maxima.info-4" 245026 886 "Funciones y variables para simetrías"))
("multi_orbit" . ("maxima.info-4" 245917 684 "Funciones y variables para simetrías"))
("multi_pui" . ("maxima.info-4" 246611 440 "Funciones y variables para simetrías"))
("multibernstein_poly" . ("maxima.info-5" 199090 424 "Funciones y variables para Bernstein"))
("multinomial" . ("maxima.info-4" 247056 308 "Funciones y variables para simetrías"))
("multinomial_coeff" . ("maxima.info-5" 36089 1273 "Funciones y variables para los conjuntos"))
("multinomial_coeff <1>" . ("maxima.info-5" 36090 1272 "Funciones y variables para los conjuntos"))
("multiplicative" . ("maxima.info-2" 21156 998 "Funciones y variables para simplificación"))
("multiplicities" . ("maxima.info-3" 172354 202 "Funciones y variable para las ecuaciones"))
("multiplot_mode" . ("maxima.info-6" 127479 1255 "Funciones y variables para draw"))
("multsym" . ("maxima.info-4" 247368 854 "Funciones y variables para simetrías"))
("multthru" . ("maxima.info-2" 22162 2364 "Funciones y variables para simplificación"))
("multthru <1>" . ("maxima.info-2" 22163 2363 "Funciones y variables para simplificación"))
("mycielski_graph" . ("maxima.info-7" 86318 86 "Funciones y variables para graphs"))
("myoptions" . ("maxima.info-1" 63797 238 "Funciones y variables para la línea de comandos"))
("nary" . ("maxima.info-2" 24539 703 "Funciones y variables para simplificación"))
("nary <1>" . ("maxima.info-1" 224032 472 "Operadores definidos por el usuario"))
("nary <2>" . ("maxima.info-1" 224033 471 "Operadores definidos por el usuario"))
("natural_unit" . ("maxima.info-7" 54824 248 "Funciones y variables para ezunits"))
("nc_degree" . ("maxima.info-3" 291926 182 "Funciones y variables para Afines"))
("ncexpt" . ("maxima.info-1" 79360 329 "Funciones y variables para la impresión"))
("ncharpoly" . ("maxima.info-3" 278143 829 "Funciones y variables para las matrices y el álgebra lineal"))
("negative_picture" . ("maxima.info-6" 258604 133 "Funciones y variables para picture"))
("negdistrib" . ("maxima.info-2" 25249 390 "Funciones y variables para simplificación"))
("negsumdispflag" . ("maxima.info-1" 90572 424 "Funciones y variables para la impresión"))
("neighbors" . ("maxima.info-7" 108741 277 "Funciones y variables para graphs"))
("new" . ("maxima.info-1" 174775 868 "Funciones y variables para las estructuras"))
("new <1>" . ("maxima.info-1" 174776 867 "Funciones y variables para las estructuras"))
("new_graph" . ("maxima.info-7" 86406 74 "Funciones y variables para graphs"))
("new_variable" . ("maxima.info-8" 145666 1689 "Funciones y variables para to_poly_solve"))
("newcontext" . ("maxima.info-2" 129020 284 "Funciones y variables para los hechos"))
("newdet" . ("maxima.info-3" 278985 192 "Funciones y variables para las matrices y el álgebra lineal"))
("newline" . ("maxima.info-8" 110628 55 "Funciones y variables para caracteres"))
("newline <1>" . ("maxima.info-8" 100942 269 "Funciones y variables para entrada y salida"))
("newline <2>" . ("maxima.info-8" 100943 268 "Funciones y variables para entrada y salida"))
("newton" . ("maxima.info-3" 215350 1416 "Funciones para la resolución numérica de ecuaciones"))
("newtonepsilon" . ("maxima.info-7" 261400 359 "Funciones y variables para mnewton"))
("newtonmaxiter" . ("maxima.info-7" 261767 240 "Funciones y variables para mnewton"))
("next_prime" . ("maxima.info-4" 210929 158 "Funciones y variables para teoría de números"))
("nextlayerfactor" . ("maxima.info-8" 17354 779 "Paquete facexp"))
("nicedummies" . ("maxima.info-8" 147372 616 "Funciones y variables para to_poly_solve"))
("niceindices" . ("maxima.info-4" 160968 1500 "Funciones y variables para las series"))
("niceindicespref" . ("maxima.info-4" 162481 1084 "Funciones y variables para las series"))
("ninth" . ("maxima.info-1" 136846 133 "Funciones y variables para listas"))
("nm" . ("maxima.info-4" 131454 120 "Funciones y variables para ctensor"))
("nmc" . ("maxima.info-4" 131576 127 "Funciones y variables para ctensor"))
("noeval" . ("maxima.info-1" 255535 234 "Funciones y variables para la evaluación"))
("nofix" . ("maxima.info-1" 224511 456 "Operadores definidos por el usuario"))
("nofix <1>" . ("maxima.info-1" 224512 455 "Operadores definidos por el usuario"))
("nolabels" . ("maxima.info-1" 64037 952 "Funciones y variables para la línea de comandos"))
("nonarray" . ("maxima.info-2" 106208 608 "Funciones y variables para las propiedades"))
("noncentral_moment" . ("maxima.info-5" 248582 930 "Funciones y variables de parámetros descriptivos"))
("noncentral_moment <1>" . ("maxima.info-5" 248583 929 "Funciones y variables de parámetros descriptivos"))
("nonegative_lp" . ("maxima.info-8" 12162 244 "Funciones y variables para simplex"))
("noninteger" . ("maxima.info-2" 105199 479 "Funciones y variables para las propiedades"))
("nonmetricity" . ("maxima.info-4" 104495 127 "Funciones y variables para ctensor"))
("nonnegintegerp" . ("maxima.info-1" 105770 103 "Funciones y variables para los números"))
("nonscalar" . ("maxima.info-2" 106824 196 "Funciones y variables para las propiedades"))
("nonscalarp" . ("maxima.info-2" 107025 162 "Funciones y variables para las propiedades"))
("nonzeroandfreeof" . ("maxima.info-8" 21200 242 "Paquete functs"))
("not" . ("maxima.info-1" 197522 623 "Operadores lógicos"))
("notequal" . ("maxima.info-2" 135999 932 "Funciones y variables para los predicados"))
("noun" . ("maxima.info-1" 282719 206 "Funciones y variables para expresiones"))
("noundisp" . ("maxima.info-1" 282931 243 "Funciones y variables para expresiones"))
("nounify" . ("maxima.info-1" 283177 445 "Funciones y variables para expresiones"))
("nouns" . ("maxima.info-1" 255774 329 "Funciones y variables para la evaluación"))
("np" . ("maxima.info-4" 130429 110 "Funciones y variables para ctensor"))
("npi" . ("maxima.info-4" 130541 652 "Funciones y variables para ctensor"))
("nptetrad" . ("maxima.info-4" 98252 715 "Funciones y variables para ctensor"))
("npv" . ("maxima.info-7" 67799 355 "Funciones y Variables para finance"))
("nroots" . ("maxima.info-3" 172557 444 "Funciones y variable para las ecuaciones"))
("nterms" . ("maxima.info-1" 283634 428 "Funciones y variables para expresiones"))
("ntermst" . ("maxima.info-4" 111156 84 "Funciones y variables para ctensor"))
("nthroot" . ("maxima.info-3" 173008 354 "Funciones y variable para las ecuaciones"))
("nticks" . ("maxima.info-2" 164301 370 "Opciones gráficas"))
("nticks <1>" . ("maxima.info-6" 178955 656 "Funciones y variables para draw"))
("ntrig" . ("maxima.info-2" 76716 309 "Funciones y variables para trigonometría"))
("nullity" . ("maxima.info-7" 226747 107 "Funciones y variables para linearalgebra"))
("nullspace" . ("maxima.info-7" 226518 226 "Funciones y variables para linearalgebra"))
("num" . ("maxima.info-2" 251575 192 "Funciones y variables para polinomios"))
("num_distinct_partitions" . ("maxima.info-5" 37367 880 "Funciones y variables para los conjuntos"))
("num_distinct_partitions <1>" . ("maxima.info-5" 37368 879 "Funciones y variables para los conjuntos"))
("num_partitions" . ("maxima.info-5" 38254 925 "Funciones y variables para los conjuntos"))
("num_partitions <1>" . ("maxima.info-5" 38255 924 "Funciones y variables para los conjuntos"))
("numbered_boundaries" . ("maxima.info-6" 260942 609 "Funciones y variables para worldmap"))
("numberp" . ("maxima.info-1" 105875 1300 "Funciones y variables para los números"))
("numer" . ("maxima.info-1" 107182 757 "Funciones y variables para los números"))
("numer_pbranch" . ("maxima.info-1" 107949 908 "Funciones y variables para los números"))
("numerval" . ("maxima.info-1" 108863 443 "Funciones y variables para los números"))
("numfactor" . ("maxima.info-3" 33877 480 "Funciones Gamma y factorial"))
("nusum" . ("maxima.info-4" 163568 2060 "Funciones y variables para las series"))
("nzeta" . ("maxima.info-3" 48822 121 "Funciones y variables para las funciones especiales"))
("nzetai" . ("maxima.info-3" 49011 64 "Funciones y variables para las funciones especiales"))
("nzetar" . ("maxima.info-3" 48947 62 "Funciones y variables para las funciones especiales"))
("obase" . ("maxima.info-1" 91000 1272 "Funciones y variables para la impresión"))
("odd" . ("maxima.info-2" 101143 647 "Funciones y variables para las propiedades"))
("odd_girth" . ("maxima.info-7" 109021 368 "Funciones y variables para graphs"))
("oddfun" . ("maxima.info-2" 11182 682 "Funciones y variables para simplificación"))
("oddp" . ("maxima.info-1" 109317 111 "Funciones y variables para los números"))
("ode2" . ("maxima.info-3" 196473 3968 "Funciones y variables para ecuaciones diferenciales"))
("ode_check" . ("maxima.info-5" 218716 1183 "Funciones y variables para contrib_ode"))
("odelin" . ("maxima.info-5" 217370 1340 "Funciones y variables para contrib_ode"))
("op" . ("maxima.info-1" 284073 1898 "Funciones y variables para expresiones"))
("opacity" . ("maxima.info-7" 10442 260 "Visualización usando VTK"))
("opena" . ("maxima.info-8" 101218 167 "Funciones y variables para entrada y salida"))
("opena_binary" . ("maxima.info-7" 275381 152 "Funciones y variables para entrada y salida en formato binario"))
("openr" . ("maxima.info-8" 101388 130 "Funciones y variables para entrada y salida"))
("openr_binary" . ("maxima.info-7" 275071 151 "Funciones y variables para entrada y salida en formato binario"))
("openw" . ("maxima.info-8" 101521 223 "Funciones y variables para entrada y salida"))
("openw_binary" . ("maxima.info-7" 275224 155 "Funciones y variables para entrada y salida en formato binario"))
("operatorp" . ("maxima.info-1" 285988 355 "Funciones y variables para expresiones"))
("operatorp <1>" . ("maxima.info-1" 285989 354 "Funciones y variables para expresiones"))
("opproperties" . ("maxima.info-2" 25645 355 "Funciones y variables para simplificación"))
("opsubst" . ("maxima.info-1" 286347 237 "Funciones y variables para expresiones"))
("opsubst <1>" . ("maxima.info-7" 278828 2329 "Funciones y variables para opsubst"))
("opsubst <2>" . ("maxima.info-7" 278829 2328 "Funciones y variables para opsubst"))
("opsubst <3>" . ("maxima.info-7" 278865 2293 "Funciones y variables para opsubst"))
("optimize" . ("maxima.info-1" 286587 410 "Funciones y variables para expresiones"))
("optimprefix" . ("maxima.info-1" 287004 183 "Funciones y variables para expresiones"))
("optionset" . ("maxima.info-1" 64998 704 "Funciones y variables para la línea de comandos"))
("or" . ("maxima.info-1" 198153 1079 "Operadores lógicos"))
("orbit" . ("maxima.info-4" 248234 564 "Funciones y variables para simetrías"))
("orbits" . ("maxima.info-6" 288804 1289 "Análisis gráfico de sistemas dinámicos discretos"))
("ordergreat" . ("maxima.info-1" 287190 900 "Funciones y variables para expresiones"))
("ordergreatp" . ("maxima.info-1" 288100 3955 "Funciones y variables para expresiones"))
("orderless" . ("maxima.info-1" 287191 899 "Funciones y variables para expresiones"))
("orderlessp" . ("maxima.info-1" 288101 3954 "Funciones y variables para expresiones"))
("orientation" . ("maxima.info-7" 10707 396 "Visualización usando VTK"))
("origin" . ("maxima.info-7" 11111 276 "Visualización usando VTK"))
("orthogonal_complement" . ("maxima.info-7" 226857 239 "Funciones y variables para linearalgebra"))
("orthopoly_recur" . ("maxima.info-7" 302124 1189 "Funciones y variables para polinomios ortogonales"))
("orthopoly_returns_intervals" . ("maxima.info-7" 303320 397 "Funciones y variables para polinomios ortogonales"))
("orthopoly_weight" . ("maxima.info-7" 303720 785 "Funciones y variables para polinomios ortogonales"))
("out_neighbors" . ("maxima.info-7" 109392 389 "Funciones y variables para graphs"))
("outative" . ("maxima.info-2" 26001 1360 "Funciones y variables para simplificación"))
("outchar" . ("maxima.info-1" 65709 964 "Funciones y variables para la línea de comandos"))
("outermap" . ("maxima.info-5" 166573 7020 "Funciones y variables para la programación"))
("outofpois" . ("maxima.info-4" 190356 347 "Funciones y variables para series de Poisson"))
("packagefile" . ("maxima.info-2" 194732 365 "Funciones y variables para lectura y escritura"))
("packagefile <1>" . ("maxima.info-4" 288115 359 "Funciones y variables para la miscelánea de opciones"))
("pade" . ("maxima.info-4" 165640 2961 "Funciones y variables para las series"))
("palette" . ("maxima.info-2" 164679 1416 "Opciones gráficas"))
("palette <1>" . ("maxima.info-2" 164741 1355 "Opciones gráficas"))
("palette <2>" . ("maxima.info-6" 179621 3472 "Funciones y variables para draw"))
("parGosper" . ("maxima.info-8" 198875 361 "Funciones y variables para zeilberger"))
("parabolic_cylinder_d" . ("maxima.info-3" 41421 131 "Funciones de cilindro parabólico"))
("parametric" . ("maxima.info-6" 237815 1474 "Funciones y variables para draw"))
("parametric <1>" . ("maxima.info-6" 237816 1473 "Funciones y variables para draw"))
("parametric_surface" . ("maxima.info-6" 239299 1254 "Funciones y variables para draw"))
("parg" . ("maxima.info-8" 147997 1207 "Funciones y variables para to_poly_solve"))
("parse_string" . ("maxima.info-8" 113383 710 "Funciones y variables para cadenas"))
("part" . ("maxima.info-1" 292093 1626 "Funciones y variables para expresiones"))
("part2cont" . ("maxima.info-4" 248807 512 "Funciones y variables para simetrías"))
("partfrac" . ("maxima.info-4" 211089 1246 "Funciones y variables para teoría de números"))
("partition" . ("maxima.info-1" 293743 594 "Funciones y variables para expresiones"))
("partition_set" . ("maxima.info-5" 39187 827 "Funciones y variables para los conjuntos"))
("partpol" . ("maxima.info-4" 249325 408 "Funciones y variables para simetrías"))
("partswitch" . ("maxima.info-1" 294341 243 "Funciones y variables para expresiones"))
("path_digraph" . ("maxima.info-7" 86483 82 "Funciones y variables para graphs"))
("path_graph" . ("maxima.info-7" 86568 71 "Funciones y variables para graphs"))
("pathname_directory" . ("maxima.info-2" 195101 564 "Funciones y variables para lectura y escritura"))
("pathname_name" . ("maxima.info-2" 195102 563 "Funciones y variables para lectura y escritura"))
("pathname_type" . ("maxima.info-2" 195148 518 "Funciones y variables para lectura y escritura"))
("pdf_bernoulli" . ("maxima.info-6" 107422 370 "Funciones y variables para distribuciones discretas"))
("pdf_beta" . ("maxima.info-6" 75475 233 "Funciones y variables para distribuciones continuas"))
("pdf_binomial" . ("maxima.info-6" 102080 280 "Funciones y variables para distribuciones discretas"))
("pdf_cauchy" . ("maxima.info-6" 94028 238 "Funciones y variables para distribuciones continuas"))
("pdf_chi2" . ("maxima.info-6" 57435 589 "Funciones y variables para distribuciones continuas"))
("pdf_continuous_uniform" . ("maxima.info-6" 77960 258 "Funciones y variables para distribuciones continuas"))
("pdf_discrete_uniform" . ("maxima.info-6" 113020 273 "Funciones y variables para distribuciones discretas"))
("pdf_exp" . ("maxima.info-6" 66370 386 "Funciones y variables para distribuciones continuas"))
("pdf_f" . ("maxima.info-6" 63519 227 "Funciones y variables para distribuciones continuas"))
("pdf_gamma" . ("maxima.info-6" 72690 235 "Funciones y variables para distribuciones continuas"))
("pdf_general_finite_discrete" . ("maxima.info-6" 98189 882 "Funciones y variables para distribuciones discretas"))
("pdf_geometric" . ("maxima.info-6" 110699 254 "Funciones y variables para distribuciones discretas"))
("pdf_gumbel" . ("maxima.info-6" 95233 238 "Funciones y variables para distribuciones continuas"))
("pdf_hypergeometric" . ("maxima.info-6" 115570 566 "Funciones y variables para distribuciones discretas"))
("pdf_laplace" . ("maxima.info-6" 91788 240 "Funciones y variables para distribuciones continuas"))
("pdf_logistic" . ("maxima.info-6" 80457 240 "Funciones y variables para distribuciones continuas"))
("pdf_lognormal" . ("maxima.info-6" 69992 241 "Funciones y variables para distribuciones continuas"))
("pdf_negative_binomial" . ("maxima.info-6" 118659 288 "Funciones y variables para distribuciones discretas"))
("pdf_noncentral_chi2" . ("maxima.info-6" 61080 316 "Funciones y variables para distribuciones continuas"))
("pdf_noncentral_student_t" . ("maxima.info-6" 52469 1159 "Funciones y variables para distribuciones continuas"))
("pdf_normal" . ("maxima.info-6" 46241 234 "Funciones y variables para distribuciones continuas"))
("pdf_pareto" . ("maxima.info-6" 82711 240 "Funciones y variables para distribuciones continuas"))
("pdf_poisson" . ("maxima.info-6" 104955 237 "Funciones y variables para distribuciones discretas"))
("pdf_rank_sum" . ("maxima.info-8" 88978 288 "Funciones y variables para distribuciones especiales"))
("pdf_rayleigh" . ("maxima.info-6" 87221 447 "Funciones y variables para distribuciones continuas"))
("pdf_signed_rank" . ("maxima.info-8" 88402 281 "Funciones y variables para distribuciones especiales"))
("pdf_student_t" . ("maxima.info-6" 49269 238 "Funciones y variables para distribuciones continuas"))
("pdf_weibull" . ("maxima.info-6" 84963 242 "Funciones y variables para distribuciones continuas"))
("pearson_skewness" . ("maxima.info-5" 260512 807 "Funciones y variables de parámetros descriptivos"))
("pearson_skewness <1>" . ("maxima.info-5" 260513 806 "Funciones y variables de parámetros descriptivos"))
("permanent" . ("maxima.info-3" 279180 297 "Funciones y variables para las matrices y el álgebra lineal"))
("permut" . ("maxima.info-4" 249738 83 "Funciones y variables para simetrías"))
("permutation" . ("maxima.info-8" 26314 190 "Paquete functs"))
("permutations" . ("maxima.info-5" 40021 643 "Funciones y variables para los conjuntos"))
("petersen_graph" . ("maxima.info-7" 86642 180 "Funciones y variables para graphs"))
("petersen_graph <1>" . ("maxima.info-7" 86643 179 "Funciones y variables para graphs"))
("petrov" . ("maxima.info-4" 99241 3983 "Funciones y variables para ctensor"))
("pfeformat" . ("maxima.info-1" 92283 734 "Funciones y variables para la impresión"))
("phi" . ("maxima.info-1" 118350 2469 "Funciones y variables para Constantes"))
("phiresolution" . ("maxima.info-7" 11393 322 "Visualización usando VTK"))
("pi" . ("maxima.info-1" 120832 209 "Funciones y variables para Constantes"))
("pickapart" . ("maxima.info-1" 294587 3042 "Funciones y variables para expresiones"))
("picture_equalp" . ("maxima.info-6" 258740 139 "Funciones y variables para picture"))
("picturep" . ("maxima.info-6" 258883 127 "Funciones y variables para picture"))
("piece" . ("maxima.info-1" 297637 103 "Funciones y variables para expresiones"))
("piechart" . ("maxima.info-6" 14005 1966 "Funciones y variables para gráficos estadísticos"))
("piechart <1>" . ("maxima.info-6" 14006 1965 "Funciones y variables para gráficos estadísticos"))
("piechart <2>" . ("maxima.info-6" 14038 1934 "Funciones y variables para gráficos estadísticos"))
("piechart <3>" . ("maxima.info-6" 14099 1874 "Funciones y variables para gráficos estadísticos"))
("piechart <4>" . ("maxima.info-6" 14144 1830 "Funciones y variables para gráficos estadísticos"))
("piechart <5>" . ("maxima.info-6" 14218 1757 "Funciones y variables para gráficos estadísticos"))
("piechart_description" . ("maxima.info-6" 14260 1716 "Funciones y variables para gráficos estadísticos"))
("planar_embedding" . ("maxima.info-7" 109784 532 "Funciones y variables para graphs"))
("playback" . ("maxima.info-1" 66681 2264 "Funciones y variables para la línea de comandos"))
("playback <1>" . ("maxima.info-1" 66682 2263 "Funciones y variables para la línea de comandos"))
("playback <2>" . ("maxima.info-1" 66708 2238 "Funciones y variables para la línea de comandos"))
("playback <3>" . ("maxima.info-1" 66737 2210 "Funciones y variables para la línea de comandos"))
("playback <4>" . ("maxima.info-1" 66773 2175 "Funciones y variables para la línea de comandos"))
("playback <5>" . ("maxima.info-1" 66804 2145 "Funciones y variables para la línea de comandos"))
("playback <6>" . ("maxima.info-1" 66835 2115 "Funciones y variables para la línea de comandos"))
("playback <7>" . ("maxima.info-1" 66865 2086 "Funciones y variables para la línea de comandos"))
("plog" . ("maxima.info-2" 67696 131 "Funciones radicales exponenciales y logarítmicas"))
("plot2d" . ("maxima.info-2" 145579 6550 "Funciones y variables para gráficos"))
("plot2d <1>" . ("maxima.info-2" 145580 6549 "Funciones y variables para gráficos"))
("plot2d <2>" . ("maxima.info-2" 145642 6488 "Funciones y variables para gráficos"))
("plot3d" . ("maxima.info-2" 152192 4474 "Funciones y variables para gráficos"))
("plot3d <1>" . ("maxima.info-2" 152193 4473 "Funciones y variables para gráficos"))
("plot_format" . ("maxima.info-2" 166116 313 "Opciones gráficas"))
("plot_options" . ("maxima.info-2" 156712 799 "Funciones y variables para gráficos"))
("plot_realpart" . ("maxima.info-2" 166432 593 "Opciones gráficas"))
("plotdf" . ("maxima.info-3" 218574 10080 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("plotdf <1>" . ("maxima.info-3" 218575 10079 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("plotdf <2>" . ("maxima.info-3" 218620 10035 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("plotdf <3>" . ("maxima.info-3" 218680 9976 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("ploteq" . ("maxima.info-3" 228787 1276 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("plsquares" . ("maxima.info-7" 250541 4322 "Funciones y variables para lsquares"))
("plsquares <1>" . ("maxima.info-7" 250542 4321 "Funciones y variables para lsquares"))
("plsquares <2>" . ("maxima.info-7" 250594 4270 "Funciones y variables para lsquares"))
("pochhammer" . ("maxima.info-7" 304509 1313 "Funciones y variables para polinomios ortogonales"))
("pochhammer_max_index" . ("maxima.info-7" 305833 526 "Funciones y variables para polinomios ortogonales"))
("point_size" . ("maxima.info-6" 183123 637 "Funciones y variables para draw"))
("point_type" . ("maxima.info-2" 167030 703 "Opciones gráficas"))
("point_type <1>" . ("maxima.info-6" 183770 1720 "Funciones y variables para draw"))
("points" . ("maxima.info-7" 11721 288 "Visualización usando VTK"))
("points <10>" . ("maxima.info-6" 241031 6199 "Funciones y variables para draw"))
("points <11>" . ("maxima.info-6" 241102 6129 "Funciones y variables para draw"))
("points <1>" . ("maxima.info-6" 240560 6662 "Funciones y variables para draw"))
("points <2>" . ("maxima.info-6" 240561 6661 "Funciones y variables para draw"))
("points <3>" . ("maxima.info-6" 240622 6601 "Funciones y variables para draw"))
("points <4>" . ("maxima.info-6" 240685 6539 "Funciones y variables para draw"))
("points <5>" . ("maxima.info-6" 240731 6494 "Funciones y variables para draw"))
("points <6>" . ("maxima.info-6" 240865 6362 "Funciones y variables para draw"))
("points <7>" . ("maxima.info-6" 240892 6335 "Funciones y variables para draw"))
("points <8>" . ("maxima.info-6" 240931 6297 "Funciones y variables para draw"))
("points <9>" . ("maxima.info-6" 240974 6255 "Funciones y variables para draw"))
("points_joined" . ("maxima.info-6" 185499 1205 "Funciones y variables para draw"))
("pointsize" . ("maxima.info-7" 12016 186 "Visualización usando VTK"))
("poisdiff" . ("maxima.info-4" 190710 173 "Funciones y variables para series de Poisson"))
("poisexpt" . ("maxima.info-4" 190888 123 "Funciones y variables para series de Poisson"))
("poisint" . ("maxima.info-4" 191014 93 "Funciones y variables para series de Poisson"))
("poislim" . ("maxima.info-4" 191109 327 "Funciones y variables para series de Poisson"))
("poismap" . ("maxima.info-4" 191438 348 "Funciones y variables para series de Poisson"))
("poisplus" . ("maxima.info-4" 191791 69 "Funciones y variables para series de Poisson"))
("poissimp" . ("maxima.info-4" 191863 115 "Funciones y variables para series de Poisson"))
("poisson" . ("maxima.info-4" 191981 144 "Funciones y variables para series de Poisson"))
("poissubst" . ("maxima.info-4" 192129 416 "Funciones y variables para series de Poisson"))
("poistimes" . ("maxima.info-4" 192548 72 "Funciones y variables para series de Poisson"))
("polar" . ("maxima.info-6" 247245 899 "Funciones y variables para draw"))
("polar_to_xy" . ("maxima.info-2" 145312 262 "Funciones y variables para gráficos"))
("polarform" . ("maxima.info-2" 50033 147 "Funciones para los números complejos"))
("polartorect" . ("maxima.info-3" 201796 696 "Funciones y variables para la transformada rápida de Fourier"))
("poly_add" . ("maxima.info-7" 135267 246 "Funciones y variables para grobner"))
("poly_buchberger" . ("maxima.info-7" 139843 192 "Funciones y variables para grobner"))
("poly_buchberger_criterion" . ("maxima.info-7" 139518 322 "Funciones y variables para grobner"))
("poly_coefficient_ring" . ("maxima.info-7" 132796 355 "Funciones y variables para grobner"))
("poly_colon_ideal" . ("maxima.info-7" 142178 214 "Funciones y variables para grobner"))
("poly_content" . ("maxima.info-7" 138228 201 "Funciones y variables para grobner"))
("poly_depends_p" . ("maxima.info-7" 141761 136 "Funciones y variables para grobner"))
("poly_elimination_ideal" . ("maxima.info-7" 141899 275 "Funciones y variables para grobner"))
("poly_elimination_order" . ("maxima.info-7" 133676 383 "Funciones y variables para grobner"))
("poly_exact_divide" . ("maxima.info-7" 139109 214 "Funciones y variables para grobner"))
("poly_expand" . ("maxima.info-7" 136694 1203 "Funciones y variables para grobner"))
("poly_expt" . ("maxima.info-7" 137901 323 "Funciones y variables para grobner"))
("poly_gcd" . ("maxima.info-7" 142652 604 "Funciones y variables para grobner"))
("poly_grobner" . ("maxima.info-7" 141357 189 "Funciones y variables para grobner"))
("poly_grobner_algorithm" . ("maxima.info-7" 134495 276 "Funciones y variables para grobner"))
("poly_grobner_debug" . ("maxima.info-7" 134350 143 "Funciones y variables para grobner"))
("poly_grobner_equal" . ("maxima.info-7" 143260 680 "Funciones y variables para grobner"))
("poly_grobner_member" . ("maxima.info-7" 144247 264 "Funciones y variables para grobner"))
("poly_grobner_subsetp" . ("maxima.info-7" 143944 298 "Funciones y variables para grobner"))
("poly_ideal_intersection" . ("maxima.info-7" 142394 158 "Funciones y variables para grobner"))
("poly_ideal_polysaturation" . ("maxima.info-7" 145713 311 "Funciones y variables para grobner"))
("poly_ideal_polysaturation1" . ("maxima.info-7" 145284 427 "Funciones y variables para grobner"))
("poly_ideal_saturation" . ("maxima.info-7" 144889 391 "Funciones y variables para grobner"))
("poly_ideal_saturation1" . ("maxima.info-7" 144514 371 "Funciones y variables para grobner"))
("poly_lcm" . ("maxima.info-7" 142555 95 "Funciones y variables para grobner"))
("poly_minimization" . ("maxima.info-7" 140796 320 "Funciones y variables para grobner"))
("poly_monomial_order" . ("maxima.info-7" 132558 234 "Funciones y variables para grobner"))
("poly_multiply" . ("maxima.info-7" 135777 221 "Funciones y variables para grobner"))
("poly_normal_form" . ("maxima.info-7" 139326 190 "Funciones y variables para grobner"))
("poly_normalize" . ("maxima.info-7" 136403 242 "Funciones y variables para grobner"))
("poly_normalize_list" . ("maxima.info-7" 141121 234 "Funciones y variables para grobner"))
("poly_polysaturation_extension" . ("maxima.info-7" 146188 100 "Funciones y variables para grobner"))
("poly_primary_elimination_order" . ("maxima.info-7" 133155 258 "Funciones y variables para grobner"))
("poly_primitive_part" . ("maxima.info-7" 136157 244 "Funciones y variables para grobner"))
("poly_pseudo_divide" . ("maxima.info-7" 138431 673 "Funciones y variables para grobner"))
("poly_reduced_grobner" . ("maxima.info-7" 141548 211 "Funciones y variables para grobner"))
("poly_reduction" . ("maxima.info-7" 140581 213 "Funciones y variables para grobner"))
("poly_return_term_list" . ("maxima.info-7" 134067 279 "Funciones y variables para grobner"))
("poly_s_polynomial" . ("maxima.info-7" 136000 155 "Funciones y variables para grobner"))
("poly_saturation_extension" . ("maxima.info-7" 146027 159 "Funciones y variables para grobner"))
("poly_secondary_elimination_order" . ("maxima.info-7" 133416 257 "Funciones y variables para grobner"))
("poly_subtract" . ("maxima.info-7" 135515 260 "Funciones y variables para grobner"))
("poly_top_reduction_only" . ("maxima.info-7" 134772 199 "Funciones y variables para grobner"))
("polydecomp" . ("maxima.info-2" 251773 2007 "Funciones y variables para polinomios"))
("polyfactor" . ("maxima.info-3" 173368 333 "Funciones y variable para las ecuaciones"))
("polygon" . ("maxima.info-6" 248148 839 "Funciones y variables para draw"))
("polygon <1>" . ("maxima.info-6" 248149 838 "Funciones y variables para draw"))
("polymod" . ("maxima.info-2" 253789 364 "Funciones y variables para polinomios"))
("polymod <1>" . ("maxima.info-2" 253790 363 "Funciones y variables para polinomios"))
("polynome2ele" . ("maxima.info-4" 249823 596 "Funciones y variables para simetrías"))
("polynomialp" . ("maxima.info-7" 227098 1766 "Funciones y variables para linearalgebra"))
("polynomialp <1>" . ("maxima.info-7" 227099 1765 "Funciones y variables para linearalgebra"))
("polynomialp <2>" . ("maxima.info-7" 227156 1709 "Funciones y variables para linearalgebra"))
("polytocompanion" . ("maxima.info-7" 228873 304 "Funciones y variables para linearalgebra"))
("pop" . ("maxima.info-1" 136984 335 "Funciones y variables para listas"))
("posfun" . ("maxima.info-2" 107190 186 "Funciones y variables para las propiedades"))
("position" . ("maxima.info-7" 12207 218 "Visualización usando VTK"))
("postfix" . ("maxima.info-1" 224977 269 "Operadores definidos por el usuario"))
("postfix <1>" . ("maxima.info-1" 224978 268 "Operadores definidos por el usuario"))
("power_mod" . ("maxima.info-4" 212341 559 "Funciones y variables para teoría de números"))
("powerdisp" . ("maxima.info-1" 93021 857 "Funciones y variables para la impresión"))
("powers" . ("maxima.info-2" 254160 144 "Funciones y variables para polinomios"))
("powerseries" . ("maxima.info-4" 168609 1979 "Funciones y variables para las series"))
("powerset" . ("maxima.info-5" 40669 1180 "Funciones y variables para los conjuntos"))
("powerset <1>" . ("maxima.info-5" 40670 1179 "Funciones y variables para los conjuntos"))
("pred" . ("maxima.info-1" 256112 389 "Funciones y variables para la evaluación"))
("prederror" . ("maxima.info-5" 163885 576 "Funciones y variables para la programación"))
("prefix" . ("maxima.info-1" 225252 274 "Operadores definidos por el usuario"))
("prefix <1>" . ("maxima.info-1" 225253 273 "Operadores definidos por el usuario"))
("prev_prime" . ("maxima.info-4" 213951 158 "Funciones y variables para teoría de números"))
("primep" . ("maxima.info-4" 212903 899 "Funciones y variables para teoría de números"))
("primep_number_of_tests" . ("maxima.info-4" 213811 138 "Funciones y variables para teoría de números"))
("principal_components" . ("maxima.info-5" 274546 4385 "Funciones y variables de parámetros descriptivos"))
("principal_components <1>" . ("maxima.info-5" 274547 4384 "Funciones y variables de parámetros descriptivos"))
("print" . ("maxima.info-1" 93883 1373 "Funciones y variables para la impresión"))
("print_graph" . ("maxima.info-7" 110320 706 "Funciones y variables para graphs"))
("printf" . ("maxima.info-8" 101747 4811 "Funciones y variables para entrada y salida"))
("printf <1>" . ("maxima.info-8" 101748 4810 "Funciones y variables para entrada y salida"))
("printfile" . ("maxima.info-2" 195669 568 "Funciones y variables para lectura y escritura"))
("printpois" . ("maxima.info-4" 192623 202 "Funciones y variables para series de Poisson"))
("printprops" . ("maxima.info-2" 107383 559 "Funciones y variables para las propiedades"))
("printprops <1>" . ("maxima.info-2" 107384 558 "Funciones y variables para las propiedades"))
("printprops <2>" . ("maxima.info-2" 107420 523 "Funciones y variables para las propiedades"))
("prodrac" . ("maxima.info-4" 250427 254 "Funciones y variables para simetrías"))
("product" . ("maxima.info-4" 146968 2672 "Funciones y variables para sumas y productos"))
("product_use_gamma" . ("maxima.info-8" 43042 281 "Funciones y variables para solve_rec"))
("program" . ("maxima.info-7" 129202 465 "Funciones y variables para graphs"))
("programmode" . ("maxima.info-3" 173704 1037 "Funciones y variable para las ecuaciones"))
("prompt" . ("maxima.info-1" 68970 227 "Funciones y variables para la línea de comandos"))
("properties" . ("maxima.info-2" 107952 124 "Funciones y variables para las propiedades"))
("proportional_axes" . ("maxima.info-6" 186713 1146 "Funciones y variables para draw"))
("props" . ("maxima.info-2" 108078 275 "Funciones y variables para las propiedades"))
("propvars" . ("maxima.info-2" 108357 262 "Funciones y variables para las propiedades"))
("psexpand" . ("maxima.info-4" 170593 433 "Funciones y variables para las series"))
("psfile" . ("maxima.info-2" 167738 326 "Opciones gráficas"))
("psi" . ("maxima.info-3" 31665 995 "Funciones Gamma y factorial"))
("psi <1>" . ("maxima.info-4" 98974 264 "Funciones y variables para ctensor"))
("psubst" . ("maxima.info-1" 297743 778 "Funciones y variables para expresiones"))
("psubst <1>" . ("maxima.info-1" 297744 777 "Funciones y variables para expresiones"))
("ptriangularize" . ("maxima.info-7" 229181 467 "Funciones y variables para linearalgebra"))
("pui" . ("maxima.info-4" 250686 1573 "Funciones y variables para simetrías"))
("pui2comp" . ("maxima.info-4" 252268 1504 "Funciones y variables para simetrías"))
("pui2ele" . ("maxima.info-4" 253779 526 "Funciones y variables para simetrías"))
("pui2polynome" . ("maxima.info-4" 254312 670 "Funciones y variables para simetrías"))
("pui_direct" . ("maxima.info-4" 254987 2183 "Funciones y variables para simetrías"))
("puireduc" . ("maxima.info-4" 257183 717 "Funciones y variables para simetrías"))
("push" . ("maxima.info-1" 137325 1385 "Funciones y variables para listas"))
("put" . ("maxima.info-2" 108624 904 "Funciones y variables para las propiedades"))
("pv" . ("maxima.info-7" 58951 338 "Funciones y Variables para finance"))
("qput" . ("maxima.info-2" 109535 831 "Funciones y variables para las propiedades"))
("qrange" . ("maxima.info-5" 254415 723 "Funciones y variables de parámetros descriptivos"))
("qrange <1>" . ("maxima.info-5" 254416 722 "Funciones y variables de parámetros descriptivos"))
("qty" . ("maxima.info-7" 41345 1709 "Funciones y variables para ezunits"))
("quad_control" . ("maxima.info-3" 146944 661 "Funciones y variables para QUADPACK"))
("quad_qag" . ("maxima.info-3" 122958 3185 "Funciones y variables para QUADPACK"))
("quad_qag <1>" . ("maxima.info-3" 123024 3120 "Funciones y variables para QUADPACK"))
("quad_qagi" . ("maxima.info-3" 128675 2762 "Funciones y variables para QUADPACK"))
("quad_qagi <1>" . ("maxima.info-3" 128745 2693 "Funciones y variables para QUADPACK"))
("quad_qagp" . ("maxima.info-3" 144153 2764 "Funciones y variables para QUADPACK"))
("quad_qagp <1>" . ("maxima.info-3" 144222 2696 "Funciones y variables para QUADPACK"))
("quad_qags" . ("maxima.info-3" 126185 2458 "Funciones y variables para QUADPACK"))
("quad_qags <1>" . ("maxima.info-3" 126255 2389 "Funciones y variables para QUADPACK"))
("quad_qawc" . ("maxima.info-3" 131468 3413 "Funciones y variables para QUADPACK"))
("quad_qawc <1>" . ("maxima.info-3" 131543 3339 "Funciones y variables para QUADPACK"))
("quad_qawf" . ("maxima.info-3" 134906 2740 "Funciones y variables para QUADPACK"))
("quad_qawf <1>" . ("maxima.info-3" 134978 2669 "Funciones y variables para QUADPACK"))
("quad_qawo" . ("maxima.info-3" 137673 3182 "Funciones y variables para QUADPACK"))
("quad_qawo <1>" . ("maxima.info-3" 137739 3117 "Funciones y variables para QUADPACK"))
("quad_qaws" . ("maxima.info-3" 140889 3236 "Funciones y variables para QUADPACK"))
("quad_qaws <1>" . ("maxima.info-3" 140963 3163 "Funciones y variables para QUADPACK"))
("quadrilateral" . ("maxima.info-6" 248994 1044 "Funciones y variables para draw"))
("quantile" . ("maxima.info-5" 252788 888 "Funciones y variables de parámetros descriptivos"))
("quantile <1>" . ("maxima.info-5" 252789 887 "Funciones y variables de parámetros descriptivos"))
("quantile_bernoulli" . ("maxima.info-6" 108060 326 "Funciones y variables para distribuciones discretas"))
("quantile_beta" . ("maxima.info-6" 76158 305 "Funciones y variables para distribuciones continuas"))
("quantile_binomial" . ("maxima.info-6" 102851 352 "Funciones y variables para distribuciones discretas"))
("quantile_cauchy" . ("maxima.info-6" 94518 311 "Funciones y variables para distribuciones continuas"))
("quantile_chi2" . ("maxima.info-6" 58522 429 "Funciones y variables para distribuciones continuas"))
("quantile_continuous_uniform" . ("maxima.info-6" 78496 349 "Funciones y variables para distribuciones continuas"))
("quantile_discrete_uniform" . ("maxima.info-6" 113577 353 "Funciones y variables para distribuciones discretas"))
("quantile_exp" . ("maxima.info-6" 67155 632 "Funciones y variables para distribuciones continuas"))
("quantile_f" . ("maxima.info-6" 64233 518 "Funciones y variables para distribuciones continuas"))
("quantile_gamma" . ("maxima.info-6" 73419 307 "Funciones y variables para distribuciones continuas"))
("quantile_general_finite_discrete" . ("maxima.info-6" 99979 234 "Funciones y variables para distribuciones discretas"))
("quantile_geometric" . ("maxima.info-6" 111220 321 "Funciones y variables para distribuciones discretas"))
("quantile_gumbel" . ("maxima.info-6" 95723 311 "Funciones y variables para distribuciones continuas"))
("quantile_hypergeometric" . ("maxima.info-6" 116537 396 "Funciones y variables para distribuciones discretas"))
("quantile_laplace" . ("maxima.info-6" 92282 315 "Funciones y variables para distribuciones continuas"))
("quantile_logistic" . ("maxima.info-6" 80953 315 "Funciones y variables para distribuciones continuas"))
("quantile_lognormal" . ("maxima.info-6" 70879 317 "Funciones y variables para distribuciones continuas"))
("quantile_negative_binomial" . ("maxima.info-6" 119383 369 "Funciones y variables para distribuciones discretas"))
("quantile_noncentral_chi2" . ("maxima.info-6" 61655 412 "Funciones y variables para distribuciones continuas"))
("quantile_noncentral_student_t" . ("maxima.info-6" 54187 424 "Funciones y variables para distribuciones continuas"))
("quantile_normal" . ("maxima.info-6" 47121 605 "Funciones y variables para distribuciones continuas"))
("quantile_pareto" . ("maxima.info-6" 83205 314 "Funciones y variables para distribuciones continuas"))
("quantile_poisson" . ("maxima.info-6" 105564 309 "Funciones y variables para distribuciones discretas"))
("quantile_rayleigh" . ("maxima.info-6" 88123 534 "Funciones y variables para distribuciones continuas"))
("quantile_student_t" . ("maxima.info-6" 50119 314 "Funciones y variables para distribuciones continuas"))
("quantile_weibull" . ("maxima.info-6" 85461 317 "Funciones y variables para distribuciones continuas"))
("quartile_skewness" . ("maxima.info-5" 261325 895 "Funciones y variables de parámetros descriptivos"))
("quartile_skewness <1>" . ("maxima.info-5" 261326 894 "Funciones y variables de parámetros descriptivos"))
("quit" . ("maxima.info-1" 69201 397 "Funciones y variables para la línea de comandos"))
("qunit" . ("maxima.info-4" 214111 345 "Funciones y variables para teoría de números"))
("quotient" . ("maxima.info-2" 254308 344 "Funciones y variables para polinomios"))
("quotient <1>" . ("maxima.info-2" 254309 343 "Funciones y variables para polinomios"))
("radcan" . ("maxima.info-2" 27376 1255 "Funciones y variables para simplificación"))
("radexpand" . ("maxima.info-2" 28638 882 "Funciones y variables para simplificación"))
("radius" . ("maxima.info-7" 12430 234 "Visualización usando VTK"))
("radius <1>" . ("maxima.info-7" 111029 209 "Funciones y variables para graphs"))
("radsubstflag" . ("maxima.info-2" 270497 181 "Funciones y variables para polinomios"))
("random" . ("maxima.info-2" 86022 1692 "Números aleatorios"))
("random_bernoulli" . ("maxima.info-6" 110260 428 "Funciones y variables para distribuciones discretas"))
("random_bernoulli <1>" . ("maxima.info-6" 110261 427 "Funciones y variables para distribuciones discretas"))
("random_beta" . ("maxima.info-6" 77453 499 "Funciones y variables para distribuciones continuas"))
("random_beta <1>" . ("maxima.info-6" 77454 498 "Funciones y variables para distribuciones continuas"))
("random_binomial" . ("maxima.info-6" 104397 550 "Funciones y variables para distribuciones discretas"))
("random_binomial <1>" . ("maxima.info-6" 104398 549 "Funciones y variables para distribuciones discretas"))
("random_bipartite_graph" . ("maxima.info-7" 86825 177 "Funciones y variables para graphs"))
("random_cauchy" . ("maxima.info-6" 94834 389 "Funciones y variables para distribuciones continuas"))
("random_cauchy <1>" . ("maxima.info-6" 94835 388 "Funciones y variables para distribuciones continuas"))
("random_chi2" . ("maxima.info-6" 60652 417 "Funciones y variables para distribuciones continuas"))
("random_chi2 <1>" . ("maxima.info-6" 60653 416 "Funciones y variables para distribuciones continuas"))
("random_continuous_uniform" . ("maxima.info-6" 79963 482 "Funciones y variables para distribuciones continuas"))
("random_continuous_uniform <1>" . ("maxima.info-6" 79964 481 "Funciones y variables para distribuciones continuas"))
("random_digraph" . ("maxima.info-7" 87005 152 "Funciones y variables para graphs"))
("random_discrete_uniform" . ("maxima.info-6" 115081 478 "Funciones y variables para distribuciones discretas"))
("random_discrete_uniform <1>" . ("maxima.info-6" 115082 477 "Funciones y variables para distribuciones discretas"))
("random_exp" . ("maxima.info-6" 69605 377 "Funciones y variables para distribuciones continuas"))
("random_exp <1>" . ("maxima.info-6" 69606 376 "Funciones y variables para distribuciones continuas"))
("random_f" . ("maxima.info-6" 65721 640 "Funciones y variables para distribuciones continuas"))
("random_f <1>" . ("maxima.info-6" 65722 639 "Funciones y variables para distribuciones continuas"))
("random_gamma" . ("maxima.info-6" 74723 741 "Funciones y variables para distribuciones continuas"))
("random_gamma <1>" . ("maxima.info-6" 74724 740 "Funciones y variables para distribuciones continuas"))
("random_general_finite_discrete" . ("maxima.info-6" 101398 675 "Funciones y variables para distribuciones discretas"))
("random_general_finite_discrete <1>" . ("maxima.info-6" 101399 674 "Funciones y variables para distribuciones discretas"))
("random_geometric" . ("maxima.info-6" 112603 407 "Funciones y variables para distribuciones discretas"))
("random_geometric <1>" . ("maxima.info-6" 112604 406 "Funciones y variables para distribuciones discretas"))
("random_graph" . ("maxima.info-7" 87357 141 "Funciones y variables para graphs"))
("random_graph1" . ("maxima.info-7" 87501 111 "Funciones y variables para graphs"))
("random_gumbel" . ("maxima.info-6" 97520 391 "Funciones y variables para distribuciones continuas"))
("random_gumbel <1>" . ("maxima.info-6" 97521 390 "Funciones y variables para distribuciones continuas"))
("random_hypergeometric" . ("maxima.info-6" 118029 622 "Funciones y variables para distribuciones discretas"))
("random_hypergeometric <1>" . ("maxima.info-6" 118030 621 "Funciones y variables para distribuciones discretas"))
("random_laplace" . ("maxima.info-6" 93625 393 "Funciones y variables para distribuciones continuas"))
("random_laplace <1>" . ("maxima.info-6" 93626 392 "Funciones y variables para distribuciones continuas"))
("random_logistic" . ("maxima.info-6" 82302 398 "Funciones y variables para distribuciones continuas"))
("random_logistic <1>" . ("maxima.info-6" 82303 397 "Funciones y variables para distribuciones continuas"))
("random_lognormal" . ("maxima.info-6" 72229 452 "Funciones y variables para distribuciones continuas"))
("random_lognormal <1>" . ("maxima.info-6" 72230 451 "Funciones y variables para distribuciones continuas"))
("random_negative_binomial" . ("maxima.info-6" 120988 515 "Funciones y variables para distribuciones discretas"))
("random_negative_binomial <1>" . ("maxima.info-6" 120989 514 "Funciones y variables para distribuciones discretas"))
("random_network" . ("maxima.info-7" 87615 528 "Funciones y variables para graphs"))
("random_noncentral_chi2" . ("maxima.info-6" 63112 399 "Funciones y variables para distribuciones continuas"))
("random_noncentral_chi2 <1>" . ("maxima.info-6" 63113 398 "Funciones y variables para distribuciones continuas"))
("random_noncentral_student_t" . ("maxima.info-6" 56479 948 "Funciones y variables para distribuciones continuas"))
("random_noncentral_student_t <1>" . ("maxima.info-6" 56480 947 "Funciones y variables para distribuciones continuas"))
("random_normal" . ("maxima.info-6" 48753 508 "Funciones y variables para distribuciones continuas"))
("random_normal <1>" . ("maxima.info-6" 48754 507 "Funciones y variables para distribuciones continuas"))
("random_pareto" . ("maxima.info-6" 84560 393 "Funciones y variables para distribuciones continuas"))
("random_pareto <1>" . ("maxima.info-6" 84561 392 "Funciones y variables para distribuciones continuas"))
("random_permutation" . ("maxima.info-5" 41854 787 "Funciones y variables para los conjuntos"))
("random_poisson" . ("maxima.info-6" 106868 546 "Funciones y variables para distribuciones discretas"))
("random_poisson <1>" . ("maxima.info-6" 106869 545 "Funciones y variables para distribuciones discretas"))
("random_rayleigh" . ("maxima.info-6" 91390 388 "Funciones y variables para distribuciones continuas"))
("random_rayleigh <1>" . ("maxima.info-6" 91391 387 "Funciones y variables para distribuciones continuas"))
("random_regular_graph" . ("maxima.info-7" 87160 193 "Funciones y variables para graphs"))
("random_regular_graph <1>" . ("maxima.info-7" 87161 192 "Funciones y variables para graphs"))
("random_student_t" . ("maxima.info-6" 51619 843 "Funciones y variables para distribuciones continuas"))
("random_student_t <1>" . ("maxima.info-6" 51620 842 "Funciones y variables para distribuciones continuas"))
("random_tournament" . ("maxima.info-7" 88147 88 "Funciones y variables para graphs"))
("random_tree" . ("maxima.info-7" 88238 81 "Funciones y variables para graphs"))
("random_weibull" . ("maxima.info-6" 86816 395 "Funciones y variables para distribuciones continuas"))
("random_weibull <1>" . ("maxima.info-6" 86817 394 "Funciones y variables para distribuciones continuas"))
("range" . ("maxima.info-5" 252338 447 "Funciones y variables de parámetros descriptivos"))
("range <1>" . ("maxima.info-5" 252339 446 "Funciones y variables de parámetros descriptivos"))
("rank" . ("maxima.info-3" 279482 270 "Funciones y variables para las matrices y el álgebra lineal"))
("rank <1>" . ("maxima.info-7" 229912 349 "Funciones y variables para linearalgebra"))
("rassociative" . ("maxima.info-2" 29528 232 "Funciones y variables para simplificación"))
("rat" . ("maxima.info-2" 254657 2070 "Funciones y variables para polinomios"))
("rat <1>" . ("maxima.info-2" 254658 2069 "Funciones y variables para polinomios"))
("ratalgdenom" . ("maxima.info-2" 256756 320 "Funciones y variables para polinomios"))
("ratchristof" . ("maxima.info-4" 127725 117 "Funciones y variables para ctensor"))
("ratcoef" . ("maxima.info-2" 257081 1127 "Funciones y variables para polinomios"))
("ratcoef <1>" . ("maxima.info-2" 257082 1126 "Funciones y variables para polinomios"))
("ratdenom" . ("maxima.info-2" 258222 730 "Funciones y variables para polinomios"))
("ratdenomdivide" . ("maxima.info-2" 258967 1841 "Funciones y variables para polinomios"))
("ratdiff" . ("maxima.info-2" 260814 1979 "Funciones y variables para polinomios"))
("ratdisrep" . ("maxima.info-2" 262809 481 "Funciones y variables para polinomios"))
("rateinstein" . ("maxima.info-4" 127845 275 "Funciones y variables para ctensor"))
("ratepsilon" . ("maxima.info-1" 109430 214 "Funciones y variables para los números"))
("ratexpand" . ("maxima.info-2" 263298 2243 "Funciones y variables para polinomios"))
("ratexpand <1>" . ("maxima.info-2" 263299 2242 "Funciones y variables para polinomios"))
("ratfac" . ("maxima.info-2" 265560 1138 "Funciones y variables para polinomios"))
("ratinterpol" . ("maxima.info-7" 159545 1419 "Funciones y variables para interpol"))
("ratinterpol <1>" . ("maxima.info-7" 159546 1418 "Funciones y variables para interpol"))
("rational" . ("maxima.info-2" 110373 231 "Funciones y variables para las propiedades"))
("rational <1>" . ("maxima.info-8" 20780 412 "Paquete functs"))
("rationalize" . ("maxima.info-1" 109648 1477 "Funciones y variables para los números"))
("ratmx" . ("maxima.info-3" 279756 632 "Funciones y variables para las matrices y el álgebra lineal"))
("ratnumer" . ("maxima.info-2" 266708 731 "Funciones y variables para polinomios"))
("ratnump" . ("maxima.info-1" 111137 162 "Funciones y variables para los números"))
("ratp" . ("maxima.info-2" 267455 341 "Funciones y variables para polinomios"))
("ratprint" . ("maxima.info-2" 267800 227 "Funciones y variables para polinomios"))
("ratriemann" . ("maxima.info-4" 128125 318 "Funciones y variables para ctensor"))
("ratsimp" . ("maxima.info-2" 268030 2259 "Funciones y variables para polinomios"))
("ratsimp <1>" . ("maxima.info-2" 268031 2258 "Funciones y variables para polinomios"))
("ratsimpexpons" . ("maxima.info-2" 270300 195 "Funciones y variables para polinomios"))
("ratsubst" . ("maxima.info-2" 270680 1622 "Funciones y variables para polinomios"))
("ratvars" . ("maxima.info-2" 272309 1061 "Funciones y variables para polinomios"))
("ratvars <1>" . ("maxima.info-2" 272310 1060 "Funciones y variables para polinomios"))
("ratvars <2>" . ("maxima.info-2" 272352 1019 "Funciones y variables para polinomios"))
("ratvarswitch" . ("maxima.info-2" 273392 1610 "Funciones y variables para polinomios"))
("ratweight" . ("maxima.info-2" 275009 1249 "Funciones y variables para polinomios"))
("ratweight <1>" . ("maxima.info-2" 275010 1248 "Funciones y variables para polinomios"))
("ratweights" . ("maxima.info-2" 276265 285 "Funciones y variables para polinomios"))
("ratweyl" . ("maxima.info-4" 128449 281 "Funciones y variables para ctensor"))
("ratwtlvl" . ("maxima.info-2" 276552 305 "Funciones y variables para polinomios"))
("read" . ("maxima.info-1" 69607 553 "Funciones y variables para la línea de comandos"))
("read_array" . ("maxima.info-7" 269974 631 "Funciones y variables para entrada y salida en formato texto"))
("read_array <1>" . ("maxima.info-7" 269975 630 "Funciones y variables para entrada y salida en formato texto"))
("read_binary_array" . ("maxima.info-7" 276016 541 "Funciones y variables para entrada y salida en formato binario"))
("read_binary_list" . ("maxima.info-7" 276560 737 "Funciones y variables para entrada y salida en formato binario"))
("read_binary_list <1>" . ("maxima.info-7" 276561 736 "Funciones y variables para entrada y salida en formato binario"))
("read_binary_matrix" . ("maxima.info-7" 275536 477 "Funciones y variables para entrada y salida en formato binario"))
("read_hashed_array" . ("maxima.info-7" 270611 805 "Funciones y variables para entrada y salida en formato texto"))
("read_hashed_array <1>" . ("maxima.info-7" 270612 804 "Funciones y variables para entrada y salida en formato texto"))
("read_list" . ("maxima.info-7" 272159 692 "Funciones y variables para entrada y salida en formato texto"))
("read_list <1>" . ("maxima.info-7" 272160 691 "Funciones y variables para entrada y salida en formato texto"))
("read_list <2>" . ("maxima.info-7" 272190 662 "Funciones y variables para entrada y salida en formato texto"))
("read_list <3>" . ("maxima.info-7" 272225 628 "Funciones y variables para entrada y salida en formato texto"))
("read_matrix" . ("maxima.info-7" 268894 1070 "Funciones y variables para entrada y salida en formato texto"))
("read_matrix <1>" . ("maxima.info-7" 268895 1069 "Funciones y variables para entrada y salida en formato texto"))
("read_matrix <2>" . ("maxima.info-7" 268927 1038 "Funciones y variables para entrada y salida en formato texto"))
("read_matrix <3>" . ("maxima.info-7" 268964 1002 "Funciones y variables para entrada y salida en formato texto"))
("read_nested_list" . ("maxima.info-7" 271421 732 "Funciones y variables para entrada y salida en formato texto"))
("read_nested_list <1>" . ("maxima.info-7" 271422 731 "Funciones y variables para entrada y salida en formato texto"))
("read_xpm" . ("maxima.info-6" 259012 115 "Funciones y variables para picture"))
("readchar" . ("maxima.info-8" 106586 222 "Funciones y variables para entrada y salida"))
("readline" . ("maxima.info-8" 106812 221 "Funciones y variables para entrada y salida"))
("readonly" . ("maxima.info-1" 70170 658 "Funciones y variables para la línea de comandos"))
("real" . ("maxima.info-2" 110607 289 "Funciones y variables para las propiedades"))
("real_imagpart_to_conjugate" . ("maxima.info-8" 149217 614 "Funciones y variables para to_poly_solve"))
("realonly" . ("maxima.info-3" 174743 175 "Funciones y variable para las ecuaciones"))
("realpart" . ("maxima.info-2" 50183 253 "Funciones para los números complejos"))
("realroots" . ("maxima.info-3" 174920 2327 "Funciones y variable para las ecuaciones"))
("realroots <1>" . ("maxima.info-3" 174921 2326 "Funciones y variable para las ecuaciones"))
("realroots <2>" . ("maxima.info-3" 174963 2285 "Funciones y variable para las ecuaciones"))
("realroots <3>" . ("maxima.info-3" 175004 2245 "Funciones y variable para las ecuaciones"))
("rearray" . ("maxima.info-1" 168804 376 "Funciones y variables para los arrays"))
("rectangle" . ("maxima.info-6" 250047 986 "Funciones y variables para draw"))
("rectform" . ("maxima.info-2" 50442 133 "Funciones para los números complejos"))
("rectform_log_if_constant" . ("maxima.info-8" 149838 821 "Funciones y variables para to_poly_solve"))
("recttopolar" . ("maxima.info-3" 202502 773 "Funciones y variables para la transformada rápida de Fourier"))
("rediff" . ("maxima.info-4" 31541 126 "Funciones y variables para itensor"))
("redraw" . ("maxima.info-7" 128305 226 "Funciones y variables para graphs"))
("reduce_consts" . ("maxima.info-8" 31096 1483 "Paquete rducon"))
("reduce_order" . ("maxima.info-8" 36132 1817 "Funciones y variables para solve_rec"))
("refcheck" . ("maxima.info-5" 181300 202 "Funciones y variables para depurado"))
("region" . ("maxima.info-6" 251039 746 "Funciones y variables para draw"))
("region_boundaries" . ("maxima.info-6" 265596 575 "Funciones y variables para worldmap"))
("region_boundaries_plus" . ("maxima.info-6" 266176 579 "Funciones y variables para worldmap"))
("rem" . ("maxima.info-2" 110899 313 "Funciones y variables para las propiedades"))
("remainder" . ("maxima.info-2" 276861 337 "Funciones y variables para polinomios"))
("remainder <1>" . ("maxima.info-2" 276862 336 "Funciones y variables para polinomios"))
("remarray" . ("maxima.info-1" 169185 532 "Funciones y variables para los arrays"))
("remarray <1>" . ("maxima.info-1" 169186 531 "Funciones y variables para los arrays"))
("rembox" . ("maxima.info-1" 298531 3193 "Funciones y variables para expresiones"))
("rembox <1>" . ("maxima.info-1" 298532 3192 "Funciones y variables para expresiones"))
("rembox <2>" . ("maxima.info-1" 298574 3151 "Funciones y variables para expresiones"))
("remcomps" . ("maxima.info-4" 13761 132 "Funciones y variables para itensor"))
("remcon" . ("maxima.info-4" 8652 269 "Funciones y variables para itensor"))
("remcon <1>" . ("maxima.info-4" 8653 268 "Funciones y variables para itensor"))
("remcoord" . ("maxima.info-4" 35780 305 "Funciones y variables para itensor"))
("remcoord <1>" . ("maxima.info-4" 35781 304 "Funciones y variables para itensor"))
("remfun" . ("maxima.info-4" 187114 353 "Funciones y variables para series de Fourier"))
("remfun <1>" . ("maxima.info-4" 187115 352 "Funciones y variables para series de Fourier"))
("remfunction" . ("maxima.info-5" 122135 836 "Funciones y variables para la definición de funciones"))
("remfunction <1>" . ("maxima.info-5" 122136 835 "Funciones y variables para la definición de funciones"))
("remlet" . ("maxima.info-4" 317690 1083 "Funciones y variables sobre reglas y patrones"))
("remlet <1>" . ("maxima.info-4" 317691 1082 "Funciones y variables sobre reglas y patrones"))
("remlet <2>" . ("maxima.info-4" 317731 1043 "Funciones y variables sobre reglas y patrones"))
("remlet <3>" . ("maxima.info-4" 317755 1020 "Funciones y variables sobre reglas y patrones"))
("remove" . ("maxima.info-2" 111220 1353 "Funciones y variables para las propiedades"))
("remove <1>" . ("maxima.info-2" 111221 1352 "Funciones y variables para las propiedades"))
("remove <2>" . ("maxima.info-2" 111276 1298 "Funciones y variables para las propiedades"))
("remove <3>" . ("maxima.info-2" 111345 1230 "Funciones y variables para las propiedades"))
("remove <4>" . ("maxima.info-2" 111384 1192 "Funciones y variables para las propiedades"))
("remove_constvalue" . ("maxima.info-7" 37390 1511 "Funciones y variables para ezunits"))
("remove_dimensions" . ("maxima.info-7" 46264 1138 "Funciones y variables para ezunits"))
("remove_edge" . ("maxima.info-7" 119707 364 "Funciones y variables para graphs"))
("remove_fundamental_dimensions" . ("maxima.info-7" 47411 1392 "Funciones y variables para ezunits"))
("remove_fundamental_units" . ("maxima.info-7" 48812 1684 "Funciones y variables para ezunits"))
("remove_vertex" . ("maxima.info-7" 120073 83 "Funciones y variables para graphs"))
("rempart" . ("maxima.info-8" 19842 258 "Paquete functs"))
("remrule" . ("maxima.info-4" 318794 2176 "Funciones y variables sobre reglas y patrones"))
("remrule <1>" . ("maxima.info-4" 318795 2175 "Funciones y variables sobre reglas y patrones"))
("remsym" . ("maxima.info-4" 27554 162 "Funciones y variables para itensor"))
("remvalue" . ("maxima.info-4" 288480 494 "Funciones y variables para la miscelánea de opciones"))
("remvalue <1>" . ("maxima.info-4" 288481 493 "Funciones y variables para la miscelánea de opciones"))
("rename" . ("maxima.info-4" 3316 3294 "Funciones y variables para itensor"))
("rename <1>" . ("maxima.info-4" 3317 3293 "Funciones y variables para itensor"))
("reset" . ("maxima.info-1" 70838 514 "Funciones y variables para la línea de comandos"))
("residue" . ("maxima.info-3" 116413 670 "Funciones y variables para integración"))
("resolution" . ("maxima.info-7" 12668 232 "Visualización usando VTK"))
("resolvante" . ("maxima.info-4" 257903 6968 "Funciones y variables para simetrías"))
("resolvante_alternee1" . ("maxima.info-4" 264883 372 "Funciones y variables para simetrías"))
("resolvante_bipartite" . ("maxima.info-4" 265261 561 "Funciones y variables para simetrías"))
("resolvante_diedrale" . ("maxima.info-4" 265828 689 "Funciones y variables para simetrías"))
("resolvante_klein" . ("maxima.info-4" 266523 324 "Funciones y variables para simetrías"))
("resolvante_klein3" . ("maxima.info-4" 266853 324 "Funciones y variables para simetrías"))
("resolvante_produit_sym" . ("maxima.info-4" 267183 1078 "Funciones y variables para simetrías"))
("resolvante_unitaire" . ("maxima.info-4" 268265 321 "Funciones y variables para simetrías"))
("resolvante_vierer" . ("maxima.info-4" 268590 324 "Funciones y variables para simetrías"))
("rest" . ("maxima.info-1" 138718 335 "Funciones y variables para listas"))
("rest <1>" . ("maxima.info-1" 138719 334 "Funciones y variables para listas"))
("restart" . ("maxima.info-7" 4068 268 "Visualización usando VTK"))
("resultant" . ("maxima.info-2" 278683 699 "Funciones y variables para polinomios"))
("resultant <1>" . ("maxima.info-2" 277203 1472 "Funciones y variables para polinomios"))
("return" . ("maxima.info-5" 164469 142 "Funciones y variables para la programación"))
("reveal" . ("maxima.info-1" 301730 2590 "Funciones y variables para expresiones"))
("reverse" . ("maxima.info-1" 139059 226 "Funciones y variables para listas"))
("revert" . ("maxima.info-4" 171028 1975 "Funciones y variables para las series"))
("revert2" . ("maxima.info-4" 171029 1974 "Funciones y variables para las series"))
("rgb2level" . ("maxima.info-6" 259130 129 "Funciones y variables para picture"))
("rhs" . ("maxima.info-3" 177271 1632 "Funciones y variable para las ecuaciones"))
("ric" . ("maxima.info-4" 129671 79 "Funciones y variables para ctensor"))
("ricci" . ("maxima.info-4" 86473 269 "Funciones y variables para ctensor"))
("riem" . ("maxima.info-4" 129156 298 "Funciones y variables para ctensor"))
("riemann" . ("maxima.info-4" 88506 1312 "Funciones y variables para ctensor"))
("rinvariant" . ("maxima.info-4" 90578 260 "Funciones y variables para ctensor"))
("risch" . ("maxima.info-3" 117086 1084 "Funciones y variables para integración"))
("rk" . ("maxima.info-3" 230083 2644 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("rk <1>" . ("maxima.info-3" 230084 2643 "Funciones para la resolución numérica de ecuaciones diferenciales"))
("rmxchar" . ("maxima.info-3" 280529 183 "Funciones y variables para las matrices y el álgebra lineal"))
("rncombine" . ("maxima.info-4" 288983 703 "Funciones y variables para la miscelánea de opciones"))
("romberg" . ("maxima.info-8" 392 5180 "Funciones y variables para romberg"))
("romberg <1>" . ("maxima.info-8" 393 5179 "Funciones y variables para romberg"))
("rombergabs" . ("maxima.info-8" 5614 590 "Funciones y variables para romberg"))
("rombergit" . ("maxima.info-8" 6212 540 "Funciones y variables para romberg"))
("rombergmin" . ("maxima.info-8" 6763 308 "Funciones y variables para romberg"))
("rombergtol" . ("maxima.info-8" 7077 593 "Funciones y variables para romberg"))
("room" . ("maxima.info-4" 278152 398 "Funciones y variables para el entorno de ejecución"))
("room <1>" . ("maxima.info-4" 278153 397 "Funciones y variables para el entorno de ejecución"))
("room <2>" . ("maxima.info-4" 278175 376 "Funciones y variables para el entorno de ejecución"))
("rootsconmode" . ("maxima.info-3" 178912 214 "Funciones y variable para las ecuaciones"))
("rootscontract" . ("maxima.info-3" 179130 2720 "Funciones y variable para las ecuaciones"))
("rootsepsilon" . ("maxima.info-3" 181861 222 "Funciones y variable para las ecuaciones"))
("round" . ("maxima.info-2" 44415 233 "Funciones para los números"))
("row" . ("maxima.info-3" 280395 131 "Funciones y variables para las matrices y el álgebra lineal"))
("rowop" . ("maxima.info-7" 229651 258 "Funciones y variables para linearalgebra"))
("rowswap" . ("maxima.info-7" 230264 164 "Funciones y variables para linearalgebra"))
("rreduce" . ("maxima.info-5" 42644 1526 "Funciones y variables para los conjuntos"))
("rreduce <1>" . ("maxima.info-5" 42645 1525 "Funciones y variables para los conjuntos"))
("run_testsuite" . ("maxima.info-1" 24183 1994 "Funciones y variables para la detección e informe de fallos"))
("run_viewer" . ("maxima.info-2" 168069 174 "Opciones gráficas"))
("save" . ("maxima.info-2" 196247 3037 "Funciones y variables para lectura y escritura"))
("save <1>" . ("maxima.info-2" 196248 3036 "Funciones y variables para lectura y escritura"))
("save <2>" . ("maxima.info-2" 196315 2970 "Funciones y variables para lectura y escritura"))
("save <3>" . ("maxima.info-2" 196379 2907 "Funciones y variables para lectura y escritura"))
("save <4>" . ("maxima.info-2" 196423 2864 "Funciones y variables para lectura y escritura"))
("save <5>" . ("maxima.info-2" 196479 2809 "Funciones y variables para lectura y escritura"))
("savedef" . ("maxima.info-5" 122988 400 "Funciones y variables para la definición de funciones"))
("savefactors" . ("maxima.info-2" 279387 294 "Funciones y variables para polinomios"))
("saving" . ("maxima.info-7" 66373 1422 "Funciones y Variables para finance"))
("scalar" . ("maxima.info-2" 112591 150 "Funciones y variables para las propiedades"))
("scalarmatrixp" . ("maxima.info-3" 280716 460 "Funciones y variables para las matrices y el álgebra lineal"))
("scalarp" . ("maxima.info-2" 112744 263 "Funciones y variables para las propiedades"))
("scale" . ("maxima.info-7" 12906 500 "Visualización usando VTK"))
("scaled_bessel_i" . ("maxima.info-2" 291420 472 "Funciones de Bessel"))
("scaled_bessel_i0" . ("maxima.info-2" 291903 77 "Funciones de Bessel"))
("scaled_bessel_i1" . ("maxima.info-2" 291983 77 "Funciones de Bessel"))
("scanmap" . ("maxima.info-5" 164616 1669 "Funciones y variables para la programación"))
("scanmap <1>" . ("maxima.info-5" 164617 1668 "Funciones y variables para la programación"))
("scatterplot" . ("maxima.info-6" 15998 2230 "Funciones y variables para gráficos estadísticos"))
("scatterplot <1>" . ("maxima.info-6" 15999 2229 "Funciones y variables para gráficos estadísticos"))
("scatterplot <2>" . ("maxima.info-6" 16034 2195 "Funciones y variables para gráficos estadísticos"))
("scatterplot <3>" . ("maxima.info-6" 16098 2132 "Funciones y variables para gráficos estadísticos"))
("scatterplot_description" . ("maxima.info-6" 16135 2096 "Funciones y variables para gráficos estadísticos"))
("scene" . ("maxima.info-7" 571 2296 "Visualización usando VTK"))
("schur2comp" . ("maxima.info-4" 268920 710 "Funciones y variables para simetrías"))
("sconcat" . ("maxima.info-1" 115547 330 "Funciones y variables para las cadenas de texto"))
("scopy" . ("maxima.info-8" 114106 83 "Funciones y variables para cadenas"))
("scsimp" . ("maxima.info-2" 29762 499 "Funciones y variables para simplificación"))
("scurvature" . ("maxima.info-4" 87278 165 "Funciones y variables para ctensor"))
("sdowncase" . ("maxima.info-8" 114191 210 "Funciones y variables para cadenas"))
("sdowncase <1>" . ("maxima.info-8" 114192 209 "Funciones y variables para cadenas"))
("sdowncase <2>" . ("maxima.info-8" 114227 175 "Funciones y variables para cadenas"))
("sec" . ("maxima.info-2" 77028 37 "Funciones y variables para trigonometría"))
("sech" . ("maxima.info-2" 77067 50 "Funciones y variables para trigonometría"))
("second" . ("maxima.info-1" 139289 135 "Funciones y variables para listas"))
("sequal" . ("maxima.info-8" 114409 137 "Funciones y variables para cadenas"))
("sequalignore" . ("maxima.info-8" 114548 131 "Funciones y variables para cadenas"))
("set_draw_defaults" . ("maxima.info-6" 128754 878 "Funciones y variables para draw"))
("set_edge_weight" . ("maxima.info-7" 111240 494 "Funciones y variables para graphs"))
("set_partitions" . ("maxima.info-5" 46908 1715 "Funciones y variables para los conjuntos"))
("set_partitions <1>" . ("maxima.info-5" 46909 1714 "Funciones y variables para los conjuntos"))
("set_plot_option" . ("maxima.info-2" 157523 1396 "Funciones y variables para gráficos"))
("set_random_state" . ("maxima.info-2" 85848 170 "Números aleatorios"))
("set_tex_environment" . ("maxima.info-2" 212159 1089 "Funciones y variables para salida TeX"))
("set_tex_environment_default" . ("maxima.info-2" 213257 1204 "Funciones y variables para salida TeX"))
("set_up_dot_simplifications" . ("maxima.info-3" 291065 571 "Funciones y variables para Afines"))
("set_up_dot_simplifications <1>" . ("maxima.info-3" 291066 570 "Funciones y variables para Afines"))
("set_vertex_label" . ("maxima.info-7" 111736 502 "Funciones y variables para graphs"))
("setcheck" . ("maxima.info-5" 181504 1386 "Funciones y variables para depurado"))
("setcheckbreak" . ("maxima.info-5" 182903 505 "Funciones y variables para depurado"))
("setdifference" . ("maxima.info-5" 44185 890 "Funciones y variables para los conjuntos"))
("setelmx" . ("maxima.info-3" 281178 246 "Funciones y variables para las matrices y el álgebra lineal"))
("setequalp" . ("maxima.info-5" 45078 653 "Funciones y variables para los conjuntos"))
("setify" . ("maxima.info-5" 45735 610 "Funciones y variables para los conjuntos"))
("setp" . ("maxima.info-5" 46348 555 "Funciones y variables para los conjuntos"))
("setunits" . ("maxima.info-8" 173840 5026 "Funciones y variables para units"))
("setup_autoload" . ("maxima.info-4" 289697 1496 "Funciones y variables para la miscelánea de opciones"))
("setval" . ("maxima.info-5" 183417 275 "Funciones y variables para depurado"))
("seventh" . ("maxima.info-1" 139429 136 "Funciones y variables para listas"))
("sexplode" . ("maxima.info-8" 114683 101 "Funciones y variables para cadenas"))
("sf" . ("maxima.info-4" 142072 291 "Funciones y variables para atensor"))
("shortest_path" . ("maxima.info-7" 112241 375 "Funciones y variables para graphs"))
("shortest_weighted_path" . ("maxima.info-7" 112619 647 "Funciones y variables para graphs"))
("show" . ("maxima.info-4" 6636 271 "Funciones y variables para itensor"))
("show_edge_color" . ("maxima.info-7" 127588 105 "Funciones y variables para graphs"))
("show_edge_type" . ("maxima.info-7" 127783 157 "Funciones y variables para graphs"))
("show_edge_width" . ("maxima.info-7" 127695 86 "Funciones y variables para graphs"))
("show_edges" . ("maxima.info-7" 127480 106 "Funciones y variables para graphs"))
("show_id" . ("maxima.info-7" 125212 168 "Funciones y variables para graphs"))
("show_label" . ("maxima.info-7" 125384 160 "Funciones y variables para graphs"))
("show_vertex_color" . ("maxima.info-7" 126682 111 "Funciones y variables para graphs"))
("show_vertex_size" . ("maxima.info-7" 126586 92 "Funciones y variables para graphs"))
("show_vertex_type" . ("maxima.info-7" 126415 164 "Funciones y variables para graphs"))
("show_vertices" . ("maxima.info-7" 126264 148 "Funciones y variables para graphs"))
("show_weight" . ("maxima.info-7" 125770 158 "Funciones y variables para graphs"))
("showcomps" . ("maxima.info-4" 13896 2410 "Funciones y variables para itensor"))
("showratvars" . ("maxima.info-2" 279683 177 "Funciones y variables para polinomios"))
("showtime" . ("maxima.info-1" 71355 415 "Funciones y variables para la línea de comandos"))
("sierpinskiale" . ("maxima.info-7" 70972 427 "Definiciones para IFS fractals"))
("sierpinskimap" . ("maxima.info-7" 75986 270 "Definiciones para curvas de Peano"))
("sign" . ("maxima.info-2" 129311 403 "Funciones y variables para los hechos"))
("signum" . ("maxima.info-2" 44658 720 "Funciones para los números"))
("similaritytransform" . ("maxima.info-3" 281427 1232 "Funciones y variables para las matrices y el álgebra lineal"))
("simp" . ("maxima.info-2" 30270 1568 "Funciones y variables para simplificación"))
("simp_inequality" . ("maxima.info-8" 150666 678 "Funciones y variables para to_poly_solve"))
("simplified_output" . ("maxima.info-8" 200190 209 "Funciones y variables para zeilberger"))
("simplify_products" . ("maxima.info-8" 37954 215 "Funciones y variables para solve_rec"))
("simplify_sum" . ("maxima.info-8" 38173 876 "Funciones y variables para solve_rec"))
("simplode" . ("maxima.info-8" 114788 704 "Funciones y variables para cadenas"))
("simplode <1>" . ("maxima.info-8" 114789 703 "Funciones y variables para cadenas"))
("simpmetderiv" . ("maxima.info-4" 37224 3004 "Funciones y variables para itensor"))
("simpmetderiv <1>" . ("maxima.info-4" 37225 3003 "Funciones y variables para itensor"))
("simpsum" . ("maxima.info-4" 149666 398 "Funciones y variables para sumas y productos"))
("simtran" . ("maxima.info-3" 281428 1231 "Funciones y variables para las matrices y el álgebra lineal"))
("sin" . ("maxima.info-2" 77120 34 "Funciones y variables para trigonometría"))
("sinh" . ("maxima.info-2" 77156 47 "Funciones y variables para trigonometría"))
("sinnpiflag" . ("maxima.info-4" 188740 89 "Funciones y variables para series de Fourier"))
("sinsert" . ("maxima.info-8" 115497 503 "Funciones y variables para cadenas"))
("sinvertcase" . ("maxima.info-8" 116006 556 "Funciones y variables para cadenas"))
("sinvertcase <1>" . ("maxima.info-8" 116007 555 "Funciones y variables para cadenas"))
("sinvertcase <2>" . ("maxima.info-8" 116044 519 "Funciones y variables para cadenas"))
("sixth" . ("maxima.info-1" 139571 132 "Funciones y variables para listas"))
("skewness" . ("maxima.info-5" 259636 870 "Funciones y variables de parámetros descriptivos"))
("skewness <1>" . ("maxima.info-5" 259637 869 "Funciones y variables de parámetros descriptivos"))
("skewness_bernoulli" . ("maxima.info-6" 109355 462 "Funciones y variables para distribuciones discretas"))
("skewness_beta" . ("maxima.info-6" 77035 205 "Funciones y variables para distribuciones continuas"))
("skewness_binomial" . ("maxima.info-6" 103895 243 "Funciones y variables para distribuciones discretas"))
("skewness_chi2" . ("maxima.info-6" 59799 463 "Funciones y variables para distribuciones continuas"))
("skewness_continuous_uniform" . ("maxima.info-6" 79498 230 "Funciones y variables para distribuciones continuas"))
("skewness_discrete_uniform" . ("maxima.info-6" 114601 236 "Funciones y variables para distribuciones discretas"))
("skewness_exp" . ("maxima.info-6" 68976 314 "Funciones y variables para distribuciones continuas"))
("skewness_f" . ("maxima.info-6" 65312 202 "Funciones y variables para distribuciones continuas"))
("skewness_gamma" . ("maxima.info-6" 74304 207 "Funciones y variables para distribuciones continuas"))
("skewness_general_finite_discrete" . ("maxima.info-6" 100904 243 "Funciones y variables para distribuciones discretas"))
("skewness_geometric" . ("maxima.info-6" 112159 217 "Funciones y variables para distribuciones discretas"))
("skewness_gumbel" . ("maxima.info-6" 96777 526 "Funciones y variables para distribuciones continuas"))
("skewness_hypergeometric" . ("maxima.info-6" 117742 281 "Funciones y variables para distribuciones discretas"))
("skewness_laplace" . ("maxima.info-6" 93196 212 "Funciones y variables para distribuciones continuas"))
("skewness_logistic" . ("maxima.info-6" 81871 212 "Funciones y variables para distribuciones continuas"))
("skewness_lognormal" . ("maxima.info-6" 71798 213 "Funciones y variables para distribuciones continuas"))
("skewness_negative_binomial" . ("maxima.info-6" 120474 253 "Funciones y variables para distribuciones discretas"))
("skewness_noncentral_chi2" . ("maxima.info-6" 62677 216 "Funciones y variables para distribuciones continuas"))
("skewness_noncentral_student_t" . ("maxima.info-6" 55863 303 "Funciones y variables para distribuciones continuas"))
("skewness_normal" . ("maxima.info-6" 48279 233 "Funciones y variables para distribuciones continuas"))
("skewness_pareto" . ("maxima.info-6" 84124 214 "Funciones y variables para distribuciones continuas"))
("skewness_poisson" . ("maxima.info-6" 106448 206 "Funciones y variables para distribuciones discretas"))
("skewness_rayleigh" . ("maxima.info-6" 90085 662 "Funciones y variables para distribuciones continuas"))
("skewness_student_t" . ("maxima.info-6" 51174 230 "Funciones y variables para distribuciones continuas"))
("skewness_weibull" . ("maxima.info-6" 86383 214 "Funciones y variables para distribuciones continuas"))
("slength" . ("maxima.info-8" 116571 83 "Funciones y variables para cadenas"))
("smake" . ("maxima.info-8" 116657 203 "Funciones y variables para cadenas"))
("small_rhombicosidodecahedron_graph" . ("maxima.info-7" 88323 105 "Funciones y variables para graphs"))
("small_rhombicuboctahedron_graph" . ("maxima.info-7" 88432 104 "Funciones y variables para graphs"))
("smax" . ("maxima.info-5" 251711 618 "Funciones y variables de parámetros descriptivos"))
("smax <1>" . ("maxima.info-5" 251712 617 "Funciones y variables de parámetros descriptivos"))
("smin" . ("maxima.info-5" 251087 615 "Funciones y variables de parámetros descriptivos"))
("smin <1>" . ("maxima.info-5" 251088 614 "Funciones y variables de parámetros descriptivos"))
("smismatch" . ("maxima.info-8" 116862 546 "Funciones y variables para cadenas"))
("smismatch <1>" . ("maxima.info-8" 116863 545 "Funciones y variables para cadenas"))
("snowmap" . ("maxima.info-7" 74667 750 "Definiciones para cops de Koch"))
("snub_cube_graph" . ("maxima.info-7" 88540 72 "Funciones y variables para graphs"))
("snub_dodecahedron_graph" . ("maxima.info-7" 88615 86 "Funciones y variables para graphs"))
("solve" . ("maxima.info-3" 182086 7479 "Funciones y variable para las ecuaciones"))
("solve <1>" . ("maxima.info-3" 182087 7478 "Funciones y variable para las ecuaciones"))
("solve <2>" . ("maxima.info-3" 182121 7445 "Funciones y variable para las ecuaciones"))
("solve_rec" . ("maxima.info-8" 39054 3072 "Funciones y variables para solve_rec"))
("solve_rec_rat" . ("maxima.info-8" 42141 893 "Funciones y variables para solve_rec"))
("solvedecomposes" . ("maxima.info-3" 189626 203 "Funciones y variable para las ecuaciones"))
("solveexplicit" . ("maxima.info-3" 189831 242 "Funciones y variable para las ecuaciones"))
("solvefactors" . ("maxima.info-3" 190076 256 "Funciones y variable para las ecuaciones"))
("solvenullwarn" . ("maxima.info-3" 190336 323 "Funciones y variable para las ecuaciones"))
("solveradcan" . ("maxima.info-3" 190665 292 "Funciones y variable para las ecuaciones"))
("solvetrigwarn" . ("maxima.info-3" 190961 310 "Funciones y variable para las ecuaciones"))
("some" . ("maxima.info-5" 48637 3129 "Funciones y variables para los conjuntos"))
("some <1>" . ("maxima.info-5" 48638 3128 "Funciones y variables para los conjuntos"))
("somrac" . ("maxima.info-4" 269641 260 "Funciones y variables para simetrías"))
("sort" . ("maxima.info-1" 139708 6208 "Funciones y variables para listas"))
("sort <1>" . ("maxima.info-1" 139709 6207 "Funciones y variables para listas"))
("space" . ("maxima.info-8" 110686 49 "Funciones y variables para caracteres"))
("sparse" . ("maxima.info-3" 282667 219 "Funciones y variables para las matrices y el álgebra lineal"))
("sparse6_decode" . ("maxima.info-7" 121536 109 "Funciones y variables para graphs"))
("sparse6_encode" . ("maxima.info-7" 121647 106 "Funciones y variables para graphs"))
("sparse6_export" . ("maxima.info-7" 121755 133 "Funciones y variables para graphs"))
("sparse6_import" . ("maxima.info-7" 121890 120 "Funciones y variables para graphs"))
("specint" . ("maxima.info-3" 41821 4226 "Funciones y variables para las funciones especiales"))
("sphere" . ("maxima.info-7" 7319 120 "Visualización usando VTK"))
("spherical" . ("maxima.info-6" 251794 742 "Funciones y variables para draw"))
("spherical_bessel_j" . ("maxima.info-7" 306364 192 "Funciones y variables para polinomios ortogonales"))
("spherical_bessel_y" . ("maxima.info-7" 306562 192 "Funciones y variables para polinomios ortogonales"))
("spherical_hankel1" . ("maxima.info-7" 306760 163 "Funciones y variables para polinomios ortogonales"))
("spherical_hankel2" . ("maxima.info-7" 306929 163 "Funciones y variables para polinomios ortogonales"))
("spherical_harmonic" . ("maxima.info-7" 307098 120 "Funciones y variables para polinomios ortogonales"))
("spherical_to_xyz" . ("maxima.info-2" 158932 258 "Funciones y variables para gráficos"))
("splice" . ("maxima.info-5" 76415 1659 "Macros"))
("split" . ("maxima.info-8" 117420 927 "Funciones y variables para cadenas"))
("split <1>" . ("maxima.info-8" 117421 926 "Funciones y variables para cadenas"))
("split <2>" . ("maxima.info-8" 117452 896 "Funciones y variables para cadenas"))
("sposition" . ("maxima.info-8" 118355 295 "Funciones y variables para cadenas"))
("spring_embedding_depth" . ("maxima.info-7" 128806 139 "Funciones y variables para graphs"))
("sprint" . ("maxima.info-8" 107037 658 "Funciones y variables para entrada y salida"))
("sqrt" . ("maxima.info-2" 67829 394 "Funciones radicales exponenciales y logarítmicas"))
("sqrtdenest" . ("maxima.info-8" 33903 1062 "Paquete sqdnst"))
("sqrtdispflag" . ("maxima.info-1" 95269 159 "Funciones y variables para la impresión"))
("sremove" . ("maxima.info-8" 118659 808 "Funciones y variables para cadenas"))
("sremove <1>" . ("maxima.info-8" 118660 807 "Funciones y variables para cadenas"))
("sremove <2>" . ("maxima.info-8" 118700 768 "Funciones y variables para cadenas"))
("sremove <3>" . ("maxima.info-8" 118748 721 "Funciones y variables para cadenas"))
("sremovefirst" . ("maxima.info-8" 119481 344 "Funciones y variables para cadenas"))
("sremovefirst <1>" . ("maxima.info-8" 119482 343 "Funciones y variables para cadenas"))
("sremovefirst <2>" . ("maxima.info-8" 119527 299 "Funciones y variables para cadenas"))
("sremovefirst <3>" . ("maxima.info-8" 119580 247 "Funciones y variables para cadenas"))
("sreverse" . ("maxima.info-8" 119834 118 "Funciones y variables para cadenas"))
("ssearch" . ("maxima.info-8" 119954 729 "Funciones y variables para cadenas"))
("ssearch <1>" . ("maxima.info-8" 119955 728 "Funciones y variables para cadenas"))
("ssearch <2>" . ("maxima.info-8" 119995 689 "Funciones y variables para cadenas"))
("ssearch <3>" . ("maxima.info-8" 120043 642 "Funciones y variables para cadenas"))
("ssort" . ("maxima.info-8" 120698 727 "Funciones y variables para cadenas"))
("ssort <1>" . ("maxima.info-8" 120699 726 "Funciones y variables para cadenas"))
("sstatus" . ("maxima.info-4" 278558 535 "Funciones y variables para el entorno de ejecución"))
("ssubst" . ("maxima.info-8" 121430 1046 "Funciones y variables para cadenas"))
("ssubst <1>" . ("maxima.info-8" 121431 1045 "Funciones y variables para cadenas"))
("ssubst <2>" . ("maxima.info-8" 121477 1000 "Funciones y variables para cadenas"))
("ssubst <3>" . ("maxima.info-8" 121531 947 "Funciones y variables para cadenas"))
("ssubstfirst" . ("maxima.info-8" 122491 382 "Funciones y variables para cadenas"))
("ssubstfirst <1>" . ("maxima.info-8" 122492 381 "Funciones y variables para cadenas"))
("ssubstfirst <2>" . ("maxima.info-8" 122543 331 "Funciones y variables para cadenas"))
("ssubstfirst <3>" . ("maxima.info-8" 122602 273 "Funciones y variables para cadenas"))
("staircase" . ("maxima.info-6" 290113 944 "Análisis gráfico de sistemas dinámicos discretos"))
("standardize" . ("maxima.info-5" 236439 360 "Funciones y variables para el tratamiento de datos"))
("standardize <1>" . ("maxima.info-5" 236440 359 "Funciones y variables para el tratamiento de datos"))
("standardize_inverse_trig" . ("maxima.info-8" 151351 330 "Funciones y variables para to_poly_solve"))
("stardisp" . ("maxima.info-1" 95430 173 "Funciones y variables para la impresión"))
("starplot" . ("maxima.info-6" 18248 2782 "Funciones y variables para gráficos estadísticos"))
("starplot_description" . ("maxima.info-6" 18320 2711 "Funciones y variables para gráficos estadísticos"))
("startphi" . ("maxima.info-7" 13412 462 "Visualización usando VTK"))
("starttheta" . ("maxima.info-7" 13882 465 "Visualización usando VTK"))
("stats_numer" . ("maxima.info-8" 52070 306 "Funciones y variables para stats"))
("status" . ("maxima.info-4" 279101 1083 "Funciones y variables para el entorno de ejecución"))
("status <1>" . ("maxima.info-4" 279102 1082 "Funciones y variables para el entorno de ejecución"))
("std" . ("maxima.info-5" 247372 593 "Funciones y variables de parámetros descriptivos"))
("std <1>" . ("maxima.info-5" 247373 592 "Funciones y variables de parámetros descriptivos"))
("std1" . ("maxima.info-5" 247973 601 "Funciones y variables de parámetros descriptivos"))
("std1 <1>" . ("maxima.info-5" 247974 600 "Funciones y variables de parámetros descriptivos"))
("std_bernoulli" . ("maxima.info-6" 109018 333 "Funciones y variables para distribuciones discretas"))
("std_beta" . ("maxima.info-6" 76836 193 "Funciones y variables para distribuciones continuas"))
("std_binomial" . ("maxima.info-6" 103658 231 "Funciones y variables para distribuciones discretas"))
("std_chi2" . ("maxima.info-6" 59503 292 "Funciones y variables para distribuciones continuas"))
("std_continuous_uniform" . ("maxima.info-6" 79274 218 "Funciones y variables para distribuciones continuas"))
("std_discrete_uniform" . ("maxima.info-6" 114371 224 "Funciones y variables para distribuciones discretas"))
("std_exp" . ("maxima.info-6" 68587 385 "Funciones y variables para distribuciones continuas"))
("std_f" . ("maxima.info-6" 65116 190 "Funciones y variables para distribuciones continuas"))
("std_gamma" . ("maxima.info-6" 74103 195 "Funciones y variables para distribuciones continuas"))
("std_general_finite_discrete" . ("maxima.info-6" 100667 231 "Funciones y variables para distribuciones discretas"))
("std_geometric" . ("maxima.info-6" 111947 205 "Funciones y variables para distribuciones discretas"))
("std_gumbel" . ("maxima.info-6" 96573 198 "Funciones y variables para distribuciones continuas"))
("std_hypergeometric" . ("maxima.info-6" 117466 269 "Funciones y variables para distribuciones discretas"))
("std_laplace" . ("maxima.info-6" 92990 200 "Funciones y variables para distribuciones continuas"))
("std_logistic" . ("maxima.info-6" 81664 200 "Funciones y variables para distribuciones continuas"))
("std_lognormal" . ("maxima.info-6" 71591 201 "Funciones y variables para distribuciones continuas"))
("std_negative_binomial" . ("maxima.info-6" 120227 241 "Funciones y variables para distribuciones discretas"))
("std_noncentral_chi2" . ("maxima.info-6" 62468 204 "Funciones y variables para distribuciones continuas"))
("std_noncentral_student_t" . ("maxima.info-6" 55565 291 "Funciones y variables para distribuciones continuas"))
("std_normal" . ("maxima.info-6" 48064 209 "Funciones y variables para distribuciones continuas"))
("std_pareto" . ("maxima.info-6" 83916 202 "Funciones y variables para distribuciones continuas"))
("std_poisson" . ("maxima.info-6" 106248 194 "Funciones y variables para distribuciones discretas"))
("std_rayleigh" . ("maxima.info-6" 89580 501 "Funciones y variables para distribuciones continuas"))
("std_student_t" . ("maxima.info-6" 50970 198 "Funciones y variables para distribuciones continuas"))
("std_weibull" . ("maxima.info-6" 86175 202 "Funciones y variables para distribuciones continuas"))
("stemplot" . ("maxima.info-6" 21056 997 "Funciones y variables para gráficos estadísticos"))
("stemplot <1>" . ("maxima.info-6" 21057 996 "Funciones y variables para gráficos estadísticos"))
("stirling" . ("maxima.info-8" 89904 2537 "Funciones y variables para stirling"))
("stirling <1>" . ("maxima.info-8" 89905 2536 "Funciones y variables para stirling"))
("stirling1" . ("maxima.info-5" 51790 2053 "Funciones y variables para los conjuntos"))
("stirling2" . ("maxima.info-5" 53857 2393 "Funciones y variables para los conjuntos"))
("strim" . ("maxima.info-8" 122882 328 "Funciones y variables para cadenas"))
("striml" . ("maxima.info-8" 123212 133 "Funciones y variables para cadenas"))
("strimr" . ("maxima.info-8" 123349 135 "Funciones y variables para cadenas"))
("string" . ("maxima.info-1" 115880 234 "Funciones y variables para las cadenas de texto"))
("stringdisp" . ("maxima.info-1" 116119 826 "Funciones y variables para las cadenas de texto"))
("stringout" . ("maxima.info-2" 199310 2026 "Funciones y variables para lectura y escritura"))
("stringout <1>" . ("maxima.info-2" 199311 2025 "Funciones y variables para lectura y escritura"))
("stringout <2>" . ("maxima.info-2" 199383 1954 "Funciones y variables para lectura y escritura"))
("stringout <3>" . ("maxima.info-2" 199432 1906 "Funciones y variables para lectura y escritura"))
("stringout <4>" . ("maxima.info-2" 199476 1863 "Funciones y variables para lectura y escritura"))
("stringp" . ("maxima.info-8" 123488 117 "Funciones y variables para cadenas"))
("strong_components" . ("maxima.info-7" 113270 381 "Funciones y variables para graphs"))
("structures" . ("maxima.info-1" 172943 139 "Funciones y variables para las estructuras"))
("struve_h" . ("maxima.info-3" 38153 102 "Funciones de Struve"))
("struve_l" . ("maxima.info-3" 38258 120 "Funciones de Struve"))
("style" . ("maxima.info-2" 168246 2138 "Opciones gráficas"))
("style <1>" . ("maxima.info-2" 168247 2137 "Opciones gráficas"))
("sublis" . ("maxima.info-1" 304331 481 "Funciones y variables para expresiones"))
("sublis_apply_lambda" . ("maxima.info-1" 304820 323 "Funciones y variables para expresiones"))
("sublist" . ("maxima.info-1" 145937 326 "Funciones y variables para listas"))
("sublist_indices" . ("maxima.info-1" 146265 1089 "Funciones y variables para listas"))
("submatrix" . ("maxima.info-3" 282888 323 "Funciones y variables para las matrices y el álgebra lineal"))
("submatrix <1>" . ("maxima.info-3" 282889 322 "Funciones y variables para las matrices y el álgebra lineal"))
("submatrix <2>" . ("maxima.info-3" 282957 255 "Funciones y variables para las matrices y el álgebra lineal"))
("subnumsimp" . ("maxima.info-1" 305147 600 "Funciones y variables para expresiones"))
("subsample" . ("maxima.info-5" 236806 6294 "Funciones y variables para el tratamiento de datos"))
("subsample <1>" . ("maxima.info-5" 236807 6293 "Funciones y variables para el tratamiento de datos"))
("subset" . ("maxima.info-5" 56264 699 "Funciones y variables para los conjuntos"))
("subsetp" . ("maxima.info-5" 56970 447 "Funciones y variables para los conjuntos"))
("subst" . ("maxima.info-1" 305752 2434 "Funciones y variables para expresiones"))
("subst_parallel" . ("maxima.info-8" 151687 2003 "Funciones y variables para to_poly_solve"))
("substinpart" . ("maxima.info-1" 308215 2420 "Funciones y variables para expresiones"))
("substpart" . ("maxima.info-1" 310651 1198 "Funciones y variables para expresiones"))
("substring" . ("maxima.info-8" 123609 589 "Funciones y variables para cadenas"))
("substring <1>" . ("maxima.info-8" 123610 588 "Funciones y variables para cadenas"))
("subvar" . ("maxima.info-1" 169723 733 "Funciones y variables para los arrays"))
("subvarp" . ("maxima.info-1" 170465 103 "Funciones y variables para los arrays"))
("sum" . ("maxima.info-4" 150070 4687 "Funciones y variables para sumas y productos"))
("sumcontract" . ("maxima.info-4" 154796 1374 "Funciones y variables para sumas y productos"))
("sumexpand" . ("maxima.info-4" 156181 983 "Funciones y variables para sumas y productos"))
("summand_to_rec" . ("maxima.info-8" 43328 1981 "Funciones y variables para solve_rec"))
("sumsplitfact" . ("maxima.info-2" 57913 160 "Funciones combinatorias"))
("supcase" . ("maxima.info-8" 124208 449 "Funciones y variables para cadenas"))
("supcase <1>" . ("maxima.info-8" 124209 448 "Funciones y variables para cadenas"))
("supcase <2>" . ("maxima.info-8" 124242 416 "Funciones y variables para cadenas"))
("supcontext" . ("maxima.info-2" 129716 277 "Funciones y variables para los hechos"))
("supcontext <1>" . ("maxima.info-2" 129717 276 "Funciones y variables para los hechos"))
("surface" . ("maxima.info-7" 14354 283 "Visualización usando VTK"))
("surface_hide" . ("maxima.info-6" 187871 577 "Funciones y variables para draw"))
("symbolp" . ("maxima.info-1" 311855 226 "Funciones y variables para expresiones"))
("symmdifference" . ("maxima.info-5" 57421 1093 "Funciones y variables para los conjuntos"))
("symmetric" . ("maxima.info-2" 31864 264 "Funciones y variables para simplificación"))
("symmetricp" . ("maxima.info-4" 111037 116 "Funciones y variables para ctensor"))
("system" . ("maxima.info-4" 280202 865 "Funciones y variables para el entorno de ejecución"))
("tab" . ("maxima.info-8" 110737 50 "Funciones y variables para caracteres"))
("take_channel" . ("maxima.info-6" 259261 955 "Funciones y variables para picture"))
("take_inference" . ("maxima.info-8" 49885 1910 "Funciones y variables para inference_result"))
("take_inference <1>" . ("maxima.info-8" 49886 1909 "Funciones y variables para inference_result"))
("take_inference <2>" . ("maxima.info-8" 49928 1868 "Funciones y variables para inference_result"))
("tan" . ("maxima.info-2" 77206 38 "Funciones y variables para trigonometría"))
("tanh" . ("maxima.info-2" 77246 51 "Funciones y variables para trigonometría"))
("taylor" . ("maxima.info-4" 173009 8179 "Funciones y variables para las series"))
("taylor <1>" . ("maxima.info-4" 173010 8178 "Funciones y variables para las series"))
("taylor <2>" . ("maxima.info-4" 173055 8134 "Funciones y variables para las series"))
("taylor <3>" . ("maxima.info-4" 173116 8074 "Funciones y variables para las series"))
("taylor <4>" . ("maxima.info-4" 173243 7949 "Funciones y variables para las series"))
("taylor_logexpand" . ("maxima.info-4" 182581 495 "Funciones y variables para las series"))
("taylor_order_coefficients" . ("maxima.info-4" 183080 332 "Funciones y variables para las series"))
("taylor_simplifier" . ("maxima.info-4" 183415 163 "Funciones y variables para las series"))
("taylor_truncate_polynomials" . ("maxima.info-4" 183582 362 "Funciones y variables para las series"))
("taylordepth" . ("maxima.info-4" 181212 291 "Funciones y variables para las series"))
("taylorinfo" . ("maxima.info-4" 181507 944 "Funciones y variables para las series"))
("taylorp" . ("maxima.info-4" 182457 122 "Funciones y variables para las series"))
("taytorat" . ("maxima.info-4" 183947 216 "Funciones y variables para las series"))
("tcl_output" . ("maxima.info-4" 291203 1247 "Funciones y variables para la miscelánea de opciones"))
("tcl_output <1>" . ("maxima.info-4" 291204 1246 "Funciones y variables para la miscelánea de opciones"))
("tcl_output <2>" . ("maxima.info-4" 291252 1199 "Funciones y variables para la miscelánea de opciones"))
("tcontract" . ("maxima.info-4" 269909 386 "Funciones y variables para simetrías"))
("tellrat" . ("maxima.info-2" 279866 2296 "Funciones y variables para polinomios"))
("tellrat <1>" . ("maxima.info-2" 279867 2295 "Funciones y variables para polinomios"))
("tellsimp" . ("maxima.info-4" 320975 2555 "Funciones y variables sobre reglas y patrones"))
("tellsimpafter" . ("maxima.info-4" 323544 9923 "Funciones y variables sobre reglas y patrones"))
("tensorkill" . ("maxima.info-4" 131705 174 "Funciones y variables para ctensor"))
("tentex" . ("maxima.info-4" 67750 1273 "Funciones y variables para itensor"))
("tenth" . ("maxima.info-1" 147790 133 "Funciones y variables para listas"))
("terminal" . ("maxima.info-6" 188455 3272 "Funciones y variables para draw"))
("terminal <1>" . ("maxima.info-7" 128948 131 "Funciones y variables para graphs"))
("test_mean" . ("maxima.info-8" 52379 4556 "Funciones y variables para stats"))
("test_mean <1>" . ("maxima.info-8" 52380 4555 "Funciones y variables para stats"))
("test_means_difference" . ("maxima.info-8" 56968 5041 "Funciones y variables para stats"))
("test_means_difference <1>" . ("maxima.info-8" 56969 5040 "Funciones y variables para stats"))
("test_normality" . ("maxima.info-8" 83077 1124 "Funciones y variables para stats"))
("test_proportion" . ("maxima.info-8" 67548 3564 "Funciones y variables para stats"))
("test_proportion <1>" . ("maxima.info-8" 67549 3563 "Funciones y variables para stats"))
("test_proportions_difference" . ("maxima.info-8" 71138 3801 "Funciones y variables para stats"))
("test_proportions_difference <1>" . ("maxima.info-8" 71139 3800 "Funciones y variables para stats"))
("test_rank_sum" . ("maxima.info-8" 80247 2811 "Funciones y variables para stats"))
("test_rank_sum <1>" . ("maxima.info-8" 80248 2810 "Funciones y variables para stats"))
("test_sign" . ("maxima.info-8" 74970 2028 "Funciones y variables para stats"))
("test_sign <1>" . ("maxima.info-8" 74971 2027 "Funciones y variables para stats"))
("test_signed_rank" . ("maxima.info-8" 77013 3213 "Funciones y variables para stats"))
("test_signed_rank <1>" . ("maxima.info-8" 77014 3212 "Funciones y variables para stats"))
("test_variance" . ("maxima.info-8" 62043 2617 "Funciones y variables para stats"))
("test_variance <1>" . ("maxima.info-8" 62044 2616 "Funciones y variables para stats"))
("test_variance_ratio" . ("maxima.info-8" 64677 2856 "Funciones y variables para stats"))
("test_variance_ratio <1>" . ("maxima.info-8" 64678 2855 "Funciones y variables para stats"))
("testsuite_files" . ("maxima.info-1" 26188 749 "Funciones y variables para la detección e informe de fallos"))
("tex" . ("maxima.info-2" 204180 2539 "Funciones y variables para salida TeX"))
("tex <1>" . ("maxima.info-2" 204181 2538 "Funciones y variables para salida TeX"))
("tex <2>" . ("maxima.info-2" 204208 2512 "Funciones y variables para salida TeX"))
("tex <3>" . ("maxima.info-2" 204250 2471 "Funciones y variables para salida TeX"))
("tex <4>" . ("maxima.info-2" 204284 2438 "Funciones y variables para salida TeX"))
("tex <5>" . ("maxima.info-2" 204312 2411 "Funciones y variables para salida TeX"))
("tex1" . ("maxima.info-2" 206746 304 "Funciones y variables para salida TeX"))
("texput" . ("maxima.info-2" 207056 5063 "Funciones y variables para salida TeX"))
("texput <1>" . ("maxima.info-2" 207057 5062 "Funciones y variables para salida TeX"))
("texput <2>" . ("maxima.info-2" 207089 5031 "Funciones y variables para salida TeX"))
("texput <3>" . ("maxima.info-2" 207121 5000 "Funciones y variables para salida TeX"))
("texput <4>" . ("maxima.info-2" 207170 4952 "Funciones y variables para salida TeX"))
("thetaresolution" . ("maxima.info-7" 14642 337 "Visualización usando VTK"))
("third" . ("maxima.info-1" 147929 135 "Funciones y variables para listas"))
("throw" . ("maxima.info-5" 166298 191 "Funciones y variables para la programación"))
("time" . ("maxima.info-4" 281075 575 "Funciones y variables para el entorno de ejecución"))
("timedate" . ("maxima.info-4" 281659 1279 "Funciones y variables para el entorno de ejecución"))
("timedate <1>" . ("maxima.info-4" 281660 1278 "Funciones y variables para el entorno de ejecución"))
("timer" . ("maxima.info-5" 183698 1377 "Funciones y variables para depurado"))
("timer <1>" . ("maxima.info-5" 183699 1376 "Funciones y variables para depurado"))
("timer <2>" . ("maxima.info-5" 183739 1337 "Funciones y variables para depurado"))
("timer_devalue" . ("maxima.info-5" 185785 623 "Funciones y variables para depurado"))
("timer_info" . ("maxima.info-5" 186420 976 "Funciones y variables para depurado"))
("timer_info <1>" . ("maxima.info-5" 186421 975 "Funciones y variables para depurado"))
("title" . ("maxima.info-6" 191746 460 "Funciones y variables para draw"))
("tldefint" . ("maxima.info-3" 118178 107 "Funciones y variables para integración"))
("tlimit" . ("maxima.info-3" 61341 251 "Funciones y variables para límites"))
("tlimit <1>" . ("maxima.info-3" 61342 250 "Funciones y variables para límites"))
("tlimit <2>" . ("maxima.info-3" 61391 202 "Funciones y variables para límites"))
("tlimswitch" . ("maxima.info-3" 61599 475 "Funciones y variables para límites"))
("to_lisp" . ("maxima.info-1" 71777 1130 "Funciones y variables para la línea de comandos"))
("to_poly" . ("maxima.info-8" 153707 2469 "Funciones y variables para to_poly_solve"))
("to_poly_solve" . ("maxima.info-8" 156198 13841 "Funciones y variables para to_poly_solve"))
("todd_coxeter" . ("maxima.info-4" 271825 1805 "Funciones y variables para grupos"))
("todd_coxeter <1>" . ("maxima.info-4" 271826 1804 "Funciones y variables para grupos"))
("toeplitz" . ("maxima.info-7" 230430 841 "Funciones y variables para linearalgebra"))
("toeplitz <1>" . ("maxima.info-7" 230431 840 "Funciones y variables para linearalgebra"))
("tokens" . ("maxima.info-8" 124664 747 "Funciones y variables para cadenas"))
("tokens <1>" . ("maxima.info-8" 124665 746 "Funciones y variables para cadenas"))
("topological_sort" . ("maxima.info-7" 113653 546 "Funciones y variables para graphs"))
("totaldisrep" . ("maxima.info-2" 282170 409 "Funciones y variables para polinomios"))
("totalfourier" . ("maxima.info-4" 189489 128 "Funciones y variables para series de Fourier"))
("totient" . ("maxima.info-4" 214460 125 "Funciones y variables para teoría de números"))
("tpartpol" . ("maxima.info-4" 270301 388 "Funciones y variables para simetrías"))
("tr" . ("maxima.info-4" 131195 141 "Funciones y variables para ctensor"))
("tr_array_as_ref" . ("maxima.info-5" 129310 358 "Funciones y variables para la definición de funciones"))
("tr_bound_function_applyp" . ("maxima.info-5" 129671 448 "Funciones y variables para la definición de funciones"))
("tr_file_tty_messagesp" . ("maxima.info-5" 130126 385 "Funciones y variables para la definición de funciones"))
("tr_float_can_branch_complex" . ("maxima.info-5" 130515 216 "Funciones y variables para la definición de funciones"))
("tr_function_call_default" . ("maxima.info-5" 130732 838 "Funciones y variables para la definición de funciones"))
("tr_numer" . ("maxima.info-5" 131577 189 "Funciones y variables para la definición de funciones"))
("tr_optimize_max_loop" . ("maxima.info-5" 131769 251 "Funciones y variables para la definición de funciones"))
("tr_semicompile" . ("maxima.info-5" 132027 250 "Funciones y variables para la definición de funciones"))
("tr_state_vars" . ("maxima.info-5" 132281 638 "Funciones y variables para la definición de funciones"))
("tr_warn_bad_function_calls" . ("maxima.info-5" 133023 251 "Funciones y variables para la definición de funciones"))
("tr_warn_fexpr" . ("maxima.info-5" 133277 190 "Funciones y variables para la definición de funciones"))
("tr_warn_meval" . ("maxima.info-5" 133470 216 "Funciones y variables para la definición de funciones"))
("tr_warn_mode" . ("maxima.info-5" 133690 168 "Funciones y variables para la definición de funciones"))
("tr_warn_undeclared" . ("maxima.info-5" 133859 146 "Funciones y variables para la definición de funciones"))
("tr_warn_undefined_variable" . ("maxima.info-5" 134006 161 "Funciones y variables para la definición de funciones"))
("tr_warnings_get" . ("maxima.info-5" 132925 96 "Funciones y variables para la definición de funciones"))
("trace" . ("maxima.info-5" 187413 1192 "Funciones y variables para depurado"))
("trace <1>" . ("maxima.info-5" 187414 1191 "Funciones y variables para depurado"))
("trace <2>" . ("maxima.info-5" 187454 1152 "Funciones y variables para depurado"))
("trace_options" . ("maxima.info-5" 188623 2618 "Funciones y variables para depurado"))
("trace_options <1>" . ("maxima.info-5" 188624 2617 "Funciones y variables para depurado"))
("tracematrix" . ("maxima.info-8" 20599 178 "Paquete functs"))
("track" . ("maxima.info-7" 14985 1074 "Visualización usando VTK"))
("transcompile" . ("maxima.info-5" 123394 297 "Funciones y variables para la definición de funciones"))
("transform" . ("maxima.info-6" 192217 1518 "Funciones y variables para draw"))
("transform_sample" . ("maxima.info-5" 243137 1615 "Funciones y variables para el tratamiento de datos"))
("transform_xy" . ("maxima.info-2" 170417 359 "Opciones gráficas"))
("translate" . ("maxima.info-5" 123697 2573 "Funciones y variables para la definición de funciones"))
("translate <1>" . ("maxima.info-5" 123698 2572 "Funciones y variables para la definición de funciones"))
("translate <2>" . ("maxima.info-5" 123742 2529 "Funciones y variables para la definición de funciones"))
("translate_file" . ("maxima.info-5" 126310 2746 "Funciones y variables para la definición de funciones"))
("translate_file <1>" . ("maxima.info-5" 126311 2745 "Funciones y variables para la definición de funciones"))
("transparent" . ("maxima.info-6" 193744 542 "Funciones y variables para draw"))
("transpose" . ("maxima.info-3" 283215 396 "Funciones y variables para las matrices y el álgebra lineal"))
("transrun" . ("maxima.info-5" 129089 219 "Funciones y variables para la definición de funciones"))
("tree_reduce" . ("maxima.info-5" 58518 1387 "Funciones y variables para los conjuntos"))
("tree_reduce <1>" . ("maxima.info-5" 58519 1386 "Funciones y variables para los conjuntos"))
("treefale" . ("maxima.info-7" 71404 326 "Definiciones para IFS fractals"))
("treillis" . ("maxima.info-4" 270694 235 "Funciones y variables para simetrías"))
("treinat" . ("maxima.info-4" 270933 555 "Funciones y variables para simetrías"))
("triangle" . ("maxima.info-6" 252542 911 "Funciones y variables para draw"))
("triangularize" . ("maxima.info-3" 283614 1054 "Funciones y variables para las matrices y el álgebra lineal"))
("trigexpand" . ("maxima.info-2" 77300 1441 "Funciones y variables para trigonometría"))
("trigexpandplus" . ("maxima.info-2" 78764 347 "Funciones y variables para trigonometría"))
("trigexpandtimes" . ("maxima.info-2" 79117 356 "Funciones y variables para trigonometría"))
("triginverses" . ("maxima.info-2" 79479 516 "Funciones y variables para trigonometría"))
("trigrat" . ("maxima.info-2" 81857 2898 "Funciones y variables para trigonometría"))
("trigreduce" . ("maxima.info-2" 80001 1100 "Funciones y variables para trigonometría"))
("trigreduce <1>" . ("maxima.info-2" 80002 1099 "Funciones y variables para trigonometría"))
("trigsign" . ("maxima.info-2" 81115 278 "Funciones y variables para trigonometría"))
("trigsimp" . ("maxima.info-2" 81398 454 "Funciones y variables para trigonometría"))
("trivial_solutions" . ("maxima.info-8" 201110 174 "Funciones y variables para zeilberger"))
("true" . ("maxima.info-1" 121046 140 "Funciones y variables para Constantes"))
("trunc" . ("maxima.info-4" 184167 578 "Funciones y variables para las series"))
("truncated_cube_graph" . ("maxima.info-7" 88704 77 "Funciones y variables para graphs"))
("truncated_dodecahedron_graph" . ("maxima.info-7" 88784 91 "Funciones y variables para graphs"))
("truncated_icosahedron_graph" . ("maxima.info-7" 88878 89 "Funciones y variables para graphs"))
("truncated_tetrahedron_graph" . ("maxima.info-7" 88970 91 "Funciones y variables para graphs"))
("tstep" . ("maxima.info-7" 4341 229 "Visualización usando VTK"))
("ttyoff" . ("maxima.info-1" 95605 400 "Funciones y variables para la impresión"))
("tube" . ("maxima.info-6" 253462 786 "Funciones y variables para draw"))
("tutte_graph" . ("maxima.info-7" 89063 61 "Funciones y variables para graphs"))
("ueivects" . ("maxima.info-3" 284673 1252 "Funciones y variables para las matrices y el álgebra lineal"))
("ufg" . ("maxima.info-4" 128991 163 "Funciones y variables para ctensor"))
("uforget" . ("maxima.info-8" 178883 1448 "Funciones y variables para units"))
("ug" . ("maxima.info-4" 130017 91 "Funciones y variables para ctensor"))
("ultraspherical" . ("maxima.info-7" 307560 161 "Funciones y variables para polinomios ortogonales"))
("und" . ("maxima.info-1" 121189 199 "Funciones y variables para Constantes"))
("underlying_graph" . ("maxima.info-7" 89126 92 "Funciones y variables para graphs"))
("undiff" . ("maxima.info-4" 31670 200 "Funciones y variables para itensor"))
("union" . ("maxima.info-5" 59923 1043 "Funciones y variables para los conjuntos"))
("unique" . ("maxima.info-1" 147359 427 "Funciones y variables para listas"))
("unit_step" . ("maxima.info-7" 307223 330 "Funciones y variables para polinomios ortogonales"))
("unit_vectors" . ("maxima.info-6" 194292 728 "Funciones y variables para draw"))
("uniteigenvectors" . ("maxima.info-3" 284672 1253 "Funciones y variables para las matrices y el álgebra lineal"))
("unitp" . ("maxima.info-7" 43062 1256 "Funciones y variables para ezunits"))
("units" . ("maxima.info-7" 38909 2427 "Funciones y variables para ezunits"))
("unitvector" . ("maxima.info-3" 285936 339 "Funciones y variables para las matrices y el álgebra lineal"))
("unknown" . ("maxima.info-2" 136934 153 "Funciones y variables para los predicados"))
("unless" . ("maxima.info-5" 166495 76 "Funciones y variables para la programación"))
("unorder" . ("maxima.info-1" 312086 1436 "Funciones y variables para expresiones"))
("unsum" . ("maxima.info-4" 184751 1197 "Funciones y variables para las series"))
("untellrat" . ("maxima.info-2" 282587 128 "Funciones y variables para polinomios"))
("untimer" . ("maxima.info-5" 185100 674 "Funciones y variables para depurado"))
("untimer <1>" . ("maxima.info-5" 185101 673 "Funciones y variables para depurado"))
("untrace" . ("maxima.info-5" 191273 401 "Funciones y variables para depurado"))
("untrace <1>" . ("maxima.info-5" 191274 400 "Funciones y variables para depurado"))
("uppercasep" . ("maxima.info-8" 110790 95 "Funciones y variables para caracteres"))
("uric" . ("maxima.info-4" 129752 101 "Funciones y variables para ctensor"))
("uricci" . ("maxima.info-4" 86747 523 "Funciones y variables para ctensor"))
("uriem" . ("maxima.info-4" 129563 106 "Funciones y variables para ctensor"))
("uriemann" . ("maxima.info-4" 90380 195 "Funciones y variables para ctensor"))
("use_fast_arrays" . ("maxima.info-1" 170570 169 "Funciones y variables para los arrays"))
("user_preamble" . ("maxima.info-6" 195028 880 "Funciones y variables para draw"))
("usersetunits" . ("maxima.info-8" 183775 8803 "Funciones y variables para units"))
("uvect" . ("maxima.info-3" 285937 338 "Funciones y variables para las matrices y el álgebra lineal"))
("values" . ("maxima.info-1" 72913 1248 "Funciones y variables para la línea de comandos"))
("vandermonde_matrix" . ("maxima.info-7" 231274 163 "Funciones y variables para linearalgebra"))
("var" . ("maxima.info-5" 245939 589 "Funciones y variables de parámetros descriptivos"))
("var <1>" . ("maxima.info-5" 245940 588 "Funciones y variables de parámetros descriptivos"))
("var1" . ("maxima.info-5" 246533 834 "Funciones y variables de parámetros descriptivos"))
("var1 <1>" . ("maxima.info-5" 246534 833 "Funciones y variables de parámetros descriptivos"))
("var_bernoulli" . ("maxima.info-6" 108702 314 "Funciones y variables para distribuciones discretas"))
("var_beta" . ("maxima.info-6" 76648 184 "Funciones y variables para distribuciones continuas"))
("var_binomial" . ("maxima.info-6" 103432 222 "Funciones y variables para distribuciones discretas"))
("var_chi2" . ("maxima.info-6" 59229 272 "Funciones y variables para distribuciones continuas"))
("var_continuous_uniform" . ("maxima.info-6" 79061 209 "Funciones y variables para distribuciones continuas"))
("var_discrete_uniform" . ("maxima.info-6" 114152 215 "Funciones y variables para distribuciones discretas"))
("var_exp" . ("maxima.info-6" 68167 418 "Funciones y variables para distribuciones continuas"))
("var_f" . ("maxima.info-6" 64931 181 "Funciones y variables para distribuciones continuas"))
("var_gamma" . ("maxima.info-6" 73913 186 "Funciones y variables para distribuciones continuas"))
("var_general_finite_discrete" . ("maxima.info-6" 100441 222 "Funciones y variables para distribuciones discretas"))
("var_geometric" . ("maxima.info-6" 111746 196 "Funciones y variables para distribuciones discretas"))
("var_gumbel" . ("maxima.info-6" 96380 189 "Funciones y variables para distribuciones continuas"))
("var_hypergeometric" . ("maxima.info-6" 117201 260 "Funciones y variables para distribuciones discretas"))
("var_laplace" . ("maxima.info-6" 92795 191 "Funciones y variables para distribuciones continuas"))
("var_logistic" . ("maxima.info-6" 81468 191 "Funciones y variables para distribuciones continuas"))
("var_lognormal" . ("maxima.info-6" 71395 192 "Funciones y variables para distribuciones continuas"))
("var_negative_binomial" . ("maxima.info-6" 119991 232 "Funciones y variables para distribuciones discretas"))
("var_noncentral_chi2" . ("maxima.info-6" 62270 195 "Funciones y variables para distribuciones continuas"))
("var_noncentral_student_t" . ("maxima.info-6" 55278 282 "Funciones y variables para distribuciones continuas"))
("var_normal" . ("maxima.info-6" 47933 129 "Funciones y variables para distribuciones continuas"))
("var_pareto" . ("maxima.info-6" 83719 193 "Funciones y variables para distribuciones continuas"))
("var_poisson" . ("maxima.info-6" 106059 185 "Funciones y variables para distribuciones discretas"))
("var_rayleigh" . ("maxima.info-6" 89053 525 "Funciones y variables para distribuciones continuas"))
("var_student_t" . ("maxima.info-6" 50649 319 "Funciones y variables para distribuciones continuas"))
("var_weibull" . ("maxima.info-6" 85978 193 "Funciones y variables para distribuciones continuas"))
("vector" . ("maxima.info-6" 254256 1319 "Funciones y variables para draw"))
("vector <1>" . ("maxima.info-6" 254257 1318 "Funciones y variables para draw"))
("vectorpotential" . ("maxima.info-3" 286281 320 "Funciones y variables para las matrices y el álgebra lineal"))
("vectorsimp" . ("maxima.info-3" 286605 1369 "Funciones y variables para las matrices y el álgebra lineal"))
("verbify" . ("maxima.info-1" 313536 385 "Funciones y variables para expresiones"))
("verbose" . ("maxima.info-4" 185952 173 "Funciones y variables para las series"))
("vers" . ("maxima.info-8" 25648 115 "Paquete functs"))
("vertex_color" . ("maxima.info-7" 126183 78 "Funciones y variables para graphs"))
("vertex_coloring" . ("maxima.info-7" 127017 175 "Funciones y variables para graphs"))
("vertex_coloring <1>" . ("maxima.info-7" 116342 481 "Funciones y variables para graphs"))
("vertex_coloring <2>" . ("maxima.info-7" 120159 480 "Funciones y variables para graphs"))
("vertex_connectivity" . ("maxima.info-7" 114205 138 "Funciones y variables para graphs"))
("vertex_degree" . ("maxima.info-7" 114348 94 "Funciones y variables para graphs"))
("vertex_distance" . ("maxima.info-7" 114445 419 "Funciones y variables para graphs"))
("vertex_eccentricity" . ("maxima.info-7" 114867 276 "Funciones y variables para graphs"))
("vertex_in_degree" . ("maxima.info-7" 115146 580 "Funciones y variables para graphs"))
("vertex_out_degree" . ("maxima.info-7" 115729 381 "Funciones y variables para graphs"))
("vertex_partition" . ("maxima.info-7" 126796 215 "Funciones y variables para graphs"))
("vertex_size" . ("maxima.info-7" 126112 67 "Funciones y variables para graphs"))
("vertex_type" . ("maxima.info-7" 125931 174 "Funciones y variables para graphs"))
("vertices" . ("maxima.info-7" 116113 226 "Funciones y variables para graphs"))
("vertices_to_cycle" . ("maxima.info-7" 130028 161 "Funciones y variables para graphs"))
("vertices_to_path" . ("maxima.info-7" 129866 159 "Funciones y variables para graphs"))
("view" . ("maxima.info-6" 195924 943 "Funciones y variables para draw"))
("warnings" . ("maxima.info-8" 200635 187 "Funciones y variables para zeilberger"))
("weyl" . ("maxima.info-4" 130111 78 "Funciones y variables para ctensor"))
("weyl <1>" . ("maxima.info-4" 90842 433 "Funciones y variables para ctensor"))
("wheel_graph" . ("maxima.info-7" 89220 83 "Funciones y variables para graphs"))
("while" . ("maxima.info-5" 166494 77 "Funciones y variables para la programación"))
("width" . ("maxima.info-7" 4574 178 "Visualización usando VTK"))
("wiener_index" . ("maxima.info-7" 116831 197 "Funciones y variables para graphs"))
("windowname" . ("maxima.info-7" 4757 338 "Visualización usando VTK"))
("windowtitle" . ("maxima.info-7" 5100 226 "Visualización usando VTK"))
("wired_surface" . ("maxima.info-6" 196884 511 "Funciones y variables para draw"))
("wireframe" . ("maxima.info-7" 16775 293 "Visualización usando VTK"))
("with_stdout" . ("maxima.info-2" 201357 1130 "Funciones y variables para lectura y escritura"))
("with_stdout <1>" . ("maxima.info-2" 201358 1129 "Funciones y variables para lectura y escritura"))
("write_binary_data" . ("maxima.info-7" 277305 1184 "Funciones y variables para entrada y salida en formato binario"))
("write_data" . ("maxima.info-7" 272858 1345 "Funciones y variables para entrada y salida en formato texto"))
("write_data <1>" . ("maxima.info-7" 272859 1344 "Funciones y variables para entrada y salida en formato texto"))
("writefile" . ("maxima.info-2" 202499 1379 "Funciones y variables para lectura y escritura"))
("wronskian" . ("maxima.info-8" 20105 491 "Paquete functs"))
("x" . ("maxima.info-2" 170780 447 "Opciones gráficas"))
("x_voxel" . ("maxima.info-6" 197402 285 "Funciones y variables para draw"))
("xaxis" . ("maxima.info-6" 197695 477 "Funciones y variables para draw"))
("xaxis_color" . ("maxima.info-6" 198182 539 "Funciones y variables para draw"))
("xaxis_secondary" . ("maxima.info-6" 198732 836 "Funciones y variables para draw"))
("xaxis_type" . ("maxima.info-6" 199577 528 "Funciones y variables para draw"))
("xaxis_width" . ("maxima.info-6" 200115 503 "Funciones y variables para draw"))
("xlabel" . ("maxima.info-2" 171241 442 "Opciones gráficas"))
("xlabel <1>" . ("maxima.info-6" 200628 566 "Funciones y variables para draw"))
("xlabel_secondary" . ("maxima.info-6" 201204 742 "Funciones y variables para draw"))
("xlength" . ("maxima.info-7" 16070 230 "Visualización usando VTK"))
("xrange" . ("maxima.info-6" 201957 595 "Funciones y variables para draw"))
("xrange_secondary" . ("maxima.info-6" 202563 550 "Funciones y variables para draw"))
("xreduce" . ("maxima.info-5" 60970 1714 "Funciones y variables para los conjuntos"))
("xreduce <1>" . ("maxima.info-5" 60971 1713 "Funciones y variables para los conjuntos"))
("xthru" . ("maxima.info-2" 32133 1017 "Funciones y variables para simplificación"))
("xtics" . ("maxima.info-6" 203124 2082 "Funciones y variables para draw"))
("xtics_axis" . ("maxima.info-6" 205225 366 "Funciones y variables para draw"))
("xtics_rotate" . ("maxima.info-6" 205599 273 "Funciones y variables para draw"))
("xtics_rotate_secondary" . ("maxima.info-6" 205879 304 "Funciones y variables para draw"))
("xtics_secondary" . ("maxima.info-6" 206190 225 "Funciones y variables para draw"))
("xtics_secondary_axis" . ("maxima.info-6" 206423 397 "Funciones y variables para draw"))
("xu_grid" . ("maxima.info-6" 206828 554 "Funciones y variables para draw"))
("xy_file" . ("maxima.info-6" 207392 429 "Funciones y variables para draw"))
("xyplane" . ("maxima.info-6" 207833 582 "Funciones y variables para draw"))
("y" . ("maxima.info-2" 171689 292 "Opciones gráficas"))
("y_voxel" . ("maxima.info-6" 208423 285 "Funciones y variables para draw"))
("yaxis" . ("maxima.info-6" 208716 477 "Funciones y variables para draw"))
("yaxis_color" . ("maxima.info-6" 209203 539 "Funciones y variables para draw"))
("yaxis_secondary" . ("maxima.info-6" 209753 737 "Funciones y variables para draw"))
("yaxis_type" . ("maxima.info-6" 210499 528 "Funciones y variables para draw"))
("yaxis_width" . ("maxima.info-6" 211037 503 "Funciones y variables para draw"))
("ylabel" . ("maxima.info-2" 171985 408 "Opciones gráficas"))
("ylabel <1>" . ("maxima.info-6" 211550 567 "Funciones y variables para draw"))
("ylabel_secondary" . ("maxima.info-6" 212127 836 "Funciones y variables para draw"))
("ylength" . ("maxima.info-7" 16305 230 "Visualización usando VTK"))
("yrange" . ("maxima.info-6" 212974 635 "Funciones y variables para draw"))
("yrange_secondary" . ("maxima.info-6" 213620 903 "Funciones y variables para draw"))
("ytics" . ("maxima.info-6" 214534 204 "Funciones y variables para draw"))
("ytics_axis" . ("maxima.info-6" 214746 366 "Funciones y variables para draw"))
("ytics_rotate" . ("maxima.info-6" 215120 273 "Funciones y variables para draw"))
("ytics_rotate_secondary" . ("maxima.info-6" 215400 304 "Funciones y variables para draw"))
("ytics_secondary" . ("maxima.info-6" 215711 225 "Funciones y variables para draw"))
("ytics_secondary_axis" . ("maxima.info-6" 215944 397 "Funciones y variables para draw"))
("yv_grid" . ("maxima.info-6" 216349 554 "Funciones y variables para draw"))
("z" . ("maxima.info-2" 172399 144 "Opciones gráficas"))
("z_voxel" . ("maxima.info-6" 216913 219 "Funciones y variables para draw"))
("zaxis" . ("maxima.info-6" 217137 601 "Funciones y variables para draw"))
("zaxis_color" . ("maxima.info-6" 217749 650 "Funciones y variables para draw"))
("zaxis_type" . ("maxima.info-6" 218411 595 "Funciones y variables para draw"))
("zaxis_width" . ("maxima.info-6" 219017 609 "Funciones y variables para draw"))
("zeroa" . ("maxima.info-1" 121391 481 "Funciones y variables para Constantes"))
("zerob" . ("maxima.info-1" 121875 235 "Funciones y variables para Constantes"))
("zerobern" . ("maxima.info-4" 214588 235 "Funciones y variables para teoría de números"))
("zeroequiv" . ("maxima.info-2" 137091 1076 "Funciones y variables para los predicados"))
("zerofor" . ("maxima.info-7" 231440 551 "Funciones y variables para linearalgebra"))
("zerofor <1>" . ("maxima.info-7" 231441 550 "Funciones y variables para linearalgebra"))
("zeromatrix" . ("maxima.info-3" 287978 129 "Funciones y variables para las matrices y el álgebra lineal"))
("zeromatrixp" . ("maxima.info-7" 231996 304 "Funciones y variables para linearalgebra"))
("zeta" . ("maxima.info-4" 214827 1393 "Funciones y variables para teoría de números"))
("zeta%pi" . ("maxima.info-4" 216237 633 "Funciones y variables para teoría de números"))
("zlabel" . ("maxima.info-2" 172547 371 "Opciones gráficas"))
("zlabel <1>" . ("maxima.info-6" 219637 587 "Funciones y variables para draw"))
("zlange" . ("maxima.info-7" 175213 789 "Funciones y variables para lapack"))
("zlength" . ("maxima.info-7" 16540 230 "Visualización usando VTK"))
("zn_add_table" . ("maxima.info-4" 216874 158 "Funciones y variables para teoría de números"))
("zn_determinant" . ("maxima.info-4" 217036 1112 "Funciones y variables para teoría de números"))
("zn_invert_by_lu" . ("maxima.info-4" 218154 1052 "Funciones y variables para teoría de números"))
("zn_log" . ("maxima.info-4" 219212 2466 "Funciones y variables para teoría de números"))
("zn_log <1>" . ("maxima.info-4" 219213 2465 "Funciones y variables para teoría de números"))
("zn_mult_table" . ("maxima.info-4" 221694 917 "Funciones y variables para teoría de números"))
("zn_mult_table <1>" . ("maxima.info-4" 221695 916 "Funciones y variables para teoría de números"))
("zn_order" . ("maxima.info-4" 222618 2322 "Funciones y variables para teoría de números"))
("zn_order <1>" . ("maxima.info-4" 222619 2321 "Funciones y variables para teoría de números"))
("zn_power_table" . ("maxima.info-4" 224950 1416 "Funciones y variables para teoría de números"))
("zn_power_table <1>" . ("maxima.info-4" 224951 1415 "Funciones y variables para teoría de números"))
("zn_primroot" . ("maxima.info-4" 226373 2522 "Funciones y variables para teoría de números"))
("zn_primroot <1>" . ("maxima.info-4" 226374 2521 "Funciones y variables para teoría de números"))
("zn_primroot_limit" . ("maxima.info-4" 228915 328 "Funciones y variables para teoría de números"))
("zn_primroot_p" . ("maxima.info-4" 229246 1793 "Funciones y variables para teoría de números"))
("zn_primroot_p <1>" . ("maxima.info-4" 229247 1792 "Funciones y variables para teoría de números"))
("zn_primroot_pretest" . ("maxima.info-4" 231050 512 "Funciones y variables para teoría de números"))
("zn_primroot_verbose" . ("maxima.info-4" 231569 167 "Funciones y variables para teoría de números"))
("zrange" . ("maxima.info-6" 220234 686 "Funciones y variables para draw"))
("ztics" . ("maxima.info-6" 220931 204 "Funciones y variables para draw"))
("ztics_axis" . ("maxima.info-6" 221143 366 "Funciones y variables para draw"))
("ztics_rotate" . ("maxima.info-6" 221517 274 "Funciones y variables para draw"))
("|" . ("maxima.info-4" 62549 1323 "Funciones y variables para itensor"))
("~" . ("maxima.info-4" 60935 1610 "Funciones y variables para itensor"))
))
(section-pairs '(
; CONTENT: (<NODE NAME> . (<FILENAME> <BYTE OFFSET> <LENGTH IN CHARACTERS>))
("Análisis gráfico de sistemas dinámicos discretos" . ("maxima.info-6" 284534 6457))
("Archivos" . ("maxima.info-2" 179912 1118))
("Arrays" . ("maxima.info-1" 148174 105))
("Cadenas de texto" . ("maxima.info-1" 111418 145))
("Claves de depuración" . ("maxima.info-5" 178213 2373))
("Comentarios" . ("maxima.info-2" 178615 1155))
("Constantes" . ("maxima.info-1" 117063 86))
("Definiciones para IFS fractals" . ("maxima.info-7" 70363 1725))
("Definiciones para cops de Koch" . ("maxima.info-7" 74595 822))
("Definiciones para curvas de Peano" . ("maxima.info-7" 75543 709))
("Definiciones para fractales complejos" . ("maxima.info-7" 72258 2156))
("Depuración del código fuente" . ("maxima.info-5" 173951 4056))
("Desigualdades" . ("maxima.info-1" 263415 193))
("Documentación" . ("maxima.info-1" 29324 959))
("Ejemplos para cobyla" . ("maxima.info-5" 210418 617))
("El paquete dynamics" . ("maxima.info-6" 283776 588))
("Estructuras" . ("maxima.info-1" 170830 139))
("Formatos gráficos" . ("maxima.info-2" 140342 2721))
("Funciones" . ("maxima.info-5" 63442 5335))
("Funciones Gamma y factorial" . ("maxima.info-3" 1783 32371))
("Funciones combinatorias" . ("maxima.info-2" 50763 7262))
("Funciones de Airy" . ("maxima.info-3" 211 1408))
("Funciones de Bessel" . ("maxima.info-2" 285545 6552))
("Funciones de Struve" . ("maxima.info-3" 37988 387))
("Funciones de cilindro parabólico" . ("maxima.info-3" 41100 449))
("Funciones hipergeométricas" . ("maxima.info-3" 38534 2351))
("Funciones para el formato Gnuplot_pipes" . ("maxima.info-2" 177129 1039))
("Funciones para la resolución numérica de ecuaciones" . ("maxima.info-3" 209880 6822))
("Funciones para la resolución numérica de ecuaciones diferenciales" . ("maxima.info-3" 218430 14144))
("Funciones para los números" . ("maxima.info-2" 33716 11573))
("Funciones para los números complejos" . ("maxima.info-2" 45549 4985))
("Funciones radicales, exponenciales y logarítmicas" . ("maxima.info-2" 58249 9889))
("Funciones trigonométricas" . ("maxima.info-2" 68404 157))
("Funciones y Variables para finance" . ("maxima.info-7" 58207 11096))
("Funciones y variable para las ecuaciones" . ("maxima.info-3" 147907 43049))
("Funciones y variables de parámetros descriptivos" . ("maxima.info-5" 244971 33783))
("Funciones y variables para Afines" . ("maxima.info-3" 289575 5263))
("Funciones y variables para Bernstein" . ("maxima.info-5" 196396 4863))
("Funciones y variables para QUADPACK" . ("maxima.info-3" 122876 24491))
("Funciones y variables para atensor" . ("maxima.info-4" 138264 5618))
("Funciones y variables para augmented_lagrangian" . ("maxima.info-5" 192016 4088))
("Funciones y variables para bode" . ("maxima.info-5" 201502 3176))
("Funciones y variables para cadenas" . ("maxima.info-8" 111015 14210))
("Funciones y variables para caracteres" . ("maxima.info-8" 107880 2947))
("Funciones y variables para cobyla" . ("maxima.info-5" 207346 2930))
("Funciones y variables para contrib_ode" . ("maxima.info-5" 217082 4929))
("Funciones y variables para ctensor" . ("maxima.info-4" 76937 56613))
("Funciones y variables para depurado" . ("maxima.info-5" 180733 10801))
("Funciones y variables para diag" . ("maxima.info-6" 22285 16977))
("Funciones y variables para distribuciones continuas" . ("maxima.info-6" 46127 51135))
("Funciones y variables para distribuciones discretas" . ("maxima.info-6" 98075 23124))
("Funciones y variables para distribuciones especiales" . ("maxima.info-8" 88286 1255))
("Funciones y variables para draw" . ("maxima.info-6" 122448 131736))
("Funciones y variables para drawdf" . ("maxima.info-6" 274934 8417))
("Funciones y variables para ecuaciones diferenciales." . ("maxima.info-3" 192389 8021))
("Funciones y variables para el entorno de ejecución" . ("maxima.info-4" 276557 7879))
("Funciones y variables para el tratamiento de datos" . ("maxima.info-5" 231387 13295))
("Funciones y variables para entrada y salida" . ("maxima.info-8" 97441 10181))
("Funciones y variables para entrada y salida en formato binario" . ("maxima.info-7" 274395 4075))
("Funciones y variables para entrada y salida en formato texto" . ("maxima.info-7" 268762 5412))
("Funciones y variables para expresiones" . ("maxima.info-1" 263718 49812))
("Funciones y variables para ezunits" . ("maxima.info-7" 26314 28656))
("Funciones y variables para f90" . ("maxima.info-7" 55292 2000))
("Funciones y variables para funciones elípticas" . ("maxima.info-3" 53651 2337))
("Funciones y variables para ggf" . ("maxima.info-7" 76475 1823))
("Funciones y variables para graphs" . ("maxima.info-7" 79767 50009))
("Funciones y variables para grobner" . ("maxima.info-7" 132421 13775))
("Funciones y variables para grupos" . ("maxima.info-4" 271747 1883))
("Funciones y variables para gráficos" . ("maxima.info-2" 143234 15794))
("Funciones y variables para gráficos estadísticos" . ("maxima.info-6" 229 21651))
("Funciones y variables para impdiff" . ("maxima.info-7" 146535 549))
("Funciones y variables para inference_result" . ("maxima.info-8" 46958 4821))
("Funciones y variables para integración" . ("maxima.info-3" 92589 25486))
("Funciones y variables para integrales elípticas" . ("maxima.info-3" 56222 1758))
("Funciones y variables para interpol" . ("maxima.info-7" 147794 13123))
("Funciones y variables para itensor" . ("maxima.info-4" 183 71216))
("Funciones y variables para la ayuda" . ("maxima.info-1" 30389 9522))
("Funciones y variables para la definición de funciones" . ("maxima.info-5" 78216 57328))
("Funciones y variables para la detección e informe de fallos" . ("maxima.info-1" 24054 4933))
("Funciones y variables para la diferenciación" . ("maxima.info-3" 62399 28498))
("Funciones y variables para la evaluación" . ("maxima.info-1" 229757 26564))
("Funciones y variables para la impresión" . ("maxima.info-1" 74318 21557))
("Funciones y variables para la línea de comandos" . ("maxima.info-1" 46178 27730))
("Funciones y variables para la miscelánea de opciones" . ("maxima.info-4" 286160 6226))
("Funciones y variables para la programación" . ("maxima.info-5" 142167 31186))
("Funciones y variables para la transformada rápida de Fourier" . ("maxima.info-3" 201663 7914))
("Funciones y variables para lapack" . ("maxima.info-7" 161556 19900))
("Funciones y variables para las funciones especiales" . ("maxima.info-3" 41705 7326))
("Funciones y variables para las matrices y el álgebra lineal" . ("maxima.info-3" 237190 51838))
("Funciones y variables para las propiedades" . ("maxima.info-2" 91747 21070))
("Funciones y variables para las series" . ("maxima.info-4" 157997 28018))
("Funciones y variables para lbfgs" . ("maxima.info-7" 182881 15457))
("Funciones y variables para lectura y escritura" . ("maxima.info-2" 181200 22467))
("Funciones y variables para lindstedt" . ("maxima.info-7" 198673 1187))
("Funciones y variables para linearalgebra" . ("maxima.info-7" 204743 27383))
("Funciones y variables para los conjuntos" . ("maxima.info-5" 11245 51129))
("Funciones y variables para los hechos" . ("maxima.info-2" 113208 16659))
("Funciones y variables para los predicados" . ("maxima.info-2" 130143 7989))
("Funciones y variables para lsquares" . ("maxima.info-7" 232562 22183))
("Funciones y variables para límites" . ("maxima.info-3" 58286 3743))
("Funciones y variables para makeOrders" . ("maxima.info-7" 255156 897))
("Funciones y variables para minpack" . ("maxima.info-7" 256554 4483))
("Funciones y variables para mnewton" . ("maxima.info-7" 261320 2304))
("Funciones y variables para opsubst" . ("maxima.info-7" 278748 2409))
("Funciones y variables para picture" . ("maxima.info-6" 255737 4454))
("Funciones y variables para polinomios" . ("maxima.info-2" 221134 61158))
("Funciones y variables para polinomios ortogonales" . ("maxima.info-7" 300025 7609))
("Funciones y variables para romberg" . ("maxima.info-8" 312 7295))
("Funciones y variables para salida Fortran" . ("maxima.info-2" 214609 2806))
("Funciones y variables para salida TeX" . ("maxima.info-2" 204094 10291))
("Funciones y variables para series de Fourier" . ("maxima.info-4" 186807 3190))
("Funciones y variables para series de Poisson" . ("maxima.info-4" 190181 2614))
("Funciones y variables para simetrías" . ("maxima.info-4" 233206 38020))
("Funciones y variables para simplex" . ("maxima.info-8" 8468 3891))
("Funciones y variables para simplificación" . ("maxima.info-2" 5064 27860))
("Funciones y variables para solve_rec" . ("maxima.info-8" 36048 9225))
("Funciones y variables para stats" . ("maxima.info-8" 51994 35927))
("Funciones y variables para stirling" . ("maxima.info-8" 89822 2619))
("Funciones y variables para sumas y productos" . ("maxima.info-4" 144572 12496))
("Funciones y variables para teoría de números" . ("maxima.info-4" 193177 38247))
("Funciones y variables para to_poly_solve" . ("maxima.info-8" 125703 44060))
("Funciones y variables para units" . ("maxima.info-8" 173764 19969))
("Funciones y variables para worldmap" . ("maxima.info-6" 260337 12377))
("Funciones y variables para zeilberger" . ("maxima.info-8" 196286 5984))
("Funciones y variables sobre reglas y patrones" . ("maxima.info-4" 293976 39436))
("Función de error" . ("maxima.info-3" 36026 1799))
("Identificadores" . ("maxima.info-1" 261103 2155))
("Integral exponencial" . ("maxima.info-3" 34501 1381))
("Interrupciones" . ("maxima.info-4" 275373 1035))
("Introducción a QUADPACK" . ("maxima.info-3" 118451 4253))
("Introducción a atensor" . ("maxima.info-4" 134206 3924))
("Introducción a cobyla" . ("maxima.info-5" 205016 2175))
("Introducción a contrib_ode" . ("maxima.info-5" 211490 5386))
("Introducción a ctensor" . ("maxima.info-4" 72291 4508))
("Introducción a descriptive" . ("maxima.info-5" 226085 5080))
("Introducción a distrib" . ("maxima.info-6" 39700 6186))
("Introducción a draw" . ("maxima.info-6" 121865 436))
("Introducción a drawdf" . ("maxima.info-6" 273084 1720))
("Introducción a ezunits" . ("maxima.info-7" 17414 3785))
("Introducción a finance" . ("maxima.info-7" 57610 483))
("Introducción a fractals" . ("maxima.info-7" 69764 440))
("Introducción a graphs" . ("maxima.info-7" 78633 1015))
("Introducción a grobner" . ("maxima.info-7" 130487 1804))
("Introducción a interpol" . ("maxima.info-7" 147403 273))
("Introducción a itensor" . ("maxima.info-3" 295219 12131))
("Introducción a la base de datos de Maxima" . ("maxima.info-2" 88263 3253))
("Introducción a la definición de funciones" . ("maxima.info-5" 63208 95))
("Introducción a la evaluación" . ("maxima.info-1" 225889 3685))
("Introducción a la integración" . ("maxima.info-3" 91513 895))
("Introducción a la línea de comandos" . ("maxima.info-1" 40429 5488))
("Introducción a la miscelánea de opciones" . ("maxima.info-4" 284963 288))
("Introducción a la programación" . ("maxima.info-5" 141841 188))
("Introducción a la resolución numérica de ecuaciones diferenciales" . ("maxima.info-3" 217044 1180))
("Introducción a la simplificación" . ("maxima.info-2" 474 4391))
("Introducción a la transformada rápida de Fourier" . ("maxima.info-3" 201195 224))
("Introducción a lapack" . ("maxima.info-7" 161271 174))
("Introducción a las ecuaciones diferenciales" . ("maxima.info-3" 191757 459))
("Introducción a las expresiones" . ("maxima.info-1" 256900 1487))
("Introducción a las funciones e integrales elípticas" . ("maxima.info-3" 49598 3798))
("Introducción a las funciones especiales" . ("maxima.info-2" 283357 2011))
("Introducción a las matrices y el álgebra lineal" . ("maxima.info-3" 233265 213))
("Introducción a las series" . ("maxima.info-4" 157352 460))
("Introducción a las series de Fourier" . ("maxima.info-4" 186324 276))
("Introducción a lbfgs" . ("maxima.info-7" 181813 956))
("Introducción a linearalgebra" . ("maxima.info-7" 200226 4386))
("Introducción a los conjuntos" . ("maxima.info-5" 453 10601))
("Introducción a los gráficos" . ("maxima.info-2" 138629 1541))
("Introducción a los operadores" . ("maxima.info-1" 177886 5821))
("Introducción a los polinomios" . ("maxima.info-2" 217812 3153))
("Introducción a minpack" . ("maxima.info-7" 256294 193))
("Introducción a numericalio" . ("maxima.info-7" 264105 4392))
("Introducción a physical_constants" . ("maxima.info-7" 21389 4761))
("Introducción a polinomios ortogonales" . ("maxima.info-7" 281547 18149))
("Introducción a reglas y patrones" . ("maxima.info-4" 292874 950))
("Introducción a simplex" . ("maxima.info-8" 7989 367))
("Introducción a simplification" . ("maxima.info-8" 12837 274))
("Introducción a solve_rec" . ("maxima.info-8" 35297 631))
("Introducción a stats" . ("maxima.info-8" 45716 1077))
("Introducción a units" . ("maxima.info-8" 170418 3215))
("Introducción a zeilberger" . ("maxima.info-8" 194160 1977))
("Introducción al entorno de ejecución" . ("maxima.info-4" 274075 1110))
("Introducción al procesamiento de cadenas" . ("maxima.info-8" 92918 4300))
("Lisp y Maxima" . ("maxima.info-5" 136458 4229))
("Listas" . ("maxima.info-1" 122217 115))
("Macros" . ("maxima.info-5" 68997 9018))
("Nombres y verbos" . ("maxima.info-1" 258529 2445))
("Números" . ("maxima.info-1" 96416 109))
("Números aleatorios" . ("maxima.info-2" 84878 2825))
("Opciones gráficas" . ("maxima.info-2" 159334 13410))
("Opciones para Gnuplot" . ("maxima.info-2" 173065 3899))
("Operadores aritméticos" . ("maxima.info-1" 183887 9539))
("Operadores de asignación" . ("maxima.info-1" 204586 10018))
("Operadores definidos por el usuario" . ("maxima.info-1" 214806 10650))
("Operadores lógicos" . ("maxima.info-1" 196375 2835))
("Operadores para ecuaciones" . ("maxima.info-1" 199380 5017))
("Operadores relacionales" . ("maxima.info-1" 193641 2587))
("Paquete absimp" . ("maxima.info-8" 13241 972))
("Paquete facexp" . ("maxima.info-8" 14328 5316))
("Paquete functs" . ("maxima.info-8" 19802 6620))
("Paquete ineq" . ("maxima.info-8" 26616 4309))
("Paquete rducon" . ("maxima.info-8" 31056 1523))
("Paquete scifac" . ("maxima.info-8" 32709 1046))
("Paquete sqdnst" . ("maxima.info-8" 33863 1102))
("Posibles mejoras a contrib_ode" . ("maxima.info-5" 222225 974))
("Pruebas realizadas con contrib_ode" . ("maxima.info-5" 223371 684))
("Recolector de basura" . ("maxima.info-5" 140875 790))
("Referencias para contrib_ode" . ("maxima.info-5" 224178 1432))
("Share" . ("maxima.info-4" 285433 593))
("Visualización usando VTK" . ("maxima.info-7" 203 16632))
)))
(load-info-hashtables (maxima::maxima-load-pathname-directory) deffn-defvr-pairs section-pairs))
